@@ -1,5 +1,5 @@
 /*
- *  $Id: app.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: app.c 21435 2018-09-11 12:39:53Z yeti-dn $
  *  Copyright (C) 2003-2006,2013-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -241,20 +241,23 @@ gwy_app_main_window_get(void)
 
 /**
  * gwy_app_main_window_set:
- * @window: A window.
+ * @window: A window, presumably the Gwyddion toolbox.
  *
- * Sets Gwyddion main application window (toolbox) for
- * gwy_app_main_window_get().
+ * Sets the main application window.
  *
- * This function can be called only once and should be called at Gwyddion
- * startup so, ignore it.
+ * This function is probably only useful in Gwyddion itself and should be
+ * ignored.
+ *
+ * It needs to be called exactly once at Gwyddion startup.
+ * It restores the toolbox position and makes gwy_app_main_window_get() work.
  **/
 void
 gwy_app_main_window_set(GtkWidget *window)
 {
     g_return_if_fail(GTK_IS_WINDOW(window));
     if (gwy_app_main_window && window != gwy_app_main_window) {
-        g_critical("Trying to change app main window");
+        g_critical("The main window was already set to %p.",
+                   gwy_app_main_window);
         return;
     }
 
@@ -1684,52 +1687,25 @@ change_brick_preview(GwyDataWindow *data_window)
         return;
     }
 
-    if (type == BRICK_PREVIEW_MEAN) {
-        gwy_brick_mean_plane(brick, preview,
-                             0, 0, 0,
-                             brick->xres, brick->yres, -1,
-                             TRUE);
-    }
-    else if (type == BRICK_PREVIEW_MINIMUM) {
-        gwy_brick_min_plane(brick, preview,
-                            0, 0, 0,
-                            brick->xres, brick->yres, -1,
-                            TRUE);
-    }
-    else if (type == BRICK_PREVIEW_MAXIMUM) {
-        gwy_brick_max_plane(brick, preview,
-                            0, 0, 0,
-                            brick->xres, brick->yres, -1,
-                            TRUE);
-    }
-    else if (type == BRICK_PREVIEW_MINPOS) {
-        gwy_brick_minpos_plane(brick, preview,
-                               0, 0, 0,
-                               brick->xres, brick->yres, -1,
-                               TRUE);
-    }
-    else if (type == BRICK_PREVIEW_MAXPOS) {
-        gwy_brick_maxpos_plane(brick, preview,
-                               0, 0, 0,
-                               brick->xres, brick->yres, -1,
-                               TRUE);
-    }
-    else if (type == BRICK_PREVIEW_RMS) {
-        gwy_brick_rms_plane(brick, preview,
-                            0, 0, 0,
-                            brick->xres, brick->yres, -1,
-                            TRUE);
-    }
+    if (type == BRICK_PREVIEW_MEAN)
+        gwy_brick_mean_xy_plane(brick, preview);
+    else if (type == BRICK_PREVIEW_MINIMUM)
+        gwy_brick_min_xy_plane(brick, preview);
+    else if (type == BRICK_PREVIEW_MAXIMUM)
+        gwy_brick_max_xy_plane(brick, preview);
+    else if (type == BRICK_PREVIEW_MINPOS)
+        gwy_brick_minpos_xy_plane(brick, preview);
+    else if (type == BRICK_PREVIEW_MAXPOS)
+        gwy_brick_maxpos_xy_plane(brick, preview);
+    else if (type == BRICK_PREVIEW_RMS)
+        gwy_brick_rms_xy_plane(brick, preview);
     else if (type == BRICK_PREVIEW_CHANNEL) {
         GQuark quark = gwy_app_get_data_key_for_id(cid);
         GObject *field = gwy_container_get_object(cdata, quark);
-        gwy_serializable_clone(field, G_OBJECT(preview));
+        gwy_data_field_assign(preview, field);
     }
     else if (type == BRICK_PREVIEW_SECTION)
-        gwy_brick_extract_plane(brick, preview,
-                                0, 0, level,
-                                brick->xres, brick->yres, -1,
-                                TRUE);
+        gwy_brick_extract_xy_plane(brick, preview, level);
     else {
         g_return_if_reached();
     }
