@@ -1,5 +1,5 @@
 /*
- *  $Id: data-browser-aux.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: data-browser-aux.c 21430 2018-09-11 11:46:00Z yeti-dn $
  *  Copyright (C) 2006-2016 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1717,7 +1717,7 @@ sync_one_data_item(GwyContainer *source, GwyContainer *dest,
  *              them, %FALSE to copy only.
  * @...: 0-terminated list of #GwyDataItem values defining the items to copy.
  *
- * Synchronizes auxiliary channel items between data containers.
+ * Synchronizes auxiliary image data items between data containers.
  **/
 void
 gwy_app_sync_data_items(GwyContainer *source,
@@ -1753,7 +1753,7 @@ gwy_app_sync_data_items(GwyContainer *source,
  * @items: List of #GwyDataItem values defining the items to copy.
  * @nitems: Number of items in @items.
  *
- * Synchronizes auxiliary channel items between data containers.
+ * Synchronizes auxiliary image data items between data containers.
  *
  * Since: 2.48
  **/
@@ -1778,6 +1778,152 @@ gwy_app_sync_data_itemsv(GwyContainer *source,
         sync_one_data_item(source, dest, from_id, to_id, items[i], delete_too);
 }
 
+static void
+sync_one_volume_item(GwyContainer *source, GwyContainer *dest,
+                     gint from_id, gint to_id,
+                     GwyDataItem what, gboolean delete_too)
+{
+    gchar key_from[40];
+    gchar key_to[40];
+    const guchar *name;
+    GObject *obj;
+
+    switch (what) {
+        case GWY_DATA_ITEM_GRADIENT:
+        g_snprintf(key_from, sizeof(key_from),
+                   BRICK_PREFIX "/%d/preview/palette", from_id);
+        g_snprintf(key_to, sizeof(key_to),
+                   BRICK_PREFIX "/%d/preview/palette", to_id);
+        if (gwy_container_gis_string_by_name(source, key_from, &name))
+            gwy_container_set_string_by_name(dest, key_to, g_strdup(name));
+        else if (delete_too)
+            gwy_container_remove_by_name(dest, key_to);
+        break;
+
+        case GWY_DATA_ITEM_TITLE:
+        g_snprintf(key_from, sizeof(key_from),
+                   BRICK_PREFIX "/%d/title", from_id);
+        g_snprintf(key_to, sizeof(key_to),
+                   BRICK_PREFIX "/%d/title", to_id);
+        if (gwy_container_gis_string_by_name(source, key_from, &name))
+            gwy_container_set_string_by_name(dest, key_to, g_strdup(name));
+        else if (delete_too)
+            gwy_container_remove_by_name(dest, key_to);
+        break;
+
+        case GWY_DATA_ITEM_META:
+        g_snprintf(key_from, sizeof(key_from),
+                   BRICK_PREFIX "/%d/meta", from_id);
+        g_snprintf(key_to, sizeof(key_to),
+                   BRICK_PREFIX "/%d/meta", to_id);
+        if (gwy_container_gis_object_by_name(source, key_from, &obj)) {
+            obj = gwy_serializable_duplicate(obj);
+            gwy_container_set_object_by_name(dest, key_to, obj);
+            g_object_unref(obj);
+        }
+        else if (delete_too)
+            gwy_container_remove_by_name(dest, key_to);
+        break;
+
+        case GWY_DATA_ITEM_PREVIEW:
+        g_snprintf(key_from, sizeof(key_from),
+                   BRICK_PREFIX "/%d/preview", from_id);
+        g_snprintf(key_to, sizeof(key_to),
+                   BRICK_PREFIX "/%d/preview", to_id);
+        if (gwy_container_gis_object_by_name(source, key_from, &obj)) {
+            obj = gwy_serializable_duplicate(obj);
+            gwy_container_set_object_by_name(dest, key_to, obj);
+            g_object_unref(obj);
+        }
+        else if (delete_too)
+            gwy_container_remove_by_name(dest, key_to);
+        break;
+
+        default:
+        g_assert_not_reached();
+        break;
+    }
+}
+
+/**
+ * gwy_app_sync_volume_items:
+ * @source: Source container.
+ * @dest: Target container (may be identical to source).
+ * @from_id: Data number to copy items from.
+ * @to_id: Data number to copy items to.
+ * @delete_too: %TRUE to delete items in target if source does not contain
+ *              them, %FALSE to copy only.
+ * @...: 0-terminated list of #GwyDataItem values defining the items to copy.
+ *
+ * Synchronizes auxiliary volume data items between data containers.
+ *
+ * Only %GWY_DATA_ITEM_GRADIENT, %GWY_DATA_ITEM_TITLE, %GWY_DATA_ITEM_META
+ * and %GWY_DATA_ITEM_PREVIEW are valid items for volume data.
+ *
+ * Since: 2.51
+ **/
+void
+gwy_app_sync_volume_items(GwyContainer *source,
+                          GwyContainer *dest,
+                          gint from_id,
+                          gint to_id,
+                          gboolean delete_too,
+                          ...)
+{
+    GwyDataItem what;
+    va_list ap;
+
+    g_return_if_fail(GWY_IS_CONTAINER(source));
+    g_return_if_fail(GWY_IS_CONTAINER(dest));
+    g_return_if_fail(from_id >= 0 && to_id >= 0);
+    if (source == dest && from_id == to_id)
+        return;
+
+    va_start(ap, delete_too);
+    while ((what = va_arg(ap, GwyDataItem)))
+        sync_one_volume_item(source, dest, from_id, to_id, what, delete_too);
+    va_end(ap);
+}
+
+/**
+ * gwy_app_sync_volume_itemsv:
+ * @source: Source container.
+ * @dest: Target container (may be identical to source).
+ * @from_id: Data number to copy items from.
+ * @to_id: Data number to copy items to.
+ * @delete_too: %TRUE to delete items in target if source does not contain
+ *              them, %FALSE to copy only.
+ * @items: List of #GwyDataItem values defining the items to copy.
+ * @nitems: Number of items in @items.
+ *
+ * Synchronizes auxiliary volume data items between data containers.
+ *
+ * Only %GWY_DATA_ITEM_GRADIENT, %GWY_DATA_ITEM_TITLE, %GWY_DATA_ITEM_META
+ * and %GWY_DATA_ITEM_PREVIEW are valid items for volume data.
+ *
+ * Since: 2.51
+ **/
+void
+gwy_app_sync_volume_itemsv(GwyContainer *source,
+                           GwyContainer *dest,
+                           gint from_id,
+                           gint to_id,
+                           gboolean delete_too,
+                           const GwyDataItem *items,
+                           guint nitems)
+{
+    guint i;
+
+    g_return_if_fail(GWY_IS_CONTAINER(source));
+    g_return_if_fail(GWY_IS_CONTAINER(dest));
+    g_return_if_fail(from_id >= 0 && to_id >= 0);
+    if (source == dest && from_id == to_id)
+        return;
+
+    for (i = 0; i < nitems; i++)
+        sync_one_volume_item(source, dest, from_id, to_id, items[i], delete_too);
+}
+
 /**
  * gwy_app_data_browser_copy_channel:
  * @source: Source container.
@@ -2141,7 +2287,7 @@ _gwy_app_create_brick_preview_field(GwyBrick *brick)
     GwyDataField *preview = gwy_data_field_new(xres, yres, xreal, yreal,
                                                FALSE);
 
-    gwy_brick_mean_plane(brick, preview, 0, 0, 0, xres, yres, -1, FALSE);
+    gwy_brick_mean_xy_plane(brick, preview);
     return preview;
 }
 
@@ -2480,9 +2626,9 @@ gwy_app_get_graph_thumbnail(GwyContainer *data,
     if (!gtk_main_level())
         return NULL;
 
-    visual = gdk_visual_get_best();
+    visual = gdk_visual_get_system();
     g_return_val_if_fail(visual, NULL);
-    cmap = gdk_colormap_new(visual, FALSE);
+    cmap = gdk_colormap_get_system();
     g_return_val_if_fail(cmap, NULL);
 
     width = MAX(width, max_width);
