@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyappfilechooser.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: gwyappfilechooser.c 20867 2018-03-20 10:34:18Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -631,6 +631,9 @@ construct_glob_filter(GwyAppFileChooser *chooser, GtkBox *vbox)
     gtk_box_pack_start(GTK_BOX(hbox), check, FALSE, FALSE, 4);
     g_signal_connect(check, "toggled", G_CALLBACK(glob_case_changed), chooser);
     chooser->glob_case_check = check;
+
+    if (chooser->glob->len)
+        glob_entry_updated(GTK_ENTRY(entry), chooser);
 }
 
 static void
