@@ -1,6 +1,6 @@
 /*
- *  $Id: gwyappinternal.h 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2004-2016 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwyappinternal.h 21392 2018-09-05 07:53:07Z yeti-dn $
+ *  Copyright (C) 2004-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -116,10 +116,6 @@ typedef struct {
     gchar *message;
 } GwyAppLogMessage;
 
-/* XXX: Bad name.  Cannot fix it because according to the rules it is now
- * public API. */
-void     gwy_app_main_window_set              (GtkWidget *window);
-
 G_GNUC_INTERNAL
 gint     _gwy_app_get_n_recent_files          (void);
 
