@@ -1,5 +1,5 @@
 /*
- *  $Id: gwymoduleutils-file.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: gwymoduleutils-file.c 21435 2018-09-11 12:39:53Z yeti-dn $
  *  Copyright (C) 2007-2016 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -237,7 +237,6 @@ gwy_app_xyz_title_fall_back(GwyContainer *data,
 guint
 gwy_app_channel_remove_bad_data(GwyDataField *dfield, GwyDataField *mfield)
 {
-    GwySIUnit *funit, *munit;
     gdouble *m = gwy_data_field_get_data(mfield);
     guint i, mcount, n;
 
@@ -257,9 +256,8 @@ gwy_app_channel_remove_bad_data(GwyDataField *dfield, GwyDataField *mfield)
 
     gwy_data_field_set_xreal(mfield, gwy_data_field_get_xreal(dfield));
     gwy_data_field_set_yreal(mfield, gwy_data_field_get_yreal(dfield));
-    funit = gwy_data_field_get_si_unit_xy(dfield);
-    munit = gwy_data_field_get_si_unit_xy(mfield);
-    gwy_serializable_clone(G_OBJECT(funit), G_OBJECT(munit));
+    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(mfield),
+                       gwy_data_field_get_si_unit_xy(dfield));
 
     gwy_debug("mcount = %u", mcount);
 
@@ -1225,7 +1223,7 @@ add_import_log(GwyContainer *data,
  *                  multiple prefixes by separating them with newline ("\n").
  * @section_template: Section start template.  It must contain the character
  *                    "\x1a" in the place where the section name apprears.
- *                    Example: "[Section \x1a]".
+ *                    For example, "[Section \x1a]".
  * @endsection_template: Section end template.  It may or may not contain the
  *                       substitute character "\x1a" depending on whether the
  *                       section end markers contain the section name.  It is
