@@ -1,6 +1,6 @@
 /*
- *  $Id: gwymoduleutils.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2007-2016 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwymoduleutils.c 21435 2018-09-11 12:39:53Z yeti-dn $
+ *  Copyright (C) 2007-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -203,6 +203,158 @@ gwy_save_auxiliary_with_callback(const gchar *title,
     return FALSE;
 }
 
+/**
+ * gwy_module_data_load:
+ * @modname: Module name (determines the subdirectory).
+ * @filename: Name of the file to load.  In GLib encoding, but it really should
+ *            be just ASCII.
+ * @contents: Location to store the allocated file contents.  Use g_free() to
+ *            free it.  It is set to %NULL when the file cannot be loaded.
+ * @length: Location to the length of the contents in bytes, or %NULL.
+ * @error: Location for error, or %NULL.
+ *
+ * Load module data file from the user directory.
+ *
+ * The function wraps g_file_get_contents(), forming the full file name
+ * automatically.
+ *
+ * The error can be from %G_FILE_ERROR domain.  Usually, however, you only need
+ * the return value and consider the file simply not existing yet when the
+ * function fails.
+ *
+ * Returns: %TRUE if the file was loaded.
+ *
+ * Since: 2.51
+ **/
+gboolean
+gwy_module_data_load(const gchar *modname,
+                     const gchar *filename,
+                     gchar **contents,
+                     gsize *length,
+                     GError **error)
+{
+    gchar *path;
+    gboolean ok;
+
+    path = g_build_filename(gwy_get_user_dir(), modname, filename, NULL);
+    /* Do not bother creating the directory in the load function.  We only
+     * need it in the save function. */
+    ok = g_file_get_contents(path, contents, length, error);
+    g_free(path);
+
+    return ok;
+}
+
+static gboolean
+ensure_module_directory(const gchar *userdir,
+                        const gchar *modname,
+                        GError **error)
+{
+    gchar *path;
+
+    path = g_build_filename(userdir, modname, NULL);
+    if (g_file_test(path, G_FILE_TEST_IS_DIR))
+        return TRUE;
+
+    if (g_mkdir(path, 0700) == 0) {
+        g_free(path);
+        return TRUE;
+    }
+
+    g_set_error_literal(error, G_FILE_ERROR,
+                        g_file_error_from_errno(errno),
+                        g_strerror(errno));
+    g_free(path);
+    return FALSE;
+}
+
+/**
+ * gwy_module_data_save:
+ * @modname: Module name (determines the subdirectory).
+ * @filename: Name of the file to save.  In GLib encoding, but it really should
+ *            be just ASCII.
+ * @contents: File contents to write.
+ * @length: Length of @contents, or -1 if it is a NUL-terminated string.
+ * @error: Location for error, or %NULL.
+ *
+ * Saves module data file to the user directory.
+ *
+ * The function wraps g_file_set_contents(), forming the full file name
+ * automatically and handling subdirectory creation.
+ *
+ * The error can be from %G_FILE_ERROR domain.
+ *
+ * Returns: %TRUE if the file was saved.
+ *
+ * Since: 2.51
+ **/
+gboolean
+gwy_module_data_save(const gchar *modname,
+                     const gchar *filename,
+                     gchar *contents,
+                     gssize length,
+                     GError **error)
+{
+    const gchar *userdir;
+    gchar *path;
+    gboolean ok;
+
+    userdir = gwy_get_user_dir();
+    if (!ensure_module_directory(userdir, modname, error))
+        return FALSE;
+
+    path = g_build_filename(userdir, modname, filename, NULL);
+    ok = g_file_set_contents(path, contents, length, error);
+    g_free(path);
+
+    return ok;
+}
+
+/**
+ * gwy_module_data_fopen:
+ * @modname: Module name (determines the subdirectory).
+ * @filename: Name of the file to save.  In GLib encoding, but it really should
+ *            be just ASCII.
+ * @mode: File open mode, as in fopen().
+ * @error: Location for error, or %NULL.
+ *
+ * Opens a module data file in the user directory.
+ *
+ * The function wraps gwy_fopen(), forming the full file name automatically and
+ * handling subdirectory creation.
+ *
+ * The error can be from %G_FILE_ERROR domain.
+ *
+ * Returns: A file handle if the file was opened as requested, %NULL on
+ *          failure.
+ *
+ * Since: 2.51
+ **/
+FILE*
+gwy_module_data_fopen(const gchar *modname,
+                      const gchar *filename,
+                      const gchar *mode,
+                      GError **error)
+{
+    const gchar *userdir;
+    gchar *path;
+    FILE *fh;
+
+    userdir = gwy_get_user_dir();
+    if (!ensure_module_directory(userdir, modname, error))
+        return FALSE;
+
+    path = g_build_filename(userdir, modname, filename, NULL);
+    if (!(fh = gwy_fopen(path, mode))) {
+        g_set_error_literal(error, G_FILE_ERROR,
+                            g_file_error_from_errno(errno),
+                            g_strerror(errno));
+    }
+    g_free(path);
+
+    return fh;
+}
+
 /**
  * gwy_set_data_preview_size:
  * @data_view: A data view used for module preview.
@@ -636,8 +788,8 @@ gwy_preview_surface_to_datafield(GwySurface *surface,
 
     gwy_data_field_average_xyz(dfield, densitymap,
                                gwy_surface_get_data_const(surface), n);
-    gwy_serializable_clone(G_OBJECT(gwy_surface_get_si_unit_xy(surface)),
-                           G_OBJECT(gwy_data_field_get_si_unit_xy(dfield)));
+    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield),
+                       gwy_surface_get_si_unit_xy(surface));
 
     if (want_densitymap) {
         gwy_data_field_copy(densitymap, dfield, FALSE);
@@ -645,8 +797,8 @@ gwy_preview_surface_to_datafield(GwySurface *surface,
         gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), NULL);
     }
     else {
-        gwy_serializable_clone(G_OBJECT(gwy_surface_get_si_unit_z(surface)),
-                               G_OBJECT(gwy_data_field_get_si_unit_z(dfield)));
+        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield),
+                           gwy_surface_get_si_unit_z(surface));
     }
 }
 
