@@ -1,6 +1,6 @@
 /*
- *  $Id: gwyplaintool.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2006,2014 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwyplaintool.c 21343 2018-08-27 14:28:40Z yeti-dn $
+ *  Copyright (C) 2006-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -1223,17 +1223,27 @@ gwy_rect_selection_labels_fill(GwyRectSelectionLabels *rlabels,
         sel[2] = gwy_data_field_get_xreal(dfield);
         sel[3] = gwy_data_field_get_yreal(dfield);
         isel[0] = isel[1] = 0;
-        isel[2] = xres;
-        isel[3] = yres;
+        isel[2] = xres-1;
+        isel[3] = yres-1;
     }
 
+    sel[0] = gwy_data_field_jtor(dfield, isel[0]);
+    sel[1] = gwy_data_field_jtor(dfield, isel[1]);
+    sel[2] = gwy_data_field_jtor(dfield, isel[2]+1);
+    sel[3] = gwy_data_field_jtor(dfield, isel[3]+1);
+
+    /* Provide sorted real and pixel selection to the caller; but before we
+     * add offsets and convert to position & size.  The documentation has
+     * always said min and max pixel coordinates, so we must have res-1 there
+     * for full-size selections. */
     if (selreal)
         gwy_assign(selreal, sel, 4);
     if (selpix)
         gwy_assign(selpix, isel, 4);
 
-    sel[2] -= sel[0];
-    sel[3] -= sel[1];
+    isel[2] -= isel[0] - 1;
+    isel[3] -= isel[1] - 1;
+
     sel[0] += xoff;
     sel[1] += yoff;
 
@@ -1247,9 +1257,6 @@ gwy_rect_selection_labels_fill(GwyRectSelectionLabels *rlabels,
     }
     gwy_si_unit_value_format_free(vf);
 
-    isel[2] -= isel[0] - 1;
-    isel[3] -= isel[1] - 1;
-
     gtk_spin_button_set_range(rlabels->pix[RLABEL_X], 0.0, xres - 1.0);
     gtk_spin_button_set_range(rlabels->pix[RLABEL_W], 2.0, xres - isel[0]);
     gtk_spin_button_set_range(rlabels->pix[RLABEL_Y], 0.0, yres - 1.0);
