@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyresultsexport.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: gwyresultsexport.c 20808 2018-03-05 10:08:24Z yeti-dn $
  *  Copyright (C) 2017 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -294,7 +294,7 @@ gwy_results_export_new(GwyResultsReportType format)
     ResultsExportPriv *priv;
     GtkTooltips *tips;
 
-    rexport = g_object_newv(GWY_TYPE_RESULTS_EXPORT, 0, NULL);
+    rexport = g_object_new(GWY_TYPE_RESULTS_EXPORT, NULL);
     priv = rexport->priv;
 
     tips = gwy_app_get_tooltips();
