@@ -1,6 +1,6 @@
 /*
- *  $Id: wait.h 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2004-2017 David Necas (Yeti), Petr Klapetek.
+ *  $Id: wait.h 21407 2018-09-07 13:07:11Z yeti-dn $
+ *  Copyright (C) 2004-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,9 +29,9 @@ G_BEGIN_DECLS
 void     gwy_app_wait_start             (GtkWindow *window,
                                          const gchar *message);
 void     gwy_app_wait_finish            (void);
-gboolean gwy_app_wait_set_fraction      (gdouble fraction);
-gboolean gwy_app_wait_set_message       (const gchar *message);
-gboolean gwy_app_wait_set_message_prefix(const gchar *prefix);
+gboolean gwy_app_wait_set_fraction      (gdouble fraction)      G_GNUC_WARN_UNUSED_RESULT;
+gboolean gwy_app_wait_set_message       (const gchar *message)  G_GNUC_WARN_UNUSED_RESULT;
+gboolean gwy_app_wait_set_message_prefix(const gchar *prefix)   G_GNUC_WARN_UNUSED_RESULT;
 void     gwy_app_wait_cursor_start      (GtkWindow *window);
 void     gwy_app_wait_cursor_finish     (GtkWindow *window);
 gboolean gwy_app_wait_get_enabled       (void);
