@@ -15,8 +15,13 @@ AUTOMAKE=${AUTOMAKE:-automake}
 ACLOCAL=${ACLOCAL:-aclocal}
 AUTOHEADER=${AUTOHEADER:-autoheader}
 GETTEXT=${GETTEXT:-gettext}
-gettextize=$(which gettextize)
-GETTEXTIZE=${GETTEXTIZE:-$gettextize}
+if test -z "$GETTEXTIZE"; then
+    GETTEXTIZE=$(which gettextize 2>/dev/null)
+fi
+if test -z "$GETTEXTIZE"; then
+    # This will then presumably fail later.
+    GETTEXTIZE=gettextize
+fi
 
 get_version() {
   local v
