@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python2
 # Written by Yeti <yeti@gwyddion.net>. Public domain.
 import sys, re
 
@@ -22,6 +22,7 @@ for x in sys.argv[3:]:
 fh = file(section_file, 'r')
 lines = [line.strip() for line in fh.readlines()]
 fh.close()
+lines = [line for line in lines if line]
 
 def add_std(tostd, ucname):
     tostd.add('GWY_IS_' + ucname)
@@ -34,10 +35,6 @@ def add_std(tostd, ucname):
 sections = []
 section = None
 for line in lines:
-    if not line:
-        assert section is None
-        continue
-
     if line == '<SECTION>':
         assert section is None
         section = {'Symbols': []}
