@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwycontainer.c 19324 2016-11-28 22:24:36Z yeti-dn $
+ *  $Id: gwycontainer.c 20678 2017-12-18 18:26:55Z yeti-dn $
  *  Copyright (C) 2003,2004,2014 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -47,12 +47,19 @@ typedef struct {
     const gchar *prefix;
     gsize prefix_length;
     guint count;
-    GSList *keylist;
     gboolean closed_prefix;
     GHFunc func;
     gpointer user_data;
 } PrefixData;
 
+typedef struct {
+    const gchar *prefix;
+    gsize prefix_length;
+    GArray *keys;
+    gboolean closed_prefix;
+    gboolean by_name;
+} PrefixSearchData;
+
 typedef struct {
     GwyContainer *container;
     gint nprefixes;
@@ -94,9 +101,6 @@ static void          hash_serialize_func             (gpointer hkey,
 static GObject*      gwy_container_deserialize       (const guchar *buffer,
                                                       gsize size,
                                                       gsize *position);
-static gboolean      hash_remove_prefix_func         (gpointer hkey,
-                                                      gpointer hvalue,
-                                                      gpointer hdata);
 static void          hash_foreach_func               (gpointer hkey,
                                                       gpointer hvalue,
                                                       gpointer hdata);
@@ -106,6 +110,9 @@ static void          keys_foreach_func               (gpointer hkey,
 static void          keys_by_name_foreach_func       (gpointer hkey,
                                                       gpointer hvalue,
                                                       gpointer hdata);
+static void          keys_prefix_foreach_func        (gpointer hkey,
+                                                      gpointer hvalue,
+                                                      gpointer hdata);
 static GObject*      gwy_container_duplicate_real    (GObject *object);
 static void          gwy_container_clone_real        (GObject *source,
                                                       GObject *copy);
@@ -120,9 +127,6 @@ static GwyContainer* duplicate_by_prefix_do          (GwyContainer *container,
 static void          hash_prefix_duplicate_func      (gpointer hkey,
                                                       gpointer hvalue,
                                                       gpointer hdata);
-static void          hash_find_keys_func             (gpointer hkey,
-                                                      gpointer hvalue,
-                                                      gpointer hdata);
 
 static int      pstring_compare_callback         (const void *p,
                                                   const void *q);
@@ -355,6 +359,50 @@ gwy_container_remove(GwyContainer *container, GQuark key)
     return TRUE;
 }
 
+static gboolean
+prefix_is_closed(const gchar *prefix, guint prefix_length)
+{
+    if (!prefix || !prefix_length)
+        return TRUE;
+
+    return (prefix[prefix_length-1] == GWY_CONTAINER_PATHSEP);
+}
+
+static gboolean
+prefix_matches_key(const gchar *prefix, guint prefix_length, gboolean is_closed,
+                   GQuark key, const gchar **pname)
+{
+    const gchar *name;
+
+    /* Empty prefix matches everything.  But caller may need to obtain the
+     * string name himself. */
+    if (!prefix_length) {
+        if (pname)
+            *pname = NULL;
+        return TRUE;
+    }
+
+    name = g_quark_to_string(key);
+    if (pname)
+        *pname = name;
+
+    /* XXX: Odd case. */
+    if (!name)
+        return FALSE;
+
+    /* No match of the initial string parts means different. */
+    if (strncmp(name, prefix, prefix_length) != 0)
+        return FALSE;
+
+    /* An unclosed prefix must also terminate at the end or separator. */
+    if (!is_closed
+        && name[prefix_length] != '\0'
+        && name[prefix_length] != GWY_CONTAINER_PATHSEP)
+        return FALSE;
+
+    return TRUE;
+}
+
 /**
  * gwy_container_remove_by_prefix:
  * @container: A container.
@@ -369,51 +417,19 @@ gwy_container_remove(GwyContainer *container, GQuark key)
 guint
 gwy_container_remove_by_prefix(GwyContainer *container, const gchar *prefix)
 {
-    PrefixData pfdata;
-    GSList *l;
+    GQuark *keys;
+    guint i, n, signalid = container_signals[ITEM_CHANGED];
 
     g_return_val_if_fail(GWY_IS_CONTAINER(container), 0);
 
-    pfdata.container = container;
-    pfdata.prefix = prefix;
-    pfdata.prefix_length = prefix ? strlen(pfdata.prefix) : 0;
-    pfdata.count = 0;
-    pfdata.keylist = NULL;
-    pfdata.closed_prefix = !pfdata.prefix_length
-                           || (prefix[pfdata.prefix_length - 1]
-                               == GWY_CONTAINER_PATHSEP);
-    g_hash_table_foreach_remove(container->values, hash_remove_prefix_func,
-                                &pfdata);
-    pfdata.keylist = g_slist_reverse(pfdata.keylist);
-    for (l = pfdata.keylist; l; l = g_slist_next(l))
-        g_signal_emit(container, container_signals[ITEM_CHANGED],
-                      GPOINTER_TO_UINT(l->data), GPOINTER_TO_UINT(l->data));
-    g_slist_free(pfdata.keylist);
+    keys = gwy_container_keys_with_prefix(container, prefix, &n);
+    for (i = 0; i < n; i++)
+        g_hash_table_remove(container->values, GUINT_TO_POINTER(keys[i]));
+    for (i = 0; i < n; i++)
+        g_signal_emit(container, signalid, keys[i], keys[i]);
+    g_free(keys);
 
-    return pfdata.count;
-}
-
-static gboolean
-hash_remove_prefix_func(gpointer hkey,
-                        G_GNUC_UNUSED gpointer hvalue,
-                        gpointer hdata)
-{
-    GQuark key = GPOINTER_TO_UINT(hkey);
-    PrefixData *pfdata = (PrefixData*)hdata;
-    const gchar *name;
-
-    if (pfdata->prefix
-        && (!(name = g_quark_to_string(key))
-            || !g_str_has_prefix(name, pfdata->prefix)
-            || (!pfdata->closed_prefix
-                && name[pfdata->prefix_length] != '\0'
-                && name[pfdata->prefix_length] != GWY_CONTAINER_PATHSEP)))
-        return FALSE;
-
-    pfdata->count++;
-    pfdata->keylist = g_slist_prepend(pfdata->keylist,
-                                      GUINT_TO_POINTER(hkey));
-    return TRUE;
+    return n;
 }
 
 /**
@@ -442,13 +458,10 @@ gwy_container_foreach(GwyContainer *container,
     g_return_val_if_fail(function, 0);
 
     pfdata.container = container;
-    pfdata.prefix = prefix;
-    pfdata.prefix_length = prefix ? strlen(pfdata.prefix) : 0;
-    pfdata.closed_prefix = !pfdata.prefix_length
-                           || (prefix[pfdata.prefix_length - 1]
-                               == GWY_CONTAINER_PATHSEP);
+    pfdata.prefix_length = prefix ? strlen(prefix) : 0;
+    pfdata.prefix = pfdata.prefix_length ? prefix : NULL;
+    pfdata.closed_prefix = prefix_is_closed(prefix, pfdata.prefix_length);
     pfdata.count = 0;
-    pfdata.keylist = NULL;
     pfdata.func = function;
     pfdata.user_data = user_data;
     g_hash_table_foreach(container->values, hash_foreach_func, &pfdata);
@@ -462,18 +475,12 @@ hash_foreach_func(gpointer hkey, gpointer hvalue, gpointer hdata)
     GQuark key = GPOINTER_TO_UINT(hkey);
     GValue *value = (GValue*)hvalue;
     PrefixData *pfdata = (PrefixData*)hdata;
-    const gchar *name;
 
-    if (pfdata->prefix
-        && (!(name = g_quark_to_string(key))
-            || !g_str_has_prefix(name, pfdata->prefix)
-            || (!pfdata->closed_prefix
-                && name[pfdata->prefix_length] != '\0'
-                && name[pfdata->prefix_length] != GWY_CONTAINER_PATHSEP)))
-        return;
-
-    pfdata->func(hkey, value, pfdata->user_data);
-    pfdata->count++;
+    if (prefix_matches_key(pfdata->prefix, pfdata->prefix_length,
+                           pfdata->closed_prefix, key, NULL)) {
+        pfdata->func(hkey, value, pfdata->user_data);
+        pfdata->count++;
+    }
 }
 
 /**
@@ -565,6 +572,97 @@ keys_by_name_foreach_func(gpointer hkey,
     g_ptr_array_add(array, (gpointer)g_quark_to_string(key));
 }
 
+/**
+ * gwy_container_keys_with_prefix:
+ * @container: A container.
+ * @prefix: A nul-terminated id prefix.
+ * @n: Location to store the number of keys to.  Can be %NULL.
+ *
+ * Gets quark keys of a container that start with given prefix.
+ *
+ * Returns: A newly allocated array with quark keys, in no particular order.
+ *          The array is zero-terminated.
+ *
+ * Since: 2.50
+ **/
+GQuark*
+gwy_container_keys_with_prefix(GwyContainer *container,
+                               const gchar *prefix,
+                               guint *n)
+{
+    PrefixSearchData pfdata;
+    GQuark nokey = 0;
+
+    g_return_val_if_fail(GWY_IS_CONTAINER(container), NULL);
+    pfdata.keys = g_array_sized_new(FALSE, FALSE, sizeof(GQuark), 8);
+    pfdata.prefix_length = prefix ? strlen(prefix) : 0;
+    pfdata.prefix = pfdata.prefix_length ? prefix : NULL;
+    pfdata.closed_prefix = prefix_is_closed(prefix, pfdata.prefix_length);
+    pfdata.by_name = FALSE;
+    g_hash_table_foreach(container->values, keys_prefix_foreach_func, &pfdata);
+    if (n)
+        *n = pfdata.keys->len;
+    g_array_append_val(pfdata.keys, nokey);
+    return (GQuark*)g_array_free(pfdata.keys, FALSE);
+}
+
+/**
+ * gwy_container_keys_with_prefix_by_name:
+ * @container: A container.
+ * @prefix: A nul-terminated id prefix.
+ * @n: Location to store the number of keys to.  Can be %NULL.
+ *
+ * Gets string keys of a container that start with given prefix.
+ *
+ * Returns: A newly allocated array with string keys, in no particular order.
+ *          Unlike the array the strings are owned by GLib and must not be
+ *          freed.  The array is %NULL-terminated.
+ *
+ * Since: 2.50
+ **/
+const gchar**
+gwy_container_keys_with_prefix_by_name(GwyContainer *container,
+                                       const gchar *prefix,
+                                       guint *n)
+{
+    PrefixSearchData pfdata;
+    const gchar *nokey = NULL;
+
+    g_return_val_if_fail(GWY_IS_CONTAINER(container), NULL);
+    pfdata.keys = g_array_sized_new(FALSE, FALSE, sizeof(const gchar*), 8);
+    pfdata.prefix_length = prefix ? strlen(prefix) : 0;
+    pfdata.prefix = pfdata.prefix_length ? prefix : NULL;
+    pfdata.closed_prefix = prefix_is_closed(prefix, pfdata.prefix_length);
+    pfdata.by_name = TRUE;
+    g_hash_table_foreach(container->values, keys_prefix_foreach_func, &pfdata);
+    if (n)
+        *n = pfdata.keys->len;
+    g_array_append_val(pfdata.keys, nokey);
+    return (const gchar**)g_array_free(pfdata.keys, FALSE);
+}
+
+static void
+keys_prefix_foreach_func(gpointer hkey,
+                         G_GNUC_UNUSED gpointer hvalue,
+                         gpointer hdata)
+{
+    GQuark key = GPOINTER_TO_UINT(hkey);
+    PrefixSearchData *pfdata = (PrefixSearchData*)hdata;
+    const gchar *name;
+
+    if (!prefix_matches_key(pfdata->prefix, pfdata->prefix_length,
+                            pfdata->closed_prefix, key, &name))
+        return;
+
+    if (pfdata->by_name) {
+        if (!name)
+            name = g_quark_to_string(key);
+        g_array_append_val(pfdata->keys, name);
+    }
+    else
+        g_array_append_val(pfdata->keys, key);
+}
+
 /**
  * gwy_container_rename_by_name:
  * @c: A container.
@@ -2189,8 +2287,7 @@ duplicate_by_prefix_do(GwyContainer *container, PrefixListData *pfxlist)
         const gchar *prefix = pfxlist->prefixes[n];
         gsize len;
         len = pfxlist->pfxlengths[n] = strlen(prefix);
-        pfxlist->pfxclosed[n] = !len || (prefix[len - 1]
-                                         == GWY_CONTAINER_PATHSEP);
+        pfxlist->pfxclosed[n] = prefix_is_closed(prefix, len);
     }
 
     /* don't emit signals when no one can be connected */
@@ -2216,20 +2313,13 @@ hash_prefix_duplicate_func(gpointer hkey, gpointer hvalue, gpointer hdata)
     GwyContainer *duplicate;
     GType type = G_VALUE_TYPE(value);
     GObject *object;
-    const gchar *name;
     gsize n;
     gboolean ok = FALSE;
 
     duplicate = pfxlist->container;
-    name = g_quark_to_string(key);
-    if (!name)
-        return;
-
     for (n = 0; n < pfxlist->nprefixes; n++) {
-        if (strncmp(name, pfxlist->prefixes[n], pfxlist->pfxlengths[n]) == 0
-            && (pfxlist->pfxclosed[n]
-                || name[pfxlist->pfxlengths[n]] == '\0'
-                || name[pfxlist->pfxlengths[n]] == GWY_CONTAINER_PATHSEP)) {
+        if (prefix_matches_key(pfxlist->prefixes[n], pfxlist->pfxlengths[n],
+                               pfxlist->pfxclosed[n], key, NULL)) {
             ok = TRUE;
             break;
         }
@@ -2365,12 +2455,11 @@ gwy_container_transfer(GwyContainer *source,
                        const gchar *dest_prefix,
                        gboolean force)
 {
-    PrefixData pfdata;
     GValue *val, *copy;
     GString *key;
     GQuark quark;
-    guint dpflen;
-    GSList *l;
+    GQuark *srckeys;
+    guint n, i, count, spflen, dpflen;
 
     g_return_val_if_fail(GWY_IS_CONTAINER(source), 0);
     g_return_val_if_fail(GWY_IS_CONTAINER(dest), 0);
@@ -2384,38 +2473,32 @@ gwy_container_transfer(GwyContainer *source,
         g_return_val_if_fail(!g_str_has_prefix(dest_prefix, source_prefix), 0);
     }
 
-    pfdata.container = source;
-    pfdata.prefix = source_prefix;
-    pfdata.prefix_length = strlen(pfdata.prefix);
-    pfdata.count = 0;
-    pfdata.keylist = NULL;
-    pfdata.closed_prefix = !pfdata.prefix_length
-                           || (source_prefix[pfdata.prefix_length - 1]
-                               == GWY_CONTAINER_PATHSEP);
-    g_hash_table_foreach(source->values, hash_find_keys_func, &pfdata);
-    if (!pfdata.keylist)
+    srckeys = gwy_container_keys_with_prefix(source, source_prefix, &n);
+    if (!n) {
+        g_free(srckeys);
         return 0;
+    }
+
+    spflen = strlen(source_prefix);
+    if (prefix_is_closed(source_prefix, spflen))
+        spflen--;
 
-    pfdata.keylist = g_slist_reverse(pfdata.keylist);
     key = g_string_new(dest_prefix);
-    dpflen = strlen(dest_prefix);
+    dpflen = key->len;
     if (dest_prefix[dpflen - 1] == GWY_CONTAINER_PATHSEP)
         dpflen--;
-    if (pfdata.closed_prefix)
-        pfdata.prefix_length--;
 
-    pfdata.count = 0;
-    for (l = pfdata.keylist; l; l = g_slist_next(l)) {
-        val = (GValue*)g_hash_table_lookup(source->values, l->data);
+    count = 0;
+    for (i = 0; i < n; i++) {
+        val = (GValue*)g_hash_table_lookup(source->values,
+                                           GUINT_TO_POINTER(srckeys[i]));
         if (!val) {
             g_critical("Container contents changed during "
                        "gwy_container_transfer().");
             break;
         }
         g_string_truncate(key, dpflen);
-        g_string_append(key,
-                        g_quark_to_string(GPOINTER_TO_UINT(l->data))
-                        + pfdata.prefix_length);
+        g_string_append(key, g_quark_to_string(srckeys[i]) + spflen);
         quark = g_quark_from_string(key->str);
 
         if (!force && g_hash_table_lookup(dest->values, GUINT_TO_POINTER(quark)))
@@ -2427,33 +2510,12 @@ gwy_container_transfer(GwyContainer *source,
 
         g_hash_table_insert(dest->values, GUINT_TO_POINTER(quark), copy);
         g_signal_emit(dest, container_signals[ITEM_CHANGED], quark, quark);
-        pfdata.count++;
+        count++;
     }
-    g_slist_free(pfdata.keylist);
+    g_free(srckeys);
     g_string_free(key, TRUE);
 
-    return pfdata.count;
-}
-
-static void
-hash_find_keys_func(gpointer hkey,
-                    G_GNUC_UNUSED gpointer hvalue,
-                    gpointer hdata)
-{
-    GQuark key = GPOINTER_TO_UINT(hkey);
-    PrefixData *pfdata = (PrefixData*)hdata;
-    const gchar *name;
-
-    if (pfdata->prefix
-        && (!(name = g_quark_to_string(key))
-            || !g_str_has_prefix(name, pfdata->prefix)
-            || (!pfdata->closed_prefix
-                && name[pfdata->prefix_length] != GWY_CONTAINER_PATHSEP)))
-        return;
-
-    pfdata->count++;
-    pfdata->keylist = g_slist_prepend(pfdata->keylist,
-                                      GUINT_TO_POINTER(hkey));
+    return count;
 }
 
 static int
