@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyexpr.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: gwyexpr.c 21002 2018-04-27 09:11:40Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -41,9 +41,11 @@ typedef enum {
     GWY_EXPR_CODE_MODULO,
     GWY_EXPR_CODE_POWER,
     GWY_EXPR_CODE_POW,
+    GWY_EXPR_CODE_SPOW,
     GWY_EXPR_CODE_MIN,
     GWY_EXPR_CODE_MAX,
     GWY_EXPR_CODE_FMOD,
+    GWY_EXPR_CODE_STEP,
     GWY_EXPR_CODE_HYPOT,
     GWY_EXPR_CODE_ATAN2,
     GWY_EXPR_CODE_ABS,
@@ -54,6 +56,7 @@ typedef enum {
     GWY_EXPR_CODE_SIN,
     GWY_EXPR_CODE_COS,
     GWY_EXPR_CODE_TAN,
+    GWY_EXPR_CODE_SINC,
     GWY_EXPR_CODE_ASIN,
     GWY_EXPR_CODE_ACOS,
     GWY_EXPR_CODE_ATAN,
@@ -62,12 +65,38 @@ typedef enum {
     GWY_EXPR_CODE_LOG,
     GWY_EXPR_CODE_POW10,
     GWY_EXPR_CODE_LOG10,
+    GWY_EXPR_CODE_EXP2,
+    GWY_EXPR_CODE_LOG2,
     GWY_EXPR_CODE_COSH,
     GWY_EXPR_CODE_SINH,
     GWY_EXPR_CODE_TANH,
     GWY_EXPR_CODE_ACOSH,
     GWY_EXPR_CODE_ASINH,
     GWY_EXPR_CODE_ATANH,
+#if HAVE_ERF
+    GWY_EXPR_CODE_ERF,
+#endif
+#if HAVE_ERFC
+    GWY_EXPR_CODE_ERFC,
+#endif
+#if HAVE_LGAMMA
+    GWY_EXPR_CODE_LGAMMA,
+#endif
+#if HAVE_TGAMMA
+    GWY_EXPR_CODE_GAMMA,
+#endif
+#if HAVE_J0
+    GWY_EXPR_CODE_J0,
+#endif
+#if HAVE_J1
+    GWY_EXPR_CODE_J1,
+#endif
+#if HAVE_Y0
+    GWY_EXPR_CODE_Y0,
+#endif
+#if HAVE_Y1
+    GWY_EXPR_CODE_Y1,
+#endif
 } GwyExprOpCode;
 
 typedef enum {
@@ -128,45 +157,93 @@ struct _GwyExpr {
     guint slen;    /* allocated size */
 };
 
+static inline gdouble
+gwy_spow(gdouble x, gdouble p)
+{
+    if (x >= 0.0)
+        return pow(x, p);
+    return -pow(-x, p);
+}
+
 #define make_function_1_1(name) \
     static void gwy_expr_##name(gdouble **s) { **s = name(**s); }
 
 #define make_function_2_1(name) \
     static void gwy_expr_##name(gdouble **s) { --*s; **s = name(*(*s+1), **s); }
 
-make_function_1_1(sqrt)
-make_function_1_1(cbrt)
-make_function_1_1(sin)
-make_function_1_1(cos)
-make_function_1_1(tan)
-make_function_1_1(exp)
-make_function_1_1(log)
-make_function_1_1(pow10)
-make_function_1_1(log10)
-make_function_1_1(asin)
-make_function_1_1(acos)
-make_function_1_1(atan)
-make_function_1_1(cosh)
-make_function_1_1(sinh)
-make_function_1_1(tanh)
-make_function_1_1(acosh)
-make_function_1_1(asinh)
-make_function_1_1(atanh)
-make_function_1_1(fabs)
-make_function_1_1(floor)
-make_function_1_1(ceil)
-make_function_2_1(pow)
-make_function_2_1(hypot)
-make_function_2_1(atan2)
-make_function_2_1(fmod)
+make_function_1_1(sqrt);
+make_function_1_1(cbrt);
+make_function_1_1(sin);
+make_function_1_1(cos);
+make_function_1_1(tan);
+make_function_1_1(exp);
+make_function_1_1(log);
+make_function_1_1(pow10);
+make_function_1_1(log10);
+make_function_1_1(asin);
+make_function_1_1(acos);
+make_function_1_1(atan);
+make_function_1_1(cosh);
+make_function_1_1(sinh);
+make_function_1_1(tanh);
+make_function_1_1(acosh);
+make_function_1_1(asinh);
+make_function_1_1(atanh);
+make_function_1_1(fabs);
+make_function_1_1(floor);
+make_function_1_1(ceil);
+make_function_1_1(gwy_sinc);
+make_function_2_1(fmin);
+make_function_2_1(fmax);
+make_function_2_1(pow);
+make_function_2_1(hypot);
+make_function_2_1(atan2);
+make_function_2_1(fmod);
+make_function_2_1(gwy_spow);
+
+#if HAVE_ERF
+make_function_1_1(erf);
+#endif
+#if HAVE_ERFC
+make_function_1_1(erfc);
+#endif
+#if HAVE_LGAMMA
+make_function_1_1(lgamma);
+#endif
+#if HAVE_TGAMMA
+make_function_1_1(tgamma);
+#endif
+#if HAVE_J0
+make_function_1_1(j0);
+#endif
+#if HAVE_J1
+make_function_1_1(j1);
+#endif
+#if HAVE_Y0
+make_function_1_1(y0);
+#endif
+#if HAVE_Y1
+make_function_1_1(y1);
+#endif
 
 static void gwy_expr_negate(gdouble **s) { **s = -(**s); }
 static void gwy_expr_add(gdouble **s) { --*s; **s = *(*s+1) + **s; }
 static void gwy_expr_subtract(gdouble **s) { --*s; **s = *(*s+1) - **s; }
 static void gwy_expr_multiply(gdouble **s) { --*s; **s = *(*s+1) * **s; }
 static void gwy_expr_divide(gdouble **s) { --*s; **s = *(*s+1) / **s; }
-static void gwy_expr_max(gdouble **s) { --*s; **s = MAX(*(*s+1), **s); }
-static void gwy_expr_min(gdouble **s) { --*s; **s = MIN(*(*s+1), **s); }
+static void gwy_expr_step(gdouble **s) { **s = **s > 0.0; }
+
+#if HAVE_EXP2
+make_function_1_1(exp2)
+#else
+static void gwy_expr_exp2(gdouble **s) { **s = exp(G_LN2 * **s); }
+#endif
+
+#if HAVE_LOG2
+make_function_1_1(log2)
+#else
+static void gwy_expr_log2(gdouble **s) { **s = log(**s)/G_LN2; }
+#endif
 
 static const GwyExprFunction call_table[] = {
     { NULL,                NULL,     0,  0,  0                       },
@@ -178,9 +255,11 @@ static const GwyExprFunction call_table[] = {
     { gwy_expr_fmod,       "%",      2,  1,  GWY_EXPR_CODE_MODULO,   },
     { gwy_expr_pow,        "^",      2,  1,  GWY_EXPR_CODE_POWER,    },
     { gwy_expr_pow,        "pow",    2,  1,  GWY_EXPR_CODE_POW,      },
-    { gwy_expr_min,        "min",    2,  1,  GWY_EXPR_CODE_MIN,      },
-    { gwy_expr_max,        "max",    2,  1,  GWY_EXPR_CODE_MAX,      },
+    { gwy_expr_gwy_spow,   "spow",   2,  1,  GWY_EXPR_CODE_SPOW,     },
+    { gwy_expr_fmin,       "min",    2,  1,  GWY_EXPR_CODE_MIN,      },
+    { gwy_expr_fmax,       "max",    2,  1,  GWY_EXPR_CODE_MAX,      },
     { gwy_expr_fmod,       "mod",    2,  1,  GWY_EXPR_CODE_FMOD,     },
+    { gwy_expr_step,       "step",   1,  1,  GWY_EXPR_CODE_STEP,     },
     { gwy_expr_hypot,      "hypot",  2,  1,  GWY_EXPR_CODE_HYPOT,    },
     { gwy_expr_atan2,      "atan2",  2,  1,  GWY_EXPR_CODE_ATAN2,    },
     { gwy_expr_fabs,       "abs",    1,  1,  GWY_EXPR_CODE_ABS,      },
@@ -191,6 +270,7 @@ static const GwyExprFunction call_table[] = {
     { gwy_expr_sin,        "sin",    1,  1,  GWY_EXPR_CODE_SIN,      },
     { gwy_expr_cos,        "cos",    1,  1,  GWY_EXPR_CODE_COS,      },
     { gwy_expr_tan,        "tan",    1,  1,  GWY_EXPR_CODE_TAN,      },
+    { gwy_expr_gwy_sinc,   "sinc",   1,  1,  GWY_EXPR_CODE_SINC,     },
     { gwy_expr_asin,       "asin",   1,  1,  GWY_EXPR_CODE_ASIN,     },
     { gwy_expr_acos,       "acos",   1,  1,  GWY_EXPR_CODE_ACOS,     },
     { gwy_expr_atan,       "atan",   1,  1,  GWY_EXPR_CODE_ATAN,     },
@@ -199,12 +279,38 @@ static const GwyExprFunction call_table[] = {
     { gwy_expr_log,        "log",    1,  1,  GWY_EXPR_CODE_LOG,      },
     { gwy_expr_pow10,      "pow10",  1,  1,  GWY_EXPR_CODE_POW10,    },
     { gwy_expr_log10,      "log10",  1,  1,  GWY_EXPR_CODE_LOG10,    },
+    { gwy_expr_exp2,       "exp2",   1,  1,  GWY_EXPR_CODE_EXP2,     },
+    { gwy_expr_log2,       "log2",   1,  1,  GWY_EXPR_CODE_LOG2,     },
     { gwy_expr_cosh,       "cosh",   1,  1,  GWY_EXPR_CODE_COSH,     },
     { gwy_expr_sinh,       "sinh",   1,  1,  GWY_EXPR_CODE_SINH,     },
     { gwy_expr_tanh,       "tanh",   1,  1,  GWY_EXPR_CODE_TANH,     },
     { gwy_expr_acosh,      "acosh",  1,  1,  GWY_EXPR_CODE_ACOSH,    },
     { gwy_expr_asinh,      "asinh",  1,  1,  GWY_EXPR_CODE_ASINH,    },
     { gwy_expr_atanh,      "atanh",  1,  1,  GWY_EXPR_CODE_ATANH,    },
+#if HAVE_ERF
+    { gwy_expr_erf,        "erf",    1,  1,  GWY_EXPR_CODE_ERF,      },
+#endif
+#if HAVE_ERFC
+    { gwy_expr_erfc,       "erfc",   1,  1,  GWY_EXPR_CODE_ERFC,     },
+#endif
+#if HAVE_LGAMMA
+    { gwy_expr_lgamma,     "lGamma", 1,  1,  GWY_EXPR_CODE_LGAMMA,   },
+#endif
+#if HAVE_TGAMMA
+    { gwy_expr_tgamma,     "Gamma",  1,  1,  GWY_EXPR_CODE_GAMMA,    },
+#endif
+#if HAVE_J0
+    { gwy_expr_j0,         "J0",     1,  1,  GWY_EXPR_CODE_J0,       },
+#endif
+#if HAVE_J1
+    { gwy_expr_j1,         "J1",     1,  1,  GWY_EXPR_CODE_J1,       },
+#endif
+#if HAVE_Y0
+    { gwy_expr_y0,         "Y0",     1,  1,  GWY_EXPR_CODE_Y0,       },
+#endif
+#if HAVE_Y1
+    { gwy_expr_y1,         "Y1",     1,  1,  GWY_EXPR_CODE_Y1,       },
+#endif
 };
 
 /* Maximum number of function arguments */
