@@ -1,6 +1,6 @@
 /*
- *  $Id: gwymath.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwymath.c 21326 2018-08-23 13:55:17Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  The quicksort algorithm was copied from GNU C library,
@@ -25,6 +25,7 @@
 
 #include "config.h"
 #include <string.h>
+#include <stdlib.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
 
@@ -1647,6 +1648,194 @@ gwy_math_refine_maximum(const gdouble *z, gdouble *x, gdouble *y)
     return gwy_math_refine_maximum_2d(z, x, y);
 }
 
+static gboolean
+interpolate_parabolic(const GwyXY *xy, gdouble *x)
+{
+    gdouble u1 = (xy[1].x - xy[0].x)*(xy[2].y - xy[1].y);
+    gdouble u2 = (xy[2].x - xy[1].x)*(xy[0].y - xy[1].y);
+    gdouble tx;
+
+    if (fabs(u2 + u1) <= 1e-12*(fabs(u1) + fabs(u2)))
+        return FALSE;
+
+    tx = 0.5*(xy[1].x + (u2*xy[2].x + u1*xy[0].x)/(u2 + u1));
+    if (tx <= xy[0].x || tx >= xy[2].x)
+        return FALSE;
+
+    *x = tx;
+    return TRUE;
+}
+
+static int
+compare_double(const void *a, const void *b)
+{
+    const gdouble *da = (const gdouble*)a;
+    const gdouble *db = (const gdouble*)b;
+
+    if (*da < *db)
+        return -1;
+    if (*da > *db)
+        return 1;
+    return 0;
+}
+
+static gboolean
+find_min_in_array(const GwyXY *xy, guint n, guint *pimin)
+{
+    gboolean any_variation = FALSE;
+    gdouble y, yy;
+    guint imin, i;
+
+    imin = n/2;
+    for (i = 1; i <= n; i++) {
+        if (xy[i].y < xy[imin].y)
+            imin = i;
+    }
+
+    y = xy[imin].y;
+    if (imin > 0) {
+       yy = xy[imin-1].y;
+       if (yy - y > 6e-16*(fabs(y) + fabs(yy)))
+           any_variation = TRUE;
+    }
+    if (imin+1 < n) {
+       yy = xy[imin+1].y;
+       if (yy - y > 6e-16*(fabs(y) + fabs(yy)))
+           any_variation = TRUE;
+    }
+
+    *pimin = imin;
+    return any_variation;
+}
+
+/**
+ * gwy_math_find_minimum_1d:
+ * @function: Function to minimize.
+ * @a: First interval endpoint.
+ * @b: Second interval endpoint.
+ * @user_data: User data passed to @function.
+ *
+ * Finds a minimum of a real function in a finite interval.
+ *
+ * The function simply does what it says on the tin.  If there are multiple
+ * minima in [a,b] any of them can be returned, even though some effort to scan
+ * the interval is made.  There is no requiement for the minimum to lie
+ * inside [a,b]; if it occurrs at one of the endpoints, the endpoint is
+ * returned.
+ *
+ * Since: 2.51
+ **/
+gdouble
+gwy_math_find_minimum_1d(GwyRealFunc function,
+                         gdouble a, gdouble b,
+                         gpointer user_data)
+{
+    enum { initial_n = 12 };
+    GwyXY xy[initial_n+1];
+    gdouble x, y, xeps;
+    guint i, imin, n, iter;
+    gboolean at_left_edge = FALSE, at_right_edge = FALSE;
+
+    if (b < a)
+        GWY_SWAP(gdouble, a, b);
+    if (b-a < 1.2e-16*(fabs(a) + fabs(b)))
+        return 0.5*(a + b);
+
+    /* Initial scan of the interval. */
+    imin = 0;
+    xy[0].x = a;
+    xy[0].y = function(a, user_data);
+    for (i = 1; i <= initial_n; i++) {
+        x = (i == initial_n) ? b : b/initial_n*i + a/initial_n*(initial_n-i);
+        xy[i].x = x;
+        xy[i].y = y = function(x, user_data);
+    }
+
+    if (!find_min_in_array(xy, initial_n, &imin))
+        return 0.5*(a + b);
+
+    gwy_debug("initial minimum at %g, point #%u", xy[imin].x, imin);
+    /* Use the first 4-5 values to keep points while iterating. */
+    if (imin == 0)
+        at_left_edge = TRUE;
+    else if (imin == initial_n) {
+        at_right_edge = TRUE;
+        memmove(xy, xy + initial_n-2, 3*sizeof(GwyXY));
+    }
+    else
+        memmove(xy, xy + imin-1, 3*sizeof(GwyXY));
+
+    gwy_debug("initial subinterval [%g..%g]", xy[0].x, xy[2].x);
+
+    iter = 0;
+    while (xy[2].x - xy[0].x > 1.2e-15*(fabs(xy[0].x) + fabs(xy[2].x))) {
+        gwy_debug("new iter, interval [%.16g..%.16g] %g, edges: %d %d",
+                  xy[0].x, xy[2].x, xy[2].x - xy[0].x,
+                  at_left_edge, at_right_edge);
+        n = 3;
+        /* Just split the interval closer to edge when the minimum seems at
+         * the edge. */
+        if (at_left_edge)
+            xy[n++].x = 0.8*xy[0].x + 0.2*xy[1].x;
+        else if (at_right_edge)
+            xy[n++].x = 0.2*xy[1].x + 0.8*xy[2].x;
+        else {
+            /* Optimistic bisection of the larger interval, always try this
+             * point.  XXX: This is not very efficient, we can end up only
+             * improving the interval from the bisection side. */
+            if (xy[1].x - xy[0].x >= xy[2].x - xy[1].x) {
+                xy[n++].x = 0.2*xy[0].x + 0.8*xy[1].x;
+                gwy_debug("bisect-left %.16g (0.2)", xy[n-1].x);
+            }
+            else {
+                xy[n++].x = 0.8*xy[1].x + 0.2*xy[2].x;
+                gwy_debug("bisect-right %.16g (0.8)", xy[n-1].x);
+            }
+
+            /* Parabolic interpolation, use if it yields distinct point inside
+             * the interval. */
+            if (interpolate_parabolic(xy, &x)) {
+                xeps = 1.2e-15*(fabs(x));
+                if (x - xy[0].x > xeps
+                    && xy[2].x - x > xeps
+                    && fabs(x - xy[3].x) > xeps) {
+                    xy[n++].x = x;
+                    gwy_debug("parabolic %.16g (%g)",
+                              x, (x - xy[0].x)/(xy[2].x - xy[0].x));
+                }
+            }
+        }
+
+        /* Find the new three points bracketing the minimum.  We should not
+         * change state from non-edge to edge, but we can change state from
+         * edge to non-edge. */
+        for (i = 3; i < n; i++) {
+            xy[i].y = function(xy[i].x, user_data);
+            gwy_debug("point %.16g, value %.16g", xy[i].x, xy[i].y);
+        }
+        qsort(xy, n, sizeof(GwyXY), compare_double);
+
+        if (!find_min_in_array(xy, n, &imin))
+            return xy[imin].x;
+
+        gwy_debug("minimum at %g, point #%u (%g)", xy[imin].x, imin,
+                  (xy[imin].x - xy[0].x)/(xy[n-1].x - xy[0].x));
+        if (imin == 0)
+            at_left_edge = TRUE;
+        else if (imin == n-1) {
+            at_right_edge = TRUE;
+            memmove(xy, xy + n-3, 3*sizeof(GwyXY));
+        }
+        else
+            memmove(xy, xy + imin-1, 3*sizeof(GwyXY));
+
+        if (iter++ == 50)
+            break;
+    }
+
+    return xy[1].x;
+}
+
 static guint
 estimate_regular_res(gdouble *pos, gint n, gdouble *minpos, gdouble *maxpos)
 {
@@ -1925,6 +2114,27 @@ gwy_xlnx_int(guint x)
     return x*log(x);
 }
 
+/**
+ * gwy_sinc:
+ * @x: Value to calculate sinc (cardinal sine) of.
+ *
+ * Calculates the sinc function.
+ *
+ * The sinc function is equal to sin(@x)/@x for non-zero @x, and defined to the
+ * limit 1 for zero @x.
+ *
+ * Returns: Value of sinc(@x).
+ *
+ * Since: 2.51
+ **/
+gdouble
+gwy_sinc(gdouble x)
+{
+    if (G_LIKELY(fabs(x) > 3e-4))
+        return sin(x)/x;
+    return 1.0 - x*x/6.0;
+}
+
 /**
  * gwy_canonicalize_angle:
  * @phi: Angle to canonicalize, in radians.
@@ -2958,4 +3168,21 @@ gwy_math_median_uncertainty(gsize n, gdouble *array, gdouble *uarray)
  * Since: 2.22
  **/
 
+/**
+ * GwyPercentileInterpolationType:
+ * @GWY_PERCENTILE_INTERPOLATION_LINEAR: Linear interpolation of the two
+ *                                       nearest values.
+ * @GWY_PERCENTILE_INTERPOLATION_LOWER: Round the rank down to an integer.
+ * @GWY_PERCENTILE_INTERPOLATION_HIGHER: Round the rank up to an integer.
+ * @GWY_PERCENTILE_INTERPOLATION_NEAREST: Round the rank to nearest integer.
+ * @GWY_PERCENTILE_INTERPOLATION_MIDPOINT: Average of the two nearest values.
+ *
+ * Type of interpolation for percentile calculation.
+ *
+ * The interpolations are used when the percentile does not correspond exactly
+ * to a rank.
+ *
+ * Since: 2.50
+ **/
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
