@@ -1,6 +1,6 @@
 /*
- *  $Id: gwymath.h 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwymath.h 21052 2018-05-04 10:07:47Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -69,6 +69,8 @@ GwyXYZ* gwy_xyz_new     (gdouble x,
 GwyXYZ* gwy_xyz_copy    (const GwyXYZ *xyz) G_GNUC_MALLOC;
 void    gwy_xyz_free    (GwyXYZ *xyz);
 
+typedef gdouble (*GwyRealFunc)(gdouble x, gpointer user_data);
+
 gdouble  gwy_math_humanize_numbers     (gdouble unit,
                                         gdouble maximum,
                                         gint *precision);
@@ -129,6 +131,10 @@ gboolean gwy_math_refine_maximum_2d    (const gdouble *z,
 gboolean gwy_math_refine_maximum       (const gdouble *z,
                                         gdouble *x,
                                         gdouble *y);
+gdouble  gwy_math_find_minimum_1d      (GwyRealFunc function,
+                                        gdouble a,
+                                        gdouble b,
+                                        gpointer user_data);
 guint*   gwy_check_regular_2d_grid     (const gdouble *coords,
                                         guint stride,
                                         guint n,
@@ -168,9 +174,10 @@ gdouble  gwy_math_median_uncertainty   (gsize n,
                                         gdouble *array,
                                         gdouble *uarray);
 gdouble  gwy_xlnx_int                  (guint x)                               G_GNUC_CONST;
+gdouble  gwy_sinc                      (gdouble x)                             G_GNUC_CONST;
 gdouble  gwy_canonicalize_angle        (gdouble phi,
                                         gboolean positive,
-                                        gboolean oriented)                           G_GNUC_CONST;
+                                        gboolean oriented)                     G_GNUC_CONST;
 guint    gwy_math_histogram            (const gdouble *values,
                                         guint n,
                                         gdouble min,
