@@ -1,5 +1,5 @@
 /*
- *  $Id: gwymathfallback.h 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: gwymathfallback.h 21249 2018-07-23 13:38:45Z yeti-dn $
  *  Copyright (C) 2007-2016 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -49,7 +49,7 @@ _GWY_STATIC_INLINE int gwy_math_fallback_isnan(double x);
 static inline double
 gwy_math_fallback_pow10(double x)
 {
-    return /**/ pow(10.0, x);
+    return /**/ exp(G_LN10 * x);
 }
 
 static inline double
@@ -105,7 +105,11 @@ gwy_math_fallback_isnan(double x)
 #ifndef GWY_MATH_NAMESPACE_CLEAN
 
 #ifndef GWY_HAVE_POW10
+#ifdef GWY_HAVE_EXP10
+#define pow10 exp10
+#else
 #define pow10 gwy_math_fallback_pow10
+#endif  /* GWY_HAVE_EXP10 */
 #endif  /* GWY_HAVE_POW10 */
 
 #ifndef GWY_HAVE_CBRT
