@@ -1,5 +1,5 @@
 /*
- *  $Id: gwynlfit.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: gwynlfit.c 21443 2018-09-18 07:44:34Z yeti-dn $
  *  Copyright (C) 2000-2003 Martin Siler.
  *  Copyright (C) 2004-2016 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
@@ -386,7 +386,7 @@ gwy_math_nlfit_fit_real(GwyNLFitter *nlfit,
     GwyNLFitIdxFunc func_idx;
     GwyNLFitIdxDiffFunc diff_idx;
     gdouble mlambda = 1e-4;
-    gdouble sumr1, sumr = G_MAXDOUBLE;
+    gdouble sumr_new, sumr_best = G_MAXDOUBLE;
     gdouble *der = NULL, *v = NULL, *xr = NULL, *w = NULL,
             *saveparam = NULL, *origparam = NULL, *resid = NULL, *gcorr = NULL,
             *a = NULL, *save_a = NULL, *covar;
@@ -397,7 +397,7 @@ gwy_math_nlfit_fit_real(GwyNLFitter *nlfit,
     guint n_var_param;
     guint miter = 0;
     gboolean approx_geometric;
-    gboolean step1 = TRUE;
+    gboolean improved = TRUE;
     gboolean end = FALSE;
 
     g_return_val_if_fail(nlfit, -1.0);
@@ -455,14 +455,14 @@ gwy_math_nlfit_fit_real(GwyNLFitter *nlfit,
     for (i = 0; i < ndata; i++)
         gcorr[i] = 1.0;
 
-    sumr1 = gwy_math_nlfit_residua(func, func_idx, ndata, x, y, w, gcorr,
-                                   nparam, param, user_data, resid,
-                                   &nlfit->eval);
-    sumr = sumr1;
+    sumr_new = gwy_math_nlfit_residua(func, func_idx, ndata, x, y, w, gcorr,
+                                      nparam, param, user_data, resid,
+                                      &nlfit->eval);
+    sumr_best = sumr_new;
 
     if (!nlfit->eval) {
         g_warning("Initial residua evaluation failed");
-        sumr = -1.0;
+        sumr_best = -1.0;
         goto fail;
     }
 
@@ -471,7 +471,7 @@ gwy_math_nlfit_fit_real(GwyNLFitter *nlfit,
 
     if (set_fraction && !set_fraction(1.0/(nlfit->maxiter + 1))) {
         nlfit->eval = FALSE;
-        sumr = -2.0;
+        sumr_best = -2.0;
         goto fail;
     }
 
@@ -527,9 +527,9 @@ gwy_math_nlfit_fit_real(GwyNLFitter *nlfit,
         gboolean first_pass = TRUE;
         guint count = 0;
 
-        if (step1) {
+        if (improved) {
             mlambda *= nlfit->mdec;
-            sumr = sumr1;
+            sumr_best = sumr_new;
 
             gwy_clear(a, covar_size);
             gwy_clear(v, n_var_param);
@@ -604,7 +604,7 @@ gwy_math_nlfit_fit_real(GwyNLFitter *nlfit,
                 gwy_assign(saveparam, param, nparam);
             }
             else {
-                sumr = -1.0;
+                sumr_best = -1.0;
                 break;
             }
         }
@@ -658,36 +658,36 @@ gwy_math_nlfit_fit_real(GwyNLFitter *nlfit,
             break;
 
         /* See what the new residua is. */
-        sumr1 = gwy_math_nlfit_residua(func, func_idx, ndata, x, y, w, gcorr,
-                                       nparam, param, user_data, resid,
-                                       &nlfit->eval);
+        sumr_new = gwy_math_nlfit_residua(func, func_idx, ndata, x, y, w, gcorr,
+                                          nparam, param, user_data, resid,
+                                          &nlfit->eval);
         /* Catch failed evaluation even if it's not reported. */
-        if (gwy_isinf(sumr1) || gwy_isnan(sumr1)) {
+        if (gwy_isinf(sumr_new) || gwy_isnan(sumr_new)) {
             nlfit->eval = FALSE;
-            sumr = -1.0;
+            sumr_best = -1.0;
             break;
         }
 
         /* Good, we've finished */
-        if ((sumr1 == 0)
-            || (miter > 2 && fabs((sumr - sumr1)/sumr1) < EPS))
+        if ((sumr_new == 0)
+            || (miter > 2 && fabs((sumr_best - sumr_new)/sumr_new) < EPS))
             end = TRUE;
         /* Overshoot, increase lambda */
-        if (!nlfit->eval || sumr1 >= sumr) {
+        if (!nlfit->eval || sumr_new >= sumr_best) {
             mlambda *= nlfit->minc;
             if (mlambda == 0.0)
                 mlambda = nlfit->mtol;
-            step1 = FALSE;
+            improved = FALSE;
         }
         else
-            step1 = TRUE;
+            improved = TRUE;
 
         if (++miter >= nlfit->maxiter)
             break;
 
         if (set_fraction && !set_fraction((gdouble)miter/nlfit->maxiter)) {
             nlfit->eval = FALSE;
-            sumr = -2.0;
+            sumr_best = -2.0;
             break;
         }
     } while (!end);
@@ -717,19 +717,19 @@ gwy_math_nlfit_fit_real(GwyNLFitter *nlfit,
                 else
                     SLi(covar, i, i) = SLi(save_a, iid, iid);
             }
-            nlfit->dispersion = sumr/(ndata - n_var_param);
+            nlfit->dispersion = sumr_best/(ndata - n_var_param);
         }
         else {
             /* XXX: else what? */
             //g_warning("Cannot invert covariance matrix");
-            sumr = -1.0;
+            sumr_best = -1.0;
             GWY_FREE(nlfit->covar);
         }
     }
 
     for (i = 0; i < nparam; i++) {
         if (gwy_isinf(param[i]) || gwy_isnan(param[i])) {
-            sumr = -1.0;
+            sumr_best = -1.0;
             GWY_FREE(nlfit->covar);
             gwy_assign(param, origparam, nparam);
             break;
@@ -742,7 +742,7 @@ gwy_math_nlfit_fit_real(GwyNLFitter *nlfit,
         for (i = 0; i < nparam*(nparam + 1)/2; i++) {
              if (gwy_isinf(covar[i]) || gwy_isnan(covar[i])) {
                  priv->nparam = 0;
-                 sumr = -1.0;
+                 sumr_best = -1.0;
                  GWY_FREE(nlfit->covar);
                  gwy_assign(param, origparam, nparam);
                  break;
@@ -765,7 +765,7 @@ fail:
     g_free(w);
     g_free(lmap);
 
-    return sumr;
+    return sumr_best;
 }
 
 /**
