@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyresource.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: gwyresource.c 21205 2018-06-25 09:17:00Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -70,7 +70,7 @@ static gboolean     gwy_resource_get_is_const   (gconstpointer item);
 static const gchar* gwy_resource_get_item_name  (gpointer item);
 static gboolean     gwy_resource_compare        (gconstpointer item1,
                                                  gconstpointer item2);
-static void         gwy_resource_rename         (gpointer item,
+static void         gwy_resource_rename_impl    (gpointer item,
                                                  const gchar *new_name);
 static const GType* gwy_resource_get_traits     (gint *ntraits);
 static const gchar* gwy_resource_get_trait_name (gint i);
@@ -95,7 +95,7 @@ static const GwyInventoryItemType gwy_resource_item_type = {
     &gwy_resource_get_is_const,
     &gwy_resource_get_item_name,
     &gwy_resource_compare,
-    &gwy_resource_rename,
+    &gwy_resource_rename_impl,
     NULL,
     NULL,  /* copy needs particular class */
     &gwy_resource_get_traits,
@@ -259,8 +259,8 @@ gwy_resource_compare(gconstpointer item1,
 }
 
 static void
-gwy_resource_rename(gpointer item,
-                    const gchar *new_name)
+gwy_resource_rename_impl(gpointer item,
+                         const gchar *new_name)
 {
     GwyResource *resource = (GwyResource*)item;
 
@@ -737,6 +737,108 @@ gwy_resource_build_filename(GwyResource *resource)
                             klass->name, resource->name->str, NULL);
 }
 
+/**
+ * gwy_resource_delete:
+ * @resource: A resource.
+ *
+ * Deletes a resource, including removal from disk.
+ *
+ * The method deletes the resource both in the inventory and on disk.
+ * Constant resources cannot be deleted.
+ *
+ * Returns: %TRUE if the removal succeeded.
+ *
+ * Since: 2.51
+ **/
+gboolean
+gwy_resource_delete(GwyResource *resource)
+{
+    GwyInventory *inventory;
+    gchar *filename;
+    gint retval;
+
+    g_return_val_if_fail(GWY_IS_RESOURCE(resource), FALSE);
+
+    if (resource->is_const)
+        return FALSE;
+
+    inventory = GWY_RESOURCE_GET_CLASS(resource)->inventory;
+
+    filename = gwy_resource_build_filename(resource);
+    retval = g_remove(filename);
+    if (retval == 0)
+        gwy_inventory_delete_item(inventory, resource->name->str);
+    else {
+        /* FIXME: GUIze this */
+        g_warning("Cannot delete resource file: %s", filename);
+    }
+    g_free(filename);
+
+    return retval == 0;
+}
+
+/**
+ * gwy_resource_rename:
+ * @resource: A resource.
+ * @newname: New resource name.
+ *
+ * Renames a resource, including renaming it on disk.
+ *
+ * The method renames the resource both in the inventory and on disk. The
+ * renaming must not conflict with an existing resource, constant resources
+ * cannot be renamed, etc.  It is OK to rename a resource to the same name
+ * (nothing happens then).
+ *
+ * Returns: %TRUE if the renaming succeeded.
+ *
+ * Since: 2.51
+ **/
+gboolean
+gwy_resource_rename(GwyResource *resource,
+                    const gchar *newname)
+{
+    GwyInventory *inventory;
+    gchar *oldname, *oldfilename, *newfilename;
+    gpointer item;
+    gint retval;
+
+    g_return_val_if_fail(GWY_IS_RESOURCE(resource), FALSE);
+    g_return_val_if_fail(newname, FALSE);
+
+    if (strchr(newname, '/') || strchr(newname, '\\')) {
+        g_warning("Refusing to rename resource to name with special chars.");
+        return FALSE;
+    }
+
+    if (gwy_strequal(newname, resource->name->str))
+        return TRUE;
+    if (resource->is_const)
+        return FALSE;
+
+    inventory = GWY_RESOURCE_GET_CLASS(resource)->inventory;
+    item = gwy_inventory_get_item(inventory, newname);
+    if (item)
+        return FALSE;
+
+    oldname = g_strdup(resource->name->str);
+    oldfilename = gwy_resource_build_filename(resource);
+    gwy_inventory_rename_item(inventory, oldname, newname);
+    newfilename = gwy_resource_build_filename(resource);
+
+    retval = g_rename(oldfilename, newfilename);
+    if (retval != 0) {
+        /* FIXME: GUIze this */
+        g_warning("Cannot rename resource file: %s to %s",
+                  oldfilename, newfilename);
+        gwy_inventory_rename_item(inventory, newname, oldname);
+    }
+    g_free(newfilename);
+    g_free(oldfilename);
+    g_free(oldname);
+
+    return retval == 0;
+}
+
 /**
  * gwy_resource_class_mkdir:
  * @klass: A resource class.
