@@ -1,6 +1,6 @@
 /*
- *  $Id: gwyresults.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2017 David Necas (Yeti).
+ *  $Id: gwyresults.c 21414 2018-09-10 09:18:57Z yeti-dn $
+ *  Copyright (C) 2017-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -89,6 +89,8 @@ typedef struct {
     gboolean has_value : 1;  /* Negation of is_na, to default to zero. */
     gboolean has_error : 1;
     gshort power_t;
+    gshort power_u;
+    gshort power_v;
     gshort power_w;
     gshort power_x;
     gshort power_y;
@@ -111,6 +113,8 @@ struct _GwyResultsPrivate {
     GwySIValueFormat *vf;
     GString *str;
     GwySIUnit *tunit;
+    GwySIUnit *uunit;
+    GwySIUnit *vunit;
     GwySIUnit *wunit;
     GwySIUnit *xunit;
     GwySIUnit *yunit;
@@ -258,6 +262,8 @@ gwy_results_finalize(GObject *object)
     g_free(results->groups);
 
     GWY_OBJECT_UNREF(results->tunit);
+    GWY_OBJECT_UNREF(results->uunit);
+    GWY_OBJECT_UNREF(results->vunit);
     GWY_OBJECT_UNREF(results->wunit);
     GWY_OBJECT_UNREF(results->xunit);
     GWY_OBJECT_UNREF(results->yunit);
@@ -279,7 +285,7 @@ gwy_results_finalize(GObject *object)
 GwyResults*
 gwy_results_new(void)
 {
-    return g_object_newv(GWY_TYPE_RESULTS, 0, NULL);
+    return g_object_new(GWY_TYPE_RESULTS, NULL);
 }
 
 /**
@@ -365,6 +371,10 @@ gwy_results_copy(GwyResults *results)
 
     if (priv->tunit)
         priv->tunit = gwy_si_unit_duplicate(priv->tunit);
+    if (priv->uunit)
+        priv->uunit = gwy_si_unit_duplicate(priv->uunit);
+    if (priv->vunit)
+        priv->vunit = gwy_si_unit_duplicate(priv->vunit);
     if (priv->wunit)
         priv->wunit = gwy_si_unit_duplicate(priv->wunit);
     if (priv->xunit)
@@ -499,6 +509,10 @@ handle_value_properties(ResultLine *line, va_list ap)
         }
         else if (gwy_strequal(propname, "power-t"))
             line->power_t = va_arg(ap, gint);
+        else if (gwy_strequal(propname, "power-u"))
+            line->power_u = va_arg(ap, gint);
+        else if (gwy_strequal(propname, "power-v"))
+            line->power_v = va_arg(ap, gint);
         else if (gwy_strequal(propname, "power-w"))
             line->power_w = va_arg(ap, gint);
         else if (gwy_strequal(propname, "power-x"))
@@ -582,6 +596,8 @@ add_result_line_base(GwyResults *results,
  * <simplelist type='vert'>
  * <member><literal>"type"</literal>, value type (#GwyResultsValueType)</member>
  * <member><literal>"power-t"</literal>, power of unit @t (#gint)</member>
+ * <member><literal>"power-u"</literal>, power of unit @u (#gint, since 2.52)</member>
+ * <member><literal>"power-v"</literal>, power of unit @v (#gint, since 2.52)</member>
  * <member><literal>"power-x"</literal>, power of unit @x (#gint)</member>
  * <member><literal>"power-y"</literal>, power of unit @y (#gint)</member>
  * <member><literal>"power-z"</literal>, power of unit @z (#gint)</member>
@@ -1253,6 +1269,8 @@ transfer_unit(GwySIUnit *src, GwySIUnit **dest)
  * Sets the unit corresponding to one axis in a set of reported scalar values,
  * from an SI unit.
  *
+ * Unit names "u" and "v" are available since 2.52.
+ *
  * Since: 2.50
  **/
 void
@@ -1261,20 +1279,27 @@ gwy_results_set_unit(GwyResults *results,
                      GwySIUnit *unit)
 {
     ResultsPriv *priv;
+    gchar u;
 
     g_return_if_fail(GWY_IS_RESULTS(results));
     g_return_if_fail(name);
+    g_return_if_fail(strlen(name) == 1);
 
     priv = results->priv;
-    if (gwy_strequal(name, "t"))
+    u = name[0];
+    if (u == 't')
         transfer_unit(unit, &priv->tunit);
-    else if (gwy_strequal(name, "w"))
+    else if (u == 'u')
+        transfer_unit(unit, &priv->uunit);
+    else if (u == 'v')
+        transfer_unit(unit, &priv->vunit);
+    else if (u == 'w')
         transfer_unit(unit, &priv->wunit);
-    else if (gwy_strequal(name, "x"))
+    else if (u == 'x')
         transfer_unit(unit, &priv->xunit);
-    else if (gwy_strequal(name, "y"))
+    else if (u == 'y')
         transfer_unit(unit, &priv->yunit);
-    else if (gwy_strequal(name, "z"))
+    else if (u == 'z')
         transfer_unit(unit, &priv->zunit);
     else {
         g_warning("Ignoring unknown unit %s.", name);
@@ -1298,12 +1323,14 @@ transfer_unit_str(const gchar *unitstr, GwySIUnit **dest)
 /**
  * gwy_results_set_unit_str:
  * @results: A set of reported scalar values.
- * @name: Unit name, one of "t", "w", "x", "y" or "z".
+ * @name: Unit name, one of "t", "u", "v", "w", "x", "y" or "z".
  * @unitstr: The unit to use, as a string.  It can be %NULL.
  *
  * Sets the unit corresponding to one axis in a set of reported scalar values,
  * from a string.
  *
+ * Unit names "u" and "v" are available since 2.52.
+ *
  * Since: 2.50
  **/
 void
@@ -1312,20 +1339,27 @@ gwy_results_set_unit_str(GwyResults *results,
                          const gchar *unitstr)
 {
     ResultsPriv *priv;
+    gchar u;
 
     g_return_if_fail(GWY_IS_RESULTS(results));
     g_return_if_fail(name);
+    g_return_if_fail(strlen(name) == 1);
 
     priv = results->priv;
-    if (gwy_strequal(name, "t"))
+    u = name[0];
+    if (u == 't')
         transfer_unit_str(unitstr, &priv->tunit);
-    else if (gwy_strequal(name, "w"))
+    else if (u == 'u')
+        transfer_unit_str(unitstr, &priv->uunit);
+    else if (u == 'v')
+        transfer_unit_str(unitstr, &priv->vunit);
+    else if (u == 'w')
         transfer_unit_str(unitstr, &priv->wunit);
-    else if (gwy_strequal(name, "x"))
+    else if (u == 'x')
         transfer_unit_str(unitstr, &priv->xunit);
-    else if (gwy_strequal(name, "y"))
+    else if (u == 'y')
         transfer_unit_str(unitstr, &priv->yunit);
-    else if (gwy_strequal(name, "z"))
+    else if (u == 'z')
         transfer_unit_str(unitstr, &priv->zunit);
     else {
         g_warning("Ignoring unknown unit %s.", name);
@@ -1446,7 +1480,8 @@ gwy_results_fill_values_with_errors(GwyResults *results,
  * gwy_results_fill_format:
  * @results: A set of reported scalar values.
  * @id: Value identifier.
- * @...: 
+ * @...: List of names (string) and values (corresponding data type),
+ *       terminated by %NULL.
  *
  * Fills the values for a formatted line in a set of reported scalar values.
  *
@@ -1654,6 +1689,14 @@ construct_line_unit(ResultsPriv *results, ResultLine *line)
         gwy_si_unit_power_multiply(unit, 1, results->tunit, line->power_t,
                                    unit);
     }
+    if (line->power_u && results->uunit) {
+        gwy_si_unit_power_multiply(unit, 1, results->uunit, line->power_u,
+                                   unit);
+    }
+    if (line->power_v && results->vunit) {
+        gwy_si_unit_power_multiply(unit, 1, results->vunit, line->power_v,
+                                   unit);
+    }
     if (line->power_w && results->wunit) {
         gwy_si_unit_power_multiply(unit, 1, results->wunit, line->power_w,
                                    unit);
@@ -2129,26 +2172,6 @@ format_all_values(ResultsPriv *results,
     results->formatted_are_valid = TRUE;
 }
 
-static guint
-measure_fixed_font_width(const gchar *s)
-{
-    guint w = 0;
-    gunichar u;
-
-    while (s && *s) {
-        u = g_utf8_get_char(s);
-        if (g_unichar_iswide(u))
-            w += 2;
-        else if (g_unichar_iszerowidth(u))
-            w += 0;
-        else
-            w += 1;
-        s = g_utf8_next_char(s);
-    }
-
-    return w;
-}
-
 static void
 append_separator(GString *str, GwyResultsReportType base_type)
 {
@@ -2328,7 +2351,7 @@ append_covariance_matrix(GString *str, ResultLine *line,
             symbols[j].pos = symstr->len;
             append_approximate_pango_markup(symstr, s);
             symbols[j].len = symstr->len - symbols[j].pos;
-            symbols[j].width = measure_fixed_font_width(symstr->str
+            symbols[j].width = gwy_str_fixed_font_width(symstr->str
                                                         + symbols[j].pos);
             j++;
             g_string_append_c(symstr, '\n');
@@ -2461,7 +2484,7 @@ gwy_results_create_report(GwyResults *results,
 
             g_string_truncate(str, 0);
             append_label_with_symbol(str, line, TRUE, for_machine);
-            line->label_width = measure_fixed_font_width(str->str);
+            line->label_width = gwy_str_fixed_font_width(str->str);
             maxwidth = MAX(maxwidth, line->label_width);
         }
         padding = g_new(gchar, maxwidth+3);
@@ -3046,7 +3069,7 @@ gwy_format_result_table_strings(GString *str,
  * @str: String to append formatted header row to.
  * @report_type: Requested report style.
  * @n: Number of values.
- * @strings: Array of @n strings to format.
+ * @values: Array of @n strings to format.
  *
  * Formats a row of string tabular results.
  *
