@@ -1,6 +1,6 @@
 /*
- *  $Id: gwyserializable.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwyserializable.c 21432 2018-09-11 12:03:36Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -322,6 +322,38 @@ gwy_serializable_duplicate_hard_way(GObject *object)
     return duplicate;
 }
 
+static void
+gwy_serializable_clone_do(GObject *source,
+                          GObject *copy,
+                          GType assert_type)
+{
+    GType source_type, copy_type;
+    void (*clone_method)(GObject*, GObject*);
+
+    g_return_if_fail(GWY_IS_SERIALIZABLE(source));
+    g_return_if_fail(GWY_IS_SERIALIZABLE(copy));
+    if (!assert_type && source == copy)
+        return;
+
+    source_type = G_TYPE_FROM_INSTANCE(source);
+    copy_type = G_TYPE_FROM_INSTANCE(copy);
+    if (assert_type) {
+        g_return_if_fail(g_type_is_a(source_type, assert_type));
+        g_return_if_fail(g_type_is_a(copy_type, assert_type));
+        if (source == copy)
+            return;
+    }
+    g_return_if_fail(g_type_is_a(copy_type, source_type));
+
+    clone_method = GWY_SERIALIZABLE_GET_IFACE(copy)->clone;
+    if (!clone_method) {
+        g_critical("`%s' doesn't implement clone()",
+                   G_OBJECT_TYPE_NAME(copy));
+        return;
+    }
+    clone_method(source, copy);
+}
+
 /**
  * gwy_serializable_clone:
  * @source: An object implementing #GwySerializable interface.
@@ -336,25 +368,29 @@ void
 gwy_serializable_clone(GObject *source,
                        GObject *copy)
 {
-    GType source_type, copy_type;
-    void (*clone_method)(GObject*, GObject*);
+    gwy_serializable_clone_do(source, copy, 0);
+}
 
-    g_return_if_fail(GWY_IS_SERIALIZABLE(source));
-    g_return_if_fail(GWY_IS_SERIALIZABLE(copy));
-    if (source == copy)
-        return;
-
-    source_type = G_TYPE_FROM_INSTANCE(source);
-    copy_type = G_TYPE_FROM_INSTANCE(copy);
-    g_return_if_fail(g_type_is_a(copy_type, source_type));
-
-    clone_method = GWY_SERIALIZABLE_GET_IFACE(copy)->clone;
-    if (!clone_method) {
-        g_critical("`%s' doesn't implement clone()",
-                   G_OBJECT_TYPE_NAME(copy));
-        return;
-    }
-    clone_method(source, copy);
+/**
+ * gwy_serializable_clone_with_type:
+ * @source: An object implementing #GwySerializable interface.
+ * @copy: An object of the same type as @source to modify after it.
+ * @type: The type the objects are asserted to be.
+ *
+ * Makes an object identical to another object of the same type, with explicit
+ * type checking.
+ *
+ * This function is the same as gwy_serializable_clone(), except it asserts
+ * the object type, which must be @type.
+ *
+ * Since 2.52
+ **/
+void
+gwy_serializable_clone_with_type(GObject *source,
+                                 GObject *copy,
+                                 GType type)
+{
+    gwy_serializable_clone_do(source, copy, type);
 }
 
 /**
