@@ -1,6 +1,6 @@
 /*
- *  $Id: gwysiunit.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2004-2016 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwysiunit.c 21357 2018-08-28 09:42:31Z yeti-dn $
+ *  Copyright (C) 2004-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,17 +25,24 @@
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwyutils.h>
 #include <libgwyddion/gwymath.h>
+#include <libgwyddion/gwyenum.h>
 #include <libgwyddion/gwydebugobjects.h>
 #include <libgwyddion/gwyserializable.h>
 #include <libgwyddion/gwysiunit.h>
 
 #define GWY_SI_UNIT_TYPE_NAME "GwySIUnit"
 
+#define unit_index(u,i) g_array_index((u)->units,GwySimpleUnit,(i))
+
 enum {
     VALUE_CHANGED,
     LAST_SIGNAL
 };
 
+enum {
+    GWY_SI_UNIT_N_BASE = 7,
+};
+
 typedef struct {
     GQuark unit;
     gshort power;
@@ -58,44 +65,51 @@ typedef struct {
     const gchar *name;
 } GwyUnitLongName;
 
-static void         gwy_si_unit_finalize          (GObject *object);
-static void         gwy_si_unit_serializable_init (GwySerializableIface *iface);
-static GByteArray*  gwy_si_unit_serialize         (GObject *obj,
-                                                   GByteArray *buffer);
-static gsize        gwy_si_unit_get_size          (GObject *obj);
-static GObject*     gwy_si_unit_deserialize       (const guchar *buffer,
-                                                   gsize size,
-                                                   gsize *position);
-static GObject*     gwy_si_unit_duplicate_real    (GObject *object);
-static void         gwy_si_unit_clone_real        (GObject *source,
-                                                   GObject *copy);
-static gboolean     gwy_si_unit_parse             (GwySIUnit *siunit,
-                                                   const gchar *string);
-static GwySIUnit*   gwy_si_unit_power_real        (GwySIUnit *siunit,
-                                                   gint power,
-                                                   GwySIUnit *result);
-static GwySIUnit*   gwy_si_unit_canonicalize      (GwySIUnit *siunit);
-static const gchar* gwy_si_unit_prefix            (gint power);
-static void         gwy_si_unit_format            (GwySIUnit *siunit,
-                                                   const GwySIStyleSpec *fs,
-                                                   GwySIValueFormat *vf,
-                                                   gboolean magnitude_fixed);
-static void         format_power_plain            (GString *string,
-                                                   gint power);
-static void         format_power_pango            (GString *string,
-                                                   gint power);
-static void         format_power_TeX              (GString *string,
-                                                   gint power);
-static void         format_power_unicode          (GString *string,
-                                                   gint power);
+typedef struct {
+    const gchar *unit;
+    gint powers[GWY_SI_UNIT_N_BASE];
+    gdouble factor;
+} GwySIUnitDecomposition;
+
+static void         gwy_si_unit_finalize         (GObject *object);
+static void         gwy_si_unit_serializable_init(GwySerializableIface *iface);
+static GByteArray*  gwy_si_unit_serialize        (GObject *obj,
+                                                  GByteArray *buffer);
+static gsize        gwy_si_unit_get_size         (GObject *obj);
+static GObject*     gwy_si_unit_deserialize      (const guchar *buffer,
+                                                  gsize size,
+                                                  gsize *position);
+static GObject*     gwy_si_unit_duplicate_real   (GObject *object);
+static void         gwy_si_unit_clone_real       (GObject *source,
+                                                  GObject *copy);
+static gboolean     gwy_si_unit_parse            (GwySIUnit *siunit,
+                                                  const gchar *string);
+static GwySIUnit*   gwy_si_unit_power_real       (GwySIUnit *siunit,
+                                                  gint power,
+                                                  GwySIUnit *result);
+static GwySIUnit*   gwy_si_unit_canonicalize     (GwySIUnit *siunit);
+static gboolean     gwy_si_unit_equal_do         (GwySIUnit *siunit1,
+                                                  GwySIUnit *siunit2,
+                                                  gboolean strict);
+static gboolean     gwy_si_unit_equal_direct     (GwySIUnit *siunit1,
+                                                  GwySIUnit *siunit2);
+static const gchar* gwy_si_unit_prefix           (gint power);
+static void         gwy_si_unit_format           (GwySIUnit *siunit,
+                                                  const GwySIStyleSpec *fs,
+                                                  GwySIValueFormat *vf,
+                                                  gboolean magnitude_fixed);
+static void         format_power_plain           (GString *string,
+                                                  gint power);
+static void         format_power_pango           (GString *string,
+                                                  gint power);
+static void         format_power_TeX             (GString *string,
+                                                  gint power);
+static void         format_power_unicode         (GString *string,
+                                                  gint power);
 
-static const struct {
-    const gchar *prefix;
-    gint power10;
-}
 /* Canonical form must be always first, because this table is used for reverse
  * mapping too */
-SI_prefixes[] = {
+static const GwyEnum SI_prefixes[] = {
     { "k",     3  },
     { "c",    -2  },
     { "m",    -3  },
@@ -124,7 +138,7 @@ SI_prefixes[] = {
 
 /* Units that can conflict with prefixes */
 static const gchar *known_units[] = {
-    "deg", "Pa", "cd", "mol", "cal", "px", "pt", "cps", "cts",
+    "deg", "Pa", "cd", "mol", "cal", "px", "pt", "cps", "cts", "Gy", "Gauss"
 };
 
 /* Long names.  Keep the list sorted by length so that we can give up quickly
@@ -134,17 +148,23 @@ static const GwyUnitLongName long_names[] = {
     { 4, "V", "Volt", },
     { 4, "W", "Watt", },
     { 5, "Hz", "Hertz", },
+    { 5, "F", "Farad", },
+    { 5, "H", "Henry", },
     { 5, "J", "Joule", },
     { 5, "m", "meter", },
     { 5, "m", "metre", },
     { 5, "px", "pixel", },
+    { 5, "T", "Tesla", },
+    { 5, "Wb", "Weber", },
     { 6, "A", "Ampere", },
     { 6, "deg", "degree", },
     { 6, "K", "Kelvin", },
+    { 6, "µm", "micron", },
     { 6, "N", "Newton", },
     { 6, "Pa", "Pascal", },
     { 6, "s", "second", },
     { 7, "cd", "candela", },
+    { 7, "S", "Siemens", },
     { 8, "Å", "Angstrom", },
 };
 
@@ -158,11 +178,40 @@ static const GwyUnitLongName long_prefixes[] = {
     { 4, "p", "pico", },
     { 4, "T", "tera", },
     { 5, "c", "centi", },
-    { 5, "f", "fmeto", },
+    { 5, "f", "femto", },
     { 5, "µ", "micro", },
     { 5, "m", "milli", },
 };
 
+/* XXX: The base unit SI is kilogram.  But there is no way we can generally
+ * keep the kilo prefix there, so we decompose to prefixable units -- and
+ * prefixable unit is gram.  Yes, this means we are not techncally decomposing
+ * to *base* SI units.  */
+static const gchar *base_si_units[GWY_SI_UNIT_N_BASE] = {
+    "m", "g", "s", "A", "K", "mol", "cd",
+};
+
+static const GwySIUnitDecomposition derived_units[] = {
+    /*           m   g   s   A   K mol  cd   */
+    { "Hz",   {  0,  0, -1,  0,  0,  0,  0 }, 1.0            },
+    { "N",    {  1,  1, -2,  0,  0,  0,  0 }, 1.0e3          },
+    { "Pa",   { -1,  1, -2,  0,  0,  0,  0 }, 1.0e3          },
+    { "J",    {  2,  1, -2,  0,  0,  0,  0 }, 1.0e3          },
+    { "eV",   {  2,  1, -2,  0,  0,  0,  0 }, 1.60217653e-16 },
+    { "W",    {  2,  1, -3,  0,  0,  0,  0 }, 1.0e3          },
+    { "C",    {  0,  0,  1,  1,  0,  0,  0 }, 1.0            },
+    { "V",    {  2,  1, -3, -1,  0,  0,  0 }, 1.0e3          },
+    { "F",    { -2, -1,  4,  2,  0,  0,  0 }, 1.0e-3         },
+    { "H",    {  2,  1, -2, -2,  0,  0,  0 }, 1.0e3          },
+    { "Ω",    {  2,  1, -3, -2,  0,  0,  0 }, 1.0e3          },
+    { "S",    { -2, -1,  3,  2,  0,  0,  0 }, 1.0e-3         },
+    { "T",    {  0,  1, -2, -1,  0,  0,  0 }, 1.0e3          },
+    { "Wb",   {  2,  1, -2, -1,  0,  0,  0 }, 1.0e3          },
+    { "Bq",   {  0,  0, -1,  0,  0,  0,  0 }, 1.0            },
+    { "Sv",   {  2,  0, -2,  0,  0,  0,  0 }, 1.0            },
+    { "Gy",   {  2,  0, -2,  0,  0,  0,  0 }, 1.0            },
+};
+
 /* Unit formats */
 static const GwySIStyleSpec format_style_plain = {
     &format_power_plain, NULL, " ", "/", " "
@@ -350,7 +399,7 @@ gwy_si_unit_clone_real(GObject *source, GObject *copy)
 
     si_unit = GWY_SI_UNIT(source);
     clone = GWY_SI_UNIT(copy);
-    if (gwy_si_unit_equal(si_unit, clone))
+    if (gwy_si_unit_equal_direct(si_unit, clone))
         return;
 
     g_array_set_size(clone->units, 0);
@@ -457,7 +506,8 @@ gwy_si_unit_set_from_string_parse(GwySIUnit *siunit,
 static inline const GwySIStyleSpec*
 gwy_si_unit_find_style_spec(GwySIUnitFormatStyle style)
 {
-    if ((guint)style >= G_N_ELEMENTS(format_styles)) {
+    if (style == GWY_SI_UNIT_FORMAT_NONE
+        || (guint)style >= G_N_ELEMENTS(format_styles)) {
         g_critical("Invalid format style %d.", style);
         style = GWY_SI_UNIT_FORMAT_PLAIN;
     }
@@ -698,34 +748,8 @@ gwy_si_unit_get_format_with_digits(GwySIUnit *siunit,
 gboolean
 gwy_si_unit_equal(GwySIUnit *siunit1, GwySIUnit *siunit2)
 {
-    guint i, j;
-
-    if (siunit2 == siunit1)
-        return TRUE;
-
-    g_return_val_if_fail(GWY_IS_SI_UNIT(siunit1), FALSE);
-    g_return_val_if_fail(GWY_IS_SI_UNIT(siunit2), FALSE);
-
-    if (siunit2->units->len != siunit1->units->len)
-        return FALSE;
-
-    for (i = 0; i < siunit1->units->len; i++) {
-        GwySimpleUnit *unit = &g_array_index(siunit1->units, GwySimpleUnit, i);
-
-        for (j = 0; j < siunit2->units->len; j++) {
-            if (g_array_index(siunit2->units, GwySimpleUnit, j).unit
-                == unit->unit) {
-                if (g_array_index(siunit2->units, GwySimpleUnit, j).power
-                    != unit->power)
-                    return FALSE;
-                break;
-            }
-        }
-        if (j == siunit2->units->len)
-            return FALSE;
-    }
-
-    return TRUE;
+    /* Here we do not want eV to be equal to J. */
+    return gwy_si_unit_equal_do(siunit1, siunit2, TRUE);
 }
 
 /**
@@ -756,14 +780,14 @@ gwy_si_unit_equal_string(GwySIUnit *siunit,
         return !siunit->units->len;
 
     if (siunit->units->len == 1) {
-        GwySimpleUnit *unit = &g_array_index(siunit->units, GwySimpleUnit, 0);
+        GwySimpleUnit *unit = &unit_index(siunit, 0);
         if (gwy_strequal(g_quark_to_string(unit->unit), unit_string))
             return TRUE;
         /* If they differ we cannot make any conclusion. */
     }
 
     tmpunit = gwy_si_unit_new_parse(unit_string, NULL);
-    retval = gwy_si_unit_equal(siunit, tmpunit);
+    retval = gwy_si_unit_equal_do(siunit, tmpunit, FALSE);
     g_object_unref(tmpunit);
 
     return retval;
@@ -776,7 +800,9 @@ fix_unit_name(GString *str)
     const gchar *s = str->str;
     guint l = str->len;
 
-    if (s[0] == '\272') {
+    /* b0 = degree sign
+     * ba = masculine ordinal indicator */
+    if (s[0] == '\xb0' || s[0] == '\xba') {
         if (!s[1])
             g_string_assign(str, "deg");
         else {
@@ -951,11 +977,11 @@ gwy_si_unit_parse(GwySIUnit *siunit,
 
         if (may_split_prefix) {
             for (i = 0; i < G_N_ELEMENTS(SI_prefixes); i++) {
-                const gchar *pfx = SI_prefixes[i].prefix;
+                const gchar *pfx = SI_prefixes[i].name;
 
                 if (g_str_has_prefix(buf->str, pfx)
                     && g_ascii_isalpha(buf->str[strlen(pfx)])) {
-                    pfpower = SI_prefixes[i].power10;
+                    pfpower = SI_prefixes[i].value;
                     g_string_erase(buf, 0, strlen(pfx));
                     break;
                 }
@@ -1205,7 +1231,7 @@ gwy_si_unit_nth_root(GwySIUnit *siunit,
 
     /* Check applicability */
     for (j = 0; j < siunit->units->len; j++) {
-        unit = &g_array_index(siunit->units, GwySimpleUnit, j);
+        unit = &unit_index(siunit, j);
         if (unit->power % ipower != 0)
             return NULL;
     }
@@ -1296,10 +1322,10 @@ gwy_si_unit_power_multiply(GwySIUnit *siunit1,
 
     result->power10 += power2*siunit2->power10;
     for (i = 0; i < siunit2->units->len; i++) {
-        unit2 = &g_array_index(siunit2->units, GwySimpleUnit, i);
+        unit2 = &unit_index(siunit2, i);
 
         for (j = 0; j < result->units->len; j++) {
-            unit = &g_array_index(result->units, GwySimpleUnit, j);
+            unit = &unit_index(result, j);
             gwy_debug("[%d] %u == [%d] %u",
                       i, unit2->unit, j, unit->unit);
             if (unit2->unit == unit->unit) {
@@ -1321,6 +1347,106 @@ gwy_si_unit_power_multiply(GwySIUnit *siunit1,
     return result;
 }
 
+/**
+ * gwy_si_unit_factor_to_base:
+ * @siunit: An SI unit.
+ * @result: An SI unit to set to decomposed @siunit.  It is safe to pass
+ *          @siunit itself.  It can be %NULL too, a new SI unit is created
+ *          then and returned.
+ * @mfactor: Location to store multiplicative factor between @siunit and
+ *           @result.  For instance, for electronvolt the value
+ *           1.60217653e-16 would be stored (the factor 1000 comes from
+ *           kilogram).  Pass %NULL if you are only interested dimension
+ *           equality.
+ *
+ * Factors a possibly derived SI unit to base units.
+ *
+ * For instance, if @siunit was set to "N/m" the result will be "kg/s^2".
+ *
+ * Normally the result will consist only of the base seven SI units.  However,
+ * recognised non-SI units (and pseudounits) in @siunit, such as "px" are left
+ * intact in the decomposition.
+ *
+ * Also note that the decomposition is done to prefixable units. Kilogram is
+ * not prefixable (gram is) and there is no general way to keep the kilo- on
+ * the kilograms when deriving units for different powers of 10.  Therefore,
+ * the calculated factor corresponds to decomposition to grams.
+ *
+ * You must multiply the corresponding data with @factor if you intend to use
+ * the @result for them instead of @unit!
+ *
+ * Returns: When @result is %NULL, a newly created SI unit that has to be
+ *          dereferenced when no longer used later.  Otherwise @result itself
+ *          is simply returned, its reference count is NOT increased.
+ *
+ * Since: 2.51
+ **/
+GwySIUnit*
+gwy_si_unit_factor_to_base(GwySIUnit *siunit,
+                           GwySIUnit *result,
+                           gdouble *mfactor)
+{
+    GwySIUnit *factored = NULL;
+    GwySimpleUnit *unit;
+    GwySimpleUnit bunit;
+    gdouble mf = 1.0;
+    guint i, j, k;
+
+    g_return_val_if_fail(GWY_IS_SI_UNIT(siunit), NULL);
+    g_return_val_if_fail(!result || GWY_IS_SI_UNIT(result), NULL);
+
+    if (!result)
+        result = gwy_si_unit_new(NULL);
+    else if (result == siunit)
+        result = factored = gwy_si_unit_new(NULL);
+    else
+        g_array_set_size(result->units, 0);
+
+    for (i = 0; i < siunit->units->len; i++) {
+        unit = &unit_index(siunit, i);
+        /* Try to find the simple unit in known derived_units[] */
+        for (j = 0; j < G_N_ELEMENTS(derived_units); j++) {
+            if (gwy_strequal(g_quark_to_string(unit->unit),
+                             derived_units[j].unit)) {
+                for (k = 0; k < GWY_SI_UNIT_N_BASE; k++) {
+                    if (!derived_units[j].powers[k])
+                        continue;
+
+                    bunit.unit = g_quark_from_static_string(base_si_units[k]);
+                    bunit.power = derived_units[k].powers[k] * unit->power;
+                    bunit.traits = 0;
+                    g_array_append_val(result->units, bunit);
+                    mf *= derived_units[k].factor * unit->power;
+                }
+                break;
+            }
+        }
+        /* If we cannot find it, just copy it over. */
+        if (j == G_N_ELEMENTS(derived_units)) {
+            bunit = *unit;
+            g_array_append_val(result->units, bunit);
+        }
+    }
+
+    gwy_si_unit_canonicalize(result);
+    /* If caller passes @result == @siunit, we have to modify @siunit and
+     * throw away the allocated unit.  Otherwise @result is either some other
+     * unit or newly allocated and we just return it.  */
+    if (result == factored) {
+        g_array_set_size(siunit->units, 0);
+        g_array_append_vals(siunit->units,
+                            result->units->data, result->units->len);
+        result = siunit;
+        g_object_unref(factored);
+    }
+
+    if (mfactor)
+        *mfactor = mf;
+
+    return result;
+}
+
+/* FIXME: Consider sorting SI units g, m, s, A, cd, mol. */
 static GwySIUnit*
 gwy_si_unit_canonicalize(GwySIUnit *siunit)
 {
@@ -1330,10 +1456,10 @@ gwy_si_unit_canonicalize(GwySIUnit *siunit)
     /* consolidate multiple occurences of the same unit */
     i = 0;
     while (i < siunit->units->len) {
-        src = &g_array_index(siunit->units, GwySimpleUnit, i);
+        src = &unit_index(siunit, i);
 
         for (j = 0; j < i; j++) {
-            dst = &g_array_index(siunit->units, GwySimpleUnit, j);
+            dst = &unit_index(siunit, j);
             if (src->unit == dst->unit) {
                 dst->power += src->power;
                 g_array_remove_index(siunit->units, i);
@@ -1348,7 +1474,7 @@ gwy_si_unit_canonicalize(GwySIUnit *siunit)
     /* remove units with zero power */
     i = 0;
     while (i < siunit->units->len) {
-        if (g_array_index(siunit->units, GwySimpleUnit, i).power)
+        if (unit_index(siunit, i).power)
             i++;
         else {
             g_array_remove_index(siunit->units, i);
@@ -1358,6 +1484,64 @@ gwy_si_unit_canonicalize(GwySIUnit *siunit)
     return siunit;
 }
 
+static gboolean
+gwy_si_unit_equal_do(GwySIUnit *siunit1, GwySIUnit *siunit2,
+                     gboolean strict)
+{
+    GwySIUnit *factored1, *factored2;
+    gdouble mf1 = 0.0, mf2 = 0.0;
+    gboolean result = FALSE;
+
+    if (siunit2 == siunit1)
+        return TRUE;
+
+    g_return_val_if_fail(GWY_IS_SI_UNIT(siunit1), FALSE);
+    g_return_val_if_fail(GWY_IS_SI_UNIT(siunit2), FALSE);
+    if (gwy_si_unit_equal_direct(siunit1, siunit2))
+        return TRUE;
+
+    /* For strict comparison only accept as equal units that are identical,
+     * including any factors.  Otherwise just check dimensional equality. */
+    factored1 = gwy_si_unit_factor_to_base(siunit1, NULL, &mf1);
+    factored2 = gwy_si_unit_factor_to_base(siunit2, NULL, &mf2);
+    if ((!strict || abs(log(mf1/mf2)) < 1e-12)
+        && gwy_si_unit_equal_direct(factored1, factored2))
+        result = TRUE;
+
+    g_object_unref(factored1);
+    g_object_unref(factored2);
+
+    return result;
+}
+
+static gboolean
+gwy_si_unit_equal_direct(GwySIUnit *siunit1, GwySIUnit *siunit2)
+{
+    guint i, j;
+
+    if (siunit2 == siunit1)
+        return TRUE;
+
+    if (siunit2->units->len != siunit1->units->len)
+        return FALSE;
+
+    for (i = 0; i < siunit1->units->len; i++) {
+        GwySimpleUnit *unit = &unit_index(siunit1, i);
+
+        for (j = 0; j < siunit2->units->len; j++) {
+            if (unit_index(siunit2, j).unit == unit->unit) {
+                if (unit_index(siunit2, j).power != unit->power)
+                    return FALSE;
+                break;
+            }
+        }
+        if (j == siunit2->units->len)
+            return FALSE;
+    }
+
+    return TRUE;
+}
+
 static void
 gwy_si_unit_format(GwySIUnit *siunit,
                    const GwySIStyleSpec *fs,
@@ -1377,7 +1561,7 @@ gwy_si_unit_format(GwySIUnit *siunit,
     move_me_to_end = -1;
     if (siunit->units->len > 1) {
         for (i = 0; i < siunit->units->len; i++) {
-            unit = &g_array_index(siunit->units, GwySimpleUnit, i);
+            unit = &unit_index(siunit, i);
             if (unit->power < 0) {
                 if (move_me_to_end >= 0) {
                     move_me_to_end = -1;
@@ -1394,7 +1578,7 @@ gwy_si_unit_format(GwySIUnit *siunit,
         for (i = 0; i < siunit->units->len; i++) {
             if (i == move_me_to_end)
                 continue;
-            unit = &g_array_index(siunit->units, GwySimpleUnit, i);
+            unit = &unit_index(siunit, i);
             if (siunit->power10 % (3*abs(unit->power)) == 0) {
                 prefix_bearer = i;
                 break;
@@ -1402,7 +1586,7 @@ gwy_si_unit_format(GwySIUnit *siunit,
         }
     }
     if (siunit->power10 && prefix_bearer < 0 && move_me_to_end >= 0) {
-        unit = &g_array_index(siunit->units, GwySimpleUnit, move_me_to_end);
+        unit = &unit_index(siunit, move_me_to_end);
         if (siunit->power10 % (3*abs(unit->power)) == 0)
             prefix_bearer = move_me_to_end;
     }
@@ -1416,7 +1600,7 @@ gwy_si_unit_format(GwySIUnit *siunit,
             for (i = 0; i < siunit->units->len; i++) {
                 if (i == move_me_to_end)
                     continue;
-                unit = &g_array_index(siunit->units, GwySimpleUnit, i);
+                unit = &unit_index(siunit, i);
                 if ((siunit->power10 - moveby) % (3*abs(unit->power)) == 0) {
                     siunit->power10 -= moveby;
                     format->magnitude /= pow10(moveby);
@@ -1436,7 +1620,7 @@ gwy_si_unit_format(GwySIUnit *siunit,
             for (i = 0; i < siunit->units->len; i++) {
                 if (i == move_me_to_end)
                     continue;
-                unit = &g_array_index(siunit->units, GwySimpleUnit, i);
+                unit = &unit_index(siunit, i);
                 if ((siunit->power10 + moveby) % (3*abs(unit->power)) == 0) {
                     siunit->power10 += moveby;
                     format->magnitude *= pow10(moveby);
@@ -1449,7 +1633,7 @@ gwy_si_unit_format(GwySIUnit *siunit,
     }
     /* Check whether we are not out of prefix range. */
     if (prefix_bearer >= 0) {
-        unit = &g_array_index(siunit->units, GwySimpleUnit, prefix_bearer);
+        unit = &unit_index(siunit, prefix_bearer);
         prefix = gwy_si_unit_prefix(siunit->power10/unit->power);
         if (!prefix)
             prefix_bearer = -1;
@@ -1488,7 +1672,7 @@ gwy_si_unit_format(GwySIUnit *siunit,
         if (i > 1 || (i && move_me_to_end)) {
             g_string_append(string, fs->unit_times);
         }
-        unit = &g_array_index(siunit->units, GwySimpleUnit, i);
+        unit = &unit_index(siunit, i);
         if (i == prefix_bearer)
             g_string_append(string, prefix);
         g_string_append(string, g_quark_to_string(unit->unit));
@@ -1497,7 +1681,7 @@ gwy_si_unit_format(GwySIUnit *siunit,
     }
     if (move_me_to_end >= 0) {
         g_string_append(string, fs->unit_division);
-        unit = &g_array_index(siunit->units, GwySimpleUnit, move_me_to_end);
+        unit = &unit_index(siunit, move_me_to_end);
         if (move_me_to_end == prefix_bearer)
             g_string_append(string, prefix);
         g_string_append(string, g_quark_to_string(unit->unit));
@@ -1514,8 +1698,8 @@ gwy_si_unit_prefix(gint power)
     gint i;
 
     for (i = 0; i < G_N_ELEMENTS(SI_prefixes); i++) {
-        if (SI_prefixes[i].power10 == power)
-            return SI_prefixes[i].prefix;
+        if (SI_prefixes[i].value == power)
+            return SI_prefixes[i].name;
     }
     return NULL;
 }
@@ -1639,4 +1823,17 @@ format_power_unicode(GString *string, gint power)
  * typecasting.
  **/
 
+/**
+ * gwy_si_unit_assign:
+ * @dest: Target SI unit.
+ * @source: Source SI unit.
+ *
+ * Convenience macro making one SI unit equal to another.
+ *
+ * This is just a gwy_serializable_clone() wrapper with all the necessary
+ * typecasting.
+ *
+ * Since: 2.51
+ **/
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
