@@ -1,6 +1,6 @@
 /*
- *  $Id: gwyutils.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2003-2016 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwyutils.c 21367 2018-08-29 12:32:29Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -1057,6 +1057,40 @@ gwy_str_next_line(gchar **buffer)
     return q;
 }
 
+/**
+ * gwy_str_fixed_font_width:
+ * @str: UTF-8 encoded string.
+ *
+ * Measures the width of UTF-8 encoded string in fixed-width font.
+ *
+ * This corresponds to width of the string displayed on a text terminal, for
+ * instance.  Zero and double width characters are taken into account.  It is
+ * not guaranteed all terminals display the string with the calculated width.
+ *
+ * Returns: String width in fixed font, in character cells.
+ *
+ * Since: 2.52
+ **/
+guint
+gwy_str_fixed_font_width(const gchar *str)
+{
+    guint w = 0;
+    gunichar u;
+
+    while (str && *str) {
+        u = g_utf8_get_char(str);
+        if (g_unichar_iswide(u))
+            w += 2;
+        else if (g_unichar_iszerowidth(u))
+            w += 0;
+        else
+            w += 1;
+        str = g_utf8_next_char(str);
+    }
+
+    return w;
+}
+
 /**
  * gwy_gstring_replace:
  * @str: A #GString string to modify in place.
