@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyadjustbar.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: gwyadjustbar.c 20808 2018-03-05 10:08:24Z yeti-dn $
  *  Copyright (C) 2012-2017 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -22,6 +22,7 @@
 #include "config.h"
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
+#include <gdk/gdkkeysyms.h>
 #include <glib-object.h>
 
 #include <libgwyddion/gwymacros.h>
@@ -579,13 +580,19 @@ static gboolean
 gwy_adjust_bar_button_press(GtkWidget *widget, GdkEventButton *event)
 {
     AdjustBar *priv = GWY_ADJUST_BAR(widget)->priv;
+    GtkWidget *child;
 
     if (!priv->bar_sensitive || event->button != 1)
         return FALSE;
     priv->dragging = TRUE;
     change_value(widget, event->x);
-    if (GTK_BIN(widget)->child)
-        gtk_widget_mnemonic_activate(GTK_BIN(widget)->child, FALSE);
+    child = GTK_BIN(widget)->child;
+    if (child) {
+        if (gtk_label_get_mnemonic_keyval(GTK_LABEL(child)) != GDK_VoidSymbol)
+            gtk_widget_mnemonic_activate(child, FALSE);
+        /* XXX: If the adjbar label does not have mnemonic we do not know
+         * which widget we would like to give focus to. */
+    }
     return TRUE;
 }
 
@@ -667,7 +674,7 @@ gwy_adjust_bar_new(GtkAdjustment *adjustment, const gchar *label)
                               NULL);
     }
     else
-        adjbar = g_object_newv(GWY_TYPE_ADJUST_BAR, 0, NULL);
+        adjbar = g_object_new(GWY_TYPE_ADJUST_BAR, NULL);
 
     if (label) {
         GtkWidget *child = GTK_BIN(adjbar)->child;
