@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyaxis.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: gwyaxis.c 21435 2018-09-11 12:39:53Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1796,7 +1796,7 @@ gwy_axis_set_si_unit(GwyAxis *axis, GwySIUnit *unit)
     if (axis->unit && gwy_si_unit_equal(axis->unit, unit))
         return;
 
-    gwy_serializable_clone(G_OBJECT(unit), G_OBJECT(axis->unit));
+    gwy_si_unit_assign(axis->unit, unit);
     gwy_axis_adjust(axis, -1, -1);
 }
 
