@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwydgetutils.c 19924 2017-06-17 12:57:41Z yeti-dn $
+ *  $Id: gwydgetutils.c 20678 2017-12-18 18:26:55Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -401,7 +401,8 @@ gwy_table_attach_hscale(GtkWidget *table, gint row,
         spin = gtk_spin_button_new(adj, 0.1, MAX(digits, 0));
         u = adj->value;
         gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
-        gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(spin), TRUE);
+        gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(spin),
+                                          style & GWY_HSCALE_SNAP);
         gtk_table_attach(tab, spin, 2, 3, row, row+1, GTK_FILL, 0, 0, 0);
         gtk_adjustment_set_value(adj, u);
         middle_widget = spin;
@@ -610,7 +611,8 @@ gwy_table_attach_adjbar(GtkWidget *table, gint row,
         digits = (u > 0.0) ? (gint)floor(-log10(u)) : 0;
         spin = gtk_spin_button_new(adj, 0.1, MAX(digits, 0));
         gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
-        gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(spin), TRUE);
+        gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(spin),
+                                          style & GWY_HSCALE_SNAP);
         gtk_table_attach(tab, spin, 1, 2, row, row+1, GTK_FILL, 0, 0, 0);
         middle_widget = spin;
 
