@@ -1,5 +1,5 @@
 /*
- *  $Id: gwygraphexport.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: gwygraphexport.c 21292 2018-08-01 09:57:34Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -65,8 +65,10 @@ gwy_graph_export_pixmap(GwyGraph *graph,
     labelw = (GWY_GRAPH_LABEL(graph->area->lab))->reqwidth;
     labelh = (GWY_GRAPH_LABEL(graph->area->lab))->reqheight;
 
-    visual = gdk_visual_get_best();
-    cmap = gdk_colormap_new(visual, FALSE);
+    visual = gdk_visual_get_system();
+    g_return_val_if_fail(visual, NULL);
+    cmap = gdk_colormap_get_system();
+    g_return_val_if_fail(cmap, NULL);
 
     pixmap = gdk_pixmap_new(NULL, width, height, visual->depth);
     gdk_drawable_set_colormap(pixmap, cmap);
