@@ -1,6 +1,6 @@
 /*
- *  $Id: gwygraphmodel.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2004-2016 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwygraphmodel.c 21445 2018-09-18 09:41:45Z yeti-dn $
+ *  Copyright (C) 2004-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -105,6 +105,9 @@ static gdouble*    merge_abscissae                   (const GwyGraphModel *gmode
                                                       guint *ndata);
 static gboolean    curve_is_equispaced               (const GwyGraphCurveModel *gcmodel);
 static gchar*      ascii_name                        (const gchar *s);
+static void        gwy_graph_model_take_curve        (GwyGraphModel *gmodel,
+                                                      GwyGraphCurveModel *curve,
+                                                      gint i);
 static void        gwy_graph_model_release_curve     (GwyGraphModel *gmodel,
                                                       guint i);
 static void        gwy_graph_model_curve_data_changed(GwyGraphCurveModel *cmodel,
@@ -445,8 +448,8 @@ gwy_graph_model_init(GwyGraphModel *gmodel)
     gmodel->curves = g_ptr_array_new();
     gmodel->curveaux = g_array_new(FALSE, FALSE, sizeof(GwyGraphModelCurveAux));
 
-    gmodel->x_unit = gwy_si_unit_new("");
-    gmodel->y_unit = gwy_si_unit_new("");
+    gmodel->x_unit = gwy_si_unit_new(NULL);
+    gmodel->y_unit = gwy_si_unit_new(NULL);
 
     gmodel->title = g_string_new("Graph");
     gmodel->bottom_label = g_string_new("x");
@@ -830,14 +833,12 @@ gwy_graph_model_clone_real(GObject *source,
     }
 
     if (!gwy_si_unit_equal(clone->x_unit, gmodel->x_unit)) {
-        gwy_serializable_clone(G_OBJECT(gmodel->x_unit),
-                               G_OBJECT(clone->x_unit));
+        gwy_si_unit_assign(clone->x_unit, gmodel->x_unit);
         g_object_notify(copy, "si-unit-x");
     }
 
     if (!gwy_si_unit_equal(clone->y_unit, gmodel->y_unit)) {
-        gwy_serializable_clone(G_OBJECT(gmodel->y_unit),
-                               G_OBJECT(clone->y_unit));
+        gwy_si_unit_assign(clone->y_unit, gmodel->y_unit);
         g_object_notify(copy, "si-unit-y");
     }
 
@@ -967,15 +968,15 @@ gwy_graph_model_set_property(GObject *object,
         break;
 
         case PROP_SI_UNIT_X:
-        /* Keep the idiosyncratic semantics of gwy_graph_model_set_si_unit_x */
-        gwy_serializable_clone(g_value_get_object(value),
-                               G_OBJECT(gmodel->x_unit));
+        /* Keep the idiosyncratic semantics of gwy_graph_model_set_si_unit_x
+         * (which no longer exists in 2.x) */
+        gwy_si_unit_assign(gmodel->x_unit, g_value_get_object(value));
         break;
 
         case PROP_SI_UNIT_Y:
-        /* Keep the idiosyncratic semantics of gwy_graph_model_set_si_unit_x */
-        gwy_serializable_clone(g_value_get_object(value),
-                               G_OBJECT(gmodel->y_unit));
+        /* Keep the idiosyncratic semantics of gwy_graph_model_set_si_unit_y
+         * (which no longer exists in 2.x) */
+        gwy_si_unit_assign(gmodel->y_unit, g_value_get_object(value));
         break;
 
         case PROP_X_LOGARITHMIC:
@@ -1215,26 +1216,13 @@ gint
 gwy_graph_model_add_curve(GwyGraphModel *gmodel,
                           GwyGraphCurveModel *curve)
 {
-    GwyGraphModelCurveAux aux;
     gint idx;
 
     g_return_val_if_fail(GWY_IS_GRAPH_MODEL(gmodel), -1);
     g_return_val_if_fail(GWY_IS_GRAPH_CURVE_MODEL(curve), -1);
 
-    g_object_ref(curve);
     idx = gmodel->curves->len;
-    g_ptr_array_add(gmodel->curves, curve);
-
-    aux.data_changed_id
-        = g_signal_connect(curve, "data-changed",
-                           G_CALLBACK(gwy_graph_model_curve_data_changed),
-                           gmodel);
-    aux.notify_id
-        = g_signal_connect(curve, "notify",
-                           G_CALLBACK(gwy_graph_model_curve_notify),
-                           gmodel);
-    g_array_append_val(gmodel->curveaux, aux);
-
+    gwy_graph_model_take_curve(gmodel, curve, idx);
     /* In principle, this can change gmodel->curves->len, so we have to save
      * the index in idx. */
     g_object_notify(G_OBJECT(gmodel), "n-curves");
@@ -1469,6 +1457,38 @@ gwy_graph_model_get_curve_index(GwyGraphModel *gmodel,
     return -1;
 }
 
+/**
+ * gwy_graph_model_replace_curve:
+ * @gmodel: A graph model.
+ * @cindex: Curve index in graph model.
+ * @curve: A curve model to put into @gmodel at position @cindex.
+ *
+ * Replaces a curve in a graph model.
+ *
+ * Since: 2.51
+ **/
+void
+gwy_graph_model_replace_curve(GwyGraphModel *gmodel,
+                              gint cindex,
+                              GwyGraphCurveModel *curve)
+{
+    GwyGraphCurveModel *cmodel;
+
+    g_return_if_fail(GWY_IS_GRAPH_MODEL(gmodel));
+    g_return_if_fail(GWY_IS_GRAPH_CURVE_MODEL(curve));
+    g_return_if_fail(cindex >= 0 && cindex < gmodel->curves->len);
+
+    cmodel = g_ptr_array_index(gmodel->curves, cindex);
+    /* Avoid messy work when the curve is already there. */
+    if (curve == cmodel)
+        return;
+
+    g_object_ref(cmodel);
+    gwy_graph_model_release_curve(gmodel, cindex);
+    gwy_graph_model_take_curve(gmodel, curve, cindex);
+    g_object_unref(cmodel);
+}
+
 /**
  * gwy_graph_model_set_units_from_data_line:
  * @gmodel: A graph model.
@@ -1490,6 +1510,46 @@ gwy_graph_model_set_units_from_data_line(GwyGraphModel *gmodel,
     g_object_set(gmodel, "si-unit-x", unitx, "si-unit-y", unity, NULL);
 }
 
+/**
+ * gwy_graph_model_set_units_from_data_field:
+ * @gmodel: A graph model.
+ * @data_field: A data field.
+ * @power_xy_in_x: Power of field's lateral units to appear in graph's abscissa
+ *                 units.
+ * @power_z_in_x: Power of field's value units to appear in graph's abscissa
+ *                units.
+ * @power_xy_in_y: Power of field's lateral units to appear in graph's ordinate
+ *                 units.
+ * @power_z_in_y: Power of field's value units to appear in graph's ordinate
+ *                units.
+ *
+ * Sets x and y graph model units to units derived from a data field.
+ *
+ * Since: 2.51
+ **/
+void
+gwy_graph_model_set_units_from_data_field(GwyGraphModel *gmodel,
+                                          GwyDataField *data_field,
+                                          gint power_xy_in_x,
+                                          gint power_z_in_x,
+                                          gint power_xy_in_y,
+                                          gint power_z_in_y)
+{
+    GwySIUnit *xyunit, *zunit;
+
+    g_return_if_fail(GWY_IS_GRAPH_MODEL(gmodel));
+    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
+
+    xyunit = gwy_data_field_get_si_unit_xy(data_field);
+    zunit = gwy_data_field_get_si_unit_z(data_field);
+    gwy_si_unit_power_multiply(xyunit, power_xy_in_x,
+                               zunit, power_z_in_x,
+                               gmodel->x_unit);
+    gwy_si_unit_power_multiply(xyunit, power_xy_in_y,
+                               zunit, power_z_in_y,
+                               gmodel->y_unit);
+}
+
 /**
  * gwy_graph_model_units_are_compatible:
  * @gmodel: A graph model.
@@ -1908,7 +1968,7 @@ append_number(GString *str,
  * @string: A string to append the text dump to, or %NULL to allocate a new
  *          string.
  *
- * Exports a graph model data to a file.
+ * Exports a graph model data to a string.
  *
  * The export format is specified by parameter @export_style.
  *
@@ -2359,6 +2419,41 @@ ascii_name(const gchar *s)
     return as;
 }
 
+static void
+gwy_graph_model_take_curve(GwyGraphModel *gmodel,
+                           GwyGraphCurveModel *curve,
+                           gint i)
+{
+    GPtrArray *curves = gmodel->curves;
+    GwyGraphModelCurveAux aux;
+    gboolean appending = FALSE;
+
+    g_object_ref(curve);
+    if (i == curves->len)
+        appending = TRUE;
+
+    if (appending)
+        g_ptr_array_add(curves, curve);
+    else {
+        g_assert(g_ptr_array_index(curves, i) == NULL);
+        g_ptr_array_index(curves, i) = curve;
+    }
+
+    aux.data_changed_id
+        = g_signal_connect(curve, "data-changed",
+                           G_CALLBACK(gwy_graph_model_curve_data_changed),
+                           gmodel);
+    aux.notify_id
+        = g_signal_connect(curve, "notify",
+                           G_CALLBACK(gwy_graph_model_curve_notify),
+                           gmodel);
+
+    if (appending)
+        g_array_append_val(gmodel->curveaux, aux);
+    else
+        g_array_index(gmodel->curveaux, GwyGraphModelCurveAux, i) = aux;
+}
+
 static void
 gwy_graph_model_release_curve(GwyGraphModel *gmodel,
                               guint i)
