@@ -1,5 +1,5 @@
 /*
- *  $Id: gwystock.h 20756 2018-01-24 14:23:24Z yeti-dn $
+ *  $Id: gwystock.h 21387 2018-09-03 11:24:04Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -40,12 +40,14 @@ G_BEGIN_DECLS
 #define GWY_STOCK_COLOR_RANGE_FIXED              "gwy_color_range_fixed"
 #define GWY_STOCK_COLOR_RANGE_FULL               "gwy_color_range_full"
 #define GWY_STOCK_CONVOLUTION                    "gwy_convolution"
+#define GWY_STOCK_CONVOLVE                       "gwy_convolve"
 #define GWY_STOCK_CORRECT_AFFINE                 "gwy_correct_affine"
 #define GWY_STOCK_CORRELATION_MASK               "gwy_correlation_mask"
 #define GWY_STOCK_CROP                           "gwy_crop"
 #define GWY_STOCK_CURVATURE                      "gwy_curvature"
 #define GWY_STOCK_CWT                            "gwy_cwt"
 #define GWY_STOCK_DATA_MEASURE                   "gwy_data_measure"
+#define GWY_STOCK_DECONVOLVE                     "gwy_deconvolve"
 #define GWY_STOCK_DISCONNECTED                   "gwy_disconnected"
 #define GWY_STOCK_DISTANCE                       "gwy_distance"
 #define GWY_STOCK_DISTANCE_TRANSFORM             "gwy_distance_transform"
@@ -69,6 +71,7 @@ G_BEGIN_DECLS
 #define GWY_STOCK_FIND_PEAKS                     "gwy_find_peaks"
 #define GWY_STOCK_FIT_SHAPE                      "gwy_fit_shape"
 #define GWY_STOCK_FIX_ZERO                       "gwy_fix_zero"
+#define GWY_STOCK_FLIP_DIAGONALLY                "gwy_flip_diagonally"
 #define GWY_STOCK_FLIP_HORIZONTALLY              "gwy_flip_horizontally"
 #define GWY_STOCK_FLIP_VERTICALLY                "gwy_flip_vertically"
 #define GWY_STOCK_FRACTAL                        "gwy_fractal"
@@ -85,6 +88,7 @@ G_BEGIN_DECLS
 #define GWY_STOCK_GRAINS_EDGE_REMOVE             "gwy_grains_edge_remove"
 #define GWY_STOCK_GRAINS_GRAPH                   "gwy_grains_graph"
 #define GWY_STOCK_GRAINS_MEASURE                 "gwy_grains_measure"
+#define GWY_STOCK_GRAINS_OTSU                    "gwy_grains_otsu"
 #define GWY_STOCK_GRAINS_REMOVE                  "gwy_grains_remove"
 #define GWY_STOCK_GRAINS_STATISTICS              "gwy_grains_statistics"
 #define GWY_STOCK_GRAINS_WATER                   "gwy_grains_water"
@@ -158,9 +162,12 @@ G_BEGIN_DECLS
 #define GWY_STOCK_MASK_THIN                      "gwy_mask_thin"
 #define GWY_STOCK_MEASURE_LATTICE                "gwy_measure_lattice"
 #define GWY_STOCK_MERGE                          "gwy_merge"
+#define GWY_STOCK_MFM_CONVERT_TO_FORCE           "gwy_mfm_convert_to_force"
 #define GWY_STOCK_MFM_CURRENT_LINE               "gwy_mfm_current_line"
 #define GWY_STOCK_MFM_FIELD_FIND_SHIFT           "gwy_mfm_field_find_shift"
 #define GWY_STOCK_MFM_FIELD_SHIFT                "gwy_mfm_field_shift"
+#define GWY_STOCK_MFM_PARALLEL                   "gwy_mfm_parallel"
+#define GWY_STOCK_MFM_PERPENDICULAR              "gwy_mfm_perpendicular"
 #define GWY_STOCK_MORE                           "gwy_more"
 #define GWY_STOCK_MUTUAL_CROP                    "gwy_mutual_crop"
 #define GWY_STOCK_NEURAL_APPLY                   "gwy_neural_apply"
@@ -207,6 +214,7 @@ G_BEGIN_DECLS
 #define GWY_STOCK_SYNTHETIC_BROWNIAN_MOTION      "gwy_synthetic_brownian_motion"
 #define GWY_STOCK_SYNTHETIC_COLUMNAR             "gwy_synthetic_columnar"
 #define GWY_STOCK_SYNTHETIC_DIFFUSION            "gwy_synthetic_diffusion"
+#define GWY_STOCK_SYNTHETIC_DISCS                "gwy_synthetic_discs"
 #define GWY_STOCK_SYNTHETIC_DOMAINS              "gwy_synthetic_domains"
 #define GWY_STOCK_SYNTHETIC_FIBRES               "gwy_synthetic_fibres"
 #define GWY_STOCK_SYNTHETIC_LATTICE              "gwy_synthetic_lattice"
@@ -231,6 +239,7 @@ G_BEGIN_DECLS
 #define GWY_STOCK_UNROTATE                       "gwy_unrotate"
 #define GWY_STOCK_VALUE_INVERT                   "gwy_value_invert"
 #define GWY_STOCK_VOLUME                         "gwy_volume"
+#define GWY_STOCK_VOLUME_ARITHMETIC              "gwy_volume_arithmetic"
 #define GWY_STOCK_VOLUME_CALIBRATE               "gwy_volume_calibrate"
 #define GWY_STOCK_VOLUME_DIMENSIONS              "gwy_volume_dimensions"
 #define GWY_STOCK_VOLUME_FD                      "gwy_volume_fd"
