@@ -1,6 +1,6 @@
 /*
- *  $Id: arithmetic.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
+ *  $Id: arithmetic.c 21288 2018-08-01 08:59:43Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -303,6 +303,50 @@ gwy_data_field_hypot_of_fields(GwyDataField *result,
     gwy_data_field_invalidate(result);
 }
 
+static void
+check_basic_properties(gint res1, gint res2,
+                       gdouble real1, gdouble real2,
+                       GwyDataCompatibilityFlags check,
+                       GwyDataCompatibilityFlags *result)
+{
+    /* Resolution */
+    if ((check & GWY_DATA_COMPATIBILITY_RES)
+        && !(*result & GWY_DATA_COMPATIBILITY_RES)) {
+        if (res1 != res2)
+            *result |= GWY_DATA_COMPATIBILITY_RES;
+    }
+
+    /* Real size */
+    if (check & GWY_DATA_COMPATIBILITY_REAL
+        && !(*result & GWY_DATA_COMPATIBILITY_REAL)) {
+        /* Keeps the condition in negative form to catch NaNs and odd values
+         * as incompatible. */
+        if (!(fabs(log(real1/real2)) <= EPSILON))
+            *result |= GWY_DATA_COMPATIBILITY_REAL;
+    }
+
+    /* Measure */
+    if (check & GWY_DATA_COMPATIBILITY_MEASURE
+        && !(*result & GWY_DATA_COMPATIBILITY_MEASURE)) {
+        if (!(fabs(log(real1/res1*res2/real2)) <= EPSILON))
+            *result |= GWY_DATA_COMPATIBILITY_MEASURE;
+    }
+}
+
+/* Check if two SI Units are equal, accepting also NULLs and considering them
+ * equal to empty units. */
+static gboolean
+units_are_equal(GwySIUnit *unit1, GwySIUnit *unit2)
+{
+    if (unit1 == unit2)
+        return TRUE;
+    if (!unit1)
+        return gwy_si_unit_equal_string(unit2, NULL);
+    if (!unit2)
+        return gwy_si_unit_equal_string(unit1, NULL);
+    return gwy_si_unit_equal(unit1, unit2);
+}
+
 /**
  * gwy_data_field_check_compatibility:
  * @data_field1: A data field.
@@ -320,61 +364,24 @@ gwy_data_field_check_compatibility(GwyDataField *data_field1,
                                    GwyDataCompatibilityFlags check)
 {
     GwyDataCompatibilityFlags result = 0;
-    gint xres1, xres2, yres1, yres2;
-    gdouble xreal1, xreal2, yreal1, yreal2;
-    GwySIUnit *unit1, *unit2;
 
     g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field1), check);
     g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field2), check);
 
-    xres1 = data_field1->xres;
-    xres2 = data_field2->xres;
-    yres1 = data_field1->yres;
-    yres2 = data_field2->yres;
-    xreal1 = data_field1->xreal;
-    xreal2 = data_field2->xreal;
-    yreal1 = data_field1->yreal;
-    yreal2 = data_field2->yreal;
+    check_basic_properties(data_field1->xres, data_field2->xres,
+                           data_field1->xreal, data_field2->xreal,
+                           check, &result);
+    check_basic_properties(data_field1->yres, data_field2->yres,
+                           data_field1->yreal, data_field2->yreal,
+                           check, &result);
 
-    /* Resolution */
-    if (check & GWY_DATA_COMPATIBILITY_RES) {
-        if (xres1 != xres2 || yres1 != yres2)
-            result |= GWY_DATA_COMPATIBILITY_RES;
-    }
+    if ((check & GWY_DATA_COMPATIBILITY_LATERAL)
+        && !units_are_equal(data_field1->si_unit_xy, data_field2->si_unit_xy))
+        result |= GWY_DATA_COMPATIBILITY_LATERAL;
 
-    /* Real size */
-    if (check & GWY_DATA_COMPATIBILITY_REAL) {
-        /* Keeps the condition in negative form to catch NaNs and odd values
-         * as incompatible. */
-        if (!(fabs(log(xreal1/xreal2)) <= EPSILON)
-            || !(fabs(log(yreal1/yreal2)) <= EPSILON))
-            result |= GWY_DATA_COMPATIBILITY_REAL;
-    }
-
-    /* Measure */
-    if (check & GWY_DATA_COMPATIBILITY_MEASURE) {
-        if (!(fabs(log(xreal1/xres1*xres2/xreal2)) <= EPSILON)
-            || !(fabs(log(yreal1/yres1*yres2/yreal2)) <= EPSILON))
-            result |= GWY_DATA_COMPATIBILITY_MEASURE;
-    }
-
-    /* Lateral units */
-    if (check & GWY_DATA_COMPATIBILITY_LATERAL) {
-        /* This can cause instantiation of data_field units as a side effect */
-        unit1 = gwy_data_field_get_si_unit_xy(data_field1);
-        unit2 = gwy_data_field_get_si_unit_xy(data_field2);
-        if (!gwy_si_unit_equal(unit1, unit2))
-            result |= GWY_DATA_COMPATIBILITY_LATERAL;
-    }
-
-    /* Value units */
-    if (check & GWY_DATA_COMPATIBILITY_VALUE) {
-        /* This can cause instantiation of data_field units as a side effect */
-        unit1 = gwy_data_field_get_si_unit_z(data_field1);
-        unit2 = gwy_data_field_get_si_unit_z(data_field2);
-        if (!gwy_si_unit_equal(unit1, unit2))
-            result |= GWY_DATA_COMPATIBILITY_VALUE;
-    }
+    if ((check & GWY_DATA_COMPATIBILITY_VALUE)
+        && !units_are_equal(data_field1->si_unit_z, data_field2->si_unit_z))
+        result |= GWY_DATA_COMPATIBILITY_VALUE;
 
     return result;
 }
@@ -396,53 +403,212 @@ gwy_data_line_check_compatibility(GwyDataLine *data_line1,
                                   GwyDataCompatibilityFlags check)
 {
     GwyDataCompatibilityFlags result = 0;
-    gint res1, res2;
-    gdouble real1, real2;
-    GwySIUnit *unit1, *unit2;
 
     g_return_val_if_fail(GWY_IS_DATA_LINE(data_line1), check);
     g_return_val_if_fail(GWY_IS_DATA_LINE(data_line2), check);
 
-    res1 = data_line1->res;
-    res2 = data_line2->res;
-    real1 = data_line1->real;
-    real2 = data_line2->real;
+    check_basic_properties(data_line1->res, data_line2->res,
+                           data_line1->real, data_line2->real,
+                           check, &result);
 
-    /* Resolution */
-    if (check & GWY_DATA_COMPATIBILITY_RES) {
-        if (res1 != res2)
-            result |= GWY_DATA_COMPATIBILITY_RES;
+    if ((check & GWY_DATA_COMPATIBILITY_LATERAL)
+        && !units_are_equal(data_line1->si_unit_x, data_line2->si_unit_x))
+        result |= GWY_DATA_COMPATIBILITY_LATERAL;
+
+    if ((check & GWY_DATA_COMPATIBILITY_VALUE)
+        && !units_are_equal(data_line1->si_unit_y, data_line2->si_unit_y))
+        result |= GWY_DATA_COMPATIBILITY_VALUE;
+
+    return result;
+}
+
+static gboolean
+data_lines_equal(GwyDataLine *data_line1,
+                 GwyDataLine *data_line2)
+{
+    gint res, i;
+    const gdouble *data1, *data2;
+
+    if (gwy_data_line_check_compatibility(data_line1, data_line2,
+                                          GWY_DATA_COMPATIBILITY_RES
+                                          | GWY_DATA_COMPATIBILITY_VALUE))
+        return FALSE;
+
+    res = data_line1->res;
+    data1 = data_line1->data;
+    data2 = data_line2->data;
+    for (i = 0; i < res; i++) {
+        /* FIXME: Add a tolerance here? */
+        if (data2[i] != data1[i])
+            return FALSE;
     }
 
-    /* Real size */
-    if (check & GWY_DATA_COMPATIBILITY_REAL) {
-        if (!(fabs(log(real1/real2)) <= EPSILON))
-            result |= GWY_DATA_COMPATIBILITY_REAL;
+    return TRUE;
+}
+
+/**
+ * gwy_brick_check_compatibility:
+ * @brick1: A data brick.
+ * @brick2: Another data brick.
+ * @check: The compatibility tests to perform.
+ *
+ * Checks whether two data bricks are compatible.
+ *
+ * Real dimensions are checked without regard to calibration.  Calibrations are
+ * considered compatible if either both exist and are identical or none exists.
+ *
+ * Returns: Zero if all tested properties are compatible.  Flags corresponding
+ *          to failed tests if bricks are not compatible.
+ *
+ * Since: 2.51
+ **/
+GwyDataCompatibilityFlags
+gwy_brick_check_compatibility(GwyBrick *brick1,
+                              GwyBrick *brick2,
+                              GwyDataCompatibilityFlags check)
+{
+    GwyDataCompatibilityFlags result = 0;
+
+    g_return_val_if_fail(GWY_IS_BRICK(brick1), check);
+    g_return_val_if_fail(GWY_IS_BRICK(brick2), check);
+
+    check_basic_properties(brick1->xres, brick2->xres,
+                           brick1->xreal, brick2->xreal,
+                           check, &result);
+    check_basic_properties(brick1->yres, brick2->yres,
+                           brick1->yreal, brick2->yreal,
+                           check, &result);
+    check_basic_properties(brick1->zres, brick2->zres,
+                           brick1->zreal, brick2->zreal,
+                           check, &result);
+
+    if ((check & GWY_DATA_COMPATIBILITY_LATERAL)
+        && (!units_are_equal(brick1->si_unit_x, brick2->si_unit_x)
+            || !units_are_equal(brick1->si_unit_y, brick2->si_unit_y)
+            || !units_are_equal(brick1->si_unit_z, brick2->si_unit_z)))
+        result |= GWY_DATA_COMPATIBILITY_LATERAL;
+
+    if (check & GWY_DATA_COMPATIBILITY_VALUE
+        && !units_are_equal(brick1->si_unit_w, brick2->si_unit_w))
+        result |= GWY_DATA_COMPATIBILITY_VALUE;
+
+    /* Z-calibration. */
+    if (check & GWY_DATA_COMPATIBILITY_AXISCAL) {
+        GwyDataLine *zcal1 = gwy_brick_get_zcalibration(brick1);
+        GwyDataLine *zcal2 = gwy_brick_get_zcalibration(brick2);
+        if ((zcal1 && !zcal2) || (!zcal1 && zcal2))
+            result |= GWY_DATA_COMPATIBILITY_AXISCAL;
+        else if (zcal1 && zcal2 && !data_lines_equal(zcal1, zcal2))
+            result |= GWY_DATA_COMPATIBILITY_AXISCAL;
     }
 
-    /* Measure */
-    if (check & GWY_DATA_COMPATIBILITY_MEASURE) {
-        if (!(fabs(log(real1/res1*res2/real2)) <= EPSILON))
-            result |= GWY_DATA_COMPATIBILITY_MEASURE;
-    }
+    return result;
+}
 
-    /* Lateral units */
-    if (check & GWY_DATA_COMPATIBILITY_LATERAL) {
-        /* This can cause instantiation of data_line units as a side effect */
-        unit1 = gwy_data_line_get_si_unit_x(data_line1);
-        unit2 = gwy_data_line_get_si_unit_x(data_line2);
-        if (!gwy_si_unit_equal(unit1, unit2))
+/**
+ * gwy_data_field_check_compatibility_with_brick_xy:
+ * @data_field: A two-dimensional data field.
+ * @brick: A three-dimensional data brick.
+ * @check: The compatibility tests to perform.
+ *
+ * Checks whether a data field is compatible with brick XY-planes.
+ *
+ * Returns: Zero if all tested properties are compatible.  Flags corresponding
+ *          to failed tests if the data objects are not compatible.
+ *
+ * Since: 2.51
+ **/
+GwyDataCompatibilityFlags
+gwy_data_field_check_compatibility_with_brick_xy(GwyDataField *data_field,
+                                                 GwyBrick *brick,
+                                                 GwyDataCompatibilityFlags check)
+{
+    GwyDataCompatibilityFlags result = 0;
+
+    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), check);
+    g_return_val_if_fail(GWY_IS_BRICK(brick), check);
+
+    check_basic_properties(data_field->xres, brick->xres,
+                           data_field->xreal, brick->xreal,
+                           check, &result);
+    check_basic_properties(data_field->yres, brick->yres,
+                           data_field->yreal, brick->yreal,
+                           check, &result);
+
+    if ((check & GWY_DATA_COMPATIBILITY_LATERAL)
+        && (!units_are_equal(data_field->si_unit_xy, brick->si_unit_x)
+            || !units_are_equal(data_field->si_unit_xy, brick->si_unit_y)))
+        result |= GWY_DATA_COMPATIBILITY_LATERAL;
+
+    if (check & GWY_DATA_COMPATIBILITY_VALUE
+        && !units_are_equal(data_field->si_unit_z, brick->si_unit_w))
+        result |= GWY_DATA_COMPATIBILITY_VALUE;
+
+    return result;
+}
+
+/**
+ * gwy_data_line_check_compatibility_with_brick_z:
+ * @data_line: A one-dimensional data line.
+ * @brick: A three-dimensional data brick.
+ * @check: The compatibility tests to perform.
+ *
+ * Checks whether a data line is compatible with brick Z-profiles.
+ *
+ * If @check includes %GWY_DATA_COMPATIBILITY_REAL or
+ * %GWY_DATA_COMPATIBILITY_LATERAL but not %GWY_DATA_COMPATIBILITY_AXISCAL,
+ * @data_line is simply compared to @brick in the Z direction.
+ *
+ * If you include %GWY_DATA_COMPATIBILITY_AXISCAL and @brick has a
+ * Z-calibration data line, then the value range and units of this data line
+ * are compared to @data_line.  This may not be very useful.
+ *
+ * Returns: Zero if all tested properties are compatible.  Flags corresponding
+ *          to failed tests if the data objects are not compatible.
+ *
+ * Since: 2.51
+ **/
+GwyDataCompatibilityFlags
+gwy_data_line_check_compatibility_with_brick_z(GwyDataLine *data_line,
+                                               GwyBrick *brick,
+                                               GwyDataCompatibilityFlags check)
+{
+    GwyDataCompatibilityFlags result = 0;
+    GwyDataLine *zcal;
+
+    g_return_val_if_fail(GWY_IS_DATA_LINE(data_line), check);
+    g_return_val_if_fail(GWY_IS_BRICK(brick), check);
+
+    if (check & GWY_DATA_COMPATIBILITY_VALUE
+        && !units_are_equal(data_line->si_unit_y, brick->si_unit_w))
+        result |= GWY_DATA_COMPATIBILITY_VALUE;
+
+    /* Without zcalibration compare directly to the brick. */
+    zcal = gwy_brick_get_zcalibration(brick);
+    if (!zcal || !(check & GWY_DATA_COMPATIBILITY_AXISCAL)) {
+        check_basic_properties(data_line->res, brick->xres,
+                               data_line->real, brick->xreal,
+                               check, &result);
+
+        if ((check & GWY_DATA_COMPATIBILITY_LATERAL)
+            && !units_are_equal(data_line->si_unit_x, brick->si_unit_z))
             result |= GWY_DATA_COMPATIBILITY_LATERAL;
+
+        return result;
     }
 
-    /* Value units */
-    if (check & GWY_DATA_COMPATIBILITY_VALUE) {
-        /* This can cause instantiation of data_line units as a side effect */
-        unit1 = gwy_data_line_get_si_unit_y(data_line1);
-        unit2 = gwy_data_line_get_si_unit_y(data_line2);
-        if (!gwy_si_unit_equal(unit1, unit2))
-            result |= GWY_DATA_COMPATIBILITY_VALUE;
-    }
+    /* With Z-calibration we compare to @zcal.  The *values* of @zcal are the
+     * same thing as *coordinates* of @data_line.  So this is a mess and
+     * possibly not useful at all. */
+    g_assert(zcal->res == brick->zres);
+    check_basic_properties(data_line->res, zcal->res,
+                           data_line->real,
+                           zcal->data[zcal->res-1] - zcal->data[0],
+                           check, &result);
+
+    if ((check & GWY_DATA_COMPATIBILITY_LATERAL)
+        && !units_are_equal(data_line->si_unit_x, zcal->si_unit_y))
+        result |= GWY_DATA_COMPATIBILITY_LATERAL;
 
     return result;
 }
@@ -715,57 +881,62 @@ rect_extend_laplace(const gdouble *in, guint inrowstride,
                     G_GNUC_UNUSED gdouble value)
 {
     GwyDataField *mask, *workspace;
-    guint e2u, e2d, e2r, e2l, i;
+    guint e2u, e2d, e2r, e2l, i, extxres, extyres;
 
-    // Expand the ROI down as far as possible
+    /* Expand the ROI down as far as possible */
     e2d = MIN(extend_down, yres - (ypos + height));
     height += e2d;
     extend_down -= e2d;
 
-    // Expand the ROI up as far as possible
+    /* Expand the ROI up as far as possible */
     e2u = MIN(extend_up, ypos);
     height += e2u;
     extend_up -= e2u;
     ypos -= e2u;
 
-    // Expand the ROI to the right as far as possible
+    /* Expand the ROI to the right as far as possible */
     e2r = MIN(extend_right, xres - (xpos + width));
     width += e2r;
     extend_right -= e2r;
 
-    // Expand the ROI to the left as far as possible
+    /* Expand the ROI to the left as far as possible */
     e2l = MIN(extend_left, xpos);
     width += e2l;
     extend_left -= e2l;
     xpos -= e2l;
 
-    // Direct copy of the ROI
-    for (i = 0; i < height; i++) {
-        gwy_assign(out + (extend_up + i)*outrowstride + extend_left,
-                   in + (ypos + i)*inrowstride + xpos,
-                   width);
+    if (extend_down + extend_up + extend_right + extend_left == 0) {
+        /* Direct copy of the ROI */
+        for (i = 0; i < height; i++) {
+            gwy_assign(out + (extend_up + i)*outrowstride + extend_left,
+                       in + (ypos + i)*inrowstride + xpos,
+                       width);
+        }
+        return;
     }
 
-    if (extend_down + extend_up + extend_right + extend_left == 0)
-        return;
-
-    mask = gwy_data_field_new(width + extend_left + extend_right,
-                              height + extend_up + extend_down,
-                              1.0, 1.0, FALSE);
+    extxres = width + extend_left + extend_right;
+    extyres = height + extend_up + extend_down;
+    mask = gwy_data_field_new(extxres, extyres, 1.0, 1.0, FALSE);
     gwy_data_field_fill(mask, 1.0);
     gwy_data_field_area_clear(mask, extend_left, extend_up, width, height);
 
-    /* Avoid allocating yet another data field just to copy the data there.
-     * This is dirty. */
-    workspace = gwy_data_field_new(1, 1, 1.0, 1.0, FALSE);
-    GWY_SWAP(gdouble*, out, workspace->data);
-    workspace->xres = width + extend_left + extend_right;
-    workspace->yres = height + extend_up + extend_down;
+    /* NB: We cannot recycle out in any manner because it has different
+     * rowstride than extxres! */
+    workspace = gwy_data_field_new(extxres, extyres, 1.0, 1.0, FALSE);
 
+    for (i = 0; i < height; i++) {
+        gwy_assign(workspace->data + (extend_up + i)*extxres + extend_left,
+                   in + (ypos + i)*inrowstride + xpos,
+                   width);
+    }
     gwy_data_field_laplace_solve(workspace, mask, -1, 0.5);
+    for (i = 0; i < extyres; i++) {
+        gwy_assign(out + i*outrowstride,
+                   workspace->data + i*extxres,
+                   extxres);
+    }
 
-    GWY_SWAP(gdouble*, out, workspace->data);
-    workspace->xres = workspace->yres = 1;
     g_object_unref(workspace);
     g_object_unref(mask);
 }
@@ -877,9 +1048,10 @@ gwy_data_field_extend(GwyDataField *data_field,
  * GWY_DATA_COMPATIBILITY_RES)
  * must pass and the same must hold for the data field to store the result to.
  *
- * Functions gwy_data_field_check_compatibility() and
- * gwy_data_line_check_compatibility() simplify testing compatibility of data
- * fields and lines, respectively.
+ * Functions gwy_data_field_check_compatibility(),
+ * gwy_data_line_check_compatibility() and gwy_brick_check_compatibility()
+ * simplify testing compatibility of data fields, lines and bricks,
+ * respectively.
  **/
 
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
