@@ -1,6 +1,6 @@
 /*
- *  $Id: brick.h 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
+ *  $Id: brick.h 21553 2018-10-29 14:23:17Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -39,6 +39,7 @@ G_BEGIN_DECLS
 typedef struct _GwyBrick      GwyBrick;
 typedef struct _GwyBrickClass GwyBrickClass;
 
+/* XXX: Unused.  But Petr copied it here so we must keep it. */
 typedef struct {
     guint col;
     guint row;
@@ -48,7 +49,6 @@ typedef struct {
     guint depth;
 } GwyBrickPart;
 
-
 struct _GwyBrick {
     GObject parent_instance;
 
@@ -85,6 +85,10 @@ struct _GwyBrickClass {
 
 #define gwy_brick_duplicate(brick) \
         (GWY_BRICK(gwy_serializable_duplicate(G_OBJECT(brick))))
+#define gwy_brick_assign(dest, source) \
+        gwy_serializable_clone_with_type(G_OBJECT(source), \
+                                         G_OBJECT(dest), \
+                                         GWY_TYPE_BRICK)
 
 GType     gwy_brick_get_type    (void)                   G_GNUC_CONST;
 GwyBrick* gwy_brick_new         (gint xres,
@@ -106,6 +110,10 @@ GwyBrick* gwy_brick_new_part    (const GwyBrick *brick,
                                  gboolean keep_offsets);
 void      gwy_brick_data_changed(GwyBrick *brick);
 
+void              gwy_brick_copy              (GwyBrick *src,
+                                               GwyBrick *dest,
+                                               gboolean nondata_too);
+
 void              gwy_brick_resample          (GwyBrick *brick,
                                                gint xres,
                                                gint yres,
@@ -133,6 +141,9 @@ void              gwy_brick_set_yoffset       (GwyBrick *brick,
                                                gdouble yoffset);
 void              gwy_brick_set_zoffset       (GwyBrick *brick,
                                                gdouble zoffset);
+gdouble           gwy_brick_get_dx            (GwyBrick *brick);
+gdouble           gwy_brick_get_dy            (GwyBrick *brick);
+gdouble           gwy_brick_get_dz            (GwyBrick *brick);
 GwySIUnit*        gwy_brick_get_si_unit_x     (GwyBrick *brick);
 GwySIUnit*        gwy_brick_get_si_unit_y     (GwyBrick *brick);
 GwySIUnit*        gwy_brick_get_si_unit_z     (GwyBrick *brick);
@@ -222,6 +233,9 @@ void              gwy_brick_extract_plane     (const GwyBrick *brick,
                                                gint height,
                                                gint depth,
                                                gboolean keep_offsets);
+void              gwy_brick_extract_xy_plane  (const GwyBrick *brick,
+                                               GwyDataField *target,
+                                               gint lev);
 void              gwy_brick_sum_plane         (const GwyBrick *brick,
                                                GwyDataField *target,
                                                gint istart,
@@ -231,6 +245,8 @@ void              gwy_brick_sum_plane         (const GwyBrick *brick,
                                                gint height,
                                                gint depth,
                                                gboolean keep_offsets);
+void              gwy_brick_sum_xy_plane      (const GwyBrick *brick,
+                                               GwyDataField *target);
 void              gwy_brick_min_plane         (const GwyBrick *brick,
                                                GwyDataField *target,
                                                gint istart,
@@ -240,6 +256,8 @@ void              gwy_brick_min_plane         (const GwyBrick *brick,
                                                gint height,
                                                gint depth,
                                                gboolean keep_offsets);
+void              gwy_brick_min_xy_plane      (const GwyBrick *brick,
+                                               GwyDataField *target);
 void              gwy_brick_max_plane         (const GwyBrick *brick,
                                                GwyDataField *target,
                                                gint istart,
@@ -249,6 +267,8 @@ void              gwy_brick_max_plane         (const GwyBrick *brick,
                                                gint height,
                                                gint depth,
                                                gboolean keep_offsets);
+void              gwy_brick_max_xy_plane      (const GwyBrick *brick,
+                                               GwyDataField *target);
 void              gwy_brick_minpos_plane      (const GwyBrick *brick,
                                                GwyDataField *target,
                                                gint istart,
@@ -258,6 +278,8 @@ void              gwy_brick_minpos_plane      (const GwyBrick *brick,
                                                gint height,
                                                gint depth,
                                                gboolean keep_offsets);
+void              gwy_brick_minpos_xy_plane   (const GwyBrick *brick,
+                                               GwyDataField *target);
 void              gwy_brick_maxpos_plane      (const GwyBrick *brick,
                                                GwyDataField *target,
                                                gint istart,
@@ -267,6 +289,8 @@ void              gwy_brick_maxpos_plane      (const GwyBrick *brick,
                                                gint height,
                                                gint depth,
                                                gboolean keep_offsets);
+void              gwy_brick_maxpos_xy_plane   (const GwyBrick *brick,
+                                               GwyDataField *target);
 void              gwy_brick_mean_plane        (const GwyBrick *brick,
                                                GwyDataField *target,
                                                gint istart,
@@ -276,6 +300,8 @@ void              gwy_brick_mean_plane        (const GwyBrick *brick,
                                                gint height,
                                                gint depth,
                                                gboolean keep_offsets);
+void              gwy_brick_mean_xy_plane     (const GwyBrick *brick,
+                                               GwyDataField *target);
 void              gwy_brick_rms_plane         (const GwyBrick *brick,
                                                GwyDataField *target,
                                                gint istart,
@@ -285,6 +311,8 @@ void              gwy_brick_rms_plane         (const GwyBrick *brick,
                                                gint height,
                                                gint depth,
                                                gboolean keep_offsets);
+void              gwy_brick_rms_xy_plane      (const GwyBrick *brick,
+                                               GwyDataField *target);
 void              gwy_brick_extract_line      (const GwyBrick *brick,
                                                GwyDataLine *target,
                                                gint istart,
@@ -294,9 +322,32 @@ void              gwy_brick_extract_line      (const GwyBrick *brick,
                                                gint jend,
                                                gint kend,
                                                gboolean keep_offsets);
+void              gwy_brick_extract_z_line    (const GwyBrick *brick,
+                                               GwyDataLine *target,
+                                               gint i,
+                                               gint j);
 GwyDataLine*      gwy_brick_get_zcalibration  (const GwyBrick *brick);
 void              gwy_brick_set_zcalibration  (const GwyBrick *brick,
                                                GwyDataLine *calibration);
+void              gwy_brick_copy_zcalibration (const GwyBrick *brick,
+                                               GwyBrick *target);
+void              gwy_brick_transpose         (GwyBrick *brick,
+                                               GwyBrick *target,
+                                               GwyBrickTransposeType type,
+                                               gboolean xflipped,
+                                               gboolean yflipped,
+                                               gboolean zflipped);
+void              gwy_brick_set_plane         (const GwyBrick *brick,
+                                               GwyDataField *plane,
+                                               gint istart,
+                                               gint jstart,
+                                               gint kstart,
+                                               gint width,
+                                               gint height,
+                                               gint depth);
+void              gwy_brick_set_xy_plane      (const GwyBrick *brick,
+                                               GwyDataField *plane,
+                                               gint lev);
 
 
 G_END_DECLS
