@@ -1,5 +1,5 @@
 /*
- *  $Id: correct-laplace.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: correct-laplace.c 21334 2018-08-24 22:10:33Z yeti-dn $
  *  Copyright (C) 2004-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -947,7 +947,7 @@ iterate_conj_grad(LaplaceIterators *iterators)
         phi += (*v)*(*f);
     }
 
-    if (S < 1e-16)
+    if (S <= 1e-20*phi)
         return TRUE;
 
     // New value and f = A.z-b
