@@ -1,5 +1,5 @@
 /*
- *  $Id: correct.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: correct.c 21617 2018-11-12 15:53:09Z yeti-dn $
  *  Copyright (C) 2004-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1292,7 +1292,7 @@ refine_lattice(GwyDataField *dfield, gdouble *a1a2, gboolean is_psdf)
                   r, a1a2[0], a1a2[1], a1a2[2], a1a2[3]);
     }
 
-    return r > 0.5;
+    return r >= 0.5;
 }
 
 /**
@@ -1334,23 +1334,31 @@ gwy_data_field_measure_lattice_acf(GwyDataField *acf2d, gdouble *a1a2)
     }
 
     for (i = 0; i < 4; i++) {
-        if (gwy_isnan(a1a2[i]) || gwy_isinf(a1a2[i]))
+        if (gwy_isnan(a1a2[i]) || gwy_isinf(a1a2[i])) {
+            gwy_debug("inf/nan 1");
             return FALSE;
+        }
     }
 
-    if (!refine_lattice(acf2d, a1a2, FALSE))
+    if (!refine_lattice(acf2d, a1a2, FALSE)) {
+        gwy_debug("failed to refine");
         return FALSE;
+    }
 
     for (i = 0; i < 4; i++) {
-        if (gwy_isnan(a1a2[i]) || gwy_isinf(a1a2[i]))
+        if (gwy_isnan(a1a2[i]) || gwy_isinf(a1a2[i])) {
+            gwy_debug("inf/nan 2");
             return FALSE;
+        }
     }
 
     /* For very skewed lattices refine() can produce two of the same vector. */
     dx = acf2d->xreal/acf2d->xres;
     dy = acf2d->yreal/acf2d->yres;
-    if (hypot(a1a2[0] - a1a2[2], a1a2[1] - a1a2[3]) < 1.8*hypot(dx, dy))
+    if (hypot(a1a2[0] - a1a2[2], a1a2[1] - a1a2[3]) < 1.8*hypot(dx, dy)) {
+        gwy_debug("too skewed");
         return FALSE;
+    }
 
     return TRUE;
 }
@@ -1571,6 +1579,273 @@ gwy_data_field_mark_scars(GwyDataField *data_field,
     }
 }
 
+/**
+ * gwy_data_field_subtract_row_shifts:
+ * @data_field: A data field.
+ * @shifts: Data line containing the row shifts.
+ *
+ * Shifts entire data field rows as specified by given data line.
+ *
+ * Data line @shifts must have resolution corresponding to the number of
+ * @data_field rows.  Its values are subtracted from individual field rows.
+ *
+ * Since: 2.52
+ **/
+void
+gwy_data_field_subtract_row_shifts(GwyDataField *data_field,
+                                   GwyDataLine *shifts)
+{
+    gint xres, yres, i, j;
+    gdouble z;
+    const gdouble *s;
+    gdouble *d;
+
+    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
+    g_return_if_fail(GWY_IS_DATA_LINE(shifts));
+    xres = data_field->xres;
+    yres = data_field->yres;
+    g_return_if_fail(shifts->res == yres);
+
+    s = shifts->data;
+    d = data_field->data;
+
+    for (i = 0; i < yres; i++) {
+        z = s[i];
+        for (j = xres; j; j--, d++)
+            *d -= z;
+    }
+
+    gwy_data_field_invalidate(data_field);
+}
+
+static gdouble
+trimmed_mean_or_median(gdouble *array, guint n, gdouble p)
+{
+    guint nlowest = GWY_ROUND(p*n);
+    guint nhighest = GWY_ROUND(p*n);
+
+    if (nlowest + nhighest + 1 >= n)
+        return gwy_math_median(n, array);
+    return gwy_math_trimmed_mean(n, array, nlowest, nhighest);
+}
+
+static void
+zero_level_row_shifts(GwyDataLine *shifts)
+{
+    gwy_data_line_add(shifts, -gwy_data_line_get_avg(shifts));
+}
+
+static void
+slope_level_row_shifts(GwyDataLine *shifts)
+{
+    gdouble a, b;
+
+    gwy_data_line_get_line_coeffs(shifts, &a, &b);
+    gwy_data_line_line_level(shifts, a, b);
+}
+
+/**
+ * gwy_data_field_find_row_shifts_trimmed_mean:
+ * @data_field: A data field.
+ * @mask: Mask of values to take values into account/exclude, or %NULL for full
+ *        @data_field.
+ * @masking: Masking mode to use.  See the introduction for description of
+ *           masking modes.
+ * @trimfrac: Fraction of lowest values and highest values to discard when
+ *            trimming.
+ * @mincount: Minimum number of values in a row necessary for per-row
+ *            calculation.  Rows which are essentially completely masked are
+ *            not shifted with respect to a global value.  Pass a non-positive
+ *            number to use an automatic minimum count.
+ *
+ * Finds row shifts to misaligned row correction using trimmed row means.
+ *
+ * For zero @trimfrac the function calculates row means.  For @trimfrac of 1/2
+ * or larger it calculates row medians.  Values between correspond to trimmed
+ * means.
+ *
+ * Returns: A newly created data line containing the row shifts, for instance
+ *          row means, medians or trimmed means.
+ *
+ * Since: 2.52
+ **/
+GwyDataLine*
+gwy_data_field_find_row_shifts_trimmed_mean(GwyDataField *data_field,
+                                            GwyDataField *mask,
+                                            GwyMaskingType masking,
+                                            gdouble trimfrac,
+                                            gint mincount)
+{
+    GwyDataLine *shifts;
+    gint xres, yres, i, j, count;
+    const gdouble *d, *m, *row, *mrow;
+    gdouble total_median;
+    gdouble *buf;
+
+    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), NULL);
+    g_return_val_if_fail(trimfrac >= 0.0, NULL);
+    xres = data_field->xres;
+    yres = data_field->yres;
+    if (masking == GWY_MASK_IGNORE)
+        mask = NULL;
+    else if (mask) {
+        g_return_val_if_fail(GWY_IS_DATA_FIELD(mask), NULL);
+        g_return_val_if_fail(mask->xres == xres, NULL);
+        g_return_val_if_fail(mask->yres == yres, NULL);
+    }
+    else
+        masking = GWY_MASK_IGNORE;
+
+    if (mincount <= 0)
+        mincount = GWY_ROUND(log(xres) + 1);
+
+    shifts = gwy_data_line_new(yres, data_field->yreal, FALSE);
+    shifts->off = data_field->yoff;
+    gwy_data_field_copy_units_to_data_line(data_field, shifts);
+    total_median = gwy_data_field_area_get_median_mask(data_field,
+                                                       mask, masking,
+                                                       0, 0, xres, yres);
+
+    d = gwy_data_field_get_data_const(data_field);
+    m = mask ? gwy_data_field_get_data_const(mask) : NULL;
+    buf = g_new(gdouble, xres);
+
+    for (i = 0; i < yres; i++) {
+        if (mask) {
+            row = d + i*xres;
+            mrow = m + i*xres;
+            count = 0;
+            if (masking == GWY_MASK_INCLUDE) {
+                for (j = 0; j < xres; j++) {
+                    if (mrow[j] > 0.0)
+                        buf[count++] = row[j];
+                }
+            }
+            else {
+                for (j = 0; j < xres; j++) {
+                    if (mrow[j] < 1.0)
+                        buf[count++] = row[j];
+                }
+            }
+            if (count >= mincount)
+                shifts->data[i] = trimmed_mean_or_median(buf, count, trimfrac);
+            else
+                shifts->data[i] = total_median;
+        }
+        else {
+            gwy_assign(buf, d + i*xres, xres);
+            shifts->data[i] = trimmed_mean_or_median(buf, xres, trimfrac);
+        }
+    }
+
+    g_free(buf);
+
+    zero_level_row_shifts(shifts);
+    return shifts;
+}
+
+/**
+ * gwy_data_field_find_row_shifts_trimmed_diff:
+ * @data_field: A data field.
+ * @mask: Mask of values to take values into account/exclude, or %NULL for full
+ *        @data_field.
+ * @masking: Masking mode to use.  See the introduction for description of
+ *           masking modes.
+ * @trimfrac: Fraction of lowest values and highest values to discard when
+ *            trimming.
+ * @mincount: Minimum number of values in a row necessary for per-row
+ *            calculation.  Rows which are essentially completely masked are
+ *            not shifted with respect to a global value.  Pass a non-positive
+ *            number to use an automatic minimum count.
+ *
+ * Finds row shifts to misaligned row correction using trimmed means of row
+ * differences.
+ *
+ * For zero @trimfrac the function calculates row means.  For @trimfrac of 1/2
+ * or larger it calculates row medians.  Values between correspond to trimmed
+ * means.
+ *
+ * Returns: A newly created data line containing the row shifts, for instance
+ *          row means, medians or trimmed means.
+ *
+ * Since: 2.52
+ **/
+GwyDataLine*
+gwy_data_field_find_row_shifts_trimmed_diff(GwyDataField *data_field,
+                                            GwyDataField *mask,
+                                            GwyMaskingType masking,
+                                            gdouble trimfrac,
+                                            gint mincount)
+{
+    GwyDataLine *shifts;
+    gint xres, yres, i, j, count;
+    const gdouble *d, *m, *row, *mrow;
+    gdouble z;
+    gdouble *buf;
+
+    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), NULL);
+    g_return_val_if_fail(trimfrac >= 0.0, NULL);
+    xres = data_field->xres;
+    yres = data_field->yres;
+    if (masking == GWY_MASK_IGNORE)
+        mask = NULL;
+    else if (mask) {
+        g_return_val_if_fail(GWY_IS_DATA_FIELD(mask), NULL);
+        g_return_val_if_fail(mask->xres == xres, NULL);
+        g_return_val_if_fail(mask->yres == yres, NULL);
+    }
+    else
+        masking = GWY_MASK_IGNORE;
+
+    if (mincount <= 0)
+        mincount = GWY_ROUND(log(xres) + 1);
+
+    shifts = gwy_data_line_new(yres, data_field->yreal, FALSE);
+    shifts->off = data_field->yoff;
+    gwy_data_field_copy_units_to_data_line(data_field, shifts);
+
+    d = gwy_data_field_get_data_const(data_field);
+    m = mask ? gwy_data_field_get_data_const(mask) : NULL;
+    buf = g_new(gdouble, xres);
+
+    shifts->data[0] = 0.0;
+    for (i = 0; i+1 < yres; i++) {
+        row = d + i*xres;
+        count = 0;
+        if (masking == GWY_MASK_INCLUDE) {
+            mrow = m + i*xres;
+            for (j = 0; j < xres; j++) {
+                if (mrow[j] <= 1.0 || mrow[xres + j] <= 1.0)
+                    continue;
+                buf[count++] = row[xres + j] - row[j];
+            }
+        }
+        else if (masking == GWY_MASK_EXCLUDE) {
+            mrow = m + i*xres;
+            for (j = 0; j < xres; j++) {
+                if (mrow[j] >= 1.0 || mrow[xres + j] >= 1.0)
+                    continue;
+                buf[count++] = row[xres + j] - row[j];
+            }
+        }
+        else {
+            for (j = 0; j < xres; j++)
+                buf[j] = row[xres + j] - row[j];
+            count = xres;
+        }
+
+        z = shifts->data[i];
+        if (count >= mincount)
+            z += trimmed_mean_or_median(buf, count, trimfrac);
+        shifts->data[i+1] = z;
+    }
+
+    g_free(buf);
+
+    slope_level_row_shifts(shifts);
+    return shifts;
+}
+
 /************************** Documentation ****************************/
 
 /**
