@@ -1,5 +1,5 @@
 /*
- *  $Id: correlation.c 20787 2018-02-01 21:53:31Z yeti-dn $
+ *  $Id: correlation.c 21456 2018-09-21 13:01:23Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1271,12 +1271,12 @@ gwy_data_field_correlation_search(GwyDataField *dfield,
     target->yreal = dfield->yreal;
     target->xoff = dfield->xoff;
     target->yoff = dfield->yoff;
-    _gwy_assign_si_unit(dfield->si_unit_xy, &target->si_unit_xy);
+    _gwy_copy_si_unit(dfield->si_unit_xy, &target->si_unit_xy);
 
     /* Scores are always unitless, even for HEIGHT_DIFF, because we normalise
      * by (local) rms of the same field. */
     if (is_score)
-        _gwy_assign_si_unit(NULL, &target->si_unit_z);
+        _gwy_copy_si_unit(NULL, &target->si_unit_z);
     else if (method == GWY_CORR_SEARCH_COVARIANCE_RAW
              || method == GWY_CORR_SEARCH_COVARIANCE) {
         funit = gwy_data_field_get_si_unit_z(dfield);
@@ -1380,15 +1380,15 @@ gwy_data_field_correlation_search(GwyDataField *dfield,
     make_symmetrical_extension(yres, ysize, &extend_up, &extend_down);
 
     cstride = xsize/2 + 1;
-    cbufA = g_new(fftw_complex, cstride*ysize);
-    cbufB = g_new(fftw_complex, cstride*ysize);
-    extdata = g_new(gdouble, xsize*ysize);
+    cbufA = gwy_fftw_new_complex(cstride*ysize);
+    cbufB = gwy_fftw_new_complex(cstride*ysize);
+    extdata = gwy_fftw_new_real(xsize*ysize);
     t = target->data;
     fplan = fftw_plan_dft_r2c_2d(ysize, xsize, extdata, cbufB,
-                                 FFTW_DESTROY_INPUT | _GWY_FFTW_PATIENCE);
+                                 FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
     g_assert(fplan);
     bplan = fftw_plan_dft_c2r_2d(ysize, xsize, cbufB, extdata,
-                                 FFTW_DESTROY_INPUT | _GWY_FFTW_PATIENCE);
+                                 FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
     g_assert(bplan);
 
     extend_rect(dfield->data, xres, extdata, xsize,
@@ -1465,9 +1465,9 @@ gwy_data_field_correlation_search(GwyDataField *dfield,
     }
 
     g_free(u);
-    g_free(cbufB);
-    g_free(cbufA);
-    g_free(extdata);
+    fftw_free(cbufB);
+    fftw_free(cbufA);
+    fftw_free(extdata);
     fftw_destroy_plan(bplan);
     fftw_destroy_plan(fplan);
 
