@@ -1,6 +1,6 @@
 /*
- *  $Id: datafield.h 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
+ *  $Id: datafield.h 21451 2018-09-21 09:24:20Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -75,6 +75,13 @@ void gwy_data_field_invalidate(GwyDataField *data_field);
 
 #define gwy_data_field_duplicate(data_field) \
         (GWY_DATA_FIELD(gwy_serializable_duplicate(G_OBJECT(data_field))))
+#define gwy_data_field_assign(dest, source) \
+        gwy_serializable_clone_with_type(G_OBJECT(source), \
+                                         G_OBJECT(dest), \
+                                         GWY_TYPE_DATA_FIELD)
+
+#define gwy_data_field_get_xmeasure gwy_data_field_get_dx
+#define gwy_data_field_get_ymeasure gwy_data_field_get_dy
 
 GType             gwy_data_field_get_type  (void) G_GNUC_CONST;
 
@@ -135,10 +142,8 @@ void              gwy_data_field_set_xreal           (GwyDataField *data_field,
                                                       gdouble xreal);
 void              gwy_data_field_set_yreal           (GwyDataField *data_field,
                                                       gdouble yreal);
-#define gwy_data_field_get_xmeasure(data_field) \
-    (gwy_data_field_get_xreal(data_field)/gwy_data_field_get_xres(data_field))
-#define gwy_data_field_get_ymeasure(data_field) \
-    (gwy_data_field_get_yreal(data_field)/gwy_data_field_get_yres(data_field))
+gdouble           gwy_data_field_get_dx              (GwyDataField *data_field);
+gdouble           gwy_data_field_get_dy              (GwyDataField *data_field);
 gdouble           gwy_data_field_get_xoffset         (GwyDataField *data_field);
 gdouble           gwy_data_field_get_yoffset         (GwyDataField *data_field);
 void              gwy_data_field_set_xoffset         (GwyDataField *data_field,
@@ -220,6 +225,7 @@ void          gwy_data_field_multiply        (GwyDataField *data_field,
                                               gdouble value);
 void          gwy_data_field_add             (GwyDataField *data_field,
                                               gdouble value);
+void          gwy_data_field_abs             (GwyDataField *data_field);
 void          gwy_data_field_area_fill       (GwyDataField *data_field,
                                               gint col,
                                               gint row,
@@ -251,6 +257,11 @@ void          gwy_data_field_area_add        (GwyDataField *data_field,
                                               gint width,
                                               gint height,
                                               gdouble value);
+void          gwy_data_field_area_abs        (GwyDataField *data_field,
+                                              gint col,
+                                              gint row,
+                                              gint width,
+                                              gint height);
 GwyDataLine*  gwy_data_field_get_profile     (GwyDataField *data_field,
                                               GwyDataLine* data_line,
                                               gint scol,
