@@ -1,6 +1,6 @@
 /*
- *  $Id: dataline.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
+ *  $Id: dataline.c 21426 2018-09-10 15:24:56Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -352,8 +352,8 @@ gwy_data_line_clone_real(GObject *source, GObject *copy)
     gwy_assign(clone->data, data_line->data, data_line->res);
 
     /* SI Units can be NULL */
-    _gwy_assign_si_unit(data_line->si_unit_x, &clone->si_unit_x);
-    _gwy_assign_si_unit(data_line->si_unit_y, &clone->si_unit_y);
+    _gwy_copy_si_unit(data_line->si_unit_x, &clone->si_unit_x);
+    _gwy_copy_si_unit(data_line->si_unit_y, &clone->si_unit_y);
 }
 
 /**
@@ -631,6 +631,7 @@ void
 gwy_data_line_set_real(GwyDataLine *data_line, gdouble real)
 {
     g_return_if_fail(GWY_IS_DATA_LINE(data_line));
+    g_return_if_fail(real > 0.0);
     data_line->real = real;
 }
 
@@ -667,6 +668,26 @@ gwy_data_line_set_offset(GwyDataLine *data_line,
     data_line->off = offset;
 }
 
+/**
+ * gwy_data_line_get_dx:
+ * @data_line: A data line.
+ *
+ * Gets the sample distance (pixel size) of a data line in real units.
+ *
+ * The result is the same as
+ * gwy_data_line_get_real(data_line)/gwy_data_line_get_res(data_line).
+ *
+ * Returns: Sampling step size.
+ *
+ * Since: 2.52
+ **/
+gdouble
+gwy_data_line_get_dx(GwyDataLine *data_line)
+{
+    g_return_val_if_fail(GWY_IS_DATA_LINE(data_line), 0.0);
+    return data_line->real/data_line->res;
+}
+
 /**
  * gwy_data_line_get_si_unit_x:
  * @data_line: A data line.
@@ -822,8 +843,8 @@ gwy_data_line_copy_units(GwyDataLine *data_line,
     g_return_if_fail(GWY_IS_DATA_LINE(data_line));
     g_return_if_fail(GWY_IS_DATA_LINE(target));
 
-    _gwy_assign_si_unit(data_line->si_unit_x, &target->si_unit_x);
-    _gwy_assign_si_unit(data_line->si_unit_y, &target->si_unit_y);
+    _gwy_copy_si_unit(data_line->si_unit_x, &target->si_unit_x);
+    _gwy_copy_si_unit(data_line->si_unit_y, &target->si_unit_y);
 }
 
 /**
@@ -840,8 +861,8 @@ gwy_data_line_copy_units_to_data_field(GwyDataLine *data_line,
     g_return_if_fail(GWY_IS_DATA_LINE(data_line));
     g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
 
-    _gwy_assign_si_unit(data_line->si_unit_x, &data_field->si_unit_xy);
-    _gwy_assign_si_unit(data_line->si_unit_y, &data_field->si_unit_z);
+    _gwy_copy_si_unit(data_line->si_unit_x, &data_field->si_unit_xy);
+    _gwy_copy_si_unit(data_line->si_unit_y, &data_field->si_unit_z);
 }
 
 /**
@@ -1630,4 +1651,17 @@ gwy_data_line_sqrt(GwyDataLine *data_line)
  * typecasting.
  **/
 
+/**
+ * gwy_data_line_assign:
+ * @dest: Target data line.
+ * @source: Source data line.
+ *
+ * Convenience macro making one data line identical to another.
+ *
+ * This is just a gwy_serializable_clone() wrapper with all the necessary
+ * typecasting.
+ *
+ * Since: 2.52
+ **/
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
