@@ -1,6 +1,6 @@
 /*
- *  $Id: filters-minmax.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2003-2016 David Necas (Yeti), Petr Klapetek.
+ *  $Id: filters-minmax.c 21458 2018-09-23 21:43:40Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -23,11 +23,12 @@
 #include <stdlib.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
-#include <libprocess/filters.h>
+#include <libprocess/arithmetic.h>
 #include <libprocess/elliptic.h>
 #include <libprocess/stats.h>
 #include <libprocess/linestats.h>
 #include <libprocess/grains.h>
+#include <libprocess/filters.h>
 #include "gwyprocessinternal.h"
 
 /* Data for one row.  To be used in conjuction with MinMaxPrecomputedReq. */
@@ -86,6 +87,38 @@ static void find_required_lengths_recursive(MinMaxPrecomputedReq *req,
                                             guint blocklen,
                                             gboolean is_even);
 
+/* K-th rank filter tree.  Number of levels with special storage; the rest
+ * is in ucount[][]. */
+enum { MEDIAN_RADIX_TREE_BASE_LEVELS = 8 };
+
+/* This needs somewhat less than 2n bits, i.e. n/4 bytes.  We actually never
+ * use the last element in each quadruple – but having it enables making
+ * add/remove branch-free, using just shifts and increments/decrements. */
+typedef struct {
+    guint n;
+    guint levels;
+    guint8 *bits;         /* Bits: 1 = number is present in the set. */
+    guint8 *bcount8;      /* Nibbles: total bits set in corresponding bits[] */
+    guint8 *bcount32;     /* Bytes: sum of quadruple of bcount8[]. */
+    guint8 *bcount128;    /* Bytes: sum of quadruple of bcount32[]. */
+    guint16 *scount512;   /* Shorts: sum of quadruple of bcount128[]. */
+    guint16 *scount2048;  /* Shorts: sum of quadruple of scount512[]. */
+    guint16 *scount8192;  /* Shorts: sum of quadruple of scount2048[]. */
+    guint16 *scount32768; /* Shorts: sum of quadruple of scount8192[]. */
+    guint32 **ucount;     /* Ints: sums of quadruples of lower levels. */
+} MedianRadixTree;
+
+typedef struct {
+    guint nup;
+    guint ndown;
+    guint nleft;
+    guint nright;
+    guint *up;
+    guint *down;
+    guint *left;
+    guint *right;
+} FilterKernelBoundaries;
+
 /**
  * gwy_data_field_area_filter_minimum:
  * @data_field: A data field to apply minimum filter to.
@@ -98,6 +131,10 @@ static void find_required_lengths_recursive(MinMaxPrecomputedReq *req,
  * Filters a rectangular part of a data field with minimum filter.
  *
  * This operation is often called erosion filter.
+ *
+ * This method uses a simple square kernel.  Use the general function
+ * gwy_data_field_area_filter_min_max() to perform filtering with different,
+ * for instance circular, kernel.
  **/
 void
 gwy_data_field_area_filter_minimum(GwyDataField *data_field,
@@ -113,11 +150,8 @@ gwy_data_field_area_filter_minimum(GwyDataField *data_field,
     gdouble *buf, *buf2;
     gdouble v;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width > 0 && height > 0
-                     && col + width <= data_field->xres
-                     && row + height <= data_field->yres);
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
     g_return_if_fail(size > 0);
     if (size == 1)
         return;
@@ -223,6 +257,10 @@ gwy_data_field_area_filter_minimum(GwyDataField *data_field,
  * @size: Neighbourhood size for minimum search.
  *
  * Filters a data field with minimum filter.
+ *
+ * This method uses a simple square kernel.  Use the general function
+ * gwy_data_field_area_filter_min_max() to perform filtering with different,
+ * for instance circular, kernel.
  **/
 void
 gwy_data_field_filter_minimum(GwyDataField *data_field,
@@ -245,6 +283,10 @@ gwy_data_field_filter_minimum(GwyDataField *data_field,
  * Filters a rectangular part of a data field with maximum filter.
  *
  * This operation is often called dilation filter.
+ *
+ * This method uses a simple square kernel.  Use the general function
+ * gwy_data_field_area_filter_min_max() to perform filtering with different,
+ * for instance circular, kernel.
  **/
 void
 gwy_data_field_area_filter_maximum(GwyDataField *data_field,
@@ -260,11 +302,8 @@ gwy_data_field_area_filter_maximum(GwyDataField *data_field,
     gdouble *buf, *buf2;
     gdouble v;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width > 0 && height > 0
-                     && col + width <= data_field->xres
-                     && row + height <= data_field->yres);
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
     g_return_if_fail(size > 0);
     if (size == 1)
         return;
@@ -370,6 +409,10 @@ gwy_data_field_area_filter_maximum(GwyDataField *data_field,
  * @size: Neighbourhood size for maximum search.
  *
  * Filters a data field with maximum filter.
+ *
+ * This method uses a simple square kernel.  Use the general function
+ * gwy_data_field_area_filter_min_max() to perform filtering with different,
+ * for instance circular, kernel.
  **/
 void
 gwy_data_field_filter_maximum(GwyDataField *data_field,
@@ -1314,12 +1357,6 @@ gwy_data_field_area_filter_min_max_real(GwyDataField *data_field,
     gdouble *outbuf, *d;
     gint i, j, xres, yres, kxres, kyres;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width > 0 && height > 0
-                     && col + width <= data_field->xres
-                     && row + height <= data_field->yres);
-
     xres = data_field->xres;
     yres = data_field->yres;
     kxres = kernel->xres;
@@ -1485,6 +1522,8 @@ gwy_data_field_area_filter_min_max(GwyDataField *data_field,
 {
     GwyDataField *redkernel;
 
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
     g_return_if_fail(GWY_IS_DATA_FIELD(kernel));
     redkernel = gwy_data_field_duplicate(kernel);
     gwy_data_field_grains_autocrop(redkernel, TRUE, NULL, NULL, NULL, NULL);
@@ -1532,11 +1571,8 @@ gwy_data_field_area_filter_disc_asf(GwyDataField *data_field,
     GwyMinMaxFilterType filtertype1, filtertype2;
     gint r;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width > 0 && height > 0
-                     && col + width <= data_field->xres
-                     && row + height <= data_field->yres);
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
 
     if (closing) {
         filtertype1 = GWY_MIN_MAX_FILTER_OPENING;
@@ -1559,4 +1595,804 @@ gwy_data_field_area_filter_disc_asf(GwyDataField *data_field,
     }
 }
 
+/**
+ * gwy_data_field_area_filter_median:
+ * @data_field: A data field to apply the filter to.
+ * @size: Size of area to take median of.
+ * @col: Upper-left column coordinate.
+ * @row: Upper-left row coordinate.
+ * @width: Area width (number of columns).
+ * @height: Area height (number of rows).
+ *
+ * Filters a rectangular part of a data field with median filter.
+ *
+ * This method uses a simple square kernel.  Use the general function
+ * gwy_data_field_area_filter_kth_rank() to perform filtering with different,
+ * for instance circular, kernel.
+ **/
+void
+gwy_data_field_area_filter_median(GwyDataField *data_field,
+                                  gint size,
+                                  gint col, gint row,
+                                  gint width, gint height)
+{
+
+    gint rowstride;
+    gint i, j, k, len;
+    gint xfrom, xto, yfrom, yto;
+    gdouble *buffer, *data, *kernel;
+
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
+    g_return_if_fail(size > 0);
+
+    buffer = g_new(gdouble, width*height);
+    kernel = g_new(gdouble, size*size);
+    rowstride = data_field->xres;
+    data = data_field->data + rowstride*row + col;
+
+    for (i = 0; i < height; i++) {
+        yfrom = MAX(0, i - (size-1)/2);
+        yto = MIN(height-1, i + size/2);
+        for (j = 0; j < width; j++) {
+            xfrom = MAX(0, j - (size-1)/2);
+            xto = MIN(width-1, j + size/2);
+            len = xto - xfrom + 1;
+            for (k = yfrom; k <= yto; k++)
+                gwy_assign(kernel + len*(k - yfrom),
+                           data + k*rowstride + xfrom,
+                           len);
+            buffer[i*width + j] = gwy_math_median(len*(yto - yfrom + 1),
+                                                  kernel);
+        }
+    }
+
+    g_free(kernel);
+    for (i = 0; i < height; i++)
+        gwy_assign(data + i*rowstride, buffer + i*width, width);
+    g_free(buffer);
+    gwy_data_field_invalidate(data_field);
+}
+
+/**
+ * gwy_data_field_filter_median:
+ * @data_field: A data field to apply the filter to.
+ * @size: Size of area to take median of.
+ *
+ * Filters a data field with median filter.
+ *
+ * This method uses a simple square kernel.  Use the general function
+ * gwy_data_field_area_filter_kth_rank() to perform filtering with different,
+ * for instance circular, kernel.
+ **/
+void
+gwy_data_field_filter_median(GwyDataField *data_field,
+                             gint size)
+{
+    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
+    gwy_data_field_area_filter_median(data_field, size, 0, 0,
+                                      data_field->xres, data_field->yres);
+}
+
+static void
+median_radix_tree_alloc(MedianRadixTree *mrtree, guint n)
+{
+    guint size, i;
+
+    gwy_clear(mrtree, 1);
+    mrtree->n = n;
+
+    size = n/8 + (n % 8 ? 1 : 0);
+    mrtree->bits = g_new0(guint8, size);
+
+    size = (size + 1)/2;
+    mrtree->bcount8 = g_new0(guint8, size);
+    mrtree->levels = 2;
+
+    size = (size + 1)/2;
+    if (size == 1)
+        return;
+    mrtree->bcount32 = g_new0(guint8, size);
+    mrtree->levels++;
+
+    size = (size + 3)/4;
+    if (size == 1)
+        return;
+    mrtree->bcount128 = g_new0(guint8, size);
+    mrtree->levels++;
+
+    size = (size + 3)/4;
+    if (size == 1)
+        return;
+    mrtree->scount512 = g_new0(guint16, size);
+    mrtree->levels++;
+
+    size = (size + 3)/4;
+    if (size == 1)
+        return;
+    mrtree->scount2048 = g_new0(guint16, size);
+    mrtree->levels++;
+
+    size = (size + 3)/4;
+    if (size == 1)
+        return;
+    mrtree->scount8192 = g_new0(guint16, size);
+    mrtree->levels++;
+
+    size = (size + 3)/4;
+    if (size == 1)
+        return;
+    mrtree->scount32768 = g_new0(guint16, size);
+    mrtree->levels++;
+
+    size = (size + 3)/4;
+    if (size == 1)
+        return;
+    mrtree->ucount = g_new0(guint32*, (CHAR_BIT*sizeof(guint) - 15)/2);
+    i = 0;
+    do {
+        mrtree->ucount[i] = g_new0(guint32, size);
+        mrtree->levels++;
+
+        size = (size + 3)/4;
+        i++;
+    } while (size > 1);
+}
+
+static void
+median_radix_tree_free(MedianRadixTree *mrtree)
+{
+    if (mrtree->ucount) {
+        guint i;
+
+        for (i = 0; i < mrtree->levels - MEDIAN_RADIX_TREE_BASE_LEVELS; i++)
+            g_free(mrtree->ucount[i]);
+        g_free(mrtree->ucount);
+    }
+
+    g_free(mrtree->scount32768);
+    g_free(mrtree->scount8192);
+    g_free(mrtree->scount2048);
+    g_free(mrtree->scount512);
+    g_free(mrtree->bcount128);
+    g_free(mrtree->bcount32);
+    g_free(mrtree->bcount8);
+    g_free(mrtree->bits);
+}
+
+/* The compiler should be clever enough to avoid repeated comparison of levels
+ * with the constants.  GCC seems to be. */
+static void
+median_radix_tree_add(MedianRadixTree *mrtree, guint v)
+{
+    guint j, levels = mrtree->levels;
+
+    j = (v & 0x7);
+    v >>= 3;
+    mrtree->bits[v] |= (1 << j);
+
+    j = (v & 0x1) ? 0x10 : 0x01;
+    mrtree->bcount8[v >> 1] += j;
+    if (levels == 2)
+        return;
+
+    v >>= 2;
+    mrtree->bcount32[v]++;
+    if (levels == 3)
+        return;
+
+    v >>= 2;
+    mrtree->bcount128[v]++;
+    if (levels == 4)
+        return;
+
+    v >>= 2;
+    mrtree->scount512[v]++;
+    if (levels == 5)
+        return;
+
+    v >>= 2;
+    mrtree->scount2048[v]++;
+    if (levels == 6)
+        return;
+
+    v >>= 2;
+    mrtree->scount8192[v]++;
+    if (levels == 7)
+        return;
+
+    v >>= 2;
+    mrtree->scount32768[v]++;
+    if (levels == 8)
+        return;
+
+    for (j = 0; j < levels - MEDIAN_RADIX_TREE_BASE_LEVELS; j++) {
+        v >>= 2;
+        mrtree->ucount[j][v]++;
+    }
+}
+
+static void
+median_radix_tree_remove(MedianRadixTree *mrtree, guint v)
+{
+    guint j, levels = mrtree->levels;
+
+    j = (v & 0x7);
+    v >>= 3;
+    mrtree->bits[v] &= ~(1 << j);
+
+    j = (v & 0x1) ? 0x10 : 0x01;
+    mrtree->bcount8[v >> 1] -= j;
+    if (levels == 2)
+        return;
+
+    v >>= 2;
+    mrtree->bcount32[v]--;
+    if (levels == 3)
+        return;
+
+    v >>= 2;
+    mrtree->bcount128[v]--;
+    if (levels == 4)
+        return;
+
+    v >>= 2;
+    mrtree->scount512[v]--;
+    if (levels == 5)
+        return;
+
+    v >>= 2;
+    mrtree->scount2048[v]--;
+    if (levels == 6)
+        return;
+
+    v >>= 2;
+    mrtree->scount8192[v]--;
+    if (levels == 7)
+        return;
+
+    v >>= 2;
+    mrtree->scount32768[v]--;
+    if (levels == 8)
+        return;
+
+    for (j = 0; j < levels - MEDIAN_RADIX_TREE_BASE_LEVELS; j++) {
+        v >>= 2;
+        mrtree->ucount[j][v]--;
+    }
+}
+
+static inline guint
+median_radix_tree_advance32(const guint32 *ucount, guint v, guint *k)
+{
+    guint s, t;
+
+    ucount += v;
+    s = ucount[0];
+    if (s <= *k) {
+        t = s;
+        s += ucount[1];
+        if (s <= *k) {
+            t = s;
+            s += ucount[2];
+            if (s <= *k) {
+                v += 3;
+                *k -= s;
+            }
+            else {
+                v += 2;
+                *k -= t;
+            }
+        }
+        else {
+            v += 1;
+            *k -= t;
+        }
+    }
+
+    return v << 2;
+}
+
+static inline guint
+median_radix_tree_advance16(const guint16 *scount, guint v, guint *k)
+{
+    guint s, t;
+
+    scount += v;
+    s = scount[0];
+    if (s <= *k) {
+        t = s;
+        s += scount[1];
+        if (s <= *k) {
+            t = s;
+            s += scount[2];
+            if (s <= *k) {
+                v += 3;
+                *k -= s;
+            }
+            else {
+                v += 2;
+                *k -= t;
+            }
+        }
+        else {
+            v += 1;
+            *k -= t;
+        }
+    }
+
+    return v << 2;
+}
+
+static inline guint
+median_radix_tree_advance8(const guint8 *bcount, guint v, guint *k)
+{
+    guint s, t;
+
+    bcount += v;
+    s = bcount[0];
+    if (s <= *k) {
+        t = s;
+        s += bcount[1];
+        if (s <= *k) {
+            t = s;
+            s += bcount[2];
+            if (s <= *k) {
+                v += 3;
+                *k -= s;
+            }
+            else {
+                v += 2;
+                *k -= t;
+            }
+        }
+        else {
+            v += 1;
+            *k -= t;
+        }
+    }
+
+    return v << 2;
+}
+
+/* This can be relatively complicated compared to add/remove because we run
+ * add/remove twenty or hundred times per one k-th rank invocation. */
+static guint
+median_radix_tree_kth_rank(const MedianRadixTree *mrtree, guint k)
+{
+    guint v, j, s, t, levels = mrtree->levels;
+    const guint8 *bcount8;
+
+    v = 0;
+    if (levels > MEDIAN_RADIX_TREE_BASE_LEVELS) {
+        for (j = levels - MEDIAN_RADIX_TREE_BASE_LEVELS; j; j--)
+            v = median_radix_tree_advance32(mrtree->ucount[j-1], v, &k);
+    }
+
+    if (levels >= 8)
+        v = median_radix_tree_advance16(mrtree->scount32768, v, &k);
+    if (levels >= 7)
+        v = median_radix_tree_advance16(mrtree->scount8192, v, &k);
+    if (levels >= 6)
+        v = median_radix_tree_advance16(mrtree->scount2048, v, &k);
+    if (levels >= 5)
+        v = median_radix_tree_advance16(mrtree->scount512, v, &k);
+    if (levels >= 4)
+        v = median_radix_tree_advance8(mrtree->bcount128, v, &k);
+    if (levels >= 3)
+        v = median_radix_tree_advance8(mrtree->bcount32, v, &k);
+
+    bcount8 = mrtree->bcount8 + v/2;
+    s = (bcount8[0] & 0x0f);
+    if (s <= k) {
+        t = s;
+        s += (bcount8[0] >> 4);
+        if (s <= k) {
+            t = s;
+            s += (bcount8[1] & 0x0f);
+            if (s <= k) {
+                v += 3;
+                k -= s;
+            }
+            else {
+                v += 2;
+                k -= t;
+            }
+        }
+        else {
+            v += 1;
+            k -= t;
+        }
+    }
+
+    t = mrtree->bits[v];
+    for (j = 0; j < 8; j++) {
+        if (t & 1) {
+            if (!k)
+                break;
+            k--;
+        }
+        t >>= 1;
+    }
+
+    return (v << 3) + j;
+}
+
+static void
+filter_kernel_boundaries_create(GwyDataField *kernel,
+                                FilterKernelBoundaries *kbound,
+                                guint xres)
+{
+    GArray *up, *down, *left, *right;
+    gdouble *data = kernel->data;
+    guint kxres = kernel->xres, kyres = kernel->yres;
+    guint i, j, k, kk;
+
+    up = g_array_new(FALSE, FALSE, sizeof(guint));
+    down = g_array_new(FALSE, FALSE, sizeof(guint));
+    left = g_array_new(FALSE, FALSE, sizeof(guint));
+    right = g_array_new(FALSE, FALSE, sizeof(guint));
+
+    k = 0;
+    for (i = 0; i < kyres; i++) {
+        kk = i*xres;
+        for (j = 0; j < kxres; j++, k++, kk++) {
+            if (data[k] <= 0.0)
+                continue;
+
+            /* Use xres from the image, not the kernel.  This means the indices
+             * are directly usable with the image. */
+            if (!i || data[k-kxres] <= 0.0)
+                g_array_append_val(up, kk);
+            if (!j || data[k-1] <= 0.0)
+                g_array_append_val(left, kk);
+            if (j == kxres-1 || data[k+1] <= 0.0)
+                g_array_append_val(right, kk);
+            if (i == kyres-1 || data[k+kxres] <= 0.0)
+                g_array_append_val(down, kk);
+        }
+    }
+
+    kbound->up = g_new(guint, up->len + left->len + right->len + down->len);
+    gwy_assign(kbound->up, up->data, up->len);
+    kbound->nup = up->len;
+    k = kbound->nup;
+
+    kbound->down = kbound->up + k;
+    gwy_assign(kbound->down, down->data, down->len);
+    kbound->ndown = down->len;
+    k += kbound->ndown;
+
+    kbound->left = kbound->up + k;
+    gwy_assign(kbound->left, left->data, left->len);
+    kbound->nleft = left->len;
+    k += kbound->nleft;
+
+    kbound->right = kbound->up + k;
+    gwy_assign(kbound->right, right->data, right->len);
+    kbound->nright = right->len;
+
+    g_array_free(up, TRUE);
+    g_array_free(down, TRUE);
+    g_array_free(left, TRUE);
+    g_array_free(right, TRUE);
+}
+
+static void
+filter_kernel_boundaries_free(FilterKernelBoundaries *kbound)
+{
+    g_free(kbound->up);
+}
+
+/* We do signed arithmetic with the dimensions here */
+static GwyDataField*
+crop_extend_field_for_kernel(GwyDataField *dfield,
+                             gint col, gint row, gint w, gint h,
+                             GwyDataField *kernel)
+{
+    GwyDataField *tmp;
+    gint xres = dfield->xres, yres = dfield->yres;
+    gint kxres = kernel->xres, kyres = kernel->yres;
+    gint extcol, extrow, extw, exth, extxend, extyend;
+
+    /* Extfield area origin in @dfield (can stick outside). */
+    extcol = col - kxres/2;
+    extrow = row - kyres/2;
+    /* Extfield dimensions. */
+    extw = w + kxres-1;
+    exth = h + kyres-1;
+    /* Extfield area bottom right corner+1 in @dfield (can stick outside). */
+    extxend = extcol + extw;
+    extyend = extrow + exth;
+
+    /* Crop-only; also reduces to plain duplicate() for exact size match. */
+    if (extcol >= 0 && extrow >= 0 && extxend <= xres && extyend <= yres) {
+        return gwy_data_field_area_extract(dfield,
+                                           extcol, extrow, extw, exth);
+    }
+
+    /* Extend-only */
+    if (extcol <= 0 && extrow <= 0 && extxend >= xres && extyend >= yres) {
+        return gwy_data_field_extend(dfield,
+                                     -extcol, extxend-xres,
+                                     -extrow, extyend-yres,
+                                     GWY_EXTERIOR_BORDER_EXTEND, 0.0, FALSE);
+    }
+
+    /* When we both crop and extend, crop first, then border-extend the
+     * parts we need to extend. */
+    tmp = gwy_data_field_area_extract(dfield,
+                                      MAX(extcol, 0), MAX(extrow, 0),
+                                      MIN(extxend, xres) - MAX(extcol, 0),
+                                      MIN(extyend, yres) - MAX(extrow, 0));
+    dfield = gwy_data_field_extend(tmp,
+                                   MAX(-extcol, 0), MAX(extxend-xres, 0),
+                                   MAX(-extrow, 0), MAX(extyend-yres, 0),
+                                   GWY_EXTERIOR_BORDER_EXTEND, 0.0, FALSE);
+    g_object_unref(tmp);
+
+    return dfield;
+}
+
+static gboolean
+area_rank_filter_kernel_radixtree(GwyDataField *dfield,
+                                  guint col, guint row, guint w, guint h,
+                                  GwyDataField *kernel, guint k,
+                                  GwySetFractionFunc set_fraction)
+{
+    GwyDataField *extfield;
+    guint xres = dfield->xres;
+    guint kxres = kernel->xres, kyres = kernel->yres;
+    guint i, j, n, kn, extxres, extyres, extn, workdone;
+    MedianRadixTree mrtree;
+    FilterKernelBoundaries kbound;
+    guint *revindex, *ranks, *rr, *filtered;
+    gdouble *d, *e, *kd;
+    gboolean ok = FALSE;
+
+    /* Figure out the field area we need to execute the filter. */
+    extfield = crop_extend_field_for_kernel(dfield, col, row, w, h, kernel);
+    extxres = extfield->xres;
+    extyres = extfield->yres;
+    extn = extxres * extyres;
+
+    /* Perform rank-transform of the data. */
+    revindex = g_new(guint, extn);
+    for (i = 0; i < extn; i++)
+        revindex[i] = i;
+    gwy_math_sort_with_index(extn, extfield->data, revindex);
+
+    ranks = g_new(guint, extn);
+    for (i = 0; i < extn; i++)
+        ranks[revindex[i]] = i;
+
+    /* Initialise the tree with ranks corresponding to kernel in top left
+     * corner. */
+    median_radix_tree_alloc(&mrtree, extn);
+    kd = kernel->data;
+    kn = 0;
+    for (i = 0; i < kyres; i++) {
+        for (j = 0; j < kxres; j++) {
+            if (kd[i*kxres + j] > 0.0) {
+                median_radix_tree_add(&mrtree, ranks[i*extxres + j]);
+                kn++;
+            }
+        }
+    }
+
+    /* Analyse kernel boundaries. */
+    filter_kernel_boundaries_create(kernel, &kbound, extxres);
+
+    /* Go vertically in the inner loop because then we tend to access ranks[]
+     * in contiguous blocks.  Recycle the revindex[] array as filtered[]. */
+    filtered = revindex;
+    workdone = 0;
+    j = 0;
+    while (TRUE) {
+        if (j % 2 == 0) {
+            /* Downward pass. */
+            i = 0;
+            while (i < h-1) {
+                filtered[i*w + j] = median_radix_tree_kth_rank(&mrtree, k);
+                rr = ranks + i*extxres + j;
+                for (n = 0; n < kbound.nup; n++)
+                    median_radix_tree_remove(&mrtree, rr[kbound.up[n]]);
+                i++;
+                rr = ranks + i*extxres + j;
+                for (n = 0; n < kbound.ndown; n++)
+                    median_radix_tree_add(&mrtree, rr[kbound.down[n]]);
+            }
+        }
+        else {
+            /* Upward pass. */
+            i = h-1;
+            while (i) {
+                filtered[i*w + j] = median_radix_tree_kth_rank(&mrtree, k);
+                rr = ranks + i*extxres + j;
+                for (n = 0; n < kbound.ndown; n++)
+                    median_radix_tree_remove(&mrtree, rr[kbound.down[n]]);
+                i--;
+                rr = ranks + i*extxres + j;
+                for (n = 0; n < kbound.nup; n++)
+                    median_radix_tree_add(&mrtree, rr[kbound.up[n]]);
+            }
+        }
+        filtered[i*w + j] = median_radix_tree_kth_rank(&mrtree, k);
+
+        /* Move right. */
+        if (j == w-1)
+            break;
+
+        rr = ranks + i*extxres + j;
+        for (n = 0; n < kbound.nleft; n++)
+            median_radix_tree_remove(&mrtree, rr[kbound.left[n]]);
+        j++;
+        rr = ranks + i*extxres + j;
+        for (n = 0; n < kbound.nright; n++)
+            median_radix_tree_add(&mrtree, rr[kbound.right[n]]);
+
+        workdone += kn*h;
+        if (set_fraction && workdone >= 1000000) {
+            if (!set_fraction(j/(gdouble)w))
+                goto finalize;
+            workdone = 0;
+        }
+    }
+
+    d = dfield->data + row*xres + col;
+    e = extfield->data;
+    for (i = 0; i < h; i++) {
+        for (j = 0; j < w; j++)
+            d[i*xres + j] = e[filtered[i*w + j]];
+    }
+    ok = TRUE;
+
+finalize:
+    g_free(ranks);
+    g_free(filtered);  /* == revindex */
+    g_object_unref(extfield);
+    median_radix_tree_free(&mrtree);
+    filter_kernel_boundaries_free(&kbound);
+
+    return ok;
+}
+
+static void
+area_rank_filter_kernel_direct(GwyDataField *dfield,
+                               guint col, guint row, guint w, guint h,
+                               GwyDataField *kernel, guint k,
+                               guint ksize)
+{
+    GwyDataField *extfield;
+    guint xres = dfield->xres;
+    guint kxres = kernel->xres, kyres = kernel->yres;
+    guint i, j, n, extxres;
+    guint *kindex;
+    gdouble *buf, *e, *d, *kd;
+
+    /* Figure out the field area we need to execute the filter. */
+    extfield = crop_extend_field_for_kernel(dfield, col, row, w, h, kernel);
+    extxres = extfield->xres;
+
+    kindex = g_new(guint, ksize);
+    buf = g_new(gdouble, ksize);
+    ksize = 0;
+    kd = kernel->data;
+    for (i = 0; i < kyres; i++) {
+        for (j = 0; j < kxres; j++) {
+            if (kd[i*kxres + j] > 0.0)
+                kindex[ksize++] = i*extxres + j;
+        }
+    }
+
+    d = dfield->data + row*xres + col;
+    for (i = 0; i < h; i++) {
+        for (j = 0; j < w; j++) {
+            e = extfield->data + i*extxres + j;
+            for (n = 0; n < ksize; n++)
+                buf[n] = e[kindex[n]];
+            d[i*xres + j] = gwy_math_kth_rank(ksize, buf, k);
+        }
+    }
+
+    g_free(buf);
+    g_free(kindex);
+    g_object_unref(extfield);
+}
+
+/**
+ * gwy_data_field_area_filter_kth_rank:
+ * @data_field: A data field to apply the filter to.
+ * @kernel: Data field defining the kernel shape.
+ * @col: Upper-left column coordinate.
+ * @row: Upper-left row coordinate.
+ * @width: Area width (number of columns).
+ * @height: Area height (number of rows).
+ * @k: Rank of the value to store as the output (from lowest to highest).
+ * @set_fraction: Function that sets fraction to output (or %NULL).
+ *
+ * Applies a @k-th rank filter to a part of a data field.
+ *
+ * Pass half the number of non-zero values in @kernel as @k for a median
+ * filter.
+ *
+ * The kernel field is a mask that defines the shape of the kernel.  You can
+ * use gwy_data_field_elliptic_area_fill() to create a true circular (or
+ * elliptical) kernel.  The kernel must be non-empty.
+ *
+ * The kernel is implicitly centered, i.e. it will be applied symmetrically to
+ * avoid unexpected data movement.  Even-sized kernels (generally not
+ * recommended) will extend farther towards the top left image corner for
+ * minimum (erosion) and towards the bottom right corner for maximum (dilation)
+ * operations due to the reflection.  If you need off-center structuring
+ * elements you can add empty rows or columns to one side of the kernel to
+ * counteract the symmetrisation.
+ *
+ * The exterior is always handled as %GWY_EXTERIOR_BORDER_EXTEND.
+ *
+ * If the operation is aborted the contents of @data_field is untouched.
+ *
+ * Returns: %TRUE if the operation was not aborted via @set_fraction
+ *          returning %FALSE; %FALSE if it was aborted.
+ *
+ * Since: 2.51
+ **/
+gboolean
+gwy_data_field_area_filter_kth_rank(GwyDataField *data_field,
+                                    GwyDataField *kernel,
+                                    gint col, gint row,
+                                    gint width, gint height,
+                                    gint k,
+                                    GwySetFractionFunc set_fraction)
+{
+    GwyDataField *redkernel;
+    gdouble *kd;
+    guint i, n;
+    gboolean ok;
+
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return FALSE;
+    g_return_val_if_fail(GWY_IS_DATA_FIELD(kernel), FALSE);
+
+    kd = kernel->data;
+    n = 0;
+    for (i = 0; i < kernel->xres * kernel->yres; i++) {
+        if (kd[i] > 0.0)
+            n++;
+    }
+    /* This also fails for an empty kernel. */
+    g_return_val_if_fail(k < n, FALSE);
+
+    /* Catch some simpler cases (currently cannot be aborted). */
+    if (k == 0) {
+        gwy_data_field_area_filter_min_max(data_field, kernel,
+                                           GWY_MIN_MAX_FILTER_MINIMUM,
+                                           col, row, width, height);
+        return TRUE;
+    }
+    if (k == n-1) {
+        gwy_data_field_area_filter_min_max(data_field, kernel,
+                                           GWY_MIN_MAX_FILTER_MAXIMUM,
+                                           col, row, width, height);
+        return TRUE;
+    }
+
+    redkernel = gwy_data_field_duplicate(kernel);
+    gwy_data_field_grains_autocrop(redkernel, TRUE, NULL, NULL, NULL, NULL);
+    if (n <= 25) {
+        /* The filter should be fast for tiny kernels. */
+        ok = TRUE;
+        area_rank_filter_kernel_direct(data_field, col, row, width, height,
+                                       redkernel, k, n);
+    }
+    else {
+        ok = area_rank_filter_kernel_radixtree(data_field,
+                                               col, row, width, height,
+                                               redkernel, k, set_fraction);
+    }
+    g_object_unref(redkernel);
+
+    return ok;
+}
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
