@@ -1,5 +1,5 @@
 /*
- *  $Id: grains-wshed.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: grains-wshed.c 21426 2018-09-10 15:24:56Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -822,7 +822,7 @@ gwy_data_field_waterpour(GwyDataField *data_field,
     result->xoff = data_field->xoff;
     result->yoff = data_field->yoff;
 
-    _gwy_assign_si_unit(data_field->si_unit_xy, &result->si_unit_xy);
+    _gwy_copy_si_unit(data_field->si_unit_xy, &result->si_unit_xy);
     if (result->si_unit_z)
         gwy_si_unit_set_from_string(result->si_unit_z, NULL);
 
