@@ -1,6 +1,6 @@
 /*
- *  $Id: grains.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
+ *  $Id: grains.c 21458 2018-09-23 21:43:40Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  Copyright (C) 2013 Brazilian Nanotechnology National Laboratory
@@ -55,8 +55,8 @@ static gint     gwy_data_field_fill_one_grain(gint xres,
                                               gint row,
                                               gint *visited,
                                               gint grain_no,
-                                              gint *listv,
-                                              gint *listh);
+                                              IntList *listv,
+                                              IntList *listh);
 
 /**
  * gwy_data_field_grains_mark_height:
@@ -916,6 +916,16 @@ gwy_data_field_number_grains(GwyDataField *mask_field,
  * This function differs from gwy_data_field_number_grains() by the assumption
  * of periodicity, i.e. grains can touch across the opposite field edges.
  *
+ * Note that some grain operations assume that grains are contiguous within the
+ * image and do not work with periodic grains.
+ *
+ * You can use gwy_data_field_get_grain_sizes() and there is
+ * gwy_data_field_get_grain_bounding_boxes_periodic() for bouding boxes of
+ * periodic grains.  As for grain quantities, simple quantities that do not
+ * depend on the shape (mean, median, etc.) are evaluated correctly even for
+ * periodic grains.  You cannot evaluate quantities depending on grain
+ * boundaries though.
+ *
  * Returns: The number of last grain (note they are numbered from 1).
  *
  * Since: 2.38
@@ -963,6 +973,38 @@ gwy_data_field_number_grains_periodic(GwyDataField *mask_field,
     return ngrains;
 }
 
+static inline void
+init_bbox_ranges(gint *bboxes, gint ngrains)
+{
+    gint i;
+
+    for (i = 1; i <= ngrains; i++) {
+        bboxes[4*i] = bboxes[4*i + 1] = G_MAXINT;
+        bboxes[4*i + 2] = bboxes[4*i + 3] = -1;
+    }
+}
+
+static inline void
+extend_bbox_range(gint *bboxes, gint id, gint j, gint i)
+{
+    id *= 4;
+    if (j < bboxes[id])
+        bboxes[id] = j;
+    if (i < bboxes[id + 1])
+        bboxes[id + 1] = i;
+    if (j > bboxes[id + 2])
+        bboxes[id + 2] = j;
+    if (i > bboxes[id + 3])
+        bboxes[id + 3] = i;
+}
+
+static inline void
+transform_range_to_bbox(gint *bbox)
+{
+    bbox[2] = bbox[2] + 1 - bbox[0];
+    bbox[3] = bbox[3] + 1 - bbox[1];
+}
+
 /**
  * gwy_data_field_get_grain_bounding_boxes:
  * @mask_field: Data field containing positive values in grains, nonpositive
@@ -973,13 +1015,14 @@ gwy_data_field_number_grains_periodic(GwyDataField *mask_field,
  *           gwy_data_field_number_grains().
  * @grains: Grain numbers filled with gwy_data_field_number_grains().
  * @bboxes: Array of size at least 4*(@ngrains+1) to fill with grain bounding
- *          boxes (as usual zero does not correspond to any grain, grains
- *          start from 1). The bounding boxes are stored as quadruples of
- *          indices: (xmin, ymin, width, height).  It can be %NULL to allocate
- *          a new array.
+ *          boxes.  It can be %NULL to allocate a new array.
  *
  * Find bounding boxes of all grains.
  *
+ * As usual zeroth element of @bboxes does not correspond to any grain; grain
+ * numbers start from 1. The bounding boxes are stored as quadruples of
+ * indices: (column, row, width, height).
+ *
  * Returns: Either @bboxes (if it was not %NULL), or a newly allocated array
  *          of size 4(@ngrains + 1).
  *
@@ -992,6 +1035,7 @@ gwy_data_field_get_grain_bounding_boxes(GwyDataField *mask_field,
                                         gint *bboxes)
 {
     gint xres, yres, i, j, id;
+    const gint *grow;
 
     g_return_val_if_fail(GWY_IS_DATA_FIELD(mask_field), NULL);
     g_return_val_if_fail(grains, NULL);
@@ -1001,31 +1045,226 @@ gwy_data_field_get_grain_bounding_boxes(GwyDataField *mask_field,
     if (!bboxes)
         bboxes = g_new(gint, 4*(ngrains + 1));
 
-    for (i = 1; i <= ngrains; i++) {
-        bboxes[4*i] = bboxes[4*i + 1] = G_MAXINT;
-        bboxes[4*i + 2] = bboxes[4*i + 3] = -1;
+    init_bbox_ranges(bboxes, ngrains);
+    for (i = 0; i < yres; i++) {
+        grow = grains + i*xres;
+        for (j = 0; j < xres; j++) {
+            if ((id = grow[j]))
+                extend_bbox_range(bboxes, id, j, i);
+        }
+    }
+    for (i = 1; i <= ngrains; i++)
+        transform_range_to_bbox(bboxes + 4*i);
+
+    return bboxes;
+}
+
+/* Find bounding box from projection. */
+static void
+make_bbox_from_projection(const gboolean *projection, gint len,
+                          gint *pos, gint *size)
+{
+    gint bl, br;
+
+    if (projection[0] && projection[len-1]) {
+        /* At most one gap in the middle, possibly entire width filled. */
+        for (bl = 0; bl < len && projection[bl]; bl++)
+            ;
+        /* If all projection pixels are covered no smaller bbox is possible. */
+        if (bl == len) {
+            *pos = 0;
+            *size = len;
+        }
+        else {
+            /* There must be a gap. */
+            for (br = len-1; projection[br]; br--)
+                ;
+            *pos = br+1;
+            *size = bl + len-1-br;
+        }
+    }
+    else {
+        /* One contiguous segment, not covering the entire line.  So neither
+         * cycle needs an explicit length stop-condition. */
+        for (bl = 0; !projection[bl]; bl++)
+            ;
+        for (br = len-1; !projection[br]; br--)
+            ;
+        *pos = bl;
+        *size = br+1 - bl;
+    }
+}
+
+/**
+ * gwy_data_field_get_grain_bounding_boxes_periodic:
+ * @mask_field: Data field containing positive values in grains, nonpositive
+ *              in free space.  However its contents is ignored as all
+ *              grain information is taken from @grains (its dimensions
+ *              determine the dimensions of @grains).
+ * @ngrains: The number of grains as returned by
+ *           gwy_data_field_number_grains_periodic().
+ * @grains: Grain numbers filled with gwy_data_field_number_grains_periodic().
+ * @bboxes: Array of size at least 4*(@ngrains+1) to fill with grain bounding
+ *          boxes.  It can be %NULL to allocate a new array.
+ *
+ * Find bounding boxes of all grains.
+ *
+ * As usual zeroth element of @bboxes does not correspond to any grain; grain
+ * numbers start from 1. The bounding boxes are stored as quadruples of
+ * indices: (column, row, width, height).
+ *
+ * The row and column always lie inside the the image.  However, width and
+ * height may specify an area which sticks outside.  In this case periodicity
+ * needs to be taken into account.
+ *
+ * Returns: Either @bboxes (if it was not %NULL), or a newly allocated array
+ *          of size 4(@ngrains + 1).
+ *
+ * Since: 2.51
+ **/
+gint*
+gwy_data_field_get_grain_bounding_boxes_periodic(GwyDataField *mask_field,
+                                                 gint ngrains,
+                                                 const gint *grains,
+                                                 gint *bboxes)
+{
+    gint xres, yres, i, j, ii, k, id, ntodo;
+    gint *sbboxes;
+    gboolean *todograins, *proj_storage;
+    gboolean **xprojection, **yprojection;
+    const gint *grow;
+
+    bboxes = gwy_data_field_get_grain_bounding_boxes(mask_field,
+                                                     ngrains, grains, bboxes);
+    g_return_val_if_fail(bboxes, NULL);
+
+    xres = mask_field->xres;
+    yres = mask_field->yres;
+
+    /* Grains passing through the boundary will have full image width or height
+     * according to the normal bounding box algorithm.  If there are none we
+     * can avoid the slow path. */
+    todograins = g_new0(gboolean, ngrains+1);
+    ntodo = 0;
+    for (id = 1; id <= ngrains; id++) {
+        if (bboxes[4*id + 2] == xres || bboxes[4*id + 3] == yres) {
+            todograins[id] = TRUE;
+            ntodo++;
+        }
+        else {
+            gwy_debug("normal bbox[%d] %dx%d at (%d,%d)",
+                      id,
+                      bboxes[4*id + 2], bboxes[4*id + 3],
+                      bboxes[4*id + 0], bboxes[4*id + 1]);
+        }
+    }
+    gwy_debug("after normal bboxing %d possibly cross-border grains remaining",
+              ntodo);
+    if (!ntodo) {
+        g_free(todograins);
+        return bboxes;
     }
 
+    /* Now try virtually shifting the image periodically by half (horizontally
+     * (0), vertically (1) and both (2)) and find the bounding boxes again.
+     * This catches all grains that simply cross the boundary but are not
+     * actually too large. */
+    sbboxes = g_new(gint, 4*(ngrains + 1));
+    for (k = 0; k < 3 && ntodo; k++) {
+        init_bbox_ranges(sbboxes, ngrains);
+
+        for (i = 0; i < yres; i++) {
+            grow = grains + i*xres;
+            ii = (k == 0) ? i : (i >= yres/2 ? i : i + yres);
+            if (k == 1) {
+                /* Horizontal lines are preserved. */
+                for (j = 0; j < xres; j++) {
+                    if (todograins[id = grow[j]])
+                        extend_bbox_range(sbboxes, id, j, ii);
+                }
+            }
+            else {
+                /* Horizontal lines are split and shifted. */
+                for (j = 0; j < xres/2; j++) {
+                    if (todograins[id = grow[j]])
+                        extend_bbox_range(sbboxes, id, j + xres, ii);
+                }
+                for (j = xres/2; j < xres; j++) {
+                    if (todograins[id = grow[j]])
+                        extend_bbox_range(sbboxes, id, j, ii);
+                }
+            }
+        }
+
+        for (id = 1; id <= ngrains; id++) {
+            if (!todograins[id])
+                continue;
+
+            transform_range_to_bbox(sbboxes + 4*id);
+            if (sbboxes[4*id + 2] == xres || sbboxes[4*id + 3] == yres)
+                continue;
+
+            /* We were able to find shifted finite bbox for grain @i.
+             * Furthermore if the bbox is finite now it must begin inside
+             * the unshifted half -- otherwise it would lie *entirely* within
+             * the shifted half and be found by the normal algorithm. */
+            g_assert(sbboxes[4*id + 0] < xres);
+            g_assert(sbboxes[4*id + 1] < yres);
+            gwy_assign(bboxes + 4*id, sbboxes + 4*id, 4);
+            gwy_debug("shifted-%d bbox[%d] %dx%d at (%d,%d)",
+                      k, id,
+                      bboxes[4*id + 2], bboxes[4*id + 3],
+                      bboxes[4*id + 0], bboxes[4*id + 1]);
+            todograins[id] = FALSE;
+            ntodo--;
+        }
+    }
+    g_free(sbboxes);
+
+    gwy_debug("after shifted bboxing %d possibly cross-border grains remaining",
+              ntodo);
+    if (!ntodo) {
+        g_free(todograins);
+        return bboxes;
+    }
+
+    /* For any remaining grains, hopefully relatively few, find horizontal and
+     * vertical projections. */
+    proj_storage = g_new0(gboolean, ntodo*(xres + yres));
+    xprojection = g_new0(gboolean*, 2*(ngrains + 1));
+    yprojection = xprojection + ngrains+1;
+    j = 0;
+    for (id = 1; id <= ngrains; id++) {
+        if (!todograins[id])
+            continue;
+
+        xprojection[id] = proj_storage + j*(xres + yres);
+        yprojection[id] = proj_storage + j*(xres + yres) + xres;
+        j++;
+    }
+    g_free(todograins);
+
     for (i = 0; i < yres; i++) {
+        grow = grains + i*xres;
         for (j = 0; j < xres; j++) {
-            if ((id = grains[i*xres + j])) {
-                id *= 4;
-                if (j < bboxes[id])
-                    bboxes[id] = j;
-                if (i < bboxes[id + 1])
-                    bboxes[id + 1] = i;
-                if (j > bboxes[id + 2])
-                    bboxes[id + 2] = j;
-                if (i > bboxes[id + 3])
-                    bboxes[id + 3] = i;
+            if (xprojection[id = grow[j]]) {
+                xprojection[id][j] = TRUE;
+                yprojection[id][i] = TRUE;
             }
         }
     }
 
-    for (i = 1; i <= ngrains; i++) {
-        bboxes[4*i + 2] = bboxes[4*i + 2] + 1 - bboxes[4*i];
-        bboxes[4*i + 3] = bboxes[4*i + 3] + 1 - bboxes[4*i + 1];
+    for (id = 1; id <= ngrains; id++) {
+        if (!xprojection[id])
+            continue;
+
+        make_bbox_from_projection(xprojection[id], xres,
+                                  bboxes + 4*id + 0, bboxes + 4*id + 2);
+        make_bbox_from_projection(yprojection[id], yres,
+                                  bboxes + 4*id + 1, bboxes + 4*id + 3);
     }
+    g_free(xprojection);
+    g_free(proj_storage);
 
     return bboxes;
 }
@@ -1145,18 +1384,16 @@ gwy_data_field_area_grains_tgnd_range(GwyDataField *data_field,
                                       gboolean below,
                                       gint nstats)
 {
-    gint *heights, *hindex, *nh, *grains, *listv, *listh, *m, *mm;
+    IntList *listv, *listh;
+    gint *heights, *hindex, *nh, *grains, *m, *mm;
     gdouble q;
     gint i, j, k, h, n;
     gint grain_no, last_grain_no;
     guint msize;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
     g_return_if_fail(GWY_IS_DATA_LINE(target_line));
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width >= 0 && height >= 0
-                     && col + width <= data_field->xres
-                     && row + height <= data_field->yres);
 
     if (nstats < 1) {
         nstats = floor(3.49*cbrt(width*height) + 0.5);
@@ -1230,8 +1467,8 @@ gwy_data_field_area_grains_tgnd_range(GwyDataField *data_field,
     nh[nstats] = n;    /* To avoid special-casing in cycle limits below */
 
     grains = g_new0(gint, n);
-    listv = g_new(gint, n/2 + 2);
-    listh = g_new(gint, n/2 + 2);
+    listv = int_list_new(64);
+    listh = int_list_new(64);
 
     m = g_new(gint, 1);
     mm = m;
@@ -1332,8 +1569,8 @@ gwy_data_field_area_grains_tgnd_range(GwyDataField *data_field,
     }
 
     g_free(m);
-    g_free(listv);
-    g_free(listh);
+    int_list_free(listv);
+    int_list_free(listh);
     g_free(grains);
     g_free(hindex);
     g_free(nh);
@@ -1358,7 +1595,8 @@ static gint*
 gwy_data_field_fill_grain(GwyDataField *data_field,
                           gint col, gint row, gint *nindices)
 {
-    gint *data, *visited, *listv, *listh;
+    IntList *listv, *listh;
+    gint *data, *visited;
     gint *indices;
     gint xres, yres, n, count;
     gint i, j;
@@ -1385,8 +1623,8 @@ gwy_data_field_fill_grain(GwyDataField *data_field,
     n = xres*yres;
     visited = g_new0(gint, n);
     data = g_new(gint, n);
-    listv = g_new(gint, n/2 + 2);
-    listh = g_new(gint, n/2 + 2);
+    listv = int_list_new(64);
+    listh = int_list_new(64);
 
     for (i = 0; i < n; i++)
         data[i] = data_field->data[i] > 0;
@@ -1394,8 +1632,8 @@ gwy_data_field_fill_grain(GwyDataField *data_field,
     count = gwy_data_field_fill_one_grain(xres, yres, data, col, row,
                                           visited, 1, listv, listh);
 
-    g_free(listh);
-    g_free(listv);
+    int_list_free(listh);
+    int_list_free(listv);
     g_free(data);
 
     indices = g_new(gint, count);
@@ -1443,11 +1681,10 @@ gwy_data_field_fill_one_grain(gint xres,
                               gint col, gint row,
                               gint *visited,
                               gint grain_no,
-                              gint *listv,
-                              gint *listh)
+                              IntList *listv,
+                              IntList *listh)
 {
     gint n, count;
-    gint nh, nv;
     gint i, p, j;
     gint initial;
     gint look_for;
@@ -1468,15 +1705,16 @@ gwy_data_field_fill_one_grain(gint xres,
     }
 
     n = xres*yres;
-    listv[0] = listv[1] = initial;
-    nv = 2;
-    listh[0] = listh[1] = initial;
-    nh = 2;
+    listv->len = listh->len = 0;
+    int_list_add(listv, initial);
+    int_list_add(listv, initial);
+    int_list_add(listh, initial);
+    int_list_add(listh, initial);
 
-    while (nv) {
+    while (listv->len) {
         /* go through vertical lines and expand them horizontally */
-        for (i = 0; i < nv; i += 2) {
-            for (p = listv[i]; p <= listv[i + 1]; p += xres) {
+        for (i = 0; i < listv->len; i += 2) {
+            for (p = listv->data[i]; p <= listv->data[i + 1]; p += xres) {
                 gint start, stop;
 
                 /* scan left */
@@ -1489,8 +1727,8 @@ gwy_data_field_fill_one_grain(gint xres,
                     count++;
                 }
                 if (j < start) {
-                    listh[nh++] = j + 1;
-                    listh[nh++] = start;
+                    int_list_add(listh, j + 1);
+                    int_list_add(listh, start);
                 }
 
                 /* scan right */
@@ -1503,16 +1741,16 @@ gwy_data_field_fill_one_grain(gint xres,
                     count++;
                 }
                 if (j > start) {
-                    listh[nh++] = start;
-                    listh[nh++] = j - 1;
+                    int_list_add(listh, start);
+                    int_list_add(listh, j - 1);
                 }
             }
         }
-        nv = 0;
+        listv->len = 0;
 
         /* go through horizontal lines and expand them vertically */
-        for (i = 0; i < nh; i += 2) {
-            for (p = listh[i]; p <= listh[i + 1]; p++) {
+        for (i = 0; i < listh->len; i += 2) {
+            for (p = listh->data[i]; p <= listh->data[i + 1]; p++) {
                 gint start, stop;
 
                 /* scan up */
@@ -1525,8 +1763,8 @@ gwy_data_field_fill_one_grain(gint xres,
                     count++;
                 }
                 if (j < start) {
-                    listv[nv++] = j + xres;
-                    listv[nv++] = start;
+                    int_list_add(listv, j + xres);
+                    int_list_add(listv, start);
                 }
 
                 /* scan down */
@@ -1539,12 +1777,12 @@ gwy_data_field_fill_one_grain(gint xres,
                     count++;
                 }
                 if (j > start) {
-                    listv[nv++] = start;
-                    listv[nv++] = j - xres;
+                    int_list_add(listv, start);
+                    int_list_add(listv, j - xres);
                 }
             }
         }
-        nh = 0;
+        listh->len = 0;
     }
 
     return count;
