@@ -1,6 +1,6 @@
 /*
- *  $Id: gwygrainvalue.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2007 David Necas (Yeti).
+ *  $Id: gwygrainvalue.c 20984 2018-04-26 15:45:55Z yeti-dn $
+ *  Copyright (C) 2007-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -121,6 +121,15 @@ grain_values[] = {
             0, 1, 0, 0,
         }
     },
+    {
+        N_("RMS value"),
+        GWY_GRAIN_VALUE_RMS,
+        {
+            GWY_GRAIN_VALUE_GROUP_VALUE,
+            "<i>z</i><sub>rms</sub>", "z_rms", NULL,
+            0, 1, 0, 0,
+        }
+    },
     {
         N_("Minimum value on boundary"),
         GWY_GRAIN_VALUE_BOUNDARY_MINIMUM,
