@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyprocess.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: gwyprocess.c 21435 2018-09-11 12:39:53Z yeti-dn $
  *  Copyright (C) 2003-2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -73,7 +73,7 @@ gwy_process_type_init(void)
 
 /* Transfer units by value (gwy_foo_copy_units()). */
 void
-_gwy_assign_si_unit(GwySIUnit *source, GwySIUnit **dest)
+_gwy_copy_si_unit(GwySIUnit *source, GwySIUnit **dest)
 {
     if (source == *dest)
         return;
@@ -84,7 +84,7 @@ _gwy_assign_si_unit(GwySIUnit *source, GwySIUnit **dest)
     }
 
     if (*dest)
-        gwy_serializable_clone(G_OBJECT(source), G_OBJECT(*dest));
+        gwy_si_unit_assign(*dest, source);
     else
         *dest = gwy_si_unit_duplicate(source);
 }
