@@ -1,6 +1,6 @@
 /*
- *  $Id: gwyprocessinternal.h 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwyprocessinternal.h 21458 2018-09-23 21:43:40Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,10 +29,13 @@
 #include <libprocess/gwyprocessenums.h>
 #include <libprocess/datafield.h>
 
-G_BEGIN_DECLS
+/* Do not require FFTW 3.3 just for a couple of trivial macros. */
+#define gwy_fftw_new_real(n) \
+    (gdouble*)fftw_malloc((n)*sizeof(gdouble))
+#define gwy_fftw_new_complex(n) \
+    (fftw_complex*)fftw_malloc((n)*sizeof(fftw_complex))
 
-/* Must include fftw3.h to get it actually defined */
-#define _GWY_FFTW_PATIENCE FFTW_ESTIMATE
+G_BEGIN_DECLS
 
 /* Cache operations */
 #define CVAL(datafield, b)  ((datafield)->cache[GWY_DATA_FIELD_CACHE_##b])
@@ -82,6 +85,16 @@ RowExtendFunc _gwy_get_row_extend_func(GwyExteriorType exterior);
 G_GNUC_INTERNAL
 RectExtendFunc _gwy_get_rect_extend_func(GwyExteriorType exterior);
 
+G_GNUC_INTERNAL
+gboolean _gwy_data_field_check_area(GwyDataField *data_field,
+                                    gint col, gint row,
+                                    gint width, gint height);
+
+G_GNUC_INTERNAL
+gboolean _gwy_data_field_check_mask(GwyDataField *data_field,
+                                    GwyDataField **mask,
+                                    GwyMaskingType *masking);
+
 struct _GwySurfacePrivate {
     GwySIUnit *si_unit_xy;
     GwySIUnit *si_unit_z;
@@ -228,7 +241,7 @@ extend_kernel_rect(const gdouble *kernel,
     (((unit) && !gwy_si_unit_equal_string((unit), NULL)) ? &(unit) : NULL)
 
 G_GNUC_INTERNAL
-void _gwy_assign_si_unit(GwySIUnit *source, GwySIUnit **dest);
+void _gwy_copy_si_unit(GwySIUnit *source, GwySIUnit **dest);
 
 G_GNUC_INTERNAL
 void _gwy_set_object_si_unit(GwySIUnit *unit, GwySIUnit **objmember);
