@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyshapefitpreset.c 19380 2016-12-23 11:12:56Z pecold $
+ *  $Id: gwyshapefitpreset.c 20678 2017-12-18 18:26:55Z yeti-dn $
  *  Copyright (C) 2016 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -532,6 +532,58 @@ gwy_shape_fit_preset_get_param_units(GwyShapeFitPreset *preset,
                                       NULL);
 }
 
+/**
+ * gwy_shape_fit_preset_get_param_power_xy:
+ * @preset: A 3D geometrical shape fitting function.
+ * @i: Parameter number.
+ *
+ * Gets the power of abscissa units in a shape fitting parameter.
+ *
+ * Usually it is easier to let gwy_shape_fit_preset_get_param_units() derive
+ * the parameter units.
+ *
+ * Returns: The power of abscissa in the parameter.
+ *
+ * Since: 2.50
+ **/
+gint
+gwy_shape_fit_preset_get_param_power_xy(GwyShapeFitPreset *preset,
+                                        guint i)
+{
+    const FitShapeFunc *builtin;
+
+    g_return_val_if_fail(GWY_IS_SHAPE_FIT_PRESET(preset), 0);
+    builtin = preset->priv->builtin;
+    g_return_val_if_fail(i < builtin->nparams, 0);
+    return builtin->param[i].power_xy;
+}
+
+/**
+ * gwy_shape_fit_preset_get_param_power_z:
+ * @preset: A 3D geometrical shape fitting function.
+ * @i: Parameter number.
+ *
+ * Gets the power of ordinate units in a shape fitting parameter.
+ *
+ * Usually it is easier to let gwy_shape_fit_preset_get_param_units() derive
+ * the parameter units.
+ *
+ * Returns: The power of ordinate in the parameter.
+ *
+ * Since: 2.50
+ **/
+gint
+gwy_shape_fit_preset_get_param_power_z(GwyShapeFitPreset *preset,
+                                       guint i)
+{
+    const FitShapeFunc *builtin;
+
+    g_return_val_if_fail(GWY_IS_SHAPE_FIT_PRESET(preset), 0);
+    builtin = preset->priv->builtin;
+    g_return_val_if_fail(i < builtin->nparams, 0);
+    return builtin->param[i].power_z;
+}
+
 /**
  * gwy_shape_fit_preset_get_nsecondary:
  * @preset: A 3D geometrical shape fitting function.
@@ -706,6 +758,60 @@ gwy_shape_fit_preset_get_secondary_units(GwyShapeFitPreset *preset,
                                       NULL);
 }
 
+/**
+ * gwy_shape_fit_preset_get_secondary_power_xy:
+ * @preset: A 3D geometrical shape fitting function.
+ * @i: Secondary quantity number.
+ *
+ * Gets the power of abscissa units in a shape fitting secondary (derived)
+ * quantity.
+ *
+ * Usually it is easier to let gwy_shape_fit_preset_get_secondary_units()
+ * derive the quantity units.
+ *
+ * Returns: The power of abscissa in the secondary quantity.
+ *
+ * Since: 2.50
+ **/
+gint
+gwy_shape_fit_preset_get_secondary_power_xy(GwyShapeFitPreset *preset,
+                                            guint i)
+{
+    const FitShapeFunc *builtin;
+
+    g_return_val_if_fail(GWY_IS_SHAPE_FIT_PRESET(preset), 0);
+    builtin = preset->priv->builtin;
+    g_return_val_if_fail(i < builtin->nsecondary, 0);
+    return builtin->secondary[i].power_xy;
+}
+
+/**
+ * gwy_shape_fit_preset_get_secondary_power_z:
+ * @preset: A 3D geometrical shape fitting function.
+ * @i: Secondary quantity number.
+ *
+ * Gets the power of ordinate units in a shape fitting secondary (derived)
+ * quantity.
+ *
+ * Usually it is easier to let gwy_shape_fit_preset_get_secondary_units()
+ * derive the quantity units.
+ *
+ * Returns: The power of ordinate in the secondary quantity.
+ *
+ * Since: 2.50
+ **/
+gint
+gwy_shape_fit_preset_get_secondary_power_z(GwyShapeFitPreset *preset,
+                                           guint i)
+{
+    const FitShapeFunc *builtin;
+
+    g_return_val_if_fail(GWY_IS_SHAPE_FIT_PRESET(preset), 0);
+    builtin = preset->priv->builtin;
+    g_return_val_if_fail(i < builtin->nsecondary, 0);
+    return builtin->secondary[i].power_z;
+}
+
 /**
  * gwy_shape_fit_preset_setup:
  * @preset: A 3D geometrical shape fitting function.
@@ -2737,7 +2843,7 @@ pring_estimate_projection(const GwyXYZ *xyz, guint n,
                           GwyDataLine *proj, GwyXY *projdata, guint *counts,
                           GwyPeaks *peaks, gdouble *param)
 {
-    guint i, j, res;
+    guint i, res;
     const gdouble *d;
     gdouble c, real, off, width[2];
 
@@ -2755,14 +2861,11 @@ pring_estimate_projection(const GwyXYZ *xyz, guint n,
     real = gwy_data_line_get_real(proj);
     off = gwy_data_line_get_offset(proj);
     d = gwy_data_line_get_data(proj);
-    for (i = j = 0; i < res; i++) {
-        if (counts[i] > 5) {
-            projdata[j].x = c + off + (i + 0.5)*real/res;
-            projdata[j].y = d[i];
-            j++;
-        }
+    for (i = 0; i < res; i++) {
+        projdata[i].x = c + off + (i + 0.5)*real/res;
+        projdata[i].y = d[i];
     }
-    if (gwy_peaks_analyze_xy(peaks, projdata, j, 2) != 2)
+    if (gwy_peaks_analyze_xy(peaks, projdata, res, 2) != 2)
         return FALSE;
 
     gwy_peaks_get_quantity(peaks, GWY_PEAK_ABSCISSA, param);
