@@ -1,5 +1,5 @@
 /*
- *  $Id: hough.c 20788 2018-02-02 09:41:17Z yeti-dn $
+ *  $Id: hough.c 21411 2018-09-07 14:29:17Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -157,6 +157,7 @@ gwy_data_field_hough_line_strenghten(GwyDataField *dfield,
     gdouble hmax, hmin, threshval, zdata[20];
     gint i;
     gdouble xdata[20], ydata[20];
+    gdouble dx;
 
     result = gwy_data_field_new(3*hypot(gwy_data_field_get_xres(dfield),
                                         gwy_data_field_get_yres(dfield)),
@@ -182,17 +183,15 @@ gwy_data_field_hough_line_strenghten(GwyDataField *dfield,
     threshval = hmin + (hmax - hmin)*threshold; /*FIXME do GUI for this parameter*/
     gwy_data_field_get_local_maxima_list(water, xdata, ydata, zdata, 20, 10,
                                          threshval, TRUE);
-
+    dx = gwy_data_field_get_dx(result);
     for (i = 0; i < 20; i++) {
        if (zdata[i] > threshval) {
            gwy_debug("point: %g %g (of %d %d), xreal: %g  yreal: %g\n",
                      xdata[i], ydata[i],
                      result->xres, result->yres, result->xreal, result->yreal);
-           bresenhams_line_polar(dfield,
-                                 ((gdouble)xdata[i])*gwy_data_field_get_xreal(result)/((gdouble)gwy_data_field_get_xres(result))
-                                                  - gwy_data_field_get_xreal(result)/2.0,
-                                      ((gdouble)ydata[i])*G_PI/((gdouble)gwy_data_field_get_yres(result)) + G_PI/4,
-                                      1);
+           bresenhams_line_polar(dfield, xdata[i]*dx - result->xreal/2.0,
+                                 ydata[i]*G_PI/result->yres + G_PI/4,
+                                 1);
         }
     }
     g_object_unref(water);
