@@ -1,5 +1,5 @@
 /*
- *  $Id: inttrans.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: inttrans.c 21458 2018-09-23 21:43:40Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -277,6 +277,9 @@ gwy_data_line_part_fft(GwyDataLine *rsrc, GwyDataLine *isrc,
  *
  * No leveling, windowing nor scaling is performed.
  *
+ * The normalisation of FFT is symmetrical, so transformations in both
+ * directions are unitary.
+ *
  * Since 2.8 the dimensions need not to be from the set of sizes returned
  * by gwy_fft_find_nice_size().
  *
@@ -318,8 +321,8 @@ gwy_data_line_fft_do(GwyDataLine *rsrc,
                      GwyTransformDirection direction)
 {
     fftw_iodim dims[1], howmany_dims[1];
-    gdouble *rbuf = g_memdup(rsrc->data, rsrc->res*sizeof(gdouble));
-    gdouble *ibuf = g_memdup(isrc->data, isrc->res*sizeof(gdouble));
+    gdouble *rbuf = gwy_fftw_new_real(rsrc->res);
+    gdouble *ibuf = gwy_fftw_new_real(isrc->res);
     fftw_plan plan;
 
     dims[0].n = rsrc->res;
@@ -329,26 +332,25 @@ gwy_data_line_fft_do(GwyDataLine *rsrc,
     howmany_dims[0].is = rsrc->res;
     howmany_dims[0].os = rsrc->res;
     /* Backward direction is equivalent to switching real and imaginary parts */
-    /* XXX: Planner destroys input. */
     if (direction == GWY_TRANSFORM_DIRECTION_BACKWARD) {
         plan = fftw_plan_guru_split_dft(1, dims, 1, howmany_dims,
-                                        rsrc->data, isrc->data,
+                                        rbuf, ibuf,
                                         rdest->data, idest->data,
-                                        _GWY_FFTW_PATIENCE);
+                                        FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
     }
     else {
         plan = fftw_plan_guru_split_dft(1, dims, 1, howmany_dims,
-                                        isrc->data, rsrc->data,
+                                        ibuf, rbuf,
                                         idest->data, rdest->data,
-                                        _GWY_FFTW_PATIENCE);
+                                        FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
     }
     g_return_if_fail(plan);
-    gwy_assign(rsrc->data, rbuf, rsrc->res);
-    gwy_assign(isrc->data, ibuf, isrc->res);
-    g_free(ibuf);
-    g_free(rbuf);
+    gwy_assign(rbuf, rsrc->data, rsrc->res);
+    gwy_assign(ibuf, isrc->data, isrc->res);
     fftw_execute(plan);
     fftw_destroy_plan(plan);
+    fftw_free(ibuf);
+    fftw_free(rbuf);
 
     gwy_data_line_multiply(rdest, 1.0/sqrt(rsrc->res));
     gwy_data_line_multiply(idest, 1.0/sqrt(rsrc->res));
@@ -375,7 +377,7 @@ gwy_data_line_fft_real_do(GwyDataLine *rsrc,
     plan = fftw_plan_guru_split_dft_r2c(1, dims, 1, howmany_dims,
                                         ibuf->data,
                                         rdest->data, idest->data,
-                                        _GWY_FFTW_PATIENCE);
+                                        FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
     g_return_if_fail(plan);
     /* R2C destroys input, and especially, the planner destroys input too */
     gwy_data_line_copy(rsrc, ibuf);
@@ -467,16 +469,14 @@ gwy_data_field_area_2dfft(GwyDataField *rin, GwyDataField *iin,
         return;
     }
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(rin));
+    if (!_gwy_data_field_check_area(rin, col, row, width, height))
+        return;
     g_return_if_fail(GWY_IS_DATA_FIELD(rout));
     g_return_if_fail(GWY_IS_DATA_FIELD(iin));
     g_return_if_fail(GWY_IS_DATA_FIELD(iout));
     g_return_if_fail(rin->xres == iin->xres && rin->yres == rout->yres);
     g_return_if_fail(level >= 0 && level <= 2);
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width >= 2 && height >= 2
-                     && col + width <= rin->xres
-                     && row + height <= rin->yres);
+    g_return_if_fail(width >= 2 && height >= 2);
 
     gwy_data_field_resample(rout, width, height, GWY_INTERPOLATION_NONE);
     out_rdata = rout->data;
@@ -527,6 +527,9 @@ gwy_data_field_area_2dfft(GwyDataField *rin, GwyDataField *iin,
  *
  * No leveling, windowing nor scaling is performed.
  *
+ * The normalisation of FFT is symmetrical, so transformations in both
+ * directions are unitary.
+ *
  * Since 2.8 the dimensions need not to be from the set of sizes returned
  * by gwy_fft_find_nice_size().
  *
@@ -570,6 +573,9 @@ gwy_data_field_2dfft_do(GwyDataField *rin,
                         GwyDataField *iout,
                         GwyTransformDirection direction)
 {
+    guint n = rin->xres * rin->yres;
+    gdouble *rbuf = gwy_fftw_new_real(n);
+    gdouble *ibuf = gwy_fftw_new_real(n);
     fftw_iodim dims[2], howmany_dims[1];
     fftw_plan plan;
 
@@ -580,25 +586,31 @@ gwy_data_field_2dfft_do(GwyDataField *rin,
     dims[0].is = dims[1].is * dims[1].n;
     dims[0].os = dims[1].os * dims[1].n;
     howmany_dims[0].n = 1;
-    howmany_dims[0].is = rin->xres*rin->yres;
-    howmany_dims[0].os = rin->xres*rin->yres;
+    howmany_dims[0].is = n;
+    howmany_dims[0].os = n;
     /* Backward direction is equivalent to switching real and imaginary parts */
-    if (direction == GWY_TRANSFORM_DIRECTION_BACKWARD)
+    if (direction == GWY_TRANSFORM_DIRECTION_BACKWARD) {
         plan = fftw_plan_guru_split_dft(2, dims, 1, howmany_dims,
-                                        rin->data, iin->data,
+                                        rbuf, ibuf,
                                         rout->data, iout->data,
-                                        _GWY_FFTW_PATIENCE);
-    else
+                                        FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
+    }
+    else {
         plan = fftw_plan_guru_split_dft(2, dims, 1, howmany_dims,
-                                        iin->data, rin->data,
+                                        ibuf, rbuf,
                                         iout->data, rout->data,
-                                        _GWY_FFTW_PATIENCE);
+                                        FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
+    }
     g_return_if_fail(plan);
+    gwy_assign(rbuf, rin->data, n);
+    gwy_assign(ibuf, iin->data, n);
     fftw_execute(plan);
     fftw_destroy_plan(plan);
+    fftw_free(rbuf);
+    fftw_free(ibuf);
 
-    gwy_data_field_multiply(rout, 1.0/sqrt(rin->xres*rin->yres));
-    gwy_data_field_multiply(iout, 1.0/sqrt(rin->xres*rin->yres));
+    gwy_data_field_multiply(rout, 1.0/sqrt(n));
+    gwy_data_field_multiply(iout, 1.0/sqrt(n));
     gwy_data_field_invalidate(rout);
     gwy_data_field_invalidate(iout);
 }
@@ -637,13 +649,11 @@ gwy_data_field_area_2dfft_real(GwyDataField *rin,
     gdouble *out_rdata, *out_idata;
     gdouble rmsa = 0.0, rmsb;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(rin));
+    if (!_gwy_data_field_check_area(rin, col, row, width, height))
+        return;
     g_return_if_fail(GWY_IS_DATA_FIELD(rout));
     g_return_if_fail(GWY_IS_DATA_FIELD(iout));
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width >= 2 && height >= 2
-                     && col + width <= rin->xres
-                     && row + height <= rin->yres);
+    g_return_if_fail(width >= 2 && height >= 2);
 
     gwy_data_field_resample(rout, width, height, GWY_INTERPOLATION_NONE);
     gwy_data_field_resample(iout, width, height, GWY_INTERPOLATION_NONE);
@@ -698,7 +708,7 @@ gwy_data_field_2dfft_real_do(GwyDataField *rin,
     howmany_dims[0].os = xres*yres;
     plan = fftw_plan_guru_split_dft_r2c(2, dims, 1, howmany_dims,
                                         ibuf->data, rout->data, iout->data,
-                                        _GWY_FFTW_PATIENCE);
+                                        FFTW_ESTIMATE | FFTW_DESTROY_INPUT);
     g_return_if_fail(plan);
     /* R2C destroys input, and especially, the planner destroys input too */
     gwy_data_field_copy(rin, ibuf, FALSE);
@@ -907,6 +917,10 @@ gwy_data_field_2dfft_dehumanize(GwyDataField *data_field)
  * direct-space data.  The lateral units and resolutions are updated to
  * correspond to its Fourier transform.
  *
+ * The real dimensions are set for spatial frequencies, not wavevectors.
+ * For wavevector lateral coordinates, mutiply all real dimensions and
+ * offsets by 2*%G_PI.
+ *
  * If @humanize is %TRUE gwy_data_field_2dfft_humanize() is applied to the
  * field data and the lateral offsets are set accordingly.  Otherwise the
  * offsets are cleared.
@@ -926,8 +940,8 @@ gwy_data_field_fft_postprocess(GwyDataField *dfield,
     xyunit = gwy_data_field_get_si_unit_xy(dfield);
     gwy_si_unit_power(xyunit, -1, xyunit);
 
-    gwy_data_field_set_xreal(dfield, 1.0/gwy_data_field_get_xmeasure(dfield));
-    gwy_data_field_set_yreal(dfield, 1.0/gwy_data_field_get_ymeasure(dfield));
+    gwy_data_field_set_xreal(dfield, 1.0/gwy_data_field_get_dx(dfield));
+    gwy_data_field_set_yreal(dfield, 1.0/gwy_data_field_get_dy(dfield));
 
     if (!humanize) {
         gwy_data_field_invalidate(dfield);
@@ -1110,6 +1124,9 @@ gwy_data_field_1dfft(GwyDataField *rin, GwyDataField *iin,
  *
  * No leveling, windowing nor scaling is performed.
  *
+ * The normalisation of FFT is symmetrical, so transformations in both
+ * directions are unitary.
+ *
  * Since 2.8 the dimensions need not to be from the set of sizes returned
  * by gwy_fft_find_nice_size().
  *
@@ -1197,16 +1214,14 @@ gwy_data_field_area_xfft(GwyDataField *rin, GwyDataField *iin,
     gint k;
     GwyDataField *rbuf, *ibuf;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(rin));
+    if (!_gwy_data_field_check_area(rin, col, row, width, height))
+        return;
     g_return_if_fail(GWY_IS_DATA_FIELD(rout));
     g_return_if_fail(GWY_IS_DATA_FIELD(iin));
     g_return_if_fail(GWY_IS_DATA_FIELD(iout));
     g_return_if_fail(rin->xres == iin->xres && rin->yres == rout->yres);
     g_return_if_fail(level >= 0 && level <= 2);
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width >= 2 && height >= 1
-                     && col + width <= rin->xres
-                     && row + height <= rin->yres);
+    g_return_if_fail(width >= 2);
 
     gwy_data_field_resample(rout, width, height, GWY_INTERPOLATION_NONE);
     gwy_data_field_resample(iout, width, height, GWY_INTERPOLATION_NONE);
@@ -1261,16 +1276,18 @@ gwy_data_field_xfft_do(GwyDataField *rin,
     /* XXX: Planner destroys input, we have to either allocate memory or
      * use in-place transform.  In some cases caller could provide us with
      * already allocated buffers. */
-    if (direction == GWY_TRANSFORM_DIRECTION_BACKWARD)
+    if (direction == GWY_TRANSFORM_DIRECTION_BACKWARD) {
         plan = fftw_plan_guru_split_dft(1, dims, 1, howmany_dims,
                                         rout->data, iout->data,
                                         rout->data, iout->data,
-                                        _GWY_FFTW_PATIENCE);
-    else
+                                        FFTW_ESTIMATE);
+    }
+    else {
         plan = fftw_plan_guru_split_dft(1, dims, 1, howmany_dims,
                                         iout->data, rout->data,
                                         iout->data, rout->data,
-                                        _GWY_FFTW_PATIENCE);
+                                        FFTW_ESTIMATE);
+    }
     g_return_if_fail(plan);
     gwy_data_field_copy(rin, rout, FALSE);
     gwy_data_field_copy(iin, iout, FALSE);
@@ -1316,14 +1333,12 @@ gwy_data_field_area_xfft_real(GwyDataField *rin, GwyDataField *rout,
     gint k;
     GwyDataField *rbuf, *ibuf;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(rin));
+    if (!_gwy_data_field_check_area(rin, col, row, width, height))
+        return;
     g_return_if_fail(GWY_IS_DATA_FIELD(rout));
     g_return_if_fail(GWY_IS_DATA_FIELD(iout));
     g_return_if_fail(level >= 0 && level <= 2);
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width >= 2 && height >= 1
-                     && col + width <= rin->xres
-                     && row + height <= rin->yres);
+    g_return_if_fail(width >= 2);
 
     gwy_data_field_resample(rout, width, height, GWY_INTERPOLATION_NONE);
     gwy_data_field_resample(iout, width, height, GWY_INTERPOLATION_NONE);
@@ -1371,7 +1386,7 @@ gwy_data_field_xfft_real_do(GwyDataField *rin,
     howmany_dims[0].os = rin->xres;
     plan = fftw_plan_guru_split_dft_r2c(1, dims, 1, howmany_dims,
                                         ibuf->data, rout->data, iout->data,
-                                        _GWY_FFTW_PATIENCE);
+                                        FFTW_ESTIMATE);
     g_return_if_fail(plan);
     /* R2C destroys input, and especially, the planner destroys input too */
     gwy_data_field_copy(rin, ibuf, FALSE);
@@ -1435,16 +1450,14 @@ gwy_data_field_area_yfft(GwyDataField *rin, GwyDataField *iin,
     gint k;
     GwyDataField *rbuf, *ibuf;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(rin));
+    if (!_gwy_data_field_check_area(rin, col, row, width, height))
+        return;
     g_return_if_fail(GWY_IS_DATA_FIELD(rout));
     g_return_if_fail(GWY_IS_DATA_FIELD(iin));
     g_return_if_fail(GWY_IS_DATA_FIELD(iout));
     g_return_if_fail(rin->xres == iin->xres && rin->yres == rout->yres);
     g_return_if_fail(level >= 0 && level <= 2);
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width >= 1 && height >= 2
-                     && col + width <= rin->xres
-                     && row + height <= rin->yres);
+    g_return_if_fail(height >= 2);
 
     gwy_data_field_resample(rout, width, height, GWY_INTERPOLATION_NONE);
     gwy_data_field_resample(iout, width, height, GWY_INTERPOLATION_NONE);
@@ -1499,16 +1512,18 @@ gwy_data_field_yfft_do(GwyDataField *rin,
     /* XXX: Planner destroys input, we have to either allocate memory or
      * use in-place transform.  In some cases caller could provide us with
      * already allocated buffers. */
-    if (direction == GWY_TRANSFORM_DIRECTION_BACKWARD)
+    if (direction == GWY_TRANSFORM_DIRECTION_BACKWARD) {
         plan = fftw_plan_guru_split_dft(1, dims, 1, howmany_dims,
                                         rout->data, iout->data,
                                         rout->data, iout->data,
-                                        _GWY_FFTW_PATIENCE);
-    else
+                                        FFTW_ESTIMATE);
+    }
+    else {
         plan = fftw_plan_guru_split_dft(1, dims, 1, howmany_dims,
                                         iout->data, rout->data,
                                         iout->data, rout->data,
-                                        _GWY_FFTW_PATIENCE);
+                                        FFTW_ESTIMATE);
+    }
     g_return_if_fail(plan);
     gwy_data_field_copy(rin, rout, FALSE);
     gwy_data_field_copy(iin, iout, FALSE);
@@ -1554,14 +1569,12 @@ gwy_data_field_area_yfft_real(GwyDataField *rin, GwyDataField *rout,
     gint k;
     GwyDataField *rbuf, *ibuf;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(rin));
+    if (!_gwy_data_field_check_area(rin, col, row, width, height))
+        return;
     g_return_if_fail(GWY_IS_DATA_FIELD(rout));
     g_return_if_fail(GWY_IS_DATA_FIELD(iout));
     g_return_if_fail(level >= 0 && level <= 2);
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width >= 1 && height >= 2
-                     && col + width <= rin->xres
-                     && row + height <= rin->yres);
+    g_return_if_fail(height >= 2);
 
     gwy_data_field_resample(rout, width, height, GWY_INTERPOLATION_NONE);
     gwy_data_field_resample(iout, width, height, GWY_INTERPOLATION_NONE);
@@ -1609,7 +1622,7 @@ gwy_data_field_yfft_real_do(GwyDataField *rin,
     howmany_dims[0].os = 1;
     plan = fftw_plan_guru_split_dft_r2c(1, dims, 1, howmany_dims,
                                         ibuf->data, rout->data, iout->data,
-                                        _GWY_FFTW_PATIENCE);
+                                        FFTW_ESTIMATE);
     g_return_if_fail(plan);
     /* R2C destroys input, and especially, the planner destroys input too */
     gwy_data_field_copy(rin, ibuf, FALSE);
