@@ -1,5 +1,5 @@
 /*
- *  $Id: level.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: level.c 21458 2018-09-23 21:43:40Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -24,6 +24,7 @@
 #include <libgwyddion/gwymath.h>
 #include <libprocess/datafield.h>
 #include <libprocess/level.h>
+#include "gwyprocessinternal.h"
 
 /**
  * gwy_data_field_fit_plane:
@@ -114,27 +115,17 @@ gwy_data_field_area_fit_plane(GwyDataField *data_field,
     gdouble alpha1, alpha2, alpha3, beta1, beta2, gamma1;
     gdouble det;
     gdouble a, b, c;
-    gint xres, yres;
     gdouble x, y, z;
     gdouble n;
     gdouble *datapos = NULL, *maskpos = NULL;
     gdouble *drow = NULL, *mrow = NULL;
-    gint i, j;
+    gint i, j, xres;
     gboolean skip;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height)
+        || !_gwy_data_field_check_mask(data_field, &mask, NULL))
+        return;
     xres = data_field->xres;
-    yres = data_field->yres;
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width >= 0 && height >= 0
-                     && col + width <= xres
-                     && row + height <= yres);
-    if (mask) {
-        g_return_if_fail(GWY_IS_DATA_FIELD(mask));
-        g_return_if_fail(mask->xres == xres
-                         && mask->yres == yres);
-    }
-
     n = 0;
 
     /* try to return something reasonable even in degenerate cases */
@@ -229,8 +220,8 @@ gwy_data_field_area_fit_plane(GwyDataField *data_field,
  * criterion may be sufficiently small values of the x and y plane
  * coefficients.  Note that since gwy_data_field_plane_level() uses pixel-based
  * lateral coordinates, the coefficients must be divided by
- * gwy_data_field_get_xmeasure(data_field) and
- * gwy_data_field_get_ymeasure(data_field) to obtain physical plane
+ * gwy_data_field_get_dx(data_field) and
+ * gwy_data_field_get_dy(data_field) to obtain physical plane
  * coefficients.
  *
  * Returns: %TRUE if any plane was actually fitted; %FALSE if there was an
@@ -534,11 +525,8 @@ gwy_data_field_fit_lines(GwyDataField *data_field,
     gdouble real, coefs[4];
     GwyDataLine *hlp, *xdata = NULL, *ydata = NULL;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width >= 0 && height >= 0
-                     && col + width <= data_field->xres
-                     && row + height <= data_field->yres);
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
 
     xres = data_field->xres;
     yres = data_field->yres;
@@ -644,18 +632,13 @@ gwy_data_field_area_fit_polynom(GwyDataField *data_field,
                                 gint col_degree, gint row_degree,
                                 gdouble *coeffs)
 {
-    gint r, c, i, j, size, xres, yres;
+    gint r, c, i, j, size, xres;
     gdouble *data, *sums, *m;
 
-    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), NULL);
-    xres = data_field->xres;
-    yres = data_field->yres;
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return NULL;
     g_return_val_if_fail(row_degree >= 0 && col_degree >= 0, NULL);
-    g_return_val_if_fail(col >= 0 && row >= 0
-                         && width > col_degree && height > row_degree
-                         && col + width <= xres
-                         && row + height <= yres,
-                         NULL);
+    xres = data_field->xres;
 
     data = data_field->data;
     size = (row_degree+1)*(col_degree+1);
@@ -764,19 +747,14 @@ gwy_data_field_area_subtract_polynom(GwyDataField *data_field,
                                      gint col_degree, gint row_degree,
                                      const gdouble *coeffs)
 {
-    gint r, c, i, j, xres, yres;
+    gint r, c, i, j, xres;
     gdouble *data;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
-    xres = data_field->xres;
-    yres = data_field->yres;
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
     g_return_if_fail(coeffs);
     g_return_if_fail(row_degree >= 0 && col_degree >= 0);
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width > 0 && height > 0
-                     && col + width <= xres
-                     && row + height <= yres);
-
+    xres = data_field->xres;
     data = data_field->data;
 
     for (r = row; r < row + height; r++) {
@@ -882,23 +860,18 @@ gwy_data_field_area_fit_legendre(GwyDataField *data_field,
                                  gint col_degree, gint row_degree,
                                  gdouble *coeffs)
 {
-    gint r, c, i, j, size, maxsize, xres, yres, col_n, row_n;
+    gint r, c, i, j, size, maxsize, xres, col_n, row_n;
     gint isize, jsize, thissize;
     gdouble *data, *m, *pmx, *pmy, *sumsx, *sumsy, *rhs;
 
-    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), NULL);
-    xres = data_field->xres;
-    yres = data_field->yres;
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return NULL;
     g_return_val_if_fail(row_degree >= 0 && col_degree >= 0, NULL);
-    g_return_val_if_fail(col >= 0 && row >= 0
-                         && width > col_degree && height > row_degree
-                         && col + width <= xres
-                         && row + height <= yres,
-                         NULL);
-
+    xres = data_field->xres;
     data = data_field->data;
     col_n = col_degree + 1;
     row_n = row_degree + 1;
+
     size = col_n*row_n;
     /* The maximum necessary matrix size (order), it is approximately four
      * times smaller thanks to separation of even and odd polynomials */
@@ -1172,19 +1145,14 @@ gwy_data_field_area_subtract_legendre(GwyDataField *data_field,
                                       gint col_degree, gint row_degree,
                                       const gdouble *coeffs)
 {
-    gint r, c, i, j, xres, yres, col_n, row_n;
+    gint r, c, i, j, xres, col_n, row_n;
     gdouble *data, *pmx, *pmy;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
-    xres = data_field->xres;
-    yres = data_field->yres;
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
     g_return_if_fail(coeffs);
     g_return_if_fail(row_degree >= 0 && col_degree >= 0);
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width > 0 && height > 0
-                     && col + width <= xres
-                     && row + height <= yres);
-
+    xres = data_field->xres;
     data = data_field->data;
     col_n = col_degree + 1;
     row_n = row_degree + 1;
@@ -1264,19 +1232,14 @@ gwy_data_field_area_fit_poly_max(GwyDataField *data_field,
                                  gint max_degree,
                                  gdouble *coeffs)
 {
-    gint r, c, i, j, size, xres, yres, degree_n;
+    gint r, c, i, j, size, xres, degree_n;
     gint ix, jx, iy, jy;
     gdouble *data, *m, *pmx, *pmy, *sumsx, *sumsy;
 
-    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), NULL);
-    xres = data_field->xres;
-    yres = data_field->yres;
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return NULL;
     g_return_val_if_fail(max_degree >= 0, NULL);
-    g_return_val_if_fail(col >= 0 && row >= 0
-                         && col + width <= xres
-                         && row + height <= yres,
-                         NULL);
-
+    xres = data_field->xres;
     data = data_field->data;
     degree_n = max_degree + 1;
     size = degree_n*(degree_n + 1)/2;
@@ -1416,19 +1379,14 @@ gwy_data_field_area_subtract_poly_max(GwyDataField *data_field,
                                       gint max_degree,
                                       const gdouble *coeffs)
 {
-    gint r, c, i, j, xres, yres, degree_n;
+    gint r, c, i, j, xres, degree_n;
     gdouble *data, *pmx, *pmy;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
-    xres = data_field->xres;
-    yres = data_field->yres;
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
     g_return_if_fail(coeffs);
     g_return_if_fail(max_degree >= 0);
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width > 0 && height > 0
-                     && col + width <= xres
-                     && row + height <= yres);
-
+    xres = data_field->xres;
     data = data_field->data;
     degree_n = max_degree + 1;
 
@@ -1537,22 +1495,14 @@ gwy_data_field_area_fit_poly(GwyDataField *data_field,
                              gdouble *coeffs)
 {
     const gdouble *data, *mask;
-    gint xres, yres, r, c, i, j;
+    gint xres, r, c, i, j;
     gdouble *m, *p;
 
-    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), NULL);
-    xres = data_field->xres;
-    yres = data_field->yres;
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height)
+        || !_gwy_data_field_check_mask(data_field, &mask_field, NULL))
+        return NULL;
     g_return_val_if_fail(nterms >= 0, NULL);
-    g_return_val_if_fail(col >= 0 && row >= 0
-                         && col + width <= xres
-                         && row + height <= yres,
-                         NULL);
-    if (mask_field) {
-        g_return_val_if_fail(GWY_IS_DATA_FIELD(mask_field), NULL);
-        g_return_val_if_fail(mask_field->xres == xres
-                             && mask_field->yres == yres, NULL);
-    }
+    xres = data_field->xres;
 
     if (!nterms)
         return coeffs;
@@ -1681,16 +1631,13 @@ gwy_data_field_area_subtract_poly(GwyDataField *data_field,
                                   const gdouble *coeffs)
 {
     gdouble *data;
-    gint xres, yres, r, c, i;
+    gint xres, r, c, i;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
-    xres = data_field->xres;
-    yres = data_field->yres;
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
     g_return_if_fail(nterms >= 0);
     g_return_if_fail(coeffs);
-    g_return_if_fail(col >= 0 && row >= 0
-                     && col + width <= xres
-                     && row + height <= yres);
+    xres = data_field->xres;
 
     if (!nterms)
         return;
@@ -1781,12 +1728,9 @@ gwy_data_field_area_fit_local_planes(GwyDataField *data_field,
     gdouble xreal, yreal, qx, qy, asymshfit;
     gint xres, yres, ri, i, j, ii, jj;
 
-    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), NULL);
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return NULL;
     g_return_val_if_fail(size > 1, NULL);
-    g_return_val_if_fail(col >= 0 && row >= 0
-                     && width > 0 && height > 0
-                     && col + width <= data_field->xres
-                     && row + height <= data_field->yres, NULL);
     if (!nresults)
         return NULL;
     g_return_val_if_fail(types, NULL);
