@@ -1,5 +1,5 @@
 /*
- *  $Id: linestats.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: linestats.c 21132 2018-06-11 15:40:25Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -834,7 +834,7 @@ gwy_data_line_psdf(GwyDataLine *data_line,
     data = target_line->data;
     rdata = rout->data;
     idata = iout->data;
-    q = data_line->real/(res*res*2.0*G_PI);
+    q = data_line->real/(res*2.0*G_PI);
 
     /* Calculate modulus */
     for (i = 0; i < res/2; i++)
