@@ -1,5 +1,5 @@
 /*
- *  $Id: simplefft.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: simplefft.c 21456 2018-09-21 13:01:23Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -186,15 +186,13 @@ gwy_fft_simple(GwyTransformDirection dir,
         plan = fftw_plan_guru_split_dft(1, dims, 1, howmany_dims,
                                         (gdouble*)in_im, (gdouble*)in_re,
                                         out_im, out_re,
-                                        _GWY_FFTW_PATIENCE
-                                        | FFTW_PRESERVE_INPUT);
+                                        FFTW_PRESERVE_INPUT | FFTW_ESTIMATE);
     }
     else {
         plan = fftw_plan_guru_split_dft(1, dims, 1, howmany_dims,
                                         (gdouble*)in_re, (gdouble*)in_im,
                                         out_re, out_im,
-                                        _GWY_FFTW_PATIENCE
-                                        | FFTW_PRESERVE_INPUT);
+                                        FFTW_PRESERVE_INPUT | FFTW_ESTIMATE);
     }
 
     g_return_if_fail(plan);
@@ -300,7 +298,7 @@ bessel_I0(gdouble x)
 static gdouble
 gwy_fft_window_kaiser(gint i, gint n, gdouble alpha)
 {
-    gdouble x = 2.0*i/(n - 1) - 1.0;
+    gdouble x = 2.0*i/n - 1.0;
     x = 1.0 - x*x;
     x = MAX(x, 0.0);
 
