@@ -1,6 +1,6 @@
 /*
- *  $Id: spectra.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2006 Owain Davies, David Necas (Yeti), Petr Klapetek.
+ *  $Id: spectra.c 21426 2018-09-10 15:24:56Z yeti-dn $
+ *  Copyright (C) 2006-2018 Owain Davies, David Necas (Yeti), Petr Klapetek.
  *  E-mail: owain.davies@blueyonder.co.uk
  *          yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -290,7 +290,7 @@ gwy_spectra_new_alike(GwySpectra *model)
 
     g_return_val_if_fail(GWY_IS_SPECTRA(model), NULL);
     spectra = g_object_new(GWY_TYPE_SPECTRA, NULL);
-    _gwy_assign_si_unit(model->si_unit_xy, &spectra->si_unit_xy);
+    _gwy_copy_si_unit(model->si_unit_xy, &spectra->si_unit_xy);
 
     return spectra;
 }
@@ -353,7 +353,7 @@ gwy_spectra_serialize(GObject *obj,
     guint32 *selected;
     guint32 ncurves, ncoords, nselected;
     GByteArray *retval;
-    gpointer pxyunit;
+    gpointer pxyunit, pxlabel, pylabel;
 
     spectra = GWY_SPECTRA(obj);
 
@@ -362,16 +362,18 @@ gwy_spectra_serialize(GObject *obj,
                     &ncoords, &coords,
                     &nselected, &selected);
     pxyunit = unit_pointer_if_nonempty(spectra->si_unit_xy);
+    pxlabel = spectra->spec_xlabel ? &spectra->spec_xlabel : NULL;
+    pylabel = spectra->spec_ylabel ? &spectra->spec_ylabel : NULL;
 
     {
         GwySerializeSpec spec[] = {
-            { 's', "title",       &spectra->title,       NULL,       },
-            { 'o', "si_unit_xy",  pxyunit,               NULL,       },
-            { 'D', "coords",      &coords,               &ncoords,   },
-            { 'I', "selected",    &selected,             &nselected, },
-            { 'O', "data",        &curves,               &ncurves,   },
-            { 's', "spec_xlabel", &spectra->spec_xlabel, NULL,       },
-            { 's', "spec_ylabel", &spectra->spec_ylabel, NULL,       },
+            { 's', "title",       &spectra->title, NULL,       },
+            { 'o', "si_unit_xy",  pxyunit,         NULL,       },
+            { 'D', "coords",      &coords,         &ncoords,   },
+            { 'I', "selected",    &selected,       &nselected, },
+            { 'O', "data",        &curves,         &ncurves,   },
+            { 's', "spec_xlabel", pxlabel,         NULL,       },
+            { 's', "spec_ylabel", pylabel,         NULL,       },
         };
         guint nspec = fix_serialisation_spec(spec);
 
@@ -537,7 +539,7 @@ gwy_spectra_duplicate_real(GObject *object)
     spectra = GWY_SPECTRA(object);
     duplicate = gwy_spectra_new_alike(spectra);
     duplicate->title = g_strdup(spectra->title);
-    _gwy_assign_si_unit(spectra->si_unit_xy, &duplicate->si_unit_xy);
+    _gwy_copy_si_unit(spectra->si_unit_xy, &duplicate->si_unit_xy);
     g_array_append_vals(duplicate->spectra, spectra->spectra->data,
                         spectra->spectra->len);
 
@@ -585,7 +587,7 @@ gwy_spectra_clone_real(GObject *source, GObject *copy)
             = gwy_data_line_duplicate(dataline);
     }
 
-    _gwy_assign_si_unit(spectra->si_unit_xy, &clone->si_unit_xy);
+    _gwy_copy_si_unit(spectra->si_unit_xy, &clone->si_unit_xy);
 }
 
 /**
