@@ -1,5 +1,5 @@
 /*
- *  $Id: stats-acfpsdf.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: stats-acfpsdf.c 21553 2018-10-29 14:23:17Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -24,6 +24,7 @@
 #include <libgwyddion/gwymath.h>
 #include <libprocess/simplefft.h>
 #include <libprocess/datafield.h>
+#include <libprocess/filters.h>
 #include <libprocess/correct.h>
 #include <libprocess/arithmetic.h>
 #include <libprocess/stats.h>
@@ -117,14 +118,11 @@ gwy_data_field_area_func_fft(GwyDataField *data_field,
     gint i, j, xres, yres, res = 0;
     gdouble avg;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
     g_return_if_fail(GWY_IS_DATA_LINE(target_line));
     xres = data_field->xres;
     yres = data_field->yres;
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width >= 1 && height >= 1
-                     && col + width <= xres
-                     && row + height <= yres);
     g_return_if_fail(orientation == GWY_ORIENTATION_HORIZONTAL
                      || orientation == GWY_ORIENTATION_VERTICAL);
 
@@ -146,7 +144,7 @@ gwy_data_field_area_func_fft(GwyDataField *data_field,
     dout = gwy_data_line_new(res, 1.0, FALSE);
     in = gwy_data_line_get_data(din);
     out = gwy_data_line_get_data(dout);
-    plan = fftw_plan_r2r_1d(res, in, out, FFTW_R2HC, _GWY_FFTW_PATIENCE);
+    plan = fftw_plan_r2r_1d(res, in, out, FFTW_R2HC, FFTW_ESTIMATE);
     g_return_if_fail(plan);
 
     switch (orientation) {
@@ -220,7 +218,7 @@ gwy_data_field_area_acf(GwyDataField *data_field,
                                  col, row, width, height,
                                  orientation, interpolation, nstats);
     /* Set proper units */
-    _gwy_assign_si_unit(data_field->si_unit_xy, &target_line->si_unit_x);
+    _gwy_copy_si_unit(data_field->si_unit_xy, &target_line->si_unit_x);
     gwy_si_unit_power(gwy_data_field_get_si_unit_z(data_field), 2,
                       gwy_data_line_get_si_unit_y(target_line));
 }
@@ -285,7 +283,7 @@ gwy_data_field_area_hhcf(GwyDataField *data_field,
                                  orientation, interpolation, nstats);
 
     /* Set proper units */
-    _gwy_assign_si_unit(data_field->si_unit_xy, &target_line->si_unit_x);
+    _gwy_copy_si_unit(data_field->si_unit_xy, &target_line->si_unit_x);
     gwy_si_unit_power(gwy_data_field_get_si_unit_z(data_field), 2,
                       gwy_data_line_get_si_unit_y(target_line));
 }
@@ -352,16 +350,13 @@ gwy_data_field_area_psdf(GwyDataField *data_field,
     gdouble *re, *im, *target;
     gint i, j, xres, yres, size;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
     g_return_if_fail(GWY_IS_DATA_LINE(target_line));
     xres = data_field->xres;
     yres = data_field->yres;
     size = (orientation == GWY_ORIENTATION_HORIZONTAL) ? width : height;
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width >= 1 && height >= 1
-                     && size >= 4
-                     && col + width <= xres
-                     && row + height <= yres);
+    g_return_if_fail(size >= 4);
     g_return_if_fail(orientation == GWY_ORIENTATION_HORIZONTAL
                      || orientation == GWY_ORIENTATION_VERTICAL);
 
@@ -498,14 +493,12 @@ gwy_data_field_area_rpsdf(GwyDataField *data_field,
     gint i, j, k, xres, yres;
     gdouble xreal, yreal, r;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
     g_return_if_fail(GWY_IS_DATA_LINE(target_line));
+    g_return_if_fail(width >= 4 && height >= 4);
     xres = data_field->xres;
     yres = data_field->yres;
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width >= 4 && height >= 4
-                     && col + width <= xres
-                     && row + height <= yres);
     xreal = data_field->xreal;
     yreal = data_field->yreal;
 
@@ -610,17 +603,13 @@ gwy_data_field_area_racf(GwyDataField *data_field,
                          gint nstats)
 {
     GwyDataField *acf_field;
-    gint xres, yres, size;
+    gint size;
     gdouble r;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(data_field));
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
+        return;
     g_return_if_fail(GWY_IS_DATA_LINE(target_line));
-    xres = data_field->xres;
-    yres = data_field->yres;
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width >= 4 && height >= 4
-                     && col + width <= xres
-                     && row + height <= yres);
+    g_return_if_fail(width >= 4 && height >= 4);
 
     size = MIN(width, height)/2;
     if (nstats < 1)
@@ -677,11 +666,11 @@ execute_2d_acf(GwyDataField *extfield,
     for (i = 0; i < ysize; i++) {
         row1 = extdata + i*xsize;
         row2 = extdata + ((ysize - i) % ysize)*xsize + xsize-1;
-        re = (*c)[0]*(*c)[0] + (*c)[1]*(*c)[1];
+        re = gwycreal(*c)*gwycreal(*c) + gwycimag(*c)*gwycimag(*c);
         *(row1++) = re;
         c++;
         for (j = 1; j < cstride; j++) {
-            re = (*c)[0]*(*c)[0] + (*c)[1]*(*c)[1];
+            re = gwycreal(*c)*gwycreal(*c) + gwycimag(*c)*gwycimag(*c);
             *(row1++) = re;
             *(row2--) = re;
             c++;
@@ -704,7 +693,7 @@ extract_2d_acf_real(GwyDataField *field,
         c = cbuf + ((i + ysize - (yrange-1)) % ysize)*cstride;
         row1 = field->data + i*txres;
         for (j = xrange-1; j < txres; j++) {
-            row1[j] = (*c)[0];
+            row1[j] = gwycreal(*c);
             c++;
         }
 
@@ -759,21 +748,18 @@ gwy_data_field_area_2dacf_mask(GwyDataField *field,
 {
     GwyDataField *extfield;
     fftw_plan plan;
-    gint i, j, xres, yres, xsize, ysize, cstride, txres, tyres, qi, thresh;
-    gdouble xreal, yreal;
+    gint i, j, xres, yres, xsize, ysize, cstride, txres, tyres, qi;
+    gdouble xreal, yreal, thresh;
     fftw_complex *cbuf;
     gdouble *trow, *qrow, *mrow, *drow;
 
-    g_return_if_fail(GWY_IS_DATA_FIELD(field));
+    if (!_gwy_data_field_check_area(field, col, row, width, height)
+        || !_gwy_data_field_check_mask(field, &mask, &masking))
+        return;
     g_return_if_fail(GWY_IS_DATA_FIELD(target));
-    g_return_if_fail(!mask || GWY_IS_DATA_FIELD(mask));
     g_return_if_fail(!weights || GWY_IS_DATA_FIELD(weights));
     xres = field->xres;
     yres = field->yres;
-    g_return_if_fail(col >= 0 && row >= 0
-                     && width >= 4 && height >= 4
-                     && col + width <= xres
-                     && row + height <= yres);
     if (xrange <= 0)
         xrange = width/2;
     if (yrange <= 0)
@@ -782,11 +768,6 @@ gwy_data_field_area_2dacf_mask(GwyDataField *field,
     xreal = field->xreal;
     yreal = field->yreal;
 
-    if (masking == GWY_MASK_IGNORE)
-        mask = NULL;
-    if (mask)
-        g_return_if_fail(mask->xres == xres && mask->yres == yres);
-
     xsize = gwy_fft_find_nice_size(width + xrange);
     ysize = gwy_fft_find_nice_size(height + yrange);
     cstride = xsize/2 + 1;
@@ -802,13 +783,14 @@ gwy_data_field_area_2dacf_mask(GwyDataField *field,
     else
         weights = gwy_data_field_new_alike(target, FALSE);
 
-    cbuf = g_new(fftw_complex, cstride*ysize);
+    cbuf = gwy_fftw_new_complex(cstride*ysize);
     extfield = gwy_data_field_new(xsize, ysize, 1.0, 1.0, FALSE);
     plan = fftw_plan_dft_r2c_2d(ysize, xsize, extfield->data, cbuf,
-                                FFTW_DESTROY_INPUT | _GWY_FFTW_PATIENCE);
+                                FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
     g_assert(plan);
 
     if (mask) {
+        /* Calculate unnormalised ACF of the mask, i.e. the denominators. */
         for (i = 0; i < height; i++) {
             mrow = mask->data + (i + row)*xres + col;
             trow = extfield->data + i*xsize;
@@ -825,6 +807,8 @@ gwy_data_field_area_2dacf_mask(GwyDataField *field,
         extract_2d_acf_real(weights, cbuf, cstride, ysize);
         gwy_data_field_multiply(weights, 1.0/(xsize*ysize));
 
+        /* Calculate unnormalised ACF of the premultiplied image, i.e. the
+         * numerators. */
         for (i = 0; i < height; i++) {
             mrow = mask->data + (i + row)*xres + col;
             drow = field->data + (i + row)*xres + col;
@@ -860,20 +844,19 @@ gwy_data_field_area_2dacf_mask(GwyDataField *field,
     gwy_data_field_multiply(target, 1.0/(xsize*ysize));
 
     fftw_destroy_plan(plan);
-    g_free(cbuf);
+    fftw_free(cbuf);
 
     if (mask) {
         gwy_data_field_resample(extfield, txres, tyres, GWY_INTERPOLATION_NONE);
-        thresh = GWY_ROUND(log(width*height));
+        thresh = 1.000001*GWY_ROUND(log(width*height));
         for (i = 0; i < tyres; i++) {
             qrow = weights->data + i*txres;
             trow = target->data + i*txres;
             mrow = extfield->data + i*txres;
             for (j = 0; j < txres; j++) {
-                qi = GWY_ROUND(qrow[j]);
-                if (qi > thresh) {
+                if (qrow[j] > thresh) {
                     mrow[j] = 0.0;
-                    trow[j] /= qi;
+                    trow[j] /= qrow[j];
                 }
                 else {
                     mrow[j] = 1.0;
@@ -893,7 +876,7 @@ gwy_data_field_area_2dacf_mask(GwyDataField *field,
     target->xoff = -0.5*target->xreal;
     target->yoff = -0.5*target->yreal;
 
-    _gwy_assign_si_unit(field->si_unit_xy, &target->si_unit_xy);
+    _gwy_copy_si_unit(field->si_unit_xy, &target->si_unit_xy);
     gwy_si_unit_power(gwy_data_field_get_si_unit_z(field), 2,
                       gwy_data_field_get_si_unit_z(target));
 
@@ -902,8 +885,8 @@ gwy_data_field_area_2dacf_mask(GwyDataField *field,
     weights->xoff = -0.5*weights->xreal;
     weights->yoff = -0.5*weights->yreal;
 
-    _gwy_assign_si_unit(field->si_unit_xy, &weights->si_unit_xy);
-    _gwy_assign_si_unit(NULL, &weights->si_unit_z);
+    _gwy_copy_si_unit(field->si_unit_xy, &weights->si_unit_xy);
+    _gwy_copy_si_unit(NULL, &weights->si_unit_z);
 
     gwy_data_field_invalidate(target);
     gwy_data_field_invalidate(weights);
@@ -914,7 +897,7 @@ gwy_data_field_area_2dacf_mask(GwyDataField *field,
  * gwy_data_field_area_2dacf:
  * @data_field: A data field.
  * @target_field: A data field to store the result to.  It will be resampled
- *                to (2@xrange-1)x(2@yrange-1).
+ *                to (2@xrange-1)×(2@yrange-1).
  * @col: Upper-left column coordinate.
  * @row: Upper-left row coordinate.
  * @width: Area width (number of columns).
@@ -970,6 +953,325 @@ gwy_data_field_2dacf(GwyDataField *data_field,
                               0, 0, data_field->xres, data_field->yres, 0, 0);
 }
 
+/* Assumes @plan acts on buf->data. */
+static void
+execute_2d_cacf(GwyDataField *buf, GwyDataField *target,
+                fftw_plan plan, fftw_complex *cbuf, guint cstride)
+{
+    guint i, j, xres = buf->xres, yres = buf->yres;
+    gdouble *row1, *row2, *data;
+    fftw_complex *c;
+    gdouble re;
+
+    fftw_execute(plan);
+    c = cbuf;
+    data = buf->data;
+    for (i = 0; i < yres; i++) {
+        row1 = data + i*xres;
+        row2 = data + ((yres - i) % yres)*xres + xres-1;
+        re = gwycreal(*c)*gwycreal(*c) + gwycimag(*c)*gwycimag(*c);
+        *(row1++) = re;
+        c++;
+        for (j = 1; j < cstride; j++) {
+            re = gwycreal(*c)*gwycreal(*c) + gwycimag(*c)*gwycimag(*c);
+            *(row1++) = re;
+            *(row2--) = re;
+            c++;
+        }
+    }
+
+    fftw_execute(plan);
+    c = cbuf;
+    data = target->data;
+    for (i = 0; i < yres; i++) {
+        row1 = data + i*xres;
+        row2 = data + ((yres - i) % yres)*xres + xres-1;
+        re = gwycreal(*c);
+        *(row1++) = re;
+        c++;
+        for (j = 1; j < cstride; j++) {
+            re = gwycreal(*c);
+            *(row1++) = re;
+            *(row2--) = re;
+            c++;
+        }
+    }
+}
+
+/* Ensure we produce non-negative output even in the presence of rounding
+ * errors.  Callers might not like negative PSDF much. */
+static void
+extract_2d_fft_real(GwyDataField *target,
+                    fftw_plan plan, fftw_complex *cbuf, guint cstride)
+{
+    guint i, j, xres = target->xres, yres = target->yres;
+    gdouble *row1, *row2, *data;
+    fftw_complex *c;
+    gdouble re;
+
+    fftw_execute(plan);
+    c = cbuf;
+    data = target->data;
+    for (i = 0; i < yres; i++) {
+        row1 = data + i*xres;
+        row2 = data + ((yres - i) % yres)*xres + xres-1;
+        re = gwycreal(*c);
+        *(row1++) = re;
+        c++;
+        for (j = 1; j < cstride; j++) {
+            re = gwycreal(*c);
+            *(row1++) = re;
+            *(row2--) = re;
+            c++;
+        }
+    }
+}
+
+/* Ensure we produce non-negative output even in the presence of rounding
+ * errors.  Callers might not like negative PSDF much. */
+static void
+extract_2d_fft_cnorm(GwyDataField *target,
+                     fftw_plan plan, fftw_complex *cbuf, guint cstride)
+{
+    guint i, j, xres = target->xres, yres = target->yres;
+    gdouble *row1, *row2, *data;
+    fftw_complex *c;
+    gdouble re;
+
+    fftw_execute(plan);
+    c = cbuf;
+    data = target->data;
+    for (i = 0; i < yres; i++) {
+        row1 = data + i*xres;
+        row2 = data + ((yres - i) % yres)*xres + xres-1;
+        re = gwycreal(*c)*gwycreal(*c) + gwycimag(*c)*gwycimag(*c);
+        *(row1++) = re;
+        c++;
+        for (j = 1; j < cstride; j++) {
+            re = gwycreal(*c)*gwycreal(*c) + gwycimag(*c)*gwycimag(*c);
+            *(row1++) = re;
+            *(row2--) = re;
+            c++;
+        }
+    }
+}
+
+/**
+ * gwy_data_field_area_2dpsdf_mask:
+ * @field: A data field.
+ * @mask: Mask specifying which values to take into account/exclude, or %NULL.
+ * @masking: Masking mode to use (has any effect only with non-%NULL @mask).
+ * @windowing: Windowing type to use.
+ * @level: The first polynomial degree to keep in the area; lower degrees
+ *         than @level are subtracted.  Note only values 0 (no levelling) and 1
+ *         (subtract the mean value) are available at present.
+ *         For SPM data, you usually wish to pass 1.
+ * @target_field: A data field to store the result to.  It will be resampled
+ *                to @width×@height.
+ * @col: Upper-left column coordinate.
+ * @row: Upper-left row coordinate.
+ * @width: Area width (number of columns).
+ * @height: Area height (number of rows).
+ *
+ * Calculates two-dimensional power spectrum density function of a data field
+ * area.
+ *
+ * The resulting data field has the spectrum density corresponding zero
+ * frequency (0,0) in the centre.
+ *
+ * The reduction of the total energy by windowing is compensated by multiplying
+ * the PSDF to make its sum of squares equal to the input data sum of squares.
+ *
+ * Do not assume the PSDF values are all positive, when masking is in effect.
+ * The PSDF should still have the correct integral, but it will be contaminated
+ * with noise, both positive and negative.
+ *
+ * Since: 2.51
+ **/
+void
+gwy_data_field_area_2dpsdf_mask(GwyDataField *field,
+                                GwyDataField *target,
+                                GwyDataField *mask,
+                                GwyMaskingType masking,
+                                gint col, gint row,
+                                gint width, gint height,
+                                GwyWindowingType windowing,
+                                gint level)
+{
+    GwyDataField *buf, *weights;
+    fftw_plan plan;
+    gint i, cstride, n;
+    gdouble rms, newrms, r, thresh, avg;
+    fftw_complex *cbuf;
+    gdouble *b, *m, *w, *t;
+
+    if (!_gwy_data_field_check_area(field, col, row, width, height)
+        || !_gwy_data_field_check_mask(field, &mask, &masking))
+        return;
+    g_return_if_fail(GWY_IS_DATA_FIELD(target));
+
+    if (level > 1) {
+        g_warning("Levelling degree %u is not supported, changing to 1.",
+                  level);
+        level = 1;
+    }
+
+    /* We cannot pad to a nice-for-FFT size because we want to calculate
+     * cyclic ACF. */
+    n = width*height;
+    cstride = width/2 + 1;
+    gwy_data_field_resample(target, width, height, GWY_INTERPOLATION_NONE);
+    cbuf = gwy_fftw_new_complex(cstride*height);
+
+    if (mask) {
+        buf = gwy_data_field_new_alike(target, FALSE);
+        b = buf->data;
+        plan = fftw_plan_dft_r2c_2d(height, width, b, cbuf,
+                                    FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
+        g_assert(plan);
+
+        /* Level and window the area. */
+        gwy_data_field_area_copy(field, target, col, row, width, height, 0, 0);
+        weights = gwy_data_field_new_alike(target, FALSE);
+        gwy_data_field_area_copy(mask, weights, col, row, width, height, 0, 0);
+        if (level > 0) {
+            avg = gwy_data_field_area_get_avg_mask(target,
+                                                   weights, masking,
+                                                   0, 0,
+                                                   width, height);
+            gwy_data_field_add(target, -avg);
+        }
+        rms = gwy_data_field_area_get_rms_mask(target, weights, masking,
+                                               0, 0, width, height);
+        gwy_fft_window_data_field(target, GWY_ORIENTATION_HORIZONTAL,
+                                  windowing);
+        gwy_fft_window_data_field(target, GWY_ORIENTATION_VERTICAL,
+                                  windowing);
+
+        newrms = gwy_data_field_area_get_rms_mask(target, weights, masking,
+                                                  0, 0, width, height);
+        if (newrms > 0.0)
+            gwy_data_field_multiply(target, rms/newrms);
+
+        /* Calculate unnormalised CACF of the mask, i.e. the denominators. */
+        m = mask->data;
+        if (masking == GWY_MASK_INCLUDE) {
+            for (i = 0; i < n; i++)
+                b[i] = (m[i] > 0.0);
+        }
+        else {
+            for (i = 0; i < n; i++)
+                b[i] = (m[i] <= 0.0);
+        }
+        gwy_data_field_multiply_fields(target, buf, target);
+
+        execute_2d_cacf(buf, weights, plan, cbuf, cstride);
+
+        /* Calculate unnormalised CACF of the premultiplied image, i.e. the
+         * numerators. */
+        gwy_data_field_copy(target, buf, FALSE);
+        execute_2d_cacf(buf, target, plan, cbuf, cstride);
+
+        /* Divide, with interpolation of missing values. */
+        thresh = 1.000001*GWY_ROUND(log(n));
+        w = weights->data;
+        t = target->data;
+        for (i = 0; i < n; i++) {
+            if (w[i] > thresh) {
+                b[i] = t[i]/w[i];
+                w[i] = 0.0;
+            }
+            else {
+                b[i] = 0.0;
+                w[i] = 1.0;
+            }
+        }
+        gwy_data_field_laplace_solve(buf, weights, -1, 1.0);
+
+        extract_2d_fft_real(target, plan, cbuf, cstride);
+        g_object_unref(weights);
+        g_object_unref(buf);
+    }
+    else {
+        t = target->data;
+        plan = fftw_plan_dft_r2c_2d(height, width, t, cbuf,
+                                    FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
+        g_assert(plan);
+
+        /* Level and window the area. */
+        gwy_data_field_area_copy(field, target, col, row, width, height, 0, 0);
+        if (level > 0) {
+            avg = gwy_data_field_get_avg(target);
+            gwy_data_field_add(target, -avg);
+        }
+        rms = gwy_data_field_get_rms(target);
+
+        gwy_fft_window_data_field(target, GWY_ORIENTATION_HORIZONTAL,
+                                  windowing);
+        gwy_fft_window_data_field(target, GWY_ORIENTATION_VERTICAL,
+                                  windowing);
+
+        newrms = gwy_data_field_get_rms(target);
+        if (newrms > 0.0)
+            gwy_data_field_multiply(target, rms/newrms);
+
+        /* Do the FFT and gather squared Fourier coeffs. */
+        extract_2d_fft_cnorm(target, plan, cbuf, cstride);
+        gwy_data_field_multiply(target, 1.0/n);
+    }
+    gwy_data_field_2dfft_humanize(target);
+
+    fftw_destroy_plan(plan);
+    fftw_free(cbuf);
+
+    gwy_si_unit_power(gwy_data_field_get_si_unit_xy(field), -1,
+                      gwy_data_field_get_si_unit_xy(target));
+    gwy_si_unit_power_multiply(gwy_data_field_get_si_unit_xy(field), 2,
+                               gwy_data_field_get_si_unit_z(field), 2,
+                               gwy_data_field_get_si_unit_z(target));
+
+    gwy_data_field_set_xreal(target, 2.0*G_PI/gwy_data_field_get_dx(field));
+    gwy_data_field_set_yreal(target, 2.0*G_PI/gwy_data_field_get_dy(field));
+
+    r = (width + 1 - width % 2)/2.0;
+    gwy_data_field_set_xoffset(target, -gwy_data_field_jtor(target, r));
+
+    r = (height + 1 - height % 2)/2.0;
+    gwy_data_field_set_yoffset(target, -gwy_data_field_itor(target, r));
+
+    gwy_data_field_multiply(target, 1.0/(target->xreal*target->yreal));
+    gwy_data_field_invalidate(target);
+}
+
+/**
+ * gwy_data_field_2dpsdf:
+ * @field: A data field.
+ * @windowing: Windowing type to use.
+ * @level: The first polynomial degree to keep in the area; lower degrees
+ *         than @level are subtracted.  Note only values 0 (no levelling) and 1
+ *         (subtract the mean value) are available at present.
+ *         For SPM data, you usually wish to pass 1.
+ * @target_field: A data field to store the result to.  It will be resampled
+ *                to the same size as @data_field.
+ *
+ * Calculates two-dimensional power spectrum density function of a data field.
+ *
+ * See gwy_data_field_area_2dpsdf_mask() for details and discussion.
+ *
+ * Since: 2.51
+ **/
+void
+gwy_data_field_2dpsdf(GwyDataField *field,
+                      GwyDataField *target,
+                      GwyWindowingType windowing,
+                      gint level)
+{
+    g_return_if_fail(GWY_IS_DATA_FIELD(field));
+    gwy_data_field_area_2dpsdf_mask(field, target, NULL, GWY_MASK_IGNORE,
+                                    0, 0, field->xres, field->yres,
+                                    windowing, level);
+}
+
 /* Does not really belong here, but is is used only by functions from this
  * source file, so... */
 
@@ -1276,38 +1578,6 @@ row_accum_cnorm(gdouble *accum,
     }
 }
 
-#if 0
-static void
-row2_assign_cnorm(gdouble *out,
-                  gdouble *out2,
-                  const fftw_complex *fftc,
-                  guint size,
-                  gdouble q)
-{
-    gdouble re = gwycreal(*fftc), im = gwycimag(*fftc);
-    gdouble v;
-    guint j;
-
-    q /= size;
-    v = q*(re*re + im*im);
-    *out = v;
-    out++, fftc++;
-    for (j = (size + 1)/2 - 1; j; j--, fftc++, out++, out2--) {
-        re = gwycreal(*fftc);
-        im = gwycimag(*fftc);
-        v = q*(re*re + im*im);
-        *out = v;
-        *out2 = v;
-    }
-    if (size % 2 == 0) {
-        re = gwycreal(*fftc);
-        im = gwycimag(*fftc);
-        v = q*(re*re + im*im);
-        *out = v;
-    }
-}
-#endif
-
 static void
 row_extfft_accum_cnorm(fftw_plan plan,
                        gdouble *fftr,
@@ -1399,20 +1669,6 @@ row_extfft_symmetrise_re(fftw_plan plan,
         *out = gwycreal(*fftc);
 }
 
-/* Calculate the complex absolute value of R2HC output items, excluding the
- * redundant even terms.  So the size of @out must be @size/2 + 1. */
-static void
-row_extract_cabs(const fftw_complex *in,
-                 gdouble *out,
-                 guint width)
-{
-    guint j;
-
-    *out = sqrt(gwycreal(*in)*gwycreal(*in) + gwycimag(*in)*gwycimag(*in));
-    for (j = width; j; j--, out++, in++)
-        *out = sqrt(gwycreal(*in)*gwycreal(*in) + gwycimag(*in)*gwycimag(*in));
-}
-
 static void
 row_accumulate_vk(const gdouble *data,
                   gdouble *v,
@@ -1654,13 +1910,13 @@ gwy_data_field_area_row_acf(GwyDataField *field,
      * fftw_complex, multiply it by 2 for doubles. */
     cstride = size/2 + 1;
     base = field->data + row*field->xres + col;
-    fftr = g_new(gdouble, 3*size);
-    accum_data = fftr + size;
-    accum_mask = fftr + 2*size;
-    fftc = g_new(fftw_complex, cstride);
+    fftr = gwy_fftw_new_real(size);
+    accum_data = g_new(gdouble, 2*size);
+    accum_mask = accum_data + size;
+    fftc = gwy_fftw_new_complex(cstride);
 
     plan = fftw_plan_dft_r2c_1d(size, fftr, fftc,
-                                FFTW_DESTROY_INPUT | _GWY_FFTW_PATIENCE);
+                                FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
     g_assert(plan);
     gwy_clear(accum_data, size);
     gwy_clear(accum_mask, size);
@@ -1703,7 +1959,7 @@ gwy_data_field_area_row_acf(GwyDataField *field,
     row_divide_nonzero_with_laplace(accum_data, accum_mask, line->data,
                                     line->res, GWY_ROUND(log(width*height)));
 
-    line->real = gwy_data_field_get_xmeasure(field)*line->res;
+    line->real = gwy_data_field_get_dx(field)*line->res;
     /* line->off = -0.5*line->real/line->res; */
 
     if (weights) {
@@ -1714,8 +1970,9 @@ gwy_data_field_area_row_acf(GwyDataField *field,
     }
 
     fftw_destroy_plan(plan);
-    g_free(fftc);
-    g_free(fftr);
+    fftw_free(fftc);
+    g_free(accum_data);
+    fftw_free(fftr);
 
     set_cf_units(field, line, weights);
     return line;
@@ -1748,7 +2005,7 @@ normalise_window_square(gdouble *window, guint n)
  * @mask: Mask specifying which values to take into account/exclude, or %NULL.
  * @masking: Masking mode to use (has any effect only with non-%NULL @mask).
  * @windowing: Windowing type to use.
- * @level: The first polynomial degree to keep in the rows, lower degrees than
+ * @level: The first polynomial degree to keep in the rows; lower degrees than
  *         @level are subtracted.  Note only values 0 (no levelling) and 1
  *         (subtract the mean value of each row) are available at present.  For
  *         SPM data, you usually wish to pass 1.
@@ -1768,6 +2025,10 @@ normalise_window_square(gdouble *window, guint n)
  * to the number of contributing pixels.  In other words, the weighting is
  * fair: each contributing pixel has the same influence on the result.
  *
+ * Do not assume the PSDF values are all positive, when masking is in effect.
+ * The PSDF should still have the correct integral, but it will be contaminated
+ * with noise, both positive and negative.
+ *
  * Returns: A new one-dimensional data line with the PSDF.
  **/
 GwyDataLine*
@@ -1811,11 +2072,11 @@ gwy_data_field_area_row_psdf(GwyDataField *field,
     line = gwy_data_line_new(cstride, 1.0, TRUE);
     size = (width + 3)/4*4;
     base = field->data + row*field->xres + col;
-    fftr = g_new(gdouble, 4*size);
-    accum_data = fftr + 1*size;
-    accum_mask = fftr + 2*size;
-    window = fftr + 3*size;
-    fftc = g_new(fftw_complex, cstride);
+    fftr = gwy_fftw_new_real(size);
+    accum_data = g_new(gdouble, 2*size + width);
+    accum_mask = accum_data + size;
+    window = accum_data + 2*size;
+    fftc = gwy_fftw_new_complex(cstride);
 
     gwy_clear(accum_data, size);
     gwy_clear(accum_mask, size);
@@ -1826,7 +2087,7 @@ gwy_data_field_area_row_psdf(GwyDataField *field,
     normalise_window_square(window, width);
 
     plan = fftw_plan_dft_r2c_1d(width, fftr, fftc,
-                                FFTW_DESTROY_INPUT | _GWY_FFTW_PATIENCE);
+                                FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
     g_assert(plan);
     for (i = 0; i < height; i++) {
         guint count = row_level_and_count(base + i*field->xres, fftr, width,
@@ -1869,14 +2130,16 @@ gwy_data_field_area_row_psdf(GwyDataField *field,
     /* The transform is the other way round – for complex numbers.  Since it
      * is in fact a DCT here we don't care and run it as a forward transform. */
     fftw_execute(plan);
-    row_extract_cabs(fftc, line->data, line->res);
+    for (j = 0; j < line->res; j++)
+        line->data[j] = gwycreal(fftc[j]);
 
     fftw_destroy_plan(plan);
-    g_free(fftc);
-    g_free(fftr);
+    fftw_free(fftc);
+    fftw_free(fftr);
+    g_free(accum_data);
 
-    gwy_data_line_multiply(line, gwy_data_field_get_xmeasure(field)/(2*G_PI));
-    line->real = G_PI/gwy_data_field_get_xmeasure(field);
+    gwy_data_line_multiply(line, gwy_data_field_get_dx(field)/(2*G_PI));
+    line->real = G_PI/gwy_data_field_get_dx(field);
     /* line->off = -0.5*line->real/line->res; */
 
     gwy_si_unit_power(gwy_data_field_get_si_unit_xy(field),
@@ -1963,15 +2226,15 @@ gwy_data_field_area_row_hhcf(GwyDataField *field,
      * fftw_complex, multiply it by 2 for doubles. */
     cstride = size/2 + 1;
     base = field->data + row*field->xres + col;
-    fftr = g_new(gdouble, 4*size);
-    accum_data = fftr + size;
-    accum_mask = fftr + 2*size;
-    accum_v = fftr + 3*size;
-    fftc = g_new(fftw_complex, 2*cstride);
-    tmp = fftc + cstride;
+    fftr = gwy_fftw_new_real(size);
+    accum_data = g_new(gdouble, 3*size);
+    accum_mask = accum_data + size;
+    accum_v = accum_data + 2*size;
+    fftc = gwy_fftw_new_complex(cstride);
+    tmp = g_new(fftw_complex, cstride);
 
     plan = fftw_plan_dft_r2c_1d(size, fftr, fftc,
-                                FFTW_DESTROY_INPUT | _GWY_FFTW_PATIENCE);
+                                FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
     g_assert(plan);
     gwy_clear(accum_data, size);
     gwy_clear(accum_mask, size);
@@ -2046,7 +2309,7 @@ gwy_data_field_area_row_hhcf(GwyDataField *field,
     row_divide_nonzero_with_laplace(accum_data, accum_mask, line->data,
                                     line->res, GWY_ROUND(log(width*height)));
 
-    line->real = gwy_data_field_get_xmeasure(field)*line->res;
+    line->real = gwy_data_field_get_dx(field)*line->res;
     /* line->off = -0.5*line->real/line->res; */
 
     if (weights) {
@@ -2057,8 +2320,10 @@ gwy_data_field_area_row_hhcf(GwyDataField *field,
     }
 
     fftw_destroy_plan(plan);
-    g_free(fftc);
-    g_free(fftr);
+    fftw_free(fftc);
+    fftw_free(fftr);
+    g_free(accum_data);
+    g_free(tmp);
 
     set_cf_units(field, line, weights);
     return line;
@@ -2134,7 +2399,7 @@ gwy_data_field_area_row_asg(GwyDataField *field,
         line->data[i] = asg_correction(hhcf->data[i+1]/(t*t));
     }
 
-    _gwy_assign_si_unit(field->si_unit_xy, &line->si_unit_x);
+    _gwy_copy_si_unit(field->si_unit_xy, &line->si_unit_x);
     g_object_unref(hhcf);
 
     return line;
