@@ -1,5 +1,5 @@
 /*
- *  $Id: stats-entropy.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: stats-entropy.c 21458 2018-09-23 21:43:40Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -518,15 +518,10 @@ gwy_data_field_area_get_entropy_at_scales(GwyDataField *data_field,
     gdouble min, max, S = -G_MAXDOUBLE;
     gint i;
 
-    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), S);
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height)
+        || !_gwy_data_field_check_mask(data_field, &mask, &mode))
+        return S;
     g_return_val_if_fail(GWY_IS_DATA_LINE(target_line), S);
-    g_return_val_if_fail(!mask || (GWY_IS_DATA_FIELD(mask)
-                                   && mask->xres == data_field->xres
-                                   && mask->yres == data_field->yres), S);
-    g_return_val_if_fail(col >= 0 && row >= 0
-                         && width >= 0 && height >= 0
-                         && col + width <= data_field->xres
-                         && row + height <= data_field->yres, S);
 
     ecurve = calculate_entropy_at_scales(data_field, mask, mode,
                                          col, row, width, height,
@@ -630,18 +625,12 @@ gwy_data_field_area_get_entropy(GwyDataField *data_field,
     gdouble *ecurve;
     guint maxdiv = 0;
 
-    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), S);
-    g_return_val_if_fail(!mask || (GWY_IS_DATA_FIELD(mask)
-                                   && mask->xres == data_field->xres
-                                   && mask->yres == data_field->yres), S);
-    g_return_val_if_fail(col >= 0 && row >= 0
-                         && width >= 0 && height >= 0
-                         && col + width <= data_field->xres
-                         && row + height <= data_field->yres,
-                         S);
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height)
+        || !_gwy_data_field_check_mask(data_field, &mask, &mode))
+        return S;
 
     /* The result is the same, but it can be cached. */
-    if ((!mask || mode == GWY_MASK_IGNORE)
+    if (!mask
         && row == 0 && col == 0
         && width == data_field->xres && height == data_field->yres)
         return gwy_data_field_get_entropy(data_field);
