@@ -1,6 +1,6 @@
 /*
- *  $Id: stats-sum.c 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
+ *  $Id: stats-sum.c 21458 2018-09-23 21:43:40Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -578,18 +578,12 @@ gwy_data_field_area_get_surface_area_mask(GwyDataField *data_field,
 {
     gdouble area = 0.0;
 
-    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), area);
-    g_return_val_if_fail(!mask || (GWY_IS_DATA_FIELD(mask)
-                                   && mask->xres == data_field->xres
-                                   && mask->yres == data_field->yres), area);
-    g_return_val_if_fail(col >= 0 && row >= 0
-                         && width >= 0 && height >= 0
-                         && col + width <= data_field->xres
-                         && row + height <= data_field->yres,
-                         area);
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height)
+        || !_gwy_data_field_check_mask(data_field, &mask, &mode))
+        return area;
 
     /* The result is the same, but it can be cached. */
-    if ((!mask || mode == GWY_MASK_IGNORE)
+    if (!mask
         && row == 0 && col == 0
         && width == data_field->xres && height == data_field->yres)
         return gwy_data_field_get_surface_area(data_field);
@@ -1085,18 +1079,12 @@ gwy_data_field_area_get_variation(GwyDataField *data_field,
 {
     gdouble var = 0.0;
 
-    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), var);
-    g_return_val_if_fail(!mask || (GWY_IS_DATA_FIELD(mask)
-                                   && mask->xres == data_field->xres
-                                   && mask->yres == data_field->yres), var);
-    g_return_val_if_fail(col >= 0 && row >= 0
-                         && width >= 0 && height >= 0
-                         && col + width <= data_field->xres
-                         && row + height <= data_field->yres,
-                         var);
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height)
+        || !_gwy_data_field_check_mask(data_field, &mask, &mode))
+        return var;
 
     /* The result is the same, but it can be cached. */
-    if ((!mask || mode == GWY_MASK_IGNORE)
+    if (!mask
         && row == 0 && col == 0
         && width == data_field->xres && height == data_field->yres)
         return gwy_data_field_get_variation(data_field);
@@ -1497,20 +1485,188 @@ gwy_data_field_area_get_volume(GwyDataField *data_field,
 {
     gdouble vol = 0.0;
 
-    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), vol);
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height)
+        || !_gwy_data_field_check_mask(data_field, &mask, NULL))
+        return vol;
     g_return_val_if_fail(!basis || (GWY_IS_DATA_FIELD(basis)
                                     && basis->xres == data_field->xres
                                     && basis->yres == data_field->yres), vol);
-    g_return_val_if_fail(!mask || (GWY_IS_DATA_FIELD(mask)
-                                   && mask->xres == data_field->xres
-                                   && mask->yres == data_field->yres), vol);
-    g_return_val_if_fail(col >= 0 && row >= 0
-                         && width >= 0 && height >= 0
-                         && col + width <= data_field->xres
-                         && row + height <= data_field->yres,
-                         vol);
 
     return calculate_volume(data_field, basis, mask, col, row, width, height);
 }
 
+
+/**
+ * gwy_data_field_area_get_dispersion:
+ * @data_field: A data field.
+ * @mask: Mask specifying which values to take into account/exclude, or %NULL.
+ * @masking: Masking mode to use (has any effect only with non-%NULL @mask).
+ * @col: Upper-left column coordinate.
+ * @row: Upper-left row coordinate.
+ * @width: Area width (number of columns).
+ * @height: Area height (number of rows).
+ * @xcenter: Location where to store the horizontal position of centre of mass
+ *           (in pixel coordinates), or %NULL.
+ * @ycenter: Location where to store the vertical position of centre of mass
+ *           (in pixel coordinates), or %NULL.
+ *
+ * Calculates the dispersion of a data field area, taking it as a distribution.
+ *
+ * The function takes @data_field as a distribution, finds the centre of mass
+ * in the area and then calculates the mean squared distance from this centre,
+ * weighted by @data_field values.  Normally @data_field should contain only
+ * non-negative data.
+ *
+ * The dispersion is measured in real coordinates, so horizontal and vertical
+ * pixel sizes play a role and the units are squared lateral units of
+ * @data_field.  Note, however, that @xcenter and @ycenter is returned in pixel
+ * coordinates since it is usually more convenient.
+ *
+ * Returns: Dispersion, i.e. estimated average squared distance from centre of
+ *          mass.
+ *
+ * Since: 2.52
+ **/
+gdouble
+gwy_data_field_area_get_dispersion(GwyDataField *data_field,
+                                   GwyDataField *mask,
+                                   GwyMaskingType masking,
+                                   gint col,
+                                   gint row,
+                                   gint width,
+                                   gint height,
+                                   gdouble *xcenter,
+                                   gdouble *ycenter)
+{
+    gint xres, yres, i, j;
+    gdouble dx, dy;
+    const gdouble *d, *m;
+    gdouble sx, sy, s2, sw, w, x, y;
+
+    if (!_gwy_data_field_check_area(data_field, col, row, width, height)
+        || !_gwy_data_field_check_mask(data_field, &mask, &masking))
+        return 0.0;
+
+    xres = data_field->xres;
+    yres = data_field->yres;
+    dx = data_field->xreal/xres;
+    dy = data_field->yreal/yres;
+
+    sx = sy = sw = 0.0;
+    for (i = 0; i < height; i++) {
+        d = data_field->data + (row + i)*xres + col;
+        if (masking == GWY_MASK_INCLUDE) {
+            m = mask->data + (row + i)*xres + col;
+            for (j = 0; j < width; j++) {
+                if (m[j] > 0.0) {
+                    w = d[j];
+                    sw += w;
+                    sx += j*w;
+                    sy += i*w;
+                }
+            }
+        }
+        else if (masking == GWY_MASK_EXCLUDE) {
+            m = mask->data + (row + i)*xres + col;
+            for (j = 0; j < width; j++) {
+                if (m[j] <= 0.0) {
+                    w = d[j];
+                    sw += w;
+                    sx += j*w;
+                    sy += i*w;
+                }
+            }
+        }
+        else {
+            for (j = 0; j < width; j++) {
+                w = d[j];
+                sw += w;
+                sx += j*w;
+                sy += i*w;
+            }
+        }
+    }
+
+    /* Negative values are silly but do not prevent us from continuing. */
+    if (sw == 0.0) {
+        if (xcenter)
+            *xcenter = col + 0.5*width;
+        if (ycenter)
+            *ycenter = row + 0.5*height;
+        return 0.0;
+    }
+
+    sx /= sw;
+    sy /= sw;
+    if (xcenter)
+        *xcenter = sx;
+    if (ycenter)
+        *ycenter = sy;
+    sx *= dx;
+    sy *= dy;
+
+    s2 = 0.0;
+    for (i = 0; i < height; i++) {
+        d = data_field->data + (row + i)*xres + col;
+        y = i*dy - sy;
+        if (masking == GWY_MASK_INCLUDE) {
+            m = mask->data + (row + i)*xres + col;
+            for (j = 0; j < width; j++) {
+                if (m[j] > 0.0) {
+                    x = j*dx - sx;
+                    s2 += (x*x + y*y)*d[j];
+                }
+            }
+        }
+        else if (masking == GWY_MASK_EXCLUDE) {
+            m = mask->data + (row + i)*xres + col;
+            for (j = 0; j < width; j++) {
+                if (m[j] <= 0.0) {
+                    x = j*dx - sx;
+                    s2 += (x*x + y*y)*d[j];
+                }
+            }
+        }
+        else {
+            for (j = 0; j < width; j++) {
+                x = j*dx - sx;
+                s2 += (x*x + y*y)*d[j];
+            }
+        }
+    }
+
+    return s2/sw;
+}
+
+/**
+ * gwy_data_field_get_dispersion:
+ * @data_field: A data field.
+ * @xcenter: Location where to store the horizontal position of centre of mass
+ *           (in pixel coordinates), or %NULL.
+ * @ycenter: Location where to store the vertical position of centre of mass
+ *           (in pixel coordinates), or %NULL.
+ *
+ * Calculates the dispersion of a data field, taking it as a distribution.
+ *
+ * See gwy_data_field_area_get_dispersion() for discussion.
+ *
+ * Returns: Dispersion, i.e. estimated average squared distance from centre of
+ *          mass.
+ *
+ * Since: 2.52
+ **/
+gdouble
+gwy_data_field_get_dispersion(GwyDataField *data_field,
+                              gdouble *xcenter,
+                              gdouble *ycenter)
+{
+    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), 0.0);
+    return gwy_data_field_area_get_dispersion(data_field,
+                                              NULL, GWY_MASK_IGNORE,
+                                              0, 0,
+                                              data_field->xres,
+                                              data_field->yres,
+                                              xcenter, ycenter);
+}
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
