@@ -1,6 +1,6 @@
 /*
- *  $Id: stats.h 20678 2017-12-18 18:26:55Z yeti-dn $
- *  Copyright (C) 2003-2009 David Necas (Yeti), Petr Klapetek.
+ *  $Id: stats.h 21457 2018-09-23 20:11:09Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -33,6 +33,7 @@ void    gwy_data_field_get_min_max          (GwyDataField *data_field,
                                              gdouble *max);
 gdouble gwy_data_field_get_avg              (GwyDataField *data_field);
 gdouble gwy_data_field_get_rms              (GwyDataField *data_field);
+gdouble gwy_data_field_get_mean_square      (GwyDataField *data_field);
 gdouble gwy_data_field_get_sum              (GwyDataField *data_field);
 gdouble gwy_data_field_get_median           (GwyDataField *data_field);
 gdouble gwy_data_field_get_surface_area     (GwyDataField *data_field);
@@ -153,6 +154,13 @@ gdouble gwy_data_field_area_get_surface_area_mask(GwyDataField *data_field,
                                                   gint row,
                                                   gint width,
                                                   gint height);
+gdouble gwy_data_field_area_get_mean_square (GwyDataField *data_field,
+                                             GwyDataField *mask,
+                                             GwyMaskingType mode,
+                                             gint col,
+                                             gint row,
+                                             gint width,
+                                             gint height);
 gdouble gwy_data_field_area_get_entropy_at_scales(GwyDataField *data_field,
                                                   GwyDataLine *target_line,
                                                   GwyDataField *mask,
@@ -411,6 +419,18 @@ void         gwy_data_field_area_2dacf_mask        (GwyDataField *data_field,
                                                     GwyDataField *weights);
 void         gwy_data_field_2dacf                  (GwyDataField *data_field,
                                                     GwyDataField *target_field);
+void         gwy_data_field_area_2dpsdf_mask       (GwyDataField *field,
+                                                    GwyDataField *target_field,
+                                                    GwyDataField *mask,
+                                                    GwyMaskingType masking,
+                                                    gint col, gint row,
+                                                    gint width, gint height,
+                                                    GwyWindowingType windowing,
+                                                    gint level);
+void         gwy_data_field_2dpsdf                 (GwyDataField *field,
+                                                    GwyDataField *target_field,
+                                                    GwyWindowingType windowing,
+                                                    gint level);
 void         gwy_data_field_area_racf              (GwyDataField *data_field,
                                                     GwyDataLine *target_line,
                                                     gint col,
@@ -451,6 +471,18 @@ void         gwy_data_field_area_minkowski_euler   (GwyDataField *data_field,
 void         gwy_data_field_minkowski_euler        (GwyDataField *data_field,
                                                     GwyDataLine *target_line,
                                                     gint nstats);
+gdouble      gwy_data_field_area_get_dispersion    (GwyDataField *data_field,
+                                                    GwyDataField *mask,
+                                                    GwyMaskingType masking,
+                                                    gint col,
+                                                    gint row,
+                                                    gint width,
+                                                    gint height,
+                                                    gdouble *xcenter,
+                                                    gdouble *ycenter);
+gdouble      gwy_data_field_get_dispersion         (GwyDataField *data_field,
+                                                    gdouble *xcenter,
+                                                    gdouble *ycenter);
 void         gwy_data_field_slope_distribution     (GwyDataField *data_field,
                                                     GwyDataLine *derdist,
                                                     gint kernel_size);
