@@ -1,5 +1,5 @@
 /*
- *  $Id: tip.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: tip.c 21403 2018-09-07 12:12:47Z yeti-dn $
  *  Copyright (C) 2003-2016 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -162,6 +162,23 @@ delta_guess(G_GNUC_UNUSED GwyDataField *data,
     *yres = 21;
 }
 
+static void
+sphere_guess(GwyDataField *data,
+             G_GNUC_UNUSED gdouble height,
+             gdouble radius,
+             G_GNUC_UNUSED gdouble *params,
+             gint *xres,
+             gint *yres)
+{
+    gint xpix = gwy_data_field_rtoi(data, 1.05*radius) + 1;
+
+    guess_symmetrical(data, height, radius, 1e-5, xres, yres);
+    xpix = CLAMP(xpix, 1, 600);
+
+    *xres += 2*xpix;
+    *yres += 2*xpix;
+}
+
 static void
 create_pyramid(GwyDataField *tip, gdouble alpha, gint n, gdouble theta)
 {
@@ -389,6 +406,18 @@ delta(GwyDataField *tip, gdouble height,
     gwy_data_field_invalidate(tip);
 }
 
+/* Model sphere on stick as a very sharp cone. */
+static void
+sphere(GwyDataField *tip,
+       G_GNUC_UNUSED gdouble height,
+       gdouble radius,
+       G_GNUC_UNUSED gdouble rotation,
+       G_GNUC_UNUSED gdouble *params)
+{
+    gdouble p[] = { 0.0, 1e-5 };
+    cone(tip, height, radius, rotation, p);
+}
+
 static const GwyTipParamType pyramid_params[] = {
     GWY_TIP_PARAM_RADIUS,
     GWY_TIP_PARAM_ROTATION,
@@ -425,6 +454,10 @@ static const GwyTipParamType ell_parabola_params[] = {
     GWY_TIP_PARAM_ANISOTROPY,
 };
 
+static const GwyTipParamType sphere_params[] = {
+    GWY_TIP_PARAM_RADIUS,
+};
+
 /* Must match the GwyTipTyp enum! */
 static const GwyTipModelPresetReal tip_presets[] = {
     {
@@ -469,6 +502,12 @@ static const GwyTipModelPresetReal tip_presets[] = {
         &ell_parabola, &ell_parabola_guess,
         G_N_ELEMENTS(ell_parabola_params), ell_parabola_params,
     },
+    {
+        N_("Ball on stick"),
+        N_("Symmetric"),
+        &sphere, &sphere_guess,
+        G_N_ELEMENTS(sphere_params), sphere_params,
+    },
 };
 
 /**
@@ -533,7 +572,7 @@ gwy_tip_model_get_preset_by_name(const gchar *name)
  * Returns: Preset id.
  **/
 gint
-gwy_tip_model_get_preset_id(const GwyTipModelPreset* preset)
+gwy_tip_model_get_preset_id(const GwyTipModelPreset *preset)
 {
     g_return_val_if_fail(preset, 0);
     return (const GwyTipModelPresetReal*)preset - tip_presets;
@@ -548,7 +587,7 @@ gwy_tip_model_get_preset_id(const GwyTipModelPreset* preset)
  * Returns: Preset name.
  **/
 const gchar*
-gwy_tip_model_get_preset_tip_name(const GwyTipModelPreset* preset)
+gwy_tip_model_get_preset_tip_name(const GwyTipModelPreset *preset)
 {
     g_return_val_if_fail(preset, NULL);
     return preset->tip_name;
@@ -563,7 +602,7 @@ gwy_tip_model_get_preset_tip_name(const GwyTipModelPreset* preset)
  * Returns: Preset group name.
  **/
 const gchar*
-gwy_tip_model_get_preset_group_name(const GwyTipModelPreset* preset)
+gwy_tip_model_get_preset_group_name(const GwyTipModelPreset *preset)
 {
     g_return_val_if_fail(preset, NULL);
     return preset->group_name;
@@ -586,7 +625,7 @@ gwy_tip_model_get_preset_group_name(const GwyTipModelPreset* preset)
  * Returns: Number of parameters.
  **/
 gint
-gwy_tip_model_get_preset_nparams(const GwyTipModelPreset* preset)
+gwy_tip_model_get_preset_nparams(const GwyTipModelPreset *preset)
 {
     g_return_val_if_fail(preset, 0);
     return preset->nparams;
@@ -611,7 +650,7 @@ gwy_tip_model_get_preset_nparams(const GwyTipModelPreset* preset)
  * Since: 2.47
  **/
 const GwyTipParamType*
-gwy_tip_model_get_preset_params(const GwyTipModelPreset* preset)
+gwy_tip_model_get_preset_params(const GwyTipModelPreset *preset)
 {
     g_return_val_if_fail(preset, NULL);
     return ((const GwyTipModelPresetReal*)preset)->params;
@@ -666,7 +705,7 @@ params_to_old_params(const GwyTipModelPreset *preset,
  * Since: 2.47
  **/
 void
-gwy_tip_model_preset_create(const GwyTipModelPreset* preset,
+gwy_tip_model_preset_create(const GwyTipModelPreset *preset,
                             GwyDataField *tip,
                             const gdouble *params)
 {
@@ -742,7 +781,7 @@ check_tip_zrange(GwyDataField *tip, gdouble zrange,
  * Since: 2.47
  **/
 void
-gwy_tip_model_preset_create_for_zrange(const GwyTipModelPreset* preset,
+gwy_tip_model_preset_create_for_zrange(const GwyTipModelPreset *preset,
                                        GwyDataField *tip,
                                        gdouble zrange,
                                        gboolean square,
@@ -759,8 +798,8 @@ gwy_tip_model_preset_create_for_zrange(const GwyTipModelPreset* preset,
     g_return_if_fail(preset);
 
     is_delta = (gwy_tip_model_get_preset_id(preset) == GWY_TIP_DELTA);
-    dx = gwy_data_field_get_xmeasure(tip);
-    dy = gwy_data_field_get_ymeasure(tip);
+    dx = gwy_data_field_get_dx(tip);
+    dy = gwy_data_field_get_dy(tip);
 
     /* Create tip according to the guess function.  The guess function only
      * uses the dx and dy from the data field so we pass tip itself to it. */
