@@ -1,5 +1,5 @@
 /*
- *  $Id: alicona.c 19948 2017-06-22 14:05:41Z yeti-dn $
+ *  $Id: alicona.c 20674 2017-12-18 17:58:47Z yeti-dn $
  *  Copyright (C) 2011 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -661,8 +661,7 @@ al3d_load_header(Al3DFile *afile,
     afile->ntags = (guint)atoi(afile->counter->value);
     expected_size = TAG_SIZE*afile->ntags;
     if ((gsize)(size - (p - buffer)) < expected_size + COMMENT_SIZE) {
-        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("File header is truncated."));
+        err_TRUNCATED_HEADER(error);
         return FALSE;
     }
 
