@@ -1,5 +1,5 @@
 /*
- *  $Id: dimensionfile.c 19948 2017-06-22 14:05:41Z yeti-dn $
+ *  $Id: dimensionfile.c 21472 2018-09-27 07:57:11Z yeti-dn $
  *  Copyright (C) 2016 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -40,7 +40,7 @@
 /**
  * [FILE-MAGIC-USERGUIDE]
  * Veeco Dimension 3100D
- * .001 002 etc.
+ * .001 .002 etc.
  * Read[1]
  * [1] The import module is unfinished due to the lack of documentation,
  * testing files and/or people willing to help with the testing.  If you can
@@ -197,8 +197,7 @@ dimfile_load(const gchar *filename,
     container = gwy_container_new();
     for (i = 0; i < nimages; i++) {
         dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-        gwy_serializable_clone(G_OBJECT(xyunit),
-                               G_OBJECT(gwy_data_field_get_si_unit_xy(dfield)));
+        gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), xyunit);
         gwy_convert_raw_data(buffer + HEADER_SIZE + i*imagesize, xres*yres, 1,
                              GWY_RAW_DATA_SINT16, G_LITTLE_ENDIAN,
                              gwy_data_field_get_data(dfield), 1.0, 0.0);
