@@ -13,9 +13,11 @@ static GwyModuleInfo module_info = {
 GWY_MODULE_QUERY(module_info)
 
 GwyModuleInfo* _gwy_module_query__accurexii_txt(void);
+GwyModuleInfo* _gwy_module_query__afmw_spec(void);
 GwyModuleInfo* _gwy_module_query__aistfile(void);
 GwyModuleInfo* _gwy_module_query__alicona(void);
 GwyModuleInfo* _gwy_module_query__ambfile(void);
+GwyModuleInfo* _gwy_module_query__ambprofile(void);
 GwyModuleInfo* _gwy_module_query__anfatec(void);
 GwyModuleInfo* _gwy_module_query__apefile(void);
 GwyModuleInfo* _gwy_module_query__asciiexport(void);
@@ -68,6 +70,7 @@ GwyModuleInfo* _gwy_module_query__nanonis_spec(void);
 GwyModuleInfo* _gwy_module_query__nanoscan(void);
 GwyModuleInfo* _gwy_module_query__nanoscope_ii(void);
 GwyModuleInfo* _gwy_module_query__nanoscope(void);
+GwyModuleInfo* _gwy_module_query__nanosystemz(void);
 GwyModuleInfo* _gwy_module_query__nanotop(void);
 GwyModuleInfo* _gwy_module_query__netcdf(void);
 GwyModuleInfo* _gwy_module_query__nmmxyz(void);
@@ -80,6 +83,7 @@ GwyModuleInfo* _gwy_module_query__omicron(void);
 GwyModuleInfo* _gwy_module_query__omicronflat(void);
 GwyModuleInfo* _gwy_module_query__omicronmatrix(void);
 GwyModuleInfo* _gwy_module_query__opdfile(void);
+GwyModuleInfo* _gwy_module_query__phoenix(void);
 GwyModuleInfo* _gwy_module_query__pnifile(void);
 GwyModuleInfo* _gwy_module_query__pltfile(void);
 GwyModuleInfo* _gwy_module_query__princetonspe(void);
@@ -125,9 +129,11 @@ GwyModuleInfo* _gwy_module_query__zemax(void);
 
 static const GwyModuleRecord modules[] = {
   { _gwy_module_query__accurexii_txt, "accurexii-txt", },
+  { _gwy_module_query__afmw_spec, "afmw-spec", },
   { _gwy_module_query__aistfile, "aistfile", },
   { _gwy_module_query__alicona, "alicona", },
   { _gwy_module_query__ambfile, "ambfile", },
+  { _gwy_module_query__ambprofile, "ambprofile", },
   { _gwy_module_query__anfatec, "anfatec", },
   { _gwy_module_query__apefile, "apefile", },
   { _gwy_module_query__asciiexport, "asciiexport", },
@@ -180,6 +186,7 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__nanoscan, "nanoscan", },
   { _gwy_module_query__nanoscope_ii, "nanoscope-ii", },
   { _gwy_module_query__nanoscope, "nanoscope", },
+  { _gwy_module_query__nanosystemz, "nanosystemz", },
   { _gwy_module_query__nanotop, "nanotop", },
   { _gwy_module_query__netcdf, "netcdf", },
   { _gwy_module_query__nmmxyz, "nmmxyz", },
@@ -192,6 +199,7 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__omicronflat, "omicronflat", },
   { _gwy_module_query__omicronmatrix, "omicronmatrix", },
   { _gwy_module_query__opdfile, "opdfile", },
+  { _gwy_module_query__phoenix, "phoenix", },
   { _gwy_module_query__pnifile, "pnifile", },
   { _gwy_module_query__pltfile, "pltfile", },
   { _gwy_module_query__princetonspe, "princetonspe", },
