@@ -1,5 +1,5 @@
 /*
- *  $Id: nanonis.c 20777 2018-01-30 11:44:38Z yeti-dn $
+ *  $Id: nanonis.c 21601 2018-11-07 08:52:33Z yeti-dn $
  *  Copyright (C) 2006,2015-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -55,6 +55,7 @@
 #include <libgwymodule/gwymodule-file.h>
 #include <app/data-browser.h>
 #include <app/gwymoduleutils-file.h>
+#include <app/settings.h>
 
 #include "err.h"
 
@@ -98,6 +99,10 @@ typedef struct {
     gboolean bogus_scan_time;
 } SXMFile;
 
+typedef struct {
+    gboolean preserve_coordinates;
+} SXMArgs;
+
 static gboolean      module_register(void);
 static gint          sxm_detect     (const GwyFileDetectInfo *fileinfo,
                                      gboolean only_name);
@@ -106,13 +111,21 @@ static GwyContainer* sxm_load       (const gchar *filename,
                                      GError **error);
 static GwyContainer* sxm_build_meta (const SXMFile *sxmfile,
                                      guint id);
+static void          sxm_load_args  (GwyContainer *container,
+                                     SXMArgs *args);
+static void          sxm_save_args  (GwyContainer *container,
+                                     const SXMArgs *args);
+
+static const SXMArgs sxm_defaults = {
+    FALSE,
+};
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Nanonis SXM data files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "1.2",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -390,6 +403,7 @@ read_data_field(GwyContainer *container,
                 const SXMFile *sxmfile,
                 const SXMDataInfo *data_info,
                 SXMDirection dir,
+                SXMArgs *args,
                 const guchar **p)
 {
     GwyContainer *meta;
@@ -402,6 +416,7 @@ read_data_field(GwyContainer *container,
     dfield = gwy_data_field_new(sxmfile->xres, sxmfile->yres,
                                 sxmfile->xreal, sxmfile->yreal,
                                 FALSE);
+    /* This is correct for both preserved and non-preserved coordinates? */
     gwy_data_field_set_xoffset(dfield, sxmfile->xoff - 0.5*sxmfile->xreal);
     gwy_data_field_set_yoffset(dfield, sxmfile->yoff - 0.5*sxmfile->yreal);
     data = gwy_data_field_get_data(dfield);
@@ -469,13 +484,15 @@ read_data_field(GwyContainer *container,
     if (dir == DIR_BACKWARD)
         flip_horizontally = TRUE;
 
-    /* XXX: This leaves images flipped vertically with respect to Nanonis
-     * software.  But it hopefully makes possible to put SPS points at correct
-     * positions.  We cannot do that when reading the spectra later because
-     * spectra do not know about ‘their’ images. */
-    if ((s = g_hash_table_lookup(sxmfile->meta, "SCAN_DIR"))
-        && gwy_strequal(s, "down"))
-        flip_vertically = TRUE;
+    /* With preserve_coordinates images are flipped vertically with respect to
+     * Nanonis software.  But it hopefully makes possible to put SPS points at
+     * correct positions.  We cannot do that when reading the spectra later
+     * because spectra do not know about ‘their’ images. */
+    if ((s = g_hash_table_lookup(sxmfile->meta, "SCAN_DIR"))) {
+        if ((args->preserve_coordinates && gwy_strequal(s, "down"))
+            || (!args->preserve_coordinates && gwy_strequal(s, "up")))
+            flip_vertically = TRUE;
+    }
 
     gwy_data_field_invert(dfield, flip_vertically, flip_horizontally, FALSE);
     g_object_unref(dfield);
@@ -496,6 +513,8 @@ sxm_load(const gchar *filename,
          GError **error)
 {
     SXMFile sxmfile;
+    SXMArgs args;
+    GwyContainer *settings;
     GwyContainer *container = NULL;
     GPtrArray *header_lines;
     guchar *buffer = NULL;
@@ -504,10 +523,12 @@ sxm_load(const gchar *filename,
     const guchar *p;
     gchar *header, *hp, *s, *endptr;
     gchar **columns;
-    G_GNUC_UNUSED gboolean rotated = FALSE;
     gint version;
     guint i;
 
+    settings = gwy_app_settings_get();
+    sxm_load_args(settings, &args);
+
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
         return NULL;
@@ -605,6 +626,7 @@ sxm_load(const gchar *filename,
         }
     }
 
+#if 0
     /* Check for rotated data */
     if (sxmfile.ok) {
         if ((s = g_hash_table_lookup(sxmfile.meta, "SCAN_ANGLE"))) {
@@ -614,6 +636,7 @@ sxm_load(const gchar *filename,
             }
         }
     }
+#endif
 
     /* Pixel sizes */
     if (sxmfile.ok) {
@@ -738,14 +761,14 @@ sxm_load(const gchar *filename,
             if (d == DIR_BOTH) {
                 read_data_field(container, &id, filename,
                                 &sxmfile, sxmfile.data_info + i,
-                                DIR_FORWARD, &p);
+                                DIR_FORWARD, &args, &p);
                 read_data_field(container, &id, filename,
                                 &sxmfile, sxmfile.data_info + i,
-                                DIR_BACKWARD, &p);
+                                DIR_BACKWARD, &args, &p);
             }
             else if (d == DIR_FORWARD || d == DIR_BACKWARD) {
                 read_data_field(container, &id, filename,
-                                &sxmfile, sxmfile.data_info + i, d, &p);
+                                &sxmfile, sxmfile.data_info + i, d, &args, &p);
             }
             else {
                 g_assert_not_reached();
@@ -758,6 +781,9 @@ sxm_load(const gchar *filename,
     g_hash_table_destroy(sxmfile.meta);
     g_free(header);
     gwy_file_abandon_contents(buffer, size, NULL);
+    /* This may seem pointless, but it creates the line in settings file so
+     * someone wishing to change it only has to change the value. */
+    sxm_save_args(settings, &args);
 
     return container;
 }
@@ -839,4 +865,30 @@ sxm_build_meta(const SXMFile *sxmfile,
     return NULL;
 }
 
+static const gchar preserve_coordinates_key[] = "/module/nanonis/preserve_coordinates";
+
+static void
+sxm_sanitize_args(SXMArgs *args)
+{
+    args->preserve_coordinates = !!args->preserve_coordinates;
+}
+
+static void
+sxm_save_args(GwyContainer *container, const SXMArgs *args)
+{
+    gwy_container_set_boolean_by_name(container, preserve_coordinates_key,
+                                      args->preserve_coordinates);
+}
+
+static void
+sxm_load_args(GwyContainer *container, SXMArgs *args)
+{
+    *args = sxm_defaults;
+
+    gwy_container_gis_boolean_by_name(container, preserve_coordinates_key,
+                                      &args->preserve_coordinates);
+
+    sxm_sanitize_args(args);
+}
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
