@@ -1,5 +1,5 @@
 /*
- *  $Id: nanoscantech.c 20676 2017-12-18 18:19:09Z yeti-dn $
+ *  $Id: nanoscantech.c 21430 2018-09-11 11:46:00Z yeti-dn $
  *  Copyright (C) 2012 David Necas (Yeti), Daniil Bratashov (dn2010).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -128,7 +128,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports NanoScanTech .nstdat files."),
     "Daniil Bratashov (dn2010@gmail.com)",
-    "0.12",
+    "0.13",
     "David Nečas (Yeti), Daniil Bratashov (dn2010), Antony Kikaxa",
     "2012",
 };
@@ -192,11 +192,11 @@ nst_load(const gchar *filename,
     GwyBrick *brick;
     GwyZipFile zipfile;
     guint channelno = 0;
-    gint xres, yres;
     gboolean status, is_utf;
-    gchar *buffer, *line, *p, *title, *strkey, *filename_curr;
+    gchar *buffer, *line, *p, *title, *filename_curr;
     gchar *titlestr = NULL;
     gsize size = 0;
+    GQuark quark;
 
     zipfile = gwyzip_open(filename);
     if (!zipfile) {
@@ -231,36 +231,22 @@ nst_load(const gchar *filename,
             if (gwy_strequal(line, "3d")) {
                 gwy_debug("3d: %u", channelno);
                 titlestr = NULL;
-                dfield = nst_read_3d(p, size, is_utf,
-                                     &metadata, &titlestr);
+                dfield = nst_read_3d(p, size, is_utf, &metadata, &titlestr);
                 if (dfield) {
-                    strkey = g_strdup_printf("/%d/data",
-                                             channelno);
-                    gwy_container_set_object_by_name(container,
-                                                     strkey,
-                                                     dfield);
+                    quark = gwy_app_get_data_key_for_id(channelno);
+                    gwy_container_set_object(container, quark, dfield);
                     g_object_unref(dfield);
-                    g_free(strkey);
-                    strkey = g_strdup_printf("/%d/meta",
-                                             channelno);
                     if (metadata) {
-                        gwy_container_set_object_by_name(container,
-                                                         strkey,
-                                                         metadata);
+                        quark = gwy_app_get_data_meta_key_for_id(channelno);
+                        gwy_container_set_object(container, quark, metadata);
                         g_object_unref(metadata);
                     }
-                    g_free(strkey);
-                    strkey = g_strdup_printf("/%d/data/title",
-                                             channelno);
+                    quark = gwy_app_get_data_title_key_for_id(channelno);
                     if (!titlestr)
-                        title = g_strdup_printf("Channel %u",
-                                                channelno);
+                        title = g_strdup_printf("Channel %u", channelno);
                     else
-                        title = g_strdup_printf("%s (%u)",
-                                                titlestr, channelno);
-                    gwy_container_set_string_by_name(container, strkey,
-                                                     title);
-                    g_free(strkey);
+                        title = g_strdup_printf("%s (%u)", titlestr, channelno);
+                    gwy_container_set_string(container, quark, title);
 
                     gwy_file_channel_import_log_add(container, channelno,
                                                     NULL, filename);
@@ -274,12 +260,9 @@ nst_load(const gchar *filename,
                 gwy_debug("2d: %d", channelno);
                 gmodel = nst_read_2d(p, channelno, is_utf);
                 if (gmodel) {
-                    strkey = g_strdup_printf("/0/graph/graph/%d",
-                                             channelno+1);
-                    gwy_container_set_object_by_name(container, strkey,
-                                                     gmodel);
+                    quark = gwy_app_get_graph_key_for_id(channelno+1);
+                    gwy_container_set_object(container, quark, gmodel);
                     g_object_unref(gmodel);
-                    g_free(strkey);
                 }
             }
             else if (gwy_strequal(line, "4d")) {
@@ -287,45 +270,23 @@ nst_load(const gchar *filename,
                 brick = nst_read_4d(p, size, is_utf,
                                     &metadata, &titlestr);
                 if (brick) {
-                    strkey = g_strdup_printf("/brick/%d",
-                                             channelno);
-                    gwy_container_set_object_by_name(container,
-                                                     strkey,
-                                                     brick);
-                    g_free(strkey);
-                    strkey = g_strdup_printf("/brick/%d/title",
-                                             channelno);
+                    quark = gwy_app_get_brick_key_for_id(channelno);
+                    gwy_container_set_object(container, quark, brick);
+                    quark = gwy_app_get_brick_title_key_for_id(channelno);
                     if (!titlestr)
-                        title = g_strdup_printf("Channel %u",
-                                                channelno);
+                        title = g_strdup_printf("Channel %u", channelno);
                     else
-                        title = g_strdup_printf("%s (%u)",
-                                                titlestr, channelno);
-                    gwy_container_set_string_by_name(container, strkey,
-                                                     title);
-                    g_free(strkey);
-                    strkey = g_strdup_printf("/brick/%d/meta",
-                                             channelno);
+                        title = g_strdup_printf("%s (%u)", titlestr, channelno);
+                    gwy_container_set_string(container, quark, title);
                     if (metadata) {
-                        gwy_container_set_object_by_name(container,
-                                                         strkey,
-                                                         metadata);
+                        quark = gwy_app_get_brick_meta_key_for_id(channelno);
+                        gwy_container_set_object(container, quark, metadata);
                         g_object_unref(metadata);
                     }
-                    g_free(strkey);
-                    xres = gwy_brick_get_xres(brick);
-                    yres = gwy_brick_get_yres(brick);
-                    dfield = gwy_data_field_new(xres, yres,
-                                                1.0, 1.0, FALSE);
-                    gwy_brick_mean_plane(brick, dfield,
-                                         0, 0, 0,
-                                         xres, yres, -1, FALSE);
-                    strkey = g_strdup_printf("/brick/%d/preview",
-                                             channelno);
-                    gwy_container_set_object_by_name(container,
-                                                     strkey,
-                                                     dfield);
-                    g_free(strkey);
+                    dfield = gwy_data_field_new(1, 1, 1.0, 1.0, FALSE);
+                    gwy_brick_mean_xy_plane(brick, dfield);
+                    quark = gwy_app_get_brick_preview_key_for_id(channelno);
+                    gwy_container_set_object(container, quark, dfield);
                     g_object_unref(brick);
                     g_object_unref(dfield);
                     gwy_file_volume_import_log_add(container, channelno,
