@@ -111,7 +111,7 @@ static GwyModuleInfo module_info = {
     "2006",
 };
 
-GWY_MODULE_QUERY(module_info)
+GWY_MODULE_QUERY2(module_info, nanotop)
 
 static gboolean
 module_register(void)
