@@ -1,5 +1,5 @@
 /*
- *  $Id: netcdf.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: netcdf.c 21430 2018-09-11 11:46:00Z yeti-dn $
  *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek, Niv Levy.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, nivl2000@gmail.com.
  *
@@ -284,7 +284,7 @@ gwy_brick_invert(GwyBrick *brick,
             GWY_SWAP(gdouble, *data, *flip);
     }
     else if (x && y) {
-        for (k = 0; k < brick->zres ; k++) {
+        for (k = 0; k < brick->zres; k++) {
             data = brick->data + k * n_layer;
             flip = data + n_layer -1;
             for (i = 0; i < n_layer/2; i++, data++, flip--)
@@ -295,17 +295,17 @@ gwy_brick_invert(GwyBrick *brick,
       return;
     }
     else if (z) { // TEST
-        for (i = 0 ; i < brick->xres ; i++) {
-            for (j = 0 ; j < brick->yres ; j++) {
+        for (i = 0; i < brick->xres; i++) {
+            for (j = 0; j < brick->yres; j++) {
                 data = brick->data + i + j * brick->xres;
                 flip = data + (brick->zres -1) * n_layer;
-                for (k = 0 ; k < brick->zres/2 ; k++, data+=n_layer, flip-=n_layer)
+                for (k = 0; k < brick->zres/2; k++, data+=n_layer, flip-=n_layer)
                     GWY_SWAP(gdouble, *data, *flip);
             }
         }
     }
     else if (y) {
-        for (k = 0; k < brick->zres ; k++) {
+        for (k = 0; k < brick->zres; k++) {
             for (i = 0; i < brick->yres; i++) {
                 data = brick->data + i * brick->xres + k * n_layer;
                 flip = data + brick->xres-1;
@@ -353,7 +353,7 @@ gxsm_load(const gchar *filename,
     gint dim_time, dim_value;
     gfloat *values;
     gint i, power10, value_i, time_i, frame_i;
-    const guchar *p ;
+    const guchar *p;
     guchar *pt;
     gboolean good_time_series = FALSE, good_value_Series = FALSE;
     /*allowed deviation from linearity (beyond which we don't load as volume)*/
@@ -395,14 +395,14 @@ gxsm_load(const gchar *filename,
     /* we know these exist, so just read them */
 
     dim = cdffile.dims + field_var->dimids[0];
-    dim_time = dim->length ;
+    dim_time = dim->length;
     times = g_new(gdouble, dim_time);
 
     dim = cdffile.dims + field_var->dimids[1];
-    dim_value = dim->length ;
+    dim_value = dim->length;
     values = g_new(gfloat, dim_value);
 
-    gwy_debug("dimensions: time = %d  value = %d", dim_time, dim_value) ;
+    gwy_debug("dimensions: time = %d  value = %d", dim_time, dim_value);
     /* create volume data only if equally spaced - to better than some
      * arbitrary limit currently set at 1%
      * This is meant mostly for time movies where the clock can occasionnaly
@@ -419,7 +419,7 @@ gxsm_load(const gchar *filename,
                     _("Value series values must be of type FLOAT."));
             goto gxsm_load_fail;
         }
-        p = (const guchar*) (cdffile.buffer + var->begin) ;
+        p = (const guchar*) (cdffile.buffer + var->begin);
         for (i = 0; i < dim_value; i++) {
             values[i] = gwy_get_gfloat_be(&p);
             if (i > 1 && good_value_Series && fabs(((values[i] - values[i-1])
@@ -441,7 +441,7 @@ gxsm_load(const gchar *filename,
                     _("Time values must be of type DOUBLE."));
             goto gxsm_load_fail;
         }
-        p = (const guchar*) (cdffile.buffer + var->begin) ;
+        p = (const guchar*) (cdffile.buffer + var->begin);
         for (i = 1; i < dim_time; i++) {
             times[i] = gwy_get_gdouble_be(&p);
             if (i > 1 && good_time_series &&
@@ -456,7 +456,7 @@ gxsm_load(const gchar *filename,
     }
 
     if ((var = cdffile_get_var(&cdffile, "sranger_mk2_hwi_fast_scan_flag"))) {
-        p = (const guchar*) (cdffile.buffer + var->begin) ;
+        p = (const guchar*) (cdffile.buffer + var->begin);
         if (gwy_get_gint16_be(&p) == 1) {
             g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_UNIMPLEMENTED,
                     _("Reading of fast scan files is not implemented - yet."));
@@ -471,7 +471,7 @@ gxsm_load(const gchar *filename,
     data = gwy_container_new();
     /* create meta before data since i want to use it for some conditional
      * stuff*/
-    meta = create_meta(cdffile) ;
+    meta = create_meta(cdffile);
     /* also FIXME -check if recent gxsm does put a name for the value*/
     gwy_container_set_object_by_name(data, "/brick/0/meta", meta);
     g_object_unref(meta);
@@ -488,7 +488,7 @@ gxsm_load(const gchar *filename,
                                     cdffile.dims[field_var->dimids[3]].length,
                                     cdffile.dims[field_var->dimids[2]].length,
                                     dim_value,
-                                    field_var->type) ;
+                                    field_var->type);
             /* NOTE :
              * 1. flip the Z direction so that "up" is "correctly oriented"
              * note that this opens issues is one tries to correlate voltage
@@ -501,13 +501,13 @@ gxsm_load(const gchar *filename,
              * center of a pixel, rather than the border e.g. if one take a
              * series of images at -0.95, -0.85.... 0.95V (20 steps),
              * zreal = 2.0 rather than 1.9V. In a sense, it's treating it as
-             * if we took the images in a cell at [-1,-0.9] ; it seems to be
+             * if we took the images in a cell at [-1,-0.9]; it seems to be
              * self consistent.
              */
             if ((real = (values[1] - values[0]) * dim_value) < 0 ) {
                 g_warning("Flipping brick since values have negative step");
                 gwy_brick_invert(dbrick, FALSE, FALSE, TRUE);
-                real = -real ;
+                real = -real;
                 offset = values[dim_value-1];
                 gwy_container_set_string_by_name(meta, "Z flipped",
                                                 g_strdup("True"));
@@ -528,7 +528,7 @@ gxsm_load(const gchar *filename,
                                                 values[dim_value-1]));
             }
 
-            gwy_brick_set_zoffset(dbrick, offset) ;
+            gwy_brick_set_zoffset(dbrick, offset);
             gwy_brick_set_zreal(dbrick, real);
             // duplicating code, yuck
             var = cdffile_get_var(&cdffile, "value");
@@ -546,14 +546,14 @@ gxsm_load(const gchar *filename,
                                     cdffile.dims[field_var->dimids[3]].length,
                                     cdffile.dims[field_var->dimids[2]].length,
                                     dim_time,
-                                    field_var->type) ;
+                                    field_var->type);
             /* if the times are not ordered something is very wrong - i say we fail*/
             if (times[1] < times[0]) {
                 g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                     _("Time series order is wrong."));
                 goto gxsm_load_fail;
             }
-            gwy_brick_set_zoffset(dbrick, times[0]) ;
+            gwy_brick_set_zoffset(dbrick, times[0]);
             gwy_brick_set_zreal(dbrick, (times[1] - times[0]) * dim_time);
             siunit = gwy_si_unit_new_parse("s", 0);
             gwy_brick_set_si_unit_z(dbrick, siunit);
@@ -654,14 +654,9 @@ gxsm_load(const gchar *filename,
         }
 
         //add the false 2D image in /brick/0/preview
-        dfield = gwy_data_field_new(cdffile.dims[field_var->dimids[3]].length,
-                                    cdffile.dims[field_var->dimids[2]].length,
-                                    1.0, 1.0, FALSE) ;
-        gwy_brick_extract_plane(dbrick, dfield, 0, 0, 0,
-                                    cdffile.dims[field_var->dimids[3]].length,
-                                    cdffile.dims[field_var->dimids[2]].length,
-                                    -1, TRUE);
-        gwy_container_set_object_by_name(data, "/brick/0/preview", dfield) ;
+        dfield = gwy_data_field_new(1, 1, 1.0, 1.0, FALSE);
+        gwy_brick_mean_xy_plane(dbrick, dfield);
+        gwy_container_set_object_by_name(data, "/brick/0/preview", dfield);
         if ((attr = cdffile_get_attr(field_var->attrs, field_var->nattrs, "long_name"))
                     && attr->type == NC_CHAR
                     && attr->nelems) {
@@ -675,9 +670,9 @@ gxsm_load(const gchar *filename,
     }
     // either single field, 4D (unlikely for gxsm), or not equally enough spaced
     else {
-        for (value_i = 0 ; value_i < dim_value ; value_i++) {
-            for (time_i = 0 ; time_i < dim_time ; time_i++) {
-                frame_i = value_i * dim_time + time_i ;
+        for (value_i = 0; value_i < dim_value; value_i++) {
+            for (time_i = 0; time_i < dim_time; time_i++) {
+                frame_i = value_i * dim_time + time_i;
 
                 dfield = read_data_field((const guchar*)(cdffile.buffer + field_var->begin),
                                         cdffile.dims[field_var->dimids[3]].length,
@@ -775,7 +770,7 @@ gxsm_load(const gchar *filename,
                 /*TODO copy meta instead of creating a new one when dealing with
                  * multiple frames? */
                 meta = create_meta(cdffile);
-                pt =  g_strdup_printf("/%d/meta", frame_i) ;
+                pt =  g_strdup_printf("/%d/meta", frame_i);
                 gwy_container_set_object_by_name(data, pt, meta);
                 g_free(pt);
 
@@ -909,7 +904,7 @@ read_data_field(const guchar *buffer,
     GwyDataField *dfield;
     gdouble *data;
     gint i;
-    const guchar* shifted_buffer ;
+    const guchar* shifted_buffer;
 
     gwy_debug("processing frame %d", frame_i);
     dfield = gwy_data_field_new(xres, yres, 1.0, 1.0, FALSE);
@@ -972,12 +967,12 @@ read_brick(const guchar *buffer,
           gint zres,
           NetCDFType type)
 {
-    GwyBrick *brick ;
+    GwyBrick *brick;
     gdouble * data;
     gint i;
 
-    brick = gwy_brick_new(xres, yres, zres, 1.0, 1.0, 1.0, FALSE) ;
-    data = gwy_brick_get_data(brick) ;
+    brick = gwy_brick_new(xres, yres, zres, 1.0, 1.0, 1.0, FALSE);
+    data = gwy_brick_get_data(brick);
     switch (type) {
         case NC_BYTE:
         case NC_CHAR:
@@ -1029,11 +1024,11 @@ static GwyContainer*
 create_meta
 (const NetCDF cdffile)
 {
-    GwyContainer *meta ;
+    GwyContainer *meta;
     const NetCDFVar *var;
-    GwySIUnit* siunit ;
-    gdouble real ;
-    gint power10 ;
+    GwySIUnit* siunit;
+    gdouble real;
+    gint power10;
 
     meta = gwy_container_new();
 
@@ -1090,7 +1085,7 @@ create_meta
                                          g_strndup(cdffile.buffer + var->begin,
                                                    var->vsize));
     }
-    return meta ;
+    return meta;
 }
 
 
