@@ -1,6 +1,6 @@
 /*
- *  $Id: nmmxyz.c 20544 2017-11-29 13:18:49Z yeti-dn $
- *  Copyright (C) 2016-2017 David Necas (Yeti).
+ *  $Id: nmmxyz.c 21408 2018-09-07 13:53:53Z yeti-dn $
+ *  Copyright (C) 2016-2018 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -204,7 +204,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Nano Measuring Machine profile files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.3",
+    "1.5",
     "David Nečas (Yeti)",
     "2016",
 };
@@ -312,7 +312,6 @@ nmmxyz_load(const gchar *filename,
         err_CANCELLED(error);
         goto fail;
     }
-    gwy_app_wait_set_fraction(0.0);
 
     points = create_points_with_xy(data, args.nincluded);
     if (!gwy_app_wait_set_fraction(1.0/args.nincluded)) {
@@ -660,8 +659,9 @@ create_points_with_xy(GwyDoubleArray *data, guint nincluded)
     const gdouble *d;
     gsize i, npts;
 
-    gwy_debug("data->len %u, included block size %u", data->len, nincluded);
-    d = (const gdouble*)data->data;
+    gwy_debug("data->len %u, included block size %u",
+              (guint)data->len, nincluded);
+    d = data->data;
     npts = data->len/nincluded;
     points = g_new(GwyXYZ, npts);
     gwy_debug("creating %lu XYZ points", npts);
@@ -788,7 +788,15 @@ create_data_field(GwyContainer *container,
     }
     gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), zunit);
 
-    d = data->data + i;
+    /* Map i (colulmn in data fields) to index in actually read blocks (which
+     * are only nincluded).  Do it by directly moving d to the first value
+     * of the requested channel. */
+    d = data->data;
+    for (k = 0; k < i; k++) {
+        if (args->include_channel[k])
+            d++;
+    }
+
     nincluded = args->nincluded;
     ndata = info->ndata;
     for (k = 0; k < ndata; k++) {
@@ -858,7 +866,7 @@ gather_data_files(const gchar *filename,
     gchar *dirname = NULL, *basename = NULL, *s;
     GString *str;
     gboolean ok = FALSE;
-    guint nincluded = 0, oldblocksize, oldnfiles;
+    guint oldblocksize, oldnfiles;
     gchar *filenames[2] = { NULL, NULL };
 
     oldblocksize = info->blocksize;
@@ -911,10 +919,6 @@ gather_data_files(const gchar *filename,
             }
             copy_profile_descriptions(this_dscs, dscs);
             info->blocksize = this_dscs->len;
-            nincluded = (args->include_channel
-                         ? args->nincluded
-                         : info->blocksize);
-            gwy_debug("setting nincluded to %u", nincluded);
         }
         else {
             if (!profile_descriptions_match(dscs, this_dscs)) {
@@ -948,7 +952,7 @@ gather_data_files(const gchar *filename,
             }
             read_data_file(data, str->str, info->blocksize,
                            args->include_channel, args->nincluded);
-            info->ndata = data->len/nincluded;
+            info->ndata = data->len/args->nincluded;
 
             f = (gdouble)info->nfiles/oldnfiles;
             f = CLAMP(f, 0.0, 1.0);
