@@ -1,5 +1,5 @@
 /*
- *  $Id: nrrdfile.c 20676 2017-12-18 18:19:09Z yeti-dn $
+ *  $Id: nrrdfile.c 21430 2018-09-11 11:46:00Z yeti-dn $
  *  Copyright (C) 2011 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -120,6 +120,7 @@ static void          unescape_field_value(gchar *value);
 static NRRDDataType  parse_data_type     (const gchar *value);
 static NRRDEncoding  parse_encoding      (const gchar *value);
 static gboolean      find_gwy_data_type  (NRRDDataType datatype,
+                                          NRRDEncoding encoding,
                                           const gchar *endian,
                                           GwyRawDataType *rawdatatype,
                                           GwyByteOrder *byteorder,
@@ -178,7 +179,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports and exports nearly raw raster data (NRRD) files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.6",
+    "0.7",
     "David Nečas (Yeti)",
     "2011",
 };
@@ -384,7 +385,8 @@ nrrdfile_load(const gchar *filename,
         goto fail;
     }
 
-    if (!find_gwy_data_type(data_type, g_hash_table_lookup(fields, "endian"),
+    if (!find_gwy_data_type(data_type, encoding,
+                            g_hash_table_lookup(fields, "endian"),
                             &rawdatatype, &byteorder, error))
         goto fail;
 
@@ -483,11 +485,8 @@ nrrdfile_load(const gchar *filename,
                                          fields, data_start);
         quark = gwy_app_get_brick_key_for_id(0);
         gwy_container_set_object(container, quark, brick);
-        dfield = gwy_data_field_new(xres, yres,
-                                    gwy_brick_get_xreal(brick),
-                                    gwy_brick_get_yreal(brick),
-                                    FALSE);
-        gwy_brick_mean_plane(brick, dfield, 0, 0, 0, xres, yres, -1, FALSE);
+        dfield = gwy_data_field_new(1, 1, 1.0, 1.0, FALSE);
+        gwy_brick_mean_xy_plane(brick, dfield);
         gwy_container_set_object_by_name(container, "/brick/0/preview", dfield);
         g_object_unref(brick);
         g_object_unref(dfield);
@@ -682,7 +681,8 @@ parse_encoding(const gchar *value)
 }
 
 static gboolean
-find_gwy_data_type(NRRDDataType datatype, const gchar *endian,
+find_gwy_data_type(NRRDDataType datatype, NRRDEncoding encoding,
+                   const gchar *endian,
                    GwyRawDataType *rawdatatype, GwyByteOrder *byteorder,
                    GError **error)
 {
@@ -706,7 +706,7 @@ find_gwy_data_type(NRRDDataType datatype, const gchar *endian,
 
     *rawdatatype = types[datatype];
     *byteorder = GWY_BYTE_ORDER_NATIVE;
-    if (gwy_raw_data_size(*rawdatatype) > 1) {
+    if (encoding != NRRD_ENCODING_TEXT && gwy_raw_data_size(*rawdatatype) > 1) {
         if (!endian) {
             err_MISSING_FIELD(error, "endian");
             return FALSE;
@@ -994,6 +994,7 @@ decode_text(const gchar *encoded,
             *s8 = (gint8)g_ascii_strtoll(p, &end, 10);
             if (end == p)
                 goto fail;
+            p = end;
         }
     }
     else if (rawdatatype == GWY_RAW_DATA_UINT8) {
@@ -1002,6 +1003,7 @@ decode_text(const gchar *encoded,
             *u8 = (guint8)g_ascii_strtoull(p, &end, 10);
             if (end == p)
                 goto fail;
+            p = end;
         }
     }
     else if (rawdatatype == GWY_RAW_DATA_SINT16) {
@@ -1010,6 +1012,7 @@ decode_text(const gchar *encoded,
             *s16 = (gint16)g_ascii_strtoll(p, &end, 10);
             if (end == p)
                 goto fail;
+            p = end;
         }
     }
     else if (rawdatatype == GWY_RAW_DATA_UINT16) {
@@ -1018,6 +1021,7 @@ decode_text(const gchar *encoded,
             *u16 = (guint16)g_ascii_strtoull(p, &end, 10);
             if (end == p)
                 goto fail;
+            p = end;
         }
     }
     else if (rawdatatype == GWY_RAW_DATA_SINT32) {
@@ -1026,6 +1030,7 @@ decode_text(const gchar *encoded,
             *s32 = (gint32)g_ascii_strtoll(p, &end, 10);
             if (end == p)
                 goto fail;
+            p = end;
         }
     }
     else if (rawdatatype == GWY_RAW_DATA_UINT32) {
@@ -1034,6 +1039,7 @@ decode_text(const gchar *encoded,
             *u32 = (guint32)g_ascii_strtoull(p, &end, 10);
             if (end == p)
                 goto fail;
+            p = end;
         }
     }
     else if (rawdatatype == GWY_RAW_DATA_SINT64) {
@@ -1042,6 +1048,7 @@ decode_text(const gchar *encoded,
             *s64 = g_ascii_strtoll(p, &end, 10);
             if (end == p)
                 goto fail;
+            p = end;
         }
     }
     else if (rawdatatype == GWY_RAW_DATA_UINT64) {
@@ -1050,6 +1057,7 @@ decode_text(const gchar *encoded,
             *u64 = g_ascii_strtoull(p, &end, 10);
             if (end == p)
                 goto fail;
+            p = end;
         }
     }
     else if (rawdatatype == GWY_RAW_DATA_FLOAT) {
@@ -1058,6 +1066,7 @@ decode_text(const gchar *encoded,
             *f32 = (gfloat)g_ascii_strtod(p, &end);
             if (end == p)
                 goto fail;
+            p = end;
         }
     }
     else if (rawdatatype == GWY_RAW_DATA_DOUBLE) {
@@ -1066,6 +1075,7 @@ decode_text(const gchar *encoded,
             *d64 = g_ascii_strtod(p, &end);
             if (end == p)
                 goto fail;
+            p = end;
         }
     }
     else {
