@@ -1,5 +1,5 @@
 /*
- *  $Id: oldmda.c 20676 2017-12-18 18:19:09Z yeti-dn $
+ *  $Id: oldmda.c 21430 2018-09-11 11:46:00Z yeti-dn $
  *  Copyright (C) 2010-2011 David Necas (Yeti), Petr Klapetek,
  *  Daniil Bratashov (dn2010)
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, dn2010@gmail.com
@@ -383,8 +383,7 @@ oldmda_read_data(OldMDAFile *mdafile, const gchar *buffer)
     gwy_container_set_object_by_name(mdafile->data, "/brick/0", brick);
     gwy_container_set_string_by_name(mdafile->data, "/brick/0/title",
                                      g_strdup("MDA data"));
-    gwy_brick_mean_plane(brick, dfield, 0, 0, 0,
-                         mdafile->xres, mdafile->yres, -1, FALSE);
+    gwy_brick_mean_xy_plane(brick, dfield);
     gwy_container_set_object_by_name(mdafile->data, "/brick/0/preview",
                                      dfield);
     g_object_unref(dfield);
