@@ -1,5 +1,5 @@
 /*
- *  $Id: omicronflat.c 20669 2017-12-18 12:45:41Z yeti-dn $
+ *  $Id: omicronflat.c 20808 2018-03-05 10:08:24Z yeti-dn $
  *  Copyright (C) 2015 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -916,7 +916,7 @@ load_as_channel(OmicronFlatFileList *filelist, guint fileid,
         }
 
         if (gtype && filelist->offsets->len) {
-            GwySelection *selection = g_object_newv(gtype, 0, NULL);
+            GwySelection *selection = g_object_new(gtype, NULL);
             const gdouble *offdata = (const gdouble*)filelist->offsets->data;
             gdouble *seldata = g_new(gdouble, 4*filelist->offsets->len);
             guint n = filelist->offsets->len, j;
