@@ -1,5 +1,5 @@
 /*
- *  $Id: pixmap.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: pixmap.c 21603 2018-11-07 09:13:18Z yeti-dn $
  *  Copyright (C) 2004-2014 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -79,7 +79,7 @@ typedef struct {
     gchar *zunit;
     PixmapMapType maptype;
     GdkPixbuf *pixbuf;
-} PixmapLoadArgs;
+} PixmapArgs;
 
 typedef struct {
     GtkWidget *dialog;
@@ -96,8 +96,8 @@ typedef struct {
     GtkWidget *view;
     gint xres;
     gint yres;
-    PixmapLoadArgs *args;
-} PixmapLoadControls;
+    PixmapArgs *args;
+} PixmapControls;
 
 /* there is a information duplication here,
  * however, we may invent an export format GdkPixbuf cannot load */
@@ -108,54 +108,54 @@ typedef struct {
     const GdkPixbufFormat *pixbuf_format;
 } PixmapFormatInfo;
 
-static gboolean          module_register             (void);
-static gint              pixmap_detect               (const GwyFileDetectInfo *fileinfo,
-                                                      gboolean only_name,
-                                                      const gchar *name);
-static GwyContainer*     pixmap_load                 (const gchar *filename,
-                                                      GwyRunType mode,
-                                                      GError **error,
-                                                      const gchar *name);
-static void              pixmap_load_set_field       (GwyContainer *container,
-                                                      gint id,
-                                                      GwyDataField *dfield,
-                                                      const PixmapLoadArgs *args,
-                                                      const gchar *title);
-static void              pixmap_load_pixbuf_to_dfield(GdkPixbuf *pixbuf,
-                                                      GwyDataField *dfield,
-                                                      PixmapMapType maptype);
-static gboolean          pixmap_load_dialog          (PixmapLoadArgs *args,
-                                                      const gchar *name,
-                                                      gint xres,
-                                                      gint yres,
-                                                      gboolean mapknown,
-                                                      gboolean grayscale,
-                                                      gboolean has_alpha);
-static void              pixmap_load_create_preview  (PixmapLoadArgs *args,
-                                                      PixmapLoadControls *controls);
-static void              pixmap_load_map_type_update (GtkWidget *combo,
-                                                      PixmapLoadControls *controls);
-static void              xyreal_changed_cb           (GtkAdjustment *adj,
-                                                      PixmapLoadControls *controls);
-static void              xymeasureeq_changed_cb      (PixmapLoadControls *controls);
-static void              set_combo_from_unit         (GtkWidget *combo,
-                                                      const gchar *str);
-static void              units_change_cb             (GtkWidget *button,
-                                                      PixmapLoadControls *controls);
-static void              pixmap_add_import_log       (GwyContainer *data,
-                                                      gint id,
-                                                      const gchar *filetype,
-                                                      const gchar *filename);
-static void              pixmap_load_update_controls (PixmapLoadControls *controls,
-                                                      PixmapLoadArgs *args);
-static void              pixmap_load_update_values   (PixmapLoadControls *controls,
-                                                      PixmapLoadArgs *args);
-static PixmapFormatInfo* find_format                 (const gchar *name);
-static void              pixmap_load_load_args       (GwyContainer *container,
-                                                      PixmapLoadArgs *args);
-static void              pixmap_load_save_args       (GwyContainer *container,
-                                                      PixmapLoadArgs *args);
-static void              pixmap_load_sanitize_args   (PixmapLoadArgs *args);
+static gboolean          module_register        (void);
+static gint              pixmap_detect          (const GwyFileDetectInfo *fileinfo,
+                                                 gboolean only_name,
+                                                 const gchar *name);
+static GwyContainer*     pixmap_load            (const gchar *filename,
+                                                 GwyRunType mode,
+                                                 GError **error,
+                                                 const gchar *name);
+static void              pixmap_set_field       (GwyContainer *container,
+                                                 gint id,
+                                                 GwyDataField *dfield,
+                                                 const PixmapArgs *args,
+                                                 const gchar *title);
+static void              pixmap_pixbuf_to_dfield(GdkPixbuf *pixbuf,
+                                                 GwyDataField *dfield,
+                                                 PixmapMapType maptype);
+static gboolean          pixmap_dialog          (PixmapArgs *args,
+                                                 const gchar *name,
+                                                 gint xres,
+                                                 gint yres,
+                                                 gboolean mapknown,
+                                                 gboolean grayscale,
+                                                 gboolean has_alpha);
+static void              pixmap_create_preview  (PixmapArgs *args,
+                                                 PixmapControls *controls);
+static void              pixmap_map_type_update (GtkWidget *combo,
+                                                 PixmapControls *controls);
+static void              xyreal_changed_cb      (GtkAdjustment *adj,
+                                                 PixmapControls *controls);
+static void              xymeasureeq_changed_cb (PixmapControls *controls);
+static void              set_combo_from_unit    (GtkWidget *combo,
+                                                 const gchar *str);
+static void              units_change_cb        (GtkWidget *button,
+                                                 PixmapControls *controls);
+static void              pixmap_add_import_log  (GwyContainer *data,
+                                                 gint id,
+                                                 const gchar *filetype,
+                                                 const gchar *filename);
+static void              pixmap_update_controls (PixmapControls *controls,
+                                                 PixmapArgs *args);
+static void              pixmap_update_values   (PixmapControls *controls,
+                                                 PixmapArgs *args);
+static PixmapFormatInfo* find_format            (const gchar *name);
+static void              pixmap_load_args       (GwyContainer *container,
+                                                 PixmapArgs *args);
+static void              pixmap_save_args       (GwyContainer *container,
+                                                 PixmapArgs *args);
+static void              pixmap_sanitize_args   (PixmapArgs *args);
 
 static const GwyEnum value_map_types[] = {
     { N_("All channels"), PIXMAP_MAP_ALL,   },
@@ -226,7 +226,7 @@ known_formats[] = {
 /* List of PixmapFormatInfo for all formats.  This is never freed */
 static GSList *pixmap_formats = NULL;
 
-static const PixmapLoadArgs pixmap_load_defaults = {
+static const PixmapArgs pixmap_defaults = {
     100.0, 100.0, -6, TRUE, "m", 1.0, -6, "m", PIXMAP_MAP_VALUE, NULL
 };
 
@@ -367,13 +367,13 @@ pixmap_detect(const GwyFileDetectInfo *fileinfo,
     }
     else if (gwy_strequal(name, "tiff")) {
         /* The pixbuf loader is unlikely to load BigTIFFs any time soon. */
-        GwyTIFFVersion version = GWY_TIFF_CLASSIC;
 #ifdef __WIN64
         /* The TIFF loader (supposedly GDI-based) crashes on Win64.  Unclear
          * why.  TIFF is madness.  Note there is a fallback GwyTIFF loader in
          * hdrimage which will take over when we do this.  */
         return 0;
 #else
+        GwyTIFFVersion version = GWY_TIFF_CLASSIC;
         gwy_debug("Checking TIFF header");
         if (!gwy_tiff_detect(fileinfo->head, fileinfo->buffer_len,
                              &version, NULL))
@@ -480,7 +480,7 @@ pixmap_load(const gchar *filename,
     gint i, j, width, height, rowstride;
     gboolean has_alpha, maptype_known, ok;
     gint not_grayscale, any_red, any_green, any_blue, alpha_important;
-    PixmapLoadArgs args;
+    PixmapArgs args;
 
     gwy_debug("Loading <%s> as %s", filename, name);
 
@@ -544,7 +544,7 @@ pixmap_load(const gchar *filename,
     g_object_unref(loader);
 
     settings = gwy_app_settings_get();
-    pixmap_load_load_args(settings, &args);
+    pixmap_load_args(settings, &args);
     args.pixbuf = pixbuf;
 
     width = gdk_pixbuf_get_width(pixbuf);
@@ -569,7 +569,7 @@ pixmap_load(const gchar *filename,
         }
     }
     if (!has_alpha && args.maptype == PIXMAP_MAP_ALPHA)
-        args.maptype = pixmap_load_defaults.maptype;
+        args.maptype = pixmap_defaults.maptype;
 
     maptype_known = FALSE;
     if (alpha_important) {
@@ -596,9 +596,9 @@ pixmap_load(const gchar *filename,
     if (mode == GWY_RUN_INTERACTIVE) {
         gwy_debug("Manual import is necessary.");
 
-        ok = pixmap_load_dialog(&args, name, width, height,
+        ok = pixmap_dialog(&args, name, width, height,
                                 maptype_known, !not_grayscale, alpha_important);
-        pixmap_load_save_args(settings, &args);
+        pixmap_save_args(settings, &args);
         if (!ok) {
             err_CANCELLED(error);
             g_object_unref(pixbuf);
@@ -619,8 +619,8 @@ pixmap_load(const gchar *filename,
             PixmapMapType maptype = (i < 3) ? i + 1 : 6;
             dfield = gwy_data_field_new(width, height, args.xreal, args.yreal,
                                         FALSE);
-            pixmap_load_pixbuf_to_dfield(pixbuf, dfield, maptype);
-            pixmap_load_set_field(data, i, dfield, &args,
+            pixmap_pixbuf_to_dfield(pixbuf, dfield, maptype);
+            pixmap_set_field(data, i, dfield, &args,
                                   gwy_enum_to_string(maptype,
                                                      value_map_types,
                                                      nmaptypes));
@@ -631,8 +631,8 @@ pixmap_load(const gchar *filename,
     else {
         dfield = gwy_data_field_new(width, height, args.xreal, args.yreal,
                                     FALSE);
-        pixmap_load_pixbuf_to_dfield(pixbuf, dfield, args.maptype);
-        pixmap_load_set_field(data, 0, dfield, &args,
+        pixmap_pixbuf_to_dfield(pixbuf, dfield, args.maptype);
+        pixmap_set_field(data, 0, dfield, &args,
                               gwy_enum_to_string(args.maptype,
                                                  value_map_types, nmaptypes));
         g_object_unref(dfield);
@@ -647,11 +647,11 @@ pixmap_load(const gchar *filename,
 }
 
 static void
-pixmap_load_set_field(GwyContainer *container,
-                      gint id,
-                      GwyDataField *dfield,
-                      const PixmapLoadArgs *args,
-                      const gchar *title)
+pixmap_set_field(GwyContainer *container,
+                 gint id,
+                 GwyDataField *dfield,
+                 const PixmapArgs *args,
+                 const gchar *title)
 {
     GwySIUnit *siunit;
     GQuark quark;
@@ -676,9 +676,9 @@ pixmap_load_set_field(GwyContainer *container,
 }
 
 static void
-pixmap_load_pixbuf_to_dfield(GdkPixbuf *pixbuf,
-                             GwyDataField *dfield,
-                             PixmapMapType maptype)
+pixmap_pixbuf_to_dfield(GdkPixbuf *pixbuf,
+                        GwyDataField *dfield,
+                        PixmapMapType maptype)
 {
     gint width, height, rowstride, i, j, bpp;
     guchar *pixels, *p;
@@ -751,17 +751,17 @@ describe_channels(gboolean grayscale, gboolean has_alpha)
 }
 
 static gboolean
-pixmap_load_dialog(PixmapLoadArgs *args,
-                   const gchar *name,
-                   gint xres,
-                   gint yres,
-                   gboolean mapknown,
-                   gboolean grayscale,
-                   gboolean has_alpha)
+pixmap_dialog(PixmapArgs *args,
+              const gchar *name,
+              gint xres,
+              gint yres,
+              gboolean mapknown,
+              gboolean grayscale,
+              gboolean has_alpha)
 {
     enum { RESPONSE_RESET = 1 };
 
-    PixmapLoadControls controls;
+    PixmapControls controls;
     GwyContainer *data;
     GwyPixmapLayer *layer;
     GtkObject *adj;
@@ -843,7 +843,7 @@ pixmap_load_dialog(PixmapLoadArgs *args,
     data = gwy_container_new();
     controls.view = gwy_data_view_new(data);
     g_object_unref(data);
-    pixmap_load_create_preview(args, &controls);
+    pixmap_create_preview(args, &controls);
     layer = gwy_layer_basic_new();
     gwy_pixmap_layer_set_data_key(layer, "/0/data");
     gwy_layer_basic_set_gradient_key(GWY_LAYER_BASIC(layer), "/0/base/palette");
@@ -971,7 +971,7 @@ pixmap_load_dialog(PixmapLoadArgs *args,
 
         controls.maptype
             = gwy_enum_combo_box_new(value_map_types, n,
-                                     G_CALLBACK(pixmap_load_map_type_update),
+                                     G_CALLBACK(pixmap_map_type_update),
                                      &controls,
                                      args->maptype, TRUE);
         gwy_table_attach_row(table, row++,
@@ -989,7 +989,7 @@ pixmap_load_dialog(PixmapLoadArgs *args,
     adj2 = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(controls.yreal));
     g_signal_connect(adj2, "value-changed",
                      G_CALLBACK(xyreal_changed_cb), &controls);
-    pixmap_load_update_controls(&controls, args);
+    pixmap_update_controls(&controls, args);
 
     g_object_unref(sizegroup);
 
@@ -999,7 +999,7 @@ pixmap_load_dialog(PixmapLoadArgs *args,
         switch (response) {
             case GTK_RESPONSE_CANCEL:
             case GTK_RESPONSE_DELETE_EVENT:
-            pixmap_load_update_values(&controls, args);
+            pixmap_update_values(&controls, args);
             gtk_widget_destroy(dialog);
             g_object_unref(controls.small_pixbuf);
             case GTK_RESPONSE_NONE:
@@ -1010,18 +1010,18 @@ pixmap_load_dialog(PixmapLoadArgs *args,
             break;
 
             case RESPONSE_RESET:
-            args->xreal = pixmap_load_defaults.xreal;
-            args->yreal = pixmap_load_defaults.yreal;
-            args->xyexponent = pixmap_load_defaults.xyexponent;
-            args->xymeasureeq = pixmap_load_defaults.xymeasureeq;
+            args->xreal = pixmap_defaults.xreal;
+            args->yreal = pixmap_defaults.yreal;
+            args->xyexponent = pixmap_defaults.xyexponent;
+            args->xymeasureeq = pixmap_defaults.xymeasureeq;
             g_free(args->xyunit);
-            args->xyunit = g_strdup(pixmap_load_defaults.xyunit);
-            args->zreal = pixmap_load_defaults.zreal;
-            args->zexponent = pixmap_load_defaults.zexponent;
+            args->xyunit = g_strdup(pixmap_defaults.xyunit);
+            args->zreal = pixmap_defaults.zreal;
+            args->zexponent = pixmap_defaults.zexponent;
             g_free(args->zunit);
-            args->zunit = g_strdup(pixmap_load_defaults.zunit);
-            args->maptype = pixmap_load_defaults.maptype;
-            pixmap_load_update_controls(&controls, args);
+            args->zunit = g_strdup(pixmap_defaults.zunit);
+            args->maptype = pixmap_defaults.maptype;
+            pixmap_update_controls(&controls, args);
             break;
 
             default:
@@ -1030,7 +1030,7 @@ pixmap_load_dialog(PixmapLoadArgs *args,
         }
     } while (response != GTK_RESPONSE_OK);
 
-    pixmap_load_update_values(&controls, args);
+    pixmap_update_values(&controls, args);
     gtk_widget_destroy(dialog);
     g_object_unref(controls.small_pixbuf);
 
@@ -1038,8 +1038,7 @@ pixmap_load_dialog(PixmapLoadArgs *args,
 }
 
 static void
-pixmap_load_create_preview(PixmapLoadArgs *args,
-                           PixmapLoadControls *controls)
+pixmap_create_preview(PixmapArgs *args, PixmapControls *controls)
 {
     PixmapMapType maptype = args->maptype;
     GwyContainer *data;
@@ -1053,23 +1052,21 @@ pixmap_load_create_preview(PixmapLoadArgs *args,
     }
     if (maptype == PIXMAP_MAP_ALL)
         maptype = PIXMAP_MAP_RED;
-    pixmap_load_pixbuf_to_dfield(controls->small_pixbuf, dfield, maptype);
+    pixmap_pixbuf_to_dfield(controls->small_pixbuf, dfield, maptype);
     gwy_data_field_data_changed(dfield);
 }
 
 static void
-pixmap_load_map_type_update(GtkWidget *combo,
-                            PixmapLoadControls *controls)
+pixmap_map_type_update(GtkWidget *combo, PixmapControls *controls)
 {
 
     controls->args->maptype
         = gwy_enum_combo_box_get_active(GTK_COMBO_BOX(combo));
-    pixmap_load_create_preview(controls->args, controls);
+    pixmap_create_preview(controls->args, controls);
 }
 
 static void
-pixmap_load_update_controls(PixmapLoadControls *controls,
-                            PixmapLoadArgs *args)
+pixmap_update_controls(PixmapControls *controls, PixmapArgs *args)
 {
     GtkAdjustment *adj;
 
@@ -1092,8 +1089,8 @@ pixmap_load_update_controls(PixmapLoadControls *controls,
 }
 
 static void
-pixmap_load_update_values(PixmapLoadControls *controls,
-                          PixmapLoadArgs *args)
+pixmap_update_values(PixmapControls *controls,
+                          PixmapArgs *args)
 {
     GtkAdjustment *adj;
 
@@ -1114,7 +1111,7 @@ pixmap_load_update_values(PixmapLoadControls *controls,
 
 static void
 xyreal_changed_cb(GtkAdjustment *adj,
-                  PixmapLoadControls *controls)
+                  PixmapControls *controls)
 {
     static gboolean in_update = FALSE;
     GtkAdjustment *xadj, *yadj;
@@ -1136,7 +1133,7 @@ xyreal_changed_cb(GtkAdjustment *adj,
 }
 
 static void
-xymeasureeq_changed_cb(PixmapLoadControls *controls)
+xymeasureeq_changed_cb(PixmapControls *controls)
 {
     GtkAdjustment *xadj, *yadj;
 
@@ -1165,13 +1162,13 @@ set_combo_from_unit(GtkWidget *combo,
 
 static void
 units_change_cb(GtkWidget *button,
-                PixmapLoadControls *controls)
+                PixmapControls *controls)
 {
     GtkWidget *dialog, *hbox, *label, *entry;
     const gchar *id, *unit;
     gint response;
 
-    pixmap_load_update_values(controls, controls->args);
+    pixmap_update_values(controls, controls->args);
     id = g_object_get_data(G_OBJECT(button), "id");
     dialog = gtk_dialog_new_with_buttons(_("Change Units"),
                                          GTK_WINDOW(controls->dialog),
@@ -1285,7 +1282,7 @@ static const gchar zunit_key[]       = "/module/pixmap/zunit";
 static const gchar maptype_key[]     = "/module/pixmap/maptype";
 
 static void
-pixmap_load_sanitize_args(PixmapLoadArgs *args)
+pixmap_sanitize_args(PixmapArgs *args)
 {
     args->maptype = MIN(args->maptype, PIXMAP_MAP_LAST-1);
     args->xreal = CLAMP(args->xreal, 0.01, 10000.0);
@@ -1297,10 +1294,9 @@ pixmap_load_sanitize_args(PixmapLoadArgs *args)
 }
 
 static void
-pixmap_load_load_args(GwyContainer *container,
-                      PixmapLoadArgs *args)
+pixmap_load_args(GwyContainer *container, PixmapArgs *args)
 {
-    *args = pixmap_load_defaults;
+    *args = pixmap_defaults;
 
     gwy_container_gis_double_by_name(container, xreal_key, &args->xreal);
     gwy_container_gis_double_by_name(container, yreal_key, &args->yreal);
@@ -1320,12 +1316,11 @@ pixmap_load_load_args(GwyContainer *container,
     args->xyunit = g_strdup(args->xyunit);
     args->zunit = g_strdup(args->zunit);
 
-    pixmap_load_sanitize_args(args);
+    pixmap_sanitize_args(args);
 }
 
 static void
-pixmap_load_save_args(GwyContainer *container,
-                      PixmapLoadArgs *args)
+pixmap_save_args(GwyContainer *container, PixmapArgs *args)
 {
     gwy_container_set_double_by_name(container, xreal_key, args->xreal);
     gwy_container_set_double_by_name(container, yreal_key, args->yreal);
@@ -1337,10 +1332,8 @@ pixmap_load_save_args(GwyContainer *container,
     gwy_container_set_enum_by_name(container, maptype_key, args->maptype);
     gwy_container_set_boolean_by_name(container, xymeasureeq_key,
                                       args->xymeasureeq);
-    gwy_container_set_string_by_name(container, xyunit_key,
-                                     g_strdup(args->xyunit));
-    gwy_container_set_string_by_name(container, zunit_key,
-                                     g_strdup(args->zunit));
+    gwy_container_set_const_string_by_name(container, xyunit_key, args->xyunit);
+    gwy_container_set_const_string_by_name(container, zunit_key, args->zunit);
 }
 
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
