@@ -1,5 +1,5 @@
 /*
- *  $Id: rawxyz.c 20676 2017-12-18 18:19:09Z yeti-dn $
+ *  $Id: rawxyz.c 21300 2018-08-08 14:08:32Z yeti-dn $
  *  Copyright (C) 2009-2016 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -220,8 +220,7 @@ rawxyz_load(const gchar *filename,
         }
         gwy_surface_invalidate(surface);
     }
-    gwy_serializable_clone(G_OBJECT(unit),
-                           G_OBJECT(gwy_surface_get_si_unit_xy(surface)));
+    gwy_si_unit_assign(gwy_surface_get_si_unit_xy(surface), unit);
 
     unit = gwy_si_unit_new_parse(args.z_units, &power10);
     if (power10) {
@@ -230,8 +229,7 @@ rawxyz_load(const gchar *filename,
             surface->data[k].z *= q;
         gwy_surface_invalidate(surface);
     }
-    gwy_serializable_clone(G_OBJECT(unit),
-                           G_OBJECT(gwy_surface_get_si_unit_z(surface)));
+    gwy_si_unit_assign(gwy_surface_get_si_unit_z(surface), unit);
 
     container = gwy_container_new();
     gwy_container_set_object(container, gwy_app_get_surface_key_for_id(0),
