@@ -1,5 +1,5 @@
 /*
- *  $Id: renishaw.c 20676 2017-12-18 18:19:09Z yeti-dn $
+ *  $Id: renishaw.c 21430 2018-09-11 11:46:00Z yeti-dn $
  *  Copyright (C) 2014 Daniil Bratashov (dn2010), David Necas (Yeti)..
  *  Data structures and constants are copyright (c) 2011 Renishaw plc.
  *
@@ -1014,11 +1014,8 @@ wdf_load(const gchar *filename,
             key = g_strdup_printf("/brick/%d/title", z);
             gwy_container_set_string_by_name(container, key, title);
             g_free(key);
-            dfield = gwy_data_field_new(xres, yres,
-                                        xres, yres,
-                                        TRUE);
-            gwy_brick_mean_plane(brick, dfield, 0, 0, 0,
-                                 xres, yres, -1, FALSE);
+            dfield = gwy_data_field_new(xres, yres, xres, yres, TRUE);
+            gwy_brick_mean_xy_plane(brick, dfield);
             key = g_strdup_printf("/brick/%d/preview", z);
             gwy_container_set_object_by_name(container, key, dfield);
             g_free(key);
