@@ -1,5 +1,5 @@
 /*
- *  $Id: sdfile.c 20676 2017-12-18 18:19:09Z yeti-dn $
+ *  $Id: sdfile.c 21410 2018-09-07 14:15:50Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -154,7 +154,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Surfstand group SDF (Surface Data File) files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.12",
+    "0.13",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -452,8 +452,8 @@ sdfile_export_text(G_GNUC_UNUSED GwyContainer *data,
             ttm->tm_mday, ttm->tm_mon, ttm->tm_year, ttm->tm_hour, ttm->tm_min,
             ttm->tm_mday, ttm->tm_mon, ttm->tm_year, ttm->tm_hour, ttm->tm_min,
             xres, yres,
-            gwy_data_field_get_xmeasure(dfield),
-            gwy_data_field_get_ymeasure(dfield),
+            gwy_data_field_get_dx(dfield),
+            gwy_data_field_get_dy(dfield),
             1.0,
             SDF_FLOAT);
     for (i = 0; i < xres*yres; i++) {
