@@ -1,5 +1,5 @@
 /*
- *  $Id: spmlab.c 20676 2017-12-18 18:19:09Z yeti-dn $
+ *  $Id: spmlab.c 21300 2018-08-08 14:08:32Z yeti-dn $
  *  Copyright (C) 2004-2016 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -510,10 +510,8 @@ read_data_field(SPMLabFile *slfile, const guchar *buffer,
     datablocksize = slfile->xres*slfile->yres*sizeof(guint16);
     dfield = gwy_data_field_new(xres, yres, slfile->xreal, slfile->yreal,
                                 FALSE);
-    gwy_serializable_clone(G_OBJECT(slfile->unitxy),
-                           G_OBJECT(gwy_data_field_get_si_unit_xy(dfield)));
-    gwy_serializable_clone(G_OBJECT(slfile->unitz),
-                           G_OBJECT(gwy_data_field_get_si_unit_z(dfield)));
+    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), slfile->unitxy);
+    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), slfile->unitz);
     gwy_convert_raw_data(p + i*datablocksize, xres*yres, 1,
                          GWY_RAW_DATA_UINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
                          gwy_data_field_get_data(dfield),
