@@ -1,5 +1,5 @@
 /*
- *  $Id: spmxfile.c 19044 2016-10-08 19:10:46Z yeti-dn $
+ *  $Id: spmxfile.c 21300 2018-08-08 14:08:32Z yeti-dn $
  *  Copyright (C) 2014 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -229,10 +229,10 @@ read_binary_data(const SPMXFile *spmxfile,
                                  stream->zscale, stream->zoff);
 
             unit = gwy_data_field_get_si_unit_xy(dfield);
-            gwy_serializable_clone(G_OBJECT(stream->unitxy[0]), G_OBJECT(unit));
+            gwy_si_unit_assign(unit, stream->unitxy[0]);
 
             unit = gwy_data_field_get_si_unit_z(dfield);
-            gwy_serializable_clone(G_OBJECT(stream->unitz), G_OBJECT(unit));
+            gwy_si_unit_assign(unit, stream->unitz);
 
             gwy_container_set_object(container, gwy_app_get_data_key_for_id(id),
                                      dfield);
