@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: tescan.c 19948 2017-06-22 14:05:41Z yeti-dn $
+ *  $Id: tescan.c 20677 2017-12-18 18:22:52Z yeti-dn $
  *  Copyright (C) 2013 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -303,9 +303,7 @@ tsc_get_blocks(GwyTIFF *tiff, const GwyTIFFEntry *entry, GError **error)
             g_warning("The terminating block is not really last.");
 
         if ((end - p) < 6) {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
-                        _("Parameter header is truncated"));
+            err_TRUNCATED_PART(error, "TescanBlock header");
             goto fail;
         }
 
@@ -314,9 +312,7 @@ tsc_get_blocks(GwyTIFF *tiff, const GwyTIFFEntry *entry, GError **error)
         gwy_debug("block of type %u and size %u", block.type, block.size);
         /* FIXME: Emit a better message for block.size < 2? */
         if (block.size > (gulong)(end - p) + 2 || block.size < 2) {
-            g_set_error(error, GWY_MODULE_FILE_ERROR,
-                        GWY_MODULE_FILE_ERROR_DATA,
-                        _("Parameter header is truncated"));
+            err_TRUNCATED_PART(error, "TescanBlock data");
             goto fail;
         }
         if (block.type >= TESCAN_BLOCK_NTYPES)
