@@ -1,5 +1,5 @@
 /*
- *  $Id: tiaser.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: tiaser.c 21430 2018-09-11 11:46:00Z yeti-dn $
  *  Copyright (C) 2012 David Necas (Yeti), Daniil Bratashov (dn2010).
  *  E-mail: yeti@gwyddion.net, dn2010@gmail.com.
  *
@@ -481,10 +481,8 @@ tia_load(const gchar *filename,
         if (brick) {
             gwy_container_set_object_by_name(container,
                                              "/brick/0", brick);
-            dfield = gwy_data_field_new(xres, yres,
-                                        xreal, yreal, FALSE);
-            gwy_brick_mean_plane(brick, dfield, 0, 0, 0,
-                                         xres, yres, -1, FALSE);
+            dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
+            gwy_brick_mean_xy_plane(brick, dfield);
             gwy_container_set_object_by_name(container,
                                              "/brick/0/preview",
                                              dfield);
