@@ -1,5 +1,5 @@
 /*
- *  $Id: wipfile.c 20676 2017-12-18 18:19:09Z yeti-dn $
+ *  $Id: wipfile.c 21430 2018-09-11 11:46:00Z yeti-dn $
  *  Copyright (C) 2010-2011 David Necas (Yeti), Petr Klapetek,
  *  Daniil Bratashov (dn2010)
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, dn2010@gmail.com
@@ -1621,13 +1621,10 @@ wip_read_data(GNode *node, gpointer filedata)
                 g_string_free(caption, TRUE);
                 xres = gwy_brick_get_xres(brick);
                 yres = gwy_brick_get_yres(brick);
-                image = gwy_data_field_new(xres, yres,
-                                           1.0, 1.0, FALSE);
-                gwy_brick_mean_plane(brick, image,
-                                     0, 0, 0,
-                                     xres, yres, -1, FALSE);
+                image = gwy_data_field_new(xres, yres, 1.0, 1.0, FALSE);
+                gwy_brick_mean_xy_plane(brick, image);
                 g_string_printf(key, "/brick/%d/preview",
-                                                filecontent->numbricks);
+                                filecontent->numbricks);
                 gwy_container_set_object_by_name(filecontent->data,
                                                  key->str,
                                                  image);
