@@ -1,5 +1,5 @@
 /*
- *  $Id: xyzexport.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: xyzexport.c 21410 2018-09-07 14:15:50Z yeti-dn $
  *  Copyright (C) 2015-2016 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -385,8 +385,8 @@ xyzexport_export_data_field(GwyDataField *dfield, GwyDataField *mfield,
 
     xres = gwy_data_field_get_xres(dfield);
     yres = gwy_data_field_get_yres(dfield);
-    dx = gwy_data_field_get_xmeasure(dfield);
-    dy = gwy_data_field_get_ymeasure(dfield);
+    dx = gwy_data_field_get_dx(dfield);
+    dy = gwy_data_field_get_dy(dfield);
     xoff = gwy_data_field_get_xoffset(dfield);
     yoff = gwy_data_field_get_yoffset(dfield);
 
