@@ -1,5 +1,5 @@
 /*
- *  $Id: zeisslsm.c 20676 2017-12-18 18:19:09Z yeti-dn $
+ *  $Id: zeisslsm.c 21430 2018-09-11 11:46:00Z yeti-dn $
  *  Copyright (C) 2017 David Necas (Yeti), Daniil Bratashov (dn2010).
  *  E-mail: dn2010@gwyddion.net.
  *
@@ -382,7 +382,7 @@ static GwyModuleInfo module_info = {
     module_register,
     N_("Imports Carl Zeiss CLSM images."),
     "Daniil Bratashov <dn2010@gwyddion.net>",
-    "0.3",
+    "0.4",
     "Daniil Bratashov (dn2010), David Nečas (Yeti)",
     "2017",
 };
@@ -421,12 +421,12 @@ lsm_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
         return 0;
 
     if ((tiff = gwy_tiff_load(fileinfo->name, NULL))
-     && (lsm_tag = gwy_tiff_find_tag(tiff, 0, ZEISS_LSM_HEADER_TAG))) {
+        && (lsm_tag = gwy_tiff_find_tag(tiff, 0, ZEISS_LSM_HEADER_TAG))) {
         score = 100;
     }
 
     if (tiff)
-        g_free(tiff);
+        gwy_tiff_free(tiff);
 
     return score;
 }
@@ -871,12 +871,8 @@ lsm_load_tiff(const GwyTIFF *tiff,
 
             xres = gwy_brick_get_xres(brick);
             yres = gwy_brick_get_yres(brick);
-            preview = gwy_data_field_new(xres, yres,
-                                         1.0, 1.0, FALSE);
-            gwy_brick_mean_plane(brick, preview,
-                                 0, 0, 0,
-                                 xres, yres, -1,
-                                 FALSE);
+            preview = gwy_data_field_new(xres, yres, 1.0, 1.0, FALSE);
+            gwy_brick_mean_xy_plane(brick, preview);
             key = g_strdup_printf("/brick/%d/preview", i + 1);
             gwy_container_set_object_by_name(container,
                                              key,
