@@ -1,6 +1,6 @@
 /*
- *  $Id: acf2d.c 20745 2018-01-21 17:31:21Z yeti-dn $
- *  Copyright (C) 2007-2017 David Necas (Yeti).
+ *  $Id: acf2d.c 21410 2018-09-07 14:15:50Z yeti-dn $
+ *  Copyright (C) 2007-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -65,6 +65,8 @@ typedef struct {
     LevellingType level;
     gdouble threshold;
     ZoomType zoom;
+    gboolean create_image;
+    gboolean zoomed_image;
     /* Graphing */
     gboolean separate;
     gboolean fixres;
@@ -93,6 +95,8 @@ typedef struct {
     GSList *zoom;
     GtkObject *threshold;
     GtkWidget *colour_button;
+    GtkWidget *create_image;
+    GtkWidget *zoomed_image;
     GtkWidget *guivalues[G_N_ELEMENTS(guivalues)];
     GtkWidget *rexport;
     /* Graphing */
@@ -108,73 +112,80 @@ typedef struct {
     GtkWidget *target_graph;
 } ACFControls;
 
-static gboolean module_register         (void);
-static void     acf2d                   (GwyContainer *data,
-                                         GwyRunType run);
-static void     acf2d_dialogue          (ACFArgs *args,
-                                         GwyContainer *data,
-                                         GwyDataField *dfield,
-                                         GwyDataField *mask,
-                                         gint id);
-static void     masking_changed         (GtkToggleButton *toggle,
-                                         ACFControls *controls);
-static void     levelling_changed       (GtkToggleButton *toggle,
-                                         ACFControls *controls);
-static void     threshold_changed       (ACFControls *controls,
-                                         GtkAdjustment *adjustment);
-static void     calculate_zoomed_fields (ACFControls *controls);
-static void     recalculate_acf         (ACFControls *controls);
-static void     update_parameters       (ACFControls *controls);
-static void     fixres_changed          (ACFControls *controls,
-                                         GtkToggleButton *check);
-static void     zoom_changed            (GtkRadioButton *button,
-                                         ACFControls *controls);
-static void     resolution_changed      (ACFControls *controls,
-                                         GtkAdjustment *adj);
-static void     thickness_changed       (ACFControls *controls,
-                                         GtkAdjustment *adj);
-static void     interpolation_changed   (GtkComboBox *combo,
-                                         ACFControls *controls);
-static void     separate_changed        (ACFControls *controls);
-static void     selection_changed       (ACFControls *controls,
-                                         gint hint);
-static void     update_target_graphs    (ACFControls *controls);
-static gboolean filter_target_graphs    (GwyContainer *data,
-                                         gint id,
-                                         gpointer user_data);
-static void     target_graph_changed    (ACFControls *controls);
-static void     report_style_changed    (ACFControls *controls,
-                                         GwyResultsExport *rexport);
-static void     update_curve            (ACFControls *controls,
-                                         gint i);
-static void     init_graph_model_units  (GwyGraphModel *gmodel,
-                                         GwyDataField *psdffield);
-static void     calculate_acf           (GwyDataField *dfield,
-                                         GwyDataField *mask,
-                                         const ACFArgs *args,
-                                         GwyDataField *acf,
-                                         GwyDataField *acfmask);
-static void     create_acf_mask         (GwyDataField *acf,
-                                         GwyDataField *acfmask,
-                                         const ACFArgs *args);
-static void     create_output_graphs    (ACFControls *controls,
-                                         GwyContainer *data);
-static gint     create_results          (GwyContainer *data,
-                                         GwyDataField *acf,
-                                         GwyDataField *acfmask,
-                                         gint oldid);
-static void     acf2d_load_args         (GwyContainer *container,
-                                         ACFArgs *args);
-static void     acf2d_save_args         (GwyContainer *container,
-                                         ACFArgs *args);
-static void     acf2d_sanitize_args     (ACFArgs *args);
-static void     calculate_acf_statistics(GwyDataField *acfmask,
-                                         GwyResults *results);
+static gboolean      module_register         (void);
+static void          acf2d                   (GwyContainer *data,
+                                              GwyRunType run);
+static void          acf2d_dialogue          (ACFArgs *args,
+                                              GwyContainer *data,
+                                              GwyDataField *dfield,
+                                              GwyDataField *mask,
+                                              gint id);
+static void          masking_changed         (GtkToggleButton *toggle,
+                                              ACFControls *controls);
+static void          levelling_changed       (GtkToggleButton *toggle,
+                                              ACFControls *controls);
+static void          threshold_changed       (ACFControls *controls,
+                                              GtkAdjustment *adjustment);
+static void          calculate_zoomed_fields (ACFControls *controls);
+static GwyDataField* cut_field_to_zoom       (GwyDataField *dfield,
+                                              gint zoom);
+static void          recalculate_acf         (ACFControls *controls);
+static void          update_parameters       (ACFControls *controls);
+static void          fixres_changed          (ACFControls *controls,
+                                              GtkToggleButton *check);
+static void          zoom_changed            (GtkRadioButton *button,
+                                              ACFControls *controls);
+static void          resolution_changed      (ACFControls *controls,
+                                              GtkAdjustment *adj);
+static void          thickness_changed       (ACFControls *controls,
+                                              GtkAdjustment *adj);
+static void          interpolation_changed   (GtkComboBox *combo,
+                                              ACFControls *controls);
+static void          separate_changed        (ACFControls *controls);
+static void          create_image_changed    (ACFControls *controls);
+static void          zoomed_image_changed    (ACFControls *controls);
+static void          selection_changed       (ACFControls *controls,
+                                              gint hint);
+static void          update_sensitivity      (ACFControls *controls);
+static void          update_target_graphs    (ACFControls *controls);
+static gboolean      filter_target_graphs    (GwyContainer *data,
+                                              gint id,
+                                              gpointer user_data);
+static void          target_graph_changed    (ACFControls *controls);
+static void          report_style_changed    (ACFControls *controls,
+                                              GwyResultsExport *rexport);
+static void          update_curve            (ACFControls *controls,
+                                              gint i);
+static void          init_graph_model_units  (GwyGraphModel *gmodel,
+                                              GwyDataField *psdffield);
+static void          calculate_acf           (GwyDataField *dfield,
+                                              GwyDataField *mask,
+                                              const ACFArgs *args,
+                                              GwyDataField *acf,
+                                              GwyDataField *acfmask);
+static void          create_acf_mask         (GwyDataField *acf,
+                                              GwyDataField *acfmask,
+                                              const ACFArgs *args);
+static void          create_output_graphs    (ACFControls *controls,
+                                              GwyContainer *data);
+static gint          create_results          (GwyContainer *data,
+                                              GwyDataField *acf,
+                                              GwyDataField *acfmask,
+                                              GwySelection *selection,
+                                              gint oldid,
+                                              gint zoom);
+static void          acf2d_load_args         (GwyContainer *container,
+                                              ACFArgs *args);
+static void          acf2d_save_args         (GwyContainer *container,
+                                              ACFArgs *args);
+static void          acf2d_sanitize_args     (ACFArgs *args);
+static void          calculate_acf_statistics(GwyDataField *acfmask,
+                                              GwyResults *results);
 
 static const ACFArgs acf2d_defaults = {
     GWY_RESULTS_REPORT_COLON,
     GWY_MASK_IGNORE, LEVELLING_MEAN_VALUE, 0.2,
-    ZOOM_1,
+    ZOOM_1, TRUE, TRUE,
     FALSE, FALSE, 120, 1,
     GWY_INTERPOLATION_LINEAR,
     GWY_APP_DATA_ID_NONE,
@@ -187,7 +198,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Calculates and analyzes two-dimensional autocorrelation function."),
     "Yeti <yeti@gwyddion.net>",
-    "2.0",
+    "2.2",
     "David Nečas (Yeti)",
     "2007",
 };
@@ -224,15 +235,20 @@ acf2d(GwyContainer *data, GwyRunType run)
     g_return_if_fail(dfield);
 
     if (run == GWY_RUN_IMMEDIATE) {
-        acf = gwy_data_field_new(1, 1, 1.0, 1.0, FALSE);
-        acfmask = gwy_data_field_new(1, 1, 1.0, 1.0, FALSE);
-        calculate_acf(dfield, mask, &args, acf, acfmask);
-        newid = create_results(data, acf, acfmask, id);
-        gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                GWY_DATA_ITEM_MASK_COLOR,
-                                0);
-        g_object_unref(acf);
-        g_object_unref(acfmask);
+        /* Is it reasonable to simply do nothing when settings say to not
+         * create the ACF image? */
+        if (args.create_image) {
+            acf = gwy_data_field_new(1, 1, 1.0, 1.0, FALSE);
+            acfmask = gwy_data_field_new(1, 1, 1.0, 1.0, FALSE);
+            calculate_acf(dfield, mask, &args, acf, acfmask);
+            newid = create_results(data, acf, acfmask, NULL, id,
+                                   args.zoomed_image ? args.zoom : ZOOM_1);
+            gwy_app_sync_data_items(data, data, id, newid, FALSE,
+                                    GWY_DATA_ITEM_MASK_COLOR,
+                                    0);
+            g_object_unref(acf);
+            g_object_unref(acfmask);
+        }
     }
     else {
         acf2d_dialogue(&args, data, dfield, mask, id);
@@ -354,7 +370,7 @@ acf2d_dialogue(ACFArgs *args,
     gtk_box_pack_start(GTK_BOX(vbox), notebook, FALSE, FALSE, 0);
 
     /***** ACF controls *****/
-    table = GTK_TABLE(gtk_table_new(6 + 4*(!!mask), 3, FALSE));
+    table = GTK_TABLE(gtk_table_new(8 + 4*(!!mask), 3, FALSE));
     gtk_table_set_row_spacings(table, 2);
     gtk_table_set_col_spacings(table, 6);
     gtk_container_set_border_width(GTK_CONTAINER(table), 4);
@@ -398,7 +414,28 @@ acf2d_dialogue(ACFArgs *args,
     row = gwy_radio_buttons_attach_to_table(controls.level, table, 2, row);
 
     gtk_table_set_row_spacing(table, row-1, 8);
+    controls.create_image
+        = gtk_check_button_new_with_mnemonic(_("Create ACF image"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls.create_image),
+                                 args->create_image);
+    gtk_table_attach(table, controls.create_image,
+                     0, 2, row, row+1, GTK_FILL, 0, 0, 0);
+    g_signal_connect_swapped(controls.create_image, "toggled",
+                             G_CALLBACK(create_image_changed), &controls);
+    row++;
+
+    controls.zoomed_image
+        = gtk_check_button_new_with_mnemonic(_("Only zoomed part"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls.zoomed_image),
+                                 args->zoomed_image);
+    gtk_table_attach(table, controls.zoomed_image,
+                     0, 2, row, row+1, GTK_FILL, 0, 0, 0);
+    g_signal_connect_swapped(controls.zoomed_image, "toggled",
+                             G_CALLBACK(zoomed_image_changed), &controls);
+    row++;
+
     if (mask) {
+        gtk_table_set_row_spacing(table, row-1, 8);
         gtk_table_attach(table,
                          gwy_label_new_header(_("Masking Mode")),
                          0, 2, row, row+1, GTK_FILL, 0, 0, 0);
@@ -410,7 +447,6 @@ acf2d_dialogue(ACFArgs *args,
                                        &controls, args->masking);
         row = gwy_radio_buttons_attach_to_table(controls.masking, table,
                                                 2, row);
-        gtk_table_set_row_spacing(table, row-1, 8);
     }
     else
         controls.masking = NULL;
@@ -534,6 +570,7 @@ acf2d_dialogue(ACFArgs *args,
                              G_CALLBACK(report_style_changed), &controls);
 
     update_parameters(&controls);
+    update_sensitivity(&controls);
 
     gtk_widget_show_all(controls.dialogue);
     do {
@@ -559,10 +596,14 @@ acf2d_dialogue(ACFArgs *args,
         }
     } while (response != GTK_RESPONSE_OK);
 
-    newid = create_results(data, controls.acf, controls.acfmask, id);
-    gwy_app_sync_data_items(controls.mydata, data, 0, newid, FALSE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            0);
+    if (args->create_image) {
+        newid = create_results(data, controls.acf, controls.acfmask,
+                               controls.selection, id,
+                               args->zoomed_image ? args->zoom : ZOOM_1);
+        gwy_app_sync_data_items(controls.mydata, data, 0, newid, FALSE,
+                                GWY_DATA_ITEM_MASK_COLOR,
+                                0);
+    }
     create_output_graphs(&controls, data);
     gtk_widget_destroy(controls.dialogue);
 finalize:
@@ -570,6 +611,7 @@ finalize:
     g_object_unref(controls.acf);
     g_object_unref(controls.acfmask);
     g_object_unref(controls.results);
+    g_object_unref(controls.line);
 }
 
 static void
@@ -605,50 +647,47 @@ threshold_changed(ACFControls *controls, GtkAdjustment *adjustment)
 static void
 calculate_zoomed_fields(ACFControls *controls)
 {
-    ACFArgs *args = controls->args;
-    ZoomType zoom = args->zoom;
-    GwyDataField *zoomed, *zoomedmask;
-    guint xres, yres, width, height;
+    ZoomType zoom = controls->args->zoom;
+    GwyDataField *zoomed;
 
-    zoomed = controls->acf;
-    zoomedmask = controls->acfmask;
-    xres = gwy_data_field_get_xres(zoomed);
-    yres = gwy_data_field_get_yres(zoomed);
-
-    if (zoom == ZOOM_1) {
-        g_object_ref(zoomed);
-        g_object_ref(zoomedmask);
-    }
-    else {
-        width = (xres/zoom) | 1;
-        height = (yres/zoom) | 1;
-
-        if (width < 17)
-            width = MAX(width, MIN(17, xres));
-
-        if (height < 17)
-            height = MAX(height, MIN(17, yres));
-
-        zoomed = gwy_data_field_area_extract(zoomed,
-                                             (xres - width)/2,
-                                             (yres - height)/2,
-                                             width, height);
-        gwy_data_field_set_xoffset(zoomed, -0.5*zoomed->xreal);
-        gwy_data_field_set_yoffset(zoomed, -0.5*zoomed->yreal);
-
-        zoomedmask = gwy_data_field_area_extract(zoomedmask,
-                                                 (xres - width)/2,
-                                                 (yres - height)/2,
-                                                 width, height);
-        gwy_data_field_set_xoffset(zoomedmask, -0.5*zoomedmask->xreal);
-        gwy_data_field_set_yoffset(zoomedmask, -0.5*zoomedmask->yreal);
-    }
+    zoomed = cut_field_to_zoom(controls->acf, zoom);
     gwy_container_set_object(controls->mydata,
                              gwy_app_get_data_key_for_id(0), zoomed);
-    gwy_container_set_object(controls->mydata,
-                             gwy_app_get_mask_key_for_id(0), zoomedmask);
+    gwy_data_field_data_changed(zoomed);
     g_object_unref(zoomed);
-    g_object_unref(zoomedmask);
+
+    zoomed = cut_field_to_zoom(controls->acfmask, zoom);
+    gwy_container_set_object(controls->mydata,
+                             gwy_app_get_mask_key_for_id(0), zoomed);
+    gwy_data_field_data_changed(zoomed);
+    g_object_unref(zoomed);
+}
+
+static GwyDataField*
+cut_field_to_zoom(GwyDataField *dfield, gint zoom)
+{
+    GwyDataField *zoomed;
+    guint xres, yres, width, height;
+
+    xres = gwy_data_field_get_xres(dfield);
+    yres = gwy_data_field_get_yres(dfield);
+    width = (xres/zoom) | 1;
+    height = (yres/zoom) | 1;
+    if (width < 17)
+        width = MAX(width, MIN(17, xres));
+    if (height < 17)
+        height = MAX(height, MIN(17, yres));
+
+    if (width >= xres && height >= yres)
+        return g_object_ref(dfield);
+
+    zoomed = gwy_data_field_area_extract(dfield,
+                                         (xres - width)/2, (yres - height)/2,
+                                         width, height);
+    gwy_data_field_set_xoffset(zoomed, -0.5*gwy_data_field_get_xreal(zoomed));
+    gwy_data_field_set_yoffset(zoomed, -0.5*gwy_data_field_get_yreal(zoomed));
+
+    return zoomed;
 }
 
 static void
@@ -656,8 +695,6 @@ recalculate_acf(ACFControls *controls)
 {
     calculate_acf(controls->dfield, controls->mask, controls->args,
                   controls->acf, controls->acfmask);
-    gwy_data_field_data_changed(controls->acf);
-    gwy_data_field_data_changed(controls->acfmask);
     calculate_zoomed_fields(controls);
     selection_changed(controls, -1);
     update_parameters(controls);
@@ -745,6 +782,23 @@ separate_changed(ACFControls *controls)
     update_target_graphs(controls);
 }
 
+static void
+create_image_changed(ACFControls *controls)
+{
+    ACFArgs *args = controls->args;
+    args->create_image
+        = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(controls->create_image));
+    update_sensitivity(controls);
+}
+
+static void
+zoomed_image_changed(ACFControls *controls)
+{
+    ACFArgs *args = controls->args;
+    args->zoomed_image
+        = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(controls->zoomed_image));
+}
+
 static void
 selection_changed(ACFControls *controls, gint hint)
 {
@@ -758,6 +812,21 @@ selection_changed(ACFControls *controls, gint hint)
     }
     else
         update_curve(controls, hint);
+
+    update_sensitivity(controls);
+}
+
+static void
+update_sensitivity(ACFControls *controls)
+{
+    ACFArgs *args = controls->args;
+    gint n;
+
+    n = gwy_graph_model_get_n_curves(controls->gmodel);
+    gtk_widget_set_sensitive(controls->zoomed_image, args->create_image);
+    gtk_dialog_set_response_sensitive(GTK_DIALOG(controls->dialogue),
+                                      GTK_RESPONSE_OK,
+                                      args->create_image || n);
 }
 
 static void
@@ -801,7 +870,7 @@ update_curve(ACFControls *controls, gint i)
 {
     GwyGraphCurveModel *gcmodel;
     GwyDataField *acf;
-    gdouble xy[4], xofffull, xoffzoom, yofffull, yoffzoom;
+    gdouble xy[4], xofffull, yofffull;
     gint xl0, yl0, xl1, yl1;
     gint n, lineres;
     gchar *desc;
@@ -812,16 +881,14 @@ update_curve(ACFControls *controls, gint i)
 
     acf = gwy_container_get_object(controls->mydata,
                                    gwy_app_get_data_key_for_id(0));
-    xoffzoom = gwy_data_field_get_xoffset(acf);
-    yoffzoom = gwy_data_field_get_yoffset(acf);
-    xy[0] += xoffzoom;
-    xy[1] += yoffzoom;
-
-    xofffull = gwy_data_field_get_xoffset(controls->acf);
-    yofffull = gwy_data_field_get_yoffset(controls->acf);
+    xy[0] += gwy_data_field_get_xoffset(acf);
+    xy[1] += gwy_data_field_get_yoffset(acf);
 
     xl0 = gwy_data_field_get_xres(controls->acf)/2;
     yl0 = gwy_data_field_get_yres(controls->acf)/2;
+
+    xofffull = gwy_data_field_get_xoffset(controls->acf);
+    yofffull = gwy_data_field_get_yoffset(controls->acf);
     xl1 = floor(gwy_data_field_rtoj(controls->acf, xy[0] - xofffull));
     yl1 = floor(gwy_data_field_rtoi(controls->acf, xy[1] - yofffull));
 
@@ -875,18 +942,67 @@ init_graph_model_units(GwyGraphModel *gmodel,
     g_object_unref(unit);
 }
 
+/* Convert points to lines from the origin, which is assumed to be in the
+ * centre. */
+static void
+add_line_selection_from_points(GwyContainer *data,
+                               GwyDataField *dfield,
+                               gint id,
+                               GwySelection *pointsel)
+{
+    GwySelection *linesel;
+    gint nsel, i;
+    gdouble *seldata;
+    gdouble xreal, yreal;
+    gchar *key;
+
+    if (!pointsel || !(nsel = gwy_selection_get_data(pointsel, NULL)))
+        return;
+
+    linesel = g_object_new(g_type_from_name("GwySelectionLine"), NULL);
+    g_return_if_fail(linesel);
+    gwy_selection_set_max_objects(linesel, 1024);
+    seldata = g_new(gdouble, 4*nsel);
+    xreal = gwy_data_field_get_xreal(dfield);
+    yreal = gwy_data_field_get_yreal(dfield);
+
+    for (i = 0; i < nsel; i++) {
+        seldata[4*i + 0] = 0.5*xreal;
+        seldata[4*i + 1] = 0.5*yreal;
+        gwy_selection_get_object(pointsel, i, seldata + 4*i + 2);
+    }
+
+    gwy_selection_set_data(linesel, nsel, seldata);
+    g_free(seldata);
+
+    key = g_strdup_printf("/%d/select/line", id);
+    gwy_container_set_object_by_name(data, key, linesel);
+    g_object_unref(linesel);
+}
+
 static gint
 create_results(GwyContainer *data,
                GwyDataField *acf, GwyDataField *acfmask,
-               gint oldid)
+               GwySelection *selection,
+               gint oldid, gint zoom)
 {
+    GwyDataField *zoomed, *zoomedmask;
     gint newid;
 
-    newid = gwy_app_data_browser_add_data_field(acf, data, TRUE);
-    gwy_container_set_object(data, gwy_app_get_mask_key_for_id(newid), acfmask);
+    zoomed = cut_field_to_zoom(acf, zoom);
+    newid = gwy_app_data_browser_add_data_field(zoomed, data, TRUE);
+
+    zoomedmask = cut_field_to_zoom(acfmask, zoom);
+    gwy_container_set_object(data, gwy_app_get_mask_key_for_id(newid),
+                             zoomedmask);
+
+    add_line_selection_from_points(data, zoomed, newid, selection);
     gwy_app_set_data_field_title(data, newid, _("2D ACF"));
     gwy_app_channel_log_add_proc(data, oldid, newid);
 
+    g_object_unref(zoomed);
+    g_object_unref(zoomedmask);
+
     return newid;
 }
 
@@ -998,8 +1114,8 @@ find_decays(GwyDataField *mask,
     const gdouble *data = gwy_data_field_get_data_const(mask);
     guint xres = gwy_data_field_get_xres(mask);
     guint yres = gwy_data_field_get_yres(mask);
-    gdouble dx = gwy_data_field_get_xmeasure(mask);
-    gdouble dy = gwy_data_field_get_ymeasure(mask);
+    gdouble dx = gwy_data_field_get_dx(mask);
+    gdouble dy = gwy_data_field_get_dy(mask);
     gdouble phimin = 0.0, r2min = G_MAXDOUBLE;
     gdouble phimax = 0.0, r2max = 0.0;
     gdouble xc, yc;
@@ -1067,6 +1183,7 @@ calculate_acf_statistics(GwyDataField *acfmask, GwyResults *results)
                             NULL);
 }
 
+static const gchar create_image_key[]  = "/module/acf2d/create_image";
 static const gchar fixres_key[]        = "/module/acf2d/fixres";
 static const gchar interpolation_key[] = "/module/acf2d/interpolation";
 static const gchar level_key[]         = "/module/acf2d/level";
@@ -1077,6 +1194,7 @@ static const gchar separate_key[]      = "/module/acf2d/separate";
 static const gchar thickness_key[]     = "/module/acf2d/thickness";
 static const gchar threshold_key[]     = "/module/acf2d/threshold";
 static const gchar zoom_key[]          = "/module/acf2d/zoom";
+static const gchar zoomed_image_key[]  = "/module/acf2d/zoomed_image";
 
 static void
 acf2d_sanitize_args(ACFArgs *args)
@@ -1085,6 +1203,8 @@ acf2d_sanitize_args(ACFArgs *args)
                                             GWY_TYPE_MASKING_TYPE);
     args->level = CLAMP(args->level, 0, LEVELLING_NTYPES-1);
     args->threshold = CLAMP(args->threshold, 0.0, 1.0);
+    args->create_image = !!args->create_image;
+    args->zoomed_image = !!args->zoomed_image;
     args->separate = !!args->separate;
     args->fixres = !!args->fixres;
     args->resolution = CLAMP(args->resolution, MIN_RESOLUTION, MAX_RESOLUTION);
@@ -1107,6 +1227,10 @@ acf2d_load_args(GwyContainer *container, ACFArgs *args)
     gwy_container_gis_enum_by_name(container, level_key, &args->level);
     gwy_container_gis_double_by_name(container, threshold_key,
                                      &args->threshold);
+    gwy_container_gis_boolean_by_name(container, create_image_key,
+                                      &args->create_image);
+    gwy_container_gis_boolean_by_name(container, zoomed_image_key,
+                                      &args->zoomed_image);
     gwy_container_gis_boolean_by_name(container, separate_key, &args->separate);
     gwy_container_gis_boolean_by_name(container, fixres_key, &args->fixres);
     gwy_container_gis_int32_by_name(container, resolution_key,
@@ -1127,6 +1251,10 @@ acf2d_save_args(GwyContainer *container, ACFArgs *args)
     gwy_container_set_enum_by_name(container, masking_key, args->masking);
     gwy_container_set_enum_by_name(container, level_key, args->level);
     gwy_container_set_double_by_name(container, threshold_key, args->threshold);
+    gwy_container_set_boolean_by_name(container, create_image_key,
+                                      args->create_image);
+    gwy_container_set_boolean_by_name(container, zoomed_image_key,
+                                      args->zoomed_image);
     gwy_container_set_boolean_by_name(container, separate_key, args->separate);
     gwy_container_set_boolean_by_name(container, fixres_key, args->fixres);
     gwy_container_set_int32_by_name(container, resolution_key,
