@@ -1,5 +1,5 @@
 /*
- *  $Id: arithmetic.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: arithmetic.c 21410 2018-09-07 14:15:50Z yeti-dn $
  *  Copyright (C) 2003-2013 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -21,7 +21,6 @@
 
 #include "config.h"
 #include <string.h>
-#include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
@@ -40,7 +39,7 @@
 
 enum {
     NARGS = 8,
-    HISTSIZE = 25,
+    HISTSIZE = 96,
     USER_UNITS_ID = G_MAXINT,
 };
 
@@ -58,7 +57,13 @@ enum {
 };
 
 enum {
-    ARITHMETIC_NARGS = NARGS * ARITHMETIC_NVARS + 2  /* 2 for x and y */
+    COMMON_COORD_X = 0,
+    COMMON_COORD_Y = 1,
+    COMMON_COORD_NCOORDS
+};
+
+enum {
+    ARITHMETIC_NARGS = NARGS * ARITHMETIC_NVARS + COMMON_COORD_NCOORDS
 };
 
 enum {
@@ -620,7 +625,6 @@ arithmetic_do(ArithmeticArgs *args, gint *id)
     /* We know the expression can't contain more variables */
     data_fields = g_new0(GwyDataField*, ARITHMETIC_NARGS);
     d = g_new0(const gdouble*, ARITHMETIC_NARGS + 1);
-    d[0] = NULL;
 
     /* First get all the data fields we directly have */
     for (i = 0; i < NARGS; i++) {
@@ -686,14 +690,14 @@ arithmetic_do(ArithmeticArgs *args, gint *id)
         d[args->pos[i]] = gwy_data_field_get_data_const(dfield);
     }
 
-    i = NARGS*ARITHMETIC_NVARS + 0;
+    i = NARGS*ARITHMETIC_NVARS + COMMON_COORD_X;
     if (args->pos[i]) {
         dfield = make_x(data_fields[0]);
         data_fields[i] = dfield;
         d[args->pos[i]] = gwy_data_field_get_data_const(dfield);
     }
 
-    i = NARGS*ARITHMETIC_NVARS + 1;
+    i = NARGS*ARITHMETIC_NVARS + COMMON_COORD_Y;
     if (args->pos[i]) {
         dfield = make_y(data_fields[0]);
         data_fields[i] = dfield;
@@ -716,7 +720,7 @@ arithmetic_do(ArithmeticArgs *args, gint *id)
             dfield = GWY_DATA_FIELD(gwy_container_get_object(data, quark));
         }
         unit2 = gwy_data_field_get_si_unit_z(dfield);
-        gwy_serializable_clone(G_OBJECT(unit2), G_OBJECT(unit));
+        gwy_si_unit_assign(unit, unit2);
     }
 
     /* Free stuff */
@@ -746,9 +750,12 @@ arithmetic_need_data(const ArithmeticArgs *args,
 
     // When x and y are needed, always take them from field 1.  This also
     // ensures the expression is considered to be a field expression.
-    if (args->pos[NARGS*ARITHMETIC_NVARS + 0]
-        || args->pos[NARGS*ARITHMETIC_NVARS + 1])
-        need_data[0] = TRUE;
+    for (i = NARGS*ARITHMETIC_NVARS; i < ARITHMETIC_NARGS; i++) {
+        if (args->pos[i]) {
+            need_data[0] = TRUE;
+            break;
+        }
+    }
 }
 
 static void
@@ -785,7 +792,7 @@ make_x(GwyDataField *dfield)
     result = gwy_data_field_new_alike(dfield, FALSE);
     xres = gwy_data_field_get_xres(dfield);
     yres = gwy_data_field_get_yres(dfield);
-    dx = gwy_data_field_get_xmeasure(dfield);
+    dx = gwy_data_field_get_dx(dfield);
     xoff = gwy_data_field_get_xoffset(dfield);
     data = gwy_data_field_get_data(result);
 
@@ -809,7 +816,7 @@ make_y(GwyDataField *dfield)
     result = gwy_data_field_new_alike(dfield, FALSE);
     xres = gwy_data_field_get_xres(dfield);
     yres = gwy_data_field_get_yres(dfield);
-    dy = gwy_data_field_get_ymeasure(dfield);
+    dy = gwy_data_field_get_dy(dfield);
     yoff = gwy_data_field_get_yoffset(dfield);
     data = gwy_data_field_get_data(result);
 
@@ -934,7 +941,7 @@ arithmetic_load_args(GwyContainer *settings,
 {
     GtkListStore *store;
     const guchar *str;
-    gchar *filename, *buffer, *line, *p;
+    gchar *buffer, *line, *p;
     gsize size;
     guint i;
 
@@ -952,9 +959,7 @@ arithmetic_load_args(GwyContainer *settings,
     store = gtk_list_store_new(1, G_TYPE_STRING);
     args->history = GTK_TREE_MODEL(store);
 
-    filename = g_build_filename(gwy_get_user_dir(), "arithmetic", "history",
-                                NULL);
-    if (g_file_get_contents(filename, &buffer, &size, NULL)) {
+    if (gwy_module_data_load("arithmetic", "history", &buffer, &size, NULL)) {
         p = buffer;
         for (line = gwy_str_next_line(&p); line; line = gwy_str_next_line(&p)) {
             GtkTreeIter iter;
@@ -967,7 +972,6 @@ arithmetic_load_args(GwyContainer *settings,
         }
         g_free(buffer);
     }
-    g_free(filename);
 
     for (i = 1; i < NARGS; i++) {
         args->objects[i] = object_ids[i];
@@ -984,7 +988,9 @@ static void
 arithmetic_save_args(GwyContainer *settings,
                      ArithmeticArgs *args)
 {
-    gchar *filename;
+    GtkTreeIter iter;
+    guint i = 0;
+    gchar *s;
     FILE *fh;
 
     gwy_assign(object_ids, args->objects, NARGS);
@@ -996,30 +1002,19 @@ arithmetic_save_args(GwyContainer *settings,
     gwy_container_set_int32_by_name(settings, dataunits_key,
                                      args->dataunits);
 
-    filename = g_build_filename(gwy_get_user_dir(), "arithmetic", NULL);
-    if (!g_file_test(filename, G_FILE_TEST_IS_DIR))
-        g_mkdir(filename, 0700);
-    g_free(filename);
+    if (!(fh = gwy_module_data_fopen("arithmetic", "history", "w", NULL)))
+        return;
 
-    filename = g_build_filename(gwy_get_user_dir(), "arithmetic", "history",
-                                NULL);
-    if ((fh = gwy_fopen(filename, "w"))) {
-        GtkTreeIter iter;
-        guint i = 0;
-        gchar *s;
-
-        if (gtk_tree_model_get_iter_first(args->history, &iter)) {
-            do {
-                gtk_tree_model_get(args->history, &iter, 0, &s, -1);
-                fputs(s, fh);
-                fputc('\n', fh);
-                g_free(s);
-            } while (++i < HISTSIZE
-                     && gtk_tree_model_iter_next(args->history, &iter));
-        }
-        fclose(fh);
+    if (gtk_tree_model_get_iter_first(args->history, &iter)) {
+        do {
+            gtk_tree_model_get(args->history, &iter, 0, &s, -1);
+            fputs(s, fh);
+            fputc('\n', fh);
+            g_free(s);
+        } while (++i < HISTSIZE
+                 && gtk_tree_model_iter_next(args->history, &iter));
     }
-    g_free(filename);
+    fclose(fh);
 }
 
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
