@@ -1,6 +1,6 @@
 /*
- *  $Id: bdep_synth.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2015 David Necas (Yeti).
+ *  $Id: bdep_synth.c 21408 2018-09-07 13:53:53Z yeti-dn $
+ *  Copyright (C) 2015-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -46,9 +46,9 @@ enum {
 };
 
 typedef enum {
-    GRAPH_MEAN   = 0,
-    GRAPH_RMS    = 1,
-    GRAPH_NFLAGS,
+    GRAPH_MEAN   = (1 << 0),
+    GRAPH_RMS    = (1 << 1),
+    GRAPH_ALL    = (1 << 2) - 1,
 } GraphFlags;
 
 typedef struct _BDepSynthControls BDepSynthControls;
@@ -61,7 +61,7 @@ typedef struct {
     gdouble coverage;
     gdouble height;
     gdouble height_noise;
-    gboolean graph_flags[GRAPH_NFLAGS];
+    guint graph_flags;
 } BDepSynthArgs;
 
 struct _BDepSynthControls {
@@ -79,7 +79,7 @@ struct _BDepSynthControls {
     GtkObject *height_noise;
     GtkWidget *height_units;
     GtkWidget *height_init;
-    GtkWidget *graph_flags[GRAPH_NFLAGS];
+    GSList *graph_flags;
     GwyContainer *mydata;
     GwyDataField *surface;
     gboolean in_init;
@@ -127,9 +127,13 @@ static void               bdep_synth_save_args     (GwyContainer *container,
 
 #include "synth.h"
 
-static const gchar* graph_flags[GRAPH_NFLAGS + 1/*unused*/] = {
-    N_("Mean value"),
-    N_("RMS"),
+static const GwyEnum graph_flags[] = {
+    { N_("Mean value"), GRAPH_MEAN },
+    { N_("RMS"),        GRAPH_RMS  },
+};
+
+enum {
+    GRAPH_NFLAGS = G_N_ELEMENTS(graph_flags)
 };
 
 static const BDepSynthArgs bdep_synth_defaults = {
@@ -137,7 +141,7 @@ static const BDepSynthArgs bdep_synth_defaults = {
     42, TRUE, TRUE,
     10.0,
     1.0, 0.0,
-    { FALSE, FALSE, },
+    0,
 };
 
 static const GwyDimensionArgs dims_defaults = GWY_DIMENSION_ARGS_INIT;
@@ -147,7 +151,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Generates surfaces by ballistic deposition."),
     "Yeti <yeti@gwyddion.net>",
-    "1.2",
+    "1.3",
     "David Nečas (Yeti)",
     "2015",
 };
@@ -277,7 +281,6 @@ run_noninteractive(BDepSynthArgs *args,
 
     for (i = 0; i < GRAPH_NFLAGS; i++) {
         GwyGraphModel *gmodel;
-        GwySIUnit *unit;
         gchar *s, *title;
 
         if (!gcmodels[i])
@@ -288,28 +291,25 @@ run_noninteractive(BDepSynthArgs *args,
         g_object_unref(gcmodels[i]);
 
         s = gwy_app_get_data_field_title(data, newid);
-        title = g_strdup_printf("%s (%s)", graph_flags[i], s);
+        title = g_strdup_printf("%s (%s)", _(graph_flags[i].name), s);
         g_free(s);
         g_object_set(gmodel,
                      "title", title,
                      "x-logarithmic", TRUE,
                      "y-logarithmic", TRUE,
                      "axis-label-bottom", _("Mean deposited thickness"),
-                     "axis-label-left", _(graph_flags[i]),
+                     "axis-label-left", _(graph_flags[i].name),
                      NULL);
         g_free(title);
 
-        unit = gwy_data_field_get_si_unit_z(newfield);
-        unit = gwy_si_unit_duplicate(unit);
-        g_object_set(gmodel, "si-unit-x", unit, NULL);
-        g_object_unref(unit);
-
-        if (i == GRAPH_MEAN || i == GRAPH_RMS) {
-            unit = gwy_si_unit_multiply(gwy_data_field_get_si_unit_z(newfield),
-                                        gwy_data_field_get_si_unit_xy(newfield),
-                                        NULL);
-            g_object_set(gmodel, "si-unit-y", unit, NULL);
-            g_object_unref(unit);
+        if (graph_flags[i].value == GRAPH_MEAN
+            || graph_flags[i].value == GRAPH_RMS) {
+            gwy_graph_model_set_units_from_data_field(gmodel, newfield,
+                                                      0, 1, 0, 1);
+        }
+        else {
+            gwy_graph_model_set_units_from_data_field(gmodel, newfield,
+                                                      0, 1, 0, 0);
         }
 
         gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
@@ -327,8 +327,7 @@ bdep_synth_dialog(BDepSynthArgs *args,
     BDepSynthControls controls;
     GwyDataField *dfield;
     gboolean finished;
-    gint response;
-    gint row, i;
+    gint response, row;
 
     gwy_clear(&controls, 1);
     controls.in_init = TRUE;
@@ -449,19 +448,12 @@ bdep_synth_dialog(BDepSynthArgs *args,
                      0, 2, row, row+1, GTK_FILL, 0, 0, 0);
     row++;
 
-    for (i = 0; i < GRAPH_NFLAGS; i++) {
-        GtkToggleButton *toggle;
-        controls.graph_flags[i]
-            = gtk_check_button_new_with_label(_(graph_flags[i]));
-        toggle = GTK_TOGGLE_BUTTON(controls.graph_flags[i]);
-        gtk_toggle_button_set_active(toggle, args->graph_flags[i]);
-        gtk_table_attach(GTK_TABLE(table), controls.graph_flags[i],
-                         0, 3, row, row+1, GTK_FILL, 0, 0, 0);
-        g_signal_connect(toggle, "toggled",
-                         G_CALLBACK(gwy_synth_boolean_changed_silent),
-                         &args->graph_flags[i]);
-        row++;
-    }
+    controls.graph_flags
+        = gwy_check_boxes_create(graph_flags, GRAPH_NFLAGS,
+                                 G_CALLBACK(gwy_synth_flag_changed_silent),
+                                 &args->graph_flags, args->graph_flags);
+    row = gwy_check_boxes_attach_to_table(controls.graph_flags,
+                                          GTK_TABLE(table), 3, row);
 
     gtk_widget_show_all(dialog);
     controls.in_init = FALSE;
@@ -511,8 +503,6 @@ static void
 update_controls(BDepSynthControls *controls,
                 BDepSynthArgs *args)
 {
-    guint i;
-
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->seed), args->seed);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls->randomize),
                                  args->randomize);
@@ -523,10 +513,7 @@ update_controls(BDepSynthControls *controls,
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->height), args->height);
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->height_noise),
                              args->height_noise);
-    for (i = 0; i < GRAPH_NFLAGS; i++) {
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls->graph_flags[i]),
-                                     args->graph_flags[i]);
-    }
+    gwy_check_boxes_set_selected(controls->graph_flags, args->graph_flags);
 }
 
 static void
@@ -599,7 +586,6 @@ bdep_synth_do(BDepSynthArgs *args,
 {
     GwyDataField *dfield;
     gint xres, yres, xext, yext, n;
-    guint i;
     gdouble nextgraphx;
     gdouble lasttime = 0.0, lastpreviewtime = 0.0, currtime;
     gdouble flux, height, hnoise;
@@ -607,9 +593,8 @@ bdep_synth_do(BDepSynthArgs *args,
     guint64 workdone, niter, iter;
     GwyRandGenSet *rngset;
     GRand *rng_k, *rng_height;
-    GArray **evolution = NULL;
+    GArray **evolution = NULL, *ev;
     gdouble *d;
-    gboolean any_graphs = FALSE;
     gboolean finished = FALSE;
 
     xext = gwy_data_field_get_xres(basefield)/12;
@@ -619,16 +604,9 @@ bdep_synth_do(BDepSynthArgs *args,
 
     timer = g_timer_new();
 
-    evolution = g_new0(GArray*, GRAPH_NFLAGS + 1);
     if (gcmodels) {
-        for (i = 0; i < GRAPH_NFLAGS; i++) {
-            if (args->graph_flags[i]) {
-                evolution[i] = g_array_new(FALSE, FALSE, sizeof(gdouble));
-                any_graphs = TRUE;
-            }
-        }
-        if (any_graphs)
-            evolution[GRAPH_NFLAGS] = g_array_new(FALSE, FALSE, sizeof(gdouble));
+        evolution = gwy_synth_make_evolution_arrays(graph_flags, GRAPH_NFLAGS,
+                                                    args->graph_flags);
     }
 
     xres = gwy_data_field_get_xres(dfield);
@@ -641,8 +619,8 @@ bdep_synth_do(BDepSynthArgs *args,
     rng_k = gwy_rand_gen_set_rng(rngset, 0);
     rng_height = gwy_rand_gen_set_rng(rngset, 1);
 
-    gwy_app_wait_set_message(_("Depositing particles..."));
-    gwy_app_wait_set_fraction(0.0);
+    if (!gwy_app_wait_set_message(_("Depositing particles...")))
+        goto fail;
 
     d = dfield->data;
     hnoise = args->height_noise;
@@ -689,40 +667,29 @@ bdep_synth_do(BDepSynthArgs *args,
             workdone -= WORK_UPDATE_CHECK;
         }
 
-        if (any_graphs && iter >= nextgraphx) {
+        if (evolution && iter >= nextgraphx) {
             gwy_data_field_invalidate(dfield);
             height = iter*flux*zscale;
             g_array_append_val(evolution[GRAPH_NFLAGS], height);
-            if (evolution[GRAPH_MEAN]) {
+            if ((ev = gwy_synth_find_evolution_array(evolution,
+                                                     graph_flags, GRAPH_NFLAGS,
+                                                     GRAPH_MEAN))) {
                 gdouble avg = gwy_data_field_get_avg(dfield);
-                g_array_append_val(evolution[GRAPH_MEAN], avg);
+                g_array_append_val(ev, avg);
             }
-            if (evolution[GRAPH_RMS]) {
-                gdouble avg = gwy_data_field_get_rms(dfield);
-                g_array_append_val(evolution[GRAPH_RMS], avg);
+            if ((ev = gwy_synth_find_evolution_array(evolution,
+                                                     graph_flags, GRAPH_NFLAGS,
+                                                     GRAPH_RMS))) {
+                gdouble rms = gwy_data_field_get_rms(dfield);
+                g_array_append_val(ev, rms);
             }
 
             nextgraphx += 0.0001/flux + MIN(0.2*nextgraphx, 0.08/flux);
         }
     }
 
-    if (gcmodels) {
-        const gdouble *xdata;
-        for (i = 0; i < GRAPH_NFLAGS; i++) {
-            if (!evolution[i]) {
-                gcmodels[i] = NULL;
-                continue;
-            }
-
-            xdata = (const gdouble*)evolution[GRAPH_NFLAGS]->data;
-            gcmodels[i] = gwy_graph_curve_model_new();
-            gwy_graph_curve_model_set_data(gcmodels[i],
-                                           xdata,
-                                           (gdouble*)evolution[i]->data,
-                                           evolution[GRAPH_NFLAGS]->len);
-            g_object_set(gcmodels[i], "description", _(graph_flags[i]), NULL);
-        }
-    }
+    gwy_synth_make_evolution_plots(gcmodels, evolution,
+                                   graph_flags, GRAPH_NFLAGS);
 
     gwy_data_field_area_copy(dfield, basefield,
                              xext, yext, xres - 2*xext, yres - 2*yext, 0, 0);
@@ -733,11 +700,7 @@ fail:
     g_object_unref(dfield);
     g_timer_destroy(timer);
     gwy_rand_gen_set_free(rngset);
-    for (i = 0; i <= GRAPH_NFLAGS; i++) {
-        if (evolution[i])
-            g_array_free(evolution[i], TRUE);
-    }
-    g_free(evolution);
+    gwy_synth_free_evolution_arrays(evolution, GRAPH_NFLAGS);
 
     return finished;
 }
@@ -763,6 +726,7 @@ bdep_synth_sanitize_args(BDepSynthArgs *args)
     args->height = CLAMP(args->height, 0.001, 1000.0);
     args->height_noise = CLAMP(args->height_noise, 0.0, 1.0);
     args->coverage = CLAMP(args->coverage, 0.01, 10000.0);
+    args->graph_flags &= GRAPH_ALL;
 }
 
 static void
@@ -770,9 +734,6 @@ bdep_synth_load_args(GwyContainer *container,
                      BDepSynthArgs *args,
                      GwyDimensionArgs *dimsargs)
 {
-    guint i;
-    gint gflags = 0;
-
     *args = bdep_synth_defaults;
 
     gwy_container_gis_int32_by_name(container, active_page_key,
@@ -786,11 +747,8 @@ bdep_synth_load_args(GwyContainer *container,
     gwy_container_gis_double_by_name(container, height_noise_key,
                                      &args->height_noise);
     gwy_container_gis_double_by_name(container, coverage_key, &args->coverage);
-    gwy_container_gis_int32_by_name(container, graph_flags_key, &gflags);
-
-    for (i = 0; i < GRAPH_NFLAGS; i++)
-        args->graph_flags[i] = !!(gflags & (1 << i));
-
+    gwy_container_gis_int32_by_name(container, graph_flags_key,
+                                    &args->graph_flags);
     bdep_synth_sanitize_args(args);
 
     gwy_clear(dimsargs, 1);
@@ -803,12 +761,6 @@ bdep_synth_save_args(GwyContainer *container,
                     const BDepSynthArgs *args,
                     const GwyDimensionArgs *dimsargs)
 {
-    guint i;
-    gint gflags = 0;
-
-    for (i = 0; i < GRAPH_NFLAGS; i++)
-        gflags |= (args->graph_flags[i] ? (1 << i) : 0);
-
     gwy_container_set_int32_by_name(container, active_page_key,
                                     args->active_page);
     gwy_container_set_int32_by_name(container, seed_key, args->seed);
@@ -820,7 +772,8 @@ bdep_synth_save_args(GwyContainer *container,
     gwy_container_set_double_by_name(container, height_noise_key,
                                      args->height_noise);
     gwy_container_set_double_by_name(container, coverage_key, args->coverage);
-    gwy_container_set_int32_by_name(container, graph_flags_key, gflags);
+    gwy_container_set_int32_by_name(container, graph_flags_key,
+                                    args->graph_flags);
 
     gwy_dimensions_save_args(dimsargs, container, prefix);
 }
