@@ -1,6 +1,6 @@
 /*
- *  $Id: calibrate.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2003-2013 David Necas (Yeti), Petr Klapetek.
+ *  $Id: calibrate.c 21613 2018-11-11 20:20:59Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -142,6 +142,7 @@ static void     units_change           (GtkWidget *button,
                                         CalibrateControls *controls);
 static void     match_size_changed     (GtkToggleButton *toggle,
                                         CalibrateControls *controls);
+static void     update_sensitivity     (CalibrateControls *controls);
 static void     size_channel_changed   (GwyDataChooser *toggle,
                                         CalibrateControls *controls);
 static gboolean mould_filter           (GwyContainer *data,
@@ -171,7 +172,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Recalibrates scan lateral dimensions or value range."),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "2.15",
+    "2.18",
     "David Nečas (Yeti) & Petr Klapetek",
     "2003",
 };
@@ -193,13 +194,29 @@ module_register(void)
     return TRUE;
 }
 
+static void
+calibrate_one_field(GwyDataField *dfield, const CalibrateArgs *args)
+{
+    GwySIUnit *siunitxy;
+
+    gwy_data_field_set_xreal(dfield, args->xreal);
+    gwy_data_field_set_yreal(dfield, args->yreal);
+    gwy_data_field_set_xoffset(dfield, args->x0);
+    gwy_data_field_set_yoffset(dfield, args->y0);
+    if (!gwy_strequal(args->xyunit, args->xyunitorig)) {
+        siunitxy = gwy_si_unit_new(args->xyunit);
+        gwy_data_field_set_si_unit_xy(dfield, siunitxy);
+        g_object_unref(siunitxy);
+    }
+}
+
 static void
 calibrate(GwyContainer *data, GwyRunType run)
 {
     GwyDataField *dfields[3];
     GQuark quarks[3];
     GQuark quark;
-    gint oldid, newid, datano;
+    gint n, i, oldid, newid, datano;
     CalibrateArgs args;
     gboolean ok;
     GwySIUnit *siunitxy, *siunitz;
@@ -239,11 +256,9 @@ calibrate(GwyContainer *data, GwyRunType run)
     siunitxy = gwy_data_field_get_si_unit_xy(dfields[0]);
     args.xyunitorig = gwy_si_unit_get_string(siunitxy,
                                              GWY_SI_UNIT_FORMAT_VFMARKUP);
-    args.xyunit = g_strdup(args.xyunitorig);
     siunitz = gwy_data_field_get_si_unit_z(dfields[0]);
     args.zunitorig = gwy_si_unit_get_string(siunitz,
                                             GWY_SI_UNIT_FORMAT_VFMARKUP);
-    args.zunit = g_strdup(args.zunitorig);
 
     if (run == GWY_RUN_INTERACTIVE) {
         ok = calibrate_dialog(&args, dfields[0]);
@@ -258,7 +273,7 @@ calibrate(GwyContainer *data, GwyRunType run)
     }
 
     if (!args.new_channel) {
-        guint n = 1;
+        n = 1;
         if (dfields[1]) {
             n++;
             if (dfields[2])
@@ -274,22 +289,13 @@ calibrate(GwyContainer *data, GwyRunType run)
 
     if (args.new_channel)
         dfields[0] = gwy_data_field_duplicate(dfields[0]);
-
-    gwy_data_field_set_xreal(dfields[0], args.xreal);
-    gwy_data_field_set_yreal(dfields[0], args.yreal);
+    calibrate_one_field(dfields[0], &args);
+    /* Only the image itself has Z recalibrated. */
     if (args.zratio != 1.0)
         gwy_data_field_multiply(dfields[0], args.zratio);
     if (args.zshift != 0.0)
         gwy_data_field_add(dfields[0], args.zshift);
-    gwy_data_field_set_xoffset(dfields[0], args.x0);
-    gwy_data_field_set_yoffset(dfields[0], args.y0);
-    if (!gwy_strequal(args.xyunit, args.xyunitorig)) {
-        siunitxy = gwy_si_unit_new(args.xyunit);
-        gwy_data_field_set_si_unit_xy(dfields[0], siunitxy);
-        g_object_unref(siunitxy);
-    }
-
-    if (args.zunit != args.zunitorig) {
+    if (!gwy_strequal(args.zunit, args.zunitorig)) {
         /* Ensure colour axis redraw when only the units change (but not the
          * range) by setting a new unit object. */
         siunitz = gwy_si_unit_new(args.zunit);
@@ -300,31 +306,13 @@ calibrate(GwyContainer *data, GwyRunType run)
     if (dfields[1]) {
         if (args.new_channel)
             dfields[1] = gwy_data_field_duplicate(dfields[1]);
-
-        gwy_data_field_set_xreal(dfields[1], args.xreal);
-        gwy_data_field_set_yreal(dfields[1], args.yreal);
-        gwy_data_field_set_xoffset(dfields[1], args.x0);
-        gwy_data_field_set_xoffset(dfields[1], args.y0);
-        if (!gwy_strequal(args.xyunit, args.xyunitorig)) {
-            siunitxy = gwy_si_unit_new(args.xyunit);
-            gwy_data_field_set_si_unit_xy(dfields[1], siunitxy);
-            g_object_unref(siunitxy);
-        }
+        calibrate_one_field(dfields[1], &args);
     }
 
     if (dfields[2]) {
         if (args.new_channel)
             dfields[2] = gwy_data_field_duplicate(dfields[2]);
-
-        gwy_data_field_set_xreal(dfields[2], args.xreal);
-        gwy_data_field_set_yreal(dfields[2], args.yreal);
-        gwy_data_field_set_xoffset(dfields[2], args.x0);
-        gwy_data_field_set_xoffset(dfields[2], args.y0);
-        if (!gwy_strequal(args.xyunit, args.xyunitorig)) {
-            siunitxy = gwy_si_unit_new(args.xyunit);
-            gwy_data_field_set_si_unit_xy(dfields[2], siunitxy);
-            g_object_unref(siunitxy);
-        }
+        calibrate_one_field(dfields[2], &args);
     }
 
     if (args.new_channel) {
@@ -350,8 +338,6 @@ calibrate(GwyContainer *data, GwyRunType run)
         gwy_app_channel_log_add_proc(data, oldid, newid);
     }
     else {
-        guint i;
-
         for (i = 0; i < 3; i++) {
             if (dfields[i])
                 gwy_data_field_data_changed(dfields[i]);
@@ -387,7 +373,8 @@ calibrate_dialog(CalibrateArgs *args,
                                          NULL);
     controls.dialog = dialog;
 
-    controls.ok = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_OK, GTK_RESPONSE_OK);
+    controls.ok = gtk_dialog_add_button(GTK_DIALOG(dialog),
+                                        GTK_STOCK_OK, GTK_RESPONSE_OK);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
     gwy_help_add_to_proc_dialog(GTK_DIALOG(dialog), GWY_HELP_DEFAULT);
 
@@ -520,11 +507,12 @@ calibrate_dialog(CalibrateArgs *args,
                                    controls.xreal, GWY_HSCALE_LOG);
     gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 3);
 
-    unit = gwy_data_field_get_si_unit_xy(dfield);
+    unit = gwy_si_unit_new(args->xyunit);
     controls.xyexponent
         = gwy_combo_box_metric_unit_new(G_CALLBACK(xyexponent_changed),
                                         &controls, -15, 6, unit,
                                         args->xyexponent);
+    g_object_unref(unit);
     gtk_table_attach(GTK_TABLE(table), controls.xyexponent,
                      2, 3, row, row+2, GTK_FILL | GTK_SHRINK, 0, 0, 0);
 
@@ -576,11 +564,12 @@ calibrate_dialog(CalibrateArgs *args,
                                    controls.zreal, GWY_HSCALE_SQRT);
     gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 2);
 
-    unit = gwy_data_field_get_si_unit_z(dfield);
+    unit = gwy_si_unit_new(args->zunit);
     controls.zexponent
         = gwy_combo_box_metric_unit_new(G_CALLBACK(zexponent_changed),
                                         &controls, -15, 6, unit,
                                         args->zexponent);
+    g_object_unref(unit);
     gtk_table_attach(GTK_TABLE(table), controls.zexponent,
                      2, 3, row, row+1, GTK_FILL | GTK_SHRINK, 0, 0, 0);
 
@@ -671,9 +660,8 @@ calibrate_dialog(CalibrateArgs *args,
 
     controls.in_update = FALSE;
     /* sync all fields */
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls.match_size),
-                                 args->match_size);
     calibrate_dialog_update(&controls, args);
+    update_sensitivity(&controls);
 
     gtk_widget_show_all(dialog);
     do {
@@ -1140,6 +1128,22 @@ match_size_changed(GtkToggleButton *toggle,
     gboolean matching = gtk_toggle_button_get_active(toggle);
 
     controls->args->match_size = matching;
+    if (controls->in_update)
+        return;
+
+    update_sensitivity(controls);
+    if (matching) {
+        size_channel_changed(GWY_DATA_CHOOSER(controls->size_chooser),
+                             controls);
+    }
+}
+
+static void
+update_sensitivity(CalibrateControls *controls)
+{
+    gboolean matching = controls->args->match_size;
+    gboolean isok = TRUE;
+
     gwy_table_hscale_set_sensitive(controls->xreal, !matching);
     gwy_table_hscale_set_sensitive(controls->yreal, !matching);
     gwy_table_hscale_set_sensitive(controls->xratio, !matching);
@@ -1147,15 +1151,12 @@ match_size_changed(GtkToggleButton *toggle,
     gtk_widget_set_sensitive(controls->xyunits, !matching);
     gtk_widget_set_sensitive(controls->xyexponent, !matching);
     gtk_widget_set_sensitive(controls->square, !matching);
-    /* This is fixed to sensitive again in size_channel_changed() any channel
-     * is selected. */
-    gtk_dialog_set_response_sensitive(GTK_DIALOG(controls->dialog),
-                                      GTK_RESPONSE_OK, !matching);
 
-    if (matching) {
-        size_channel_changed(GWY_DATA_CHOOSER(controls->size_chooser),
-                             controls);
-    }
+    if (matching && controls->args->sizeid.id == -1)
+        isok = FALSE;
+
+    gtk_dialog_set_response_sensitive(GTK_DIALOG(controls->dialog),
+                                      GTK_RESPONSE_OK, isok);
 }
 
 static void
@@ -1169,17 +1170,16 @@ size_channel_changed(GwyDataChooser *chooser,
     gdouble dx, dy;
     GwySIUnit *unit;
 
-    if (!gwy_data_chooser_get_active_id(chooser, &args->sizeid)) {
-        gtk_dialog_set_response_sensitive(GTK_DIALOG(controls->dialog),
-                                          GTK_RESPONSE_OK, FALSE);
+    gwy_data_chooser_get_active_id(chooser, &args->sizeid);
+    update_sensitivity(controls);
+    if (args->sizeid.id == -1)
         return;
-    }
 
     quark = gwy_app_get_data_key_for_id(args->sizeid.id);
     data = gwy_app_data_browser_get(args->sizeid.datano);
     mould = GWY_DATA_FIELD(gwy_container_get_object(data, quark));
-    dx = gwy_data_field_get_xmeasure(mould);
-    dy = gwy_data_field_get_ymeasure(mould);
+    dx = gwy_data_field_get_dx(mould);
+    dy = gwy_data_field_get_dy(mould);
     unit = gwy_data_field_get_si_unit_xy(mould);
 
     controls->in_update = TRUE;
@@ -1282,28 +1282,39 @@ calibrate_dialog_update(CalibrateControls *controls,
     else
         g_snprintf(buffer, sizeof(buffer), "× 10<sup>%d</sup>", e);
     gtk_label_set_markup(GTK_LABEL(controls->zpower10), buffer);
-
-    if (args->zratio == 0 || args->zreal == 0)
-        gtk_widget_set_sensitive(controls->ok, FALSE);
-    else
-        gtk_widget_set_sensitive(controls->ok, TRUE);
 }
 
+static const gchar match_size_key[]  = "/module/calibrate/match_size";
+static const gchar new_channel_key[] = "/module/calibrate/new_channel";
+static const gchar square_key[]      = "/module/calibrate/square";
 static const gchar xratio_key[]      = "/module/calibrate/xratio";
+static const gchar xyunit_key[]      = "/module/calibrate/xyunit";
 static const gchar yratio_key[]      = "/module/calibrate/yratio";
 static const gchar zratio_key[]      = "/module/calibrate/zratio";
 static const gchar zshift_key[]      = "/module/calibrate/zshift";
-static const gchar square_key[]      = "/module/calibrate/square";
-static const gchar new_channel_key[] = "/module/calibrate/new_channel";
-static const gchar match_size_key[]  = "/module/calibrate/match_size";
+static const gchar zunit_key[]       = "/module/calibrate/zunit";
+
+/* NB: We do not free the unit because it's meant to be taken from settings
+ * or defaults, i.e. const. */
+static void
+sanitize_unit(gchar **unitstr)
+{
+    GwySIUnit *unit;
+
+    unit = gwy_si_unit_new(*unitstr);
+    *unitstr = gwy_si_unit_get_string(unit, GWY_SI_UNIT_FORMAT_PLAIN);
+    g_object_unref(unit);
+}
 
 static void
 calibrate_sanitize_args(CalibrateArgs *args)
 {
-    args->xratio = CLAMP(args->xratio, 1e-15, 1e15);
-    args->yratio = CLAMP(args->yratio, 1e-15, 1e15);
-    args->zratio = CLAMP(args->zratio, -1e15, 1e15);
+    args->xratio = CLAMP(args->xratio, 1e-30, 1e30);
+    args->yratio = CLAMP(args->yratio, 1e-30, 1e30);
+    args->zratio = CLAMP(args->zratio, -1e30, 1e30);
     args->zshift = CLAMP(args->zshift, -1e-9, 1e9);
+    sanitize_unit(&args->xyunit);
+    sanitize_unit(&args->zunit);
     args->square = !!args->square;
     args->new_channel = !!args->new_channel;
     args->match_size = !!args->match_size;
@@ -1320,6 +1331,10 @@ calibrate_load_args(GwyContainer *container,
     gwy_container_gis_double_by_name(container, yratio_key, &args->yratio);
     gwy_container_gis_double_by_name(container, zratio_key, &args->zratio);
     gwy_container_gis_double_by_name(container, zshift_key, &args->zshift);
+    gwy_container_gis_string_by_name(container, xyunit_key,
+                                     (const guchar**)&args->xyunit);
+    gwy_container_gis_string_by_name(container, zunit_key,
+                                     (const guchar**)&args->zunit);
     gwy_container_gis_boolean_by_name(container, square_key, &args->square);
     gwy_container_gis_boolean_by_name(container, new_channel_key,
                                       &args->new_channel);
@@ -1338,6 +1353,8 @@ calibrate_save_args(GwyContainer *container,
     gwy_container_set_double_by_name(container, yratio_key, args->yratio);
     gwy_container_set_double_by_name(container, zratio_key, args->zratio);
     gwy_container_set_double_by_name(container, zshift_key, args->zshift);
+    gwy_container_set_const_string_by_name(container, xyunit_key, args->xyunit);
+    gwy_container_set_const_string_by_name(container, zunit_key, args->zunit);
     gwy_container_set_boolean_by_name(container, square_key, args->square);
     gwy_container_set_boolean_by_name(container, new_channel_key,
                                       args->new_channel);
