@@ -1,6 +1,6 @@
 /*
- *  $Id: coerce.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2016 David Necas (Yeti).
+ *  $Id: coerce.c 20809 2018-03-05 10:51:09Z yeti-dn $
+ *  Copyright (C) 2016-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -40,10 +40,11 @@
 #define COERCE_RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
 typedef enum {
-    COERCE_DISTRIBUTION_DATA     = 0,
-    COERCE_DISTRIBUTION_UNIFORM  = 1,
-    COERCE_DISTRIBUTION_GAUSSIAN = 2,
-    COERCE_DISTRIBUTION_LEVELS   = 3,
+    COERCE_DISTRIBUTION_DATA        = 0,
+    COERCE_DISTRIBUTION_UNIFORM     = 1,
+    COERCE_DISTRIBUTION_GAUSSIAN    = 2,
+    COERCE_DISTRIBUTION_LEVELS      = 3,
+    COERCE_DISTRIBUTION_SKEW_NORMAL = 4,
     COERCE_NDISTRIBUTIONS
 } CoerceDistributionType;
 
@@ -63,6 +64,7 @@ typedef struct {
     CoerceDistributionType distribution;
     CoerceLevelsType level_type;
     gint nlevels;
+    gdouble skew;
     CoerceProcessingType processing;
     gboolean update;
     GwyAppDataId template;
@@ -77,6 +79,7 @@ typedef struct {
     GSList *distribution;
     GtkWidget *level_type;
     GtkObject *nlevels;
+    GtkObject *skew;
     GSList *processing;
     GtkWidget *template;
     GtkWidget *update;
@@ -87,65 +90,72 @@ typedef struct {
     guint k;
 } ValuePos;
 
-static gboolean      module_register       (void);
-static void          coerce                (GwyContainer *data,
-                                            GwyRunType run);
-static gboolean      coerce_dialogue       (CoerceArgs *args,
-                                            GwyContainer *data,
-                                            gint id);
-static void          distribution_changed  (GtkToggleButton *toggle,
-                                            CoerceControls *controls);
-static void          level_type_changed    (GtkComboBox *combo,
-                                            CoerceControls *controls);
-static void          nlevels_changed       (GtkAdjustment *adj,
-                                            CoerceControls *controls);
-static void          processing_changed    (GtkToggleButton *toggle,
-                                            CoerceControls *controls);
-static void          template_changed      (GwyDataChooser *chooser,
-                                            CoerceControls *controls);
-static void          update_changed        (GtkToggleButton *toggle,
-                                            CoerceControls *controls);
-static void          update_sensitivity    (CoerceControls *controls);
-static gboolean      template_filter       (GwyContainer *data,
-                                            gint id,
-                                            gpointer user_data);
-static void          preview               (CoerceControls *controls);
-static GwyDataField* coerce_do             (GwyDataField *dfield,
-                                            const CoerceArgs *args);
-static void          coerce_do_field       (GwyDataField *dfield,
-                                            GwyDataField *result,
-                                            const CoerceArgs *args);
-static void          coerce_do_field_levels(GwyDataField *dfield,
-                                            GwyDataField *result,
-                                            const CoerceArgs *args);
-static void          coerce_do_rows        (GwyDataField *dfield,
-                                            GwyDataField *result,
-                                            const CoerceArgs *args);
-static void          build_values_levels   (const ValuePos *vpos,
-                                            gdouble *z,
-                                            guint n,
-                                            guint nlevels);
-static void          build_values_uniform  (gdouble *z,
-                                            guint n,
-                                            gdouble min,
-                                            gdouble max);
-static void          build_values_gaussian (gdouble *z,
-                                            guint n,
-                                            gdouble mean,
-                                            gdouble rms);
-static void          build_values_from_data(gdouble *z,
-                                            guint n,
-                                            const gdouble *data,
-                                            guint ndata);
-static void          load_args             (GwyContainer *container,
-                                            CoerceArgs *args);
-static void          save_args             (GwyContainer *container,
-                                            CoerceArgs *args);
+static gboolean      module_register         (void);
+static void          coerce                  (GwyContainer *data,
+                                              GwyRunType run);
+static gboolean      coerce_dialogue         (CoerceArgs *args,
+                                              GwyContainer *data,
+                                              gint id);
+static void          distribution_changed    (GtkToggleButton *toggle,
+                                              CoerceControls *controls);
+static void          level_type_changed      (GtkComboBox *combo,
+                                              CoerceControls *controls);
+static void          nlevels_changed         (GtkAdjustment *adj,
+                                              CoerceControls *controls);
+static void          skew_changed            (GtkAdjustment *adj,
+                                              CoerceControls *controls);
+static void          processing_changed      (GtkToggleButton *toggle,
+                                              CoerceControls *controls);
+static void          template_changed        (GwyDataChooser *chooser,
+                                              CoerceControls *controls);
+static void          update_changed          (GtkToggleButton *toggle,
+                                              CoerceControls *controls);
+static void          update_sensitivity      (CoerceControls *controls);
+static gboolean      template_filter         (GwyContainer *data,
+                                              gint id,
+                                              gpointer user_data);
+static void          preview                 (CoerceControls *controls);
+static GwyDataField* coerce_do               (GwyDataField *dfield,
+                                              const CoerceArgs *args);
+static void          coerce_do_field         (GwyDataField *dfield,
+                                              GwyDataField *result,
+                                              const CoerceArgs *args);
+static void          coerce_do_field_levels  (GwyDataField *dfield,
+                                              GwyDataField *result,
+                                              const CoerceArgs *args);
+static void          coerce_do_rows          (GwyDataField *dfield,
+                                              GwyDataField *result,
+                                              const CoerceArgs *args);
+static void          build_values_levels     (const ValuePos *vpos,
+                                              gdouble *z,
+                                              guint n,
+                                              guint nlevels);
+static void          build_values_uniform    (gdouble *z,
+                                              guint n,
+                                              gdouble min,
+                                              gdouble max);
+static void          build_values_gaussian   (gdouble *z,
+                                              guint n,
+                                              gdouble mean,
+                                              gdouble rms);
+static void          build_values_skew_normal(gdouble *z,
+                                              guint n,
+                                              gdouble mean,
+                                              gdouble rms,
+                                              gdouble skew);
+static void          build_values_from_data  (gdouble *z,
+                                              guint n,
+                                              const gdouble *data,
+                                              guint ndata);
+static void          load_args               (GwyContainer *container,
+                                              CoerceArgs *args);
+static void          save_args               (GwyContainer *container,
+                                              CoerceArgs *args);
 
 static const CoerceArgs coerce_defaults = {
     COERCE_DISTRIBUTION_UNIFORM,
     COERCE_LEVELS_EQUIAREA,
-    4,
+    4, 0.5,
     COERCE_PROCESSING_FIELD,
     TRUE,
     GWY_APP_DATA_ID_NONE,
@@ -158,7 +168,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Transforms surfaces to have prescribed statistical properties."),
     "Yeti <yeti@gwyddion.net>",
-    "1.0",
+    "1.1",
     "David Nečas (Yeti)",
     "2016",
 };
@@ -222,10 +232,11 @@ static gboolean
 coerce_dialogue(CoerceArgs *args, GwyContainer *data, gint id)
 {
     static const GwyEnum distributions[] = {
-        { N_("distribution|Uniform"),  COERCE_DISTRIBUTION_UNIFORM,  },
-        { N_("distribution|Gaussian"), COERCE_DISTRIBUTION_GAUSSIAN, },
-        { N_("As another data"),       COERCE_DISTRIBUTION_DATA,     },
-        { N_("Discrete levels"),       COERCE_DISTRIBUTION_LEVELS,   },
+        { N_("distribution|Uniform"),     COERCE_DISTRIBUTION_UNIFORM,     },
+        { N_("distribution|Gaussian"),    COERCE_DISTRIBUTION_GAUSSIAN,    },
+        { N_("distribution|Skew-normal"), COERCE_DISTRIBUTION_SKEW_NORMAL, },
+        { N_("As another data"),          COERCE_DISTRIBUTION_DATA,        },
+        { N_("Discrete levels"),          COERCE_DISTRIBUTION_LEVELS,      },
     };
 
     static const GwyEnum processings[] = {
@@ -342,6 +353,15 @@ coerce_dialogue(CoerceArgs *args, GwyContainer *data, gint id)
                              G_CALLBACK(nlevels_changed), &controls);
             row++;
         }
+        else if (dist == COERCE_DISTRIBUTION_SKEW_NORMAL) {
+            controls.skew = gtk_adjustment_new(args->skew,
+                                               -1.0, 1.0, 0.001, 0.1, 0.0);
+            gwy_table_attach_adjbar(table, row, _("_Skew:"), NULL,
+                                    controls.skew, GWY_HSCALE_LINEAR);
+            g_signal_connect(controls.skew, "value-changed",
+                             G_CALLBACK(skew_changed), &controls);
+            row++;
+        }
     }
 
     gtk_table_set_row_spacing(GTK_TABLE(table), row-1, 8);
@@ -443,6 +463,16 @@ nlevels_changed(GtkAdjustment *adj, CoerceControls *controls)
         preview(controls);
 }
 
+static void
+skew_changed(GtkAdjustment *adj, CoerceControls *controls)
+{
+    CoerceArgs *args = controls->args;
+
+    args->skew = gtk_adjustment_get_value(adj);
+    if (args->distribution == COERCE_DISTRIBUTION_SKEW_NORMAL && args->update)
+        preview(controls);
+}
+
 static void
 processing_changed(GtkToggleButton *toggle, CoerceControls *controls)
 {
@@ -483,7 +513,7 @@ update_sensitivity(CoerceControls *controls)
 {
     CoerceArgs *args = controls->args;
     GtkWidget *widget;
-    gboolean has_template, is_data, is_levels, do_update;
+    gboolean has_template, is_data, is_levels, is_skew, do_update;
 
     has_template
         = !!gwy_data_chooser_get_active(GWY_DATA_CHOOSER(controls->template),
@@ -499,6 +529,9 @@ update_sensitivity(CoerceControls *controls)
     gwy_table_hscale_set_sensitive(GTK_OBJECT(controls->level_type), is_levels);
     gwy_table_hscale_set_sensitive(controls->nlevels, is_levels);
 
+    is_skew = (args->distribution == COERCE_DISTRIBUTION_SKEW_NORMAL);
+    gwy_table_hscale_set_sensitive(controls->skew, is_skew);
+
     do_update = args->update;
     gtk_dialog_set_response_sensitive(GTK_DIALOG(controls->dialogue),
                                       RESPONSE_PREVIEW, !do_update);
@@ -610,6 +643,12 @@ coerce_do_field(GwyDataField *dfield, GwyDataField *result,
         rms = gwy_data_field_get_rms(dfield);
         build_values_gaussian(z, n, avg, rms);
     }
+    else if (args->distribution == COERCE_DISTRIBUTION_SKEW_NORMAL) {
+        gdouble avg, rms;
+        avg = gwy_data_field_get_avg(dfield);
+        rms = gwy_data_field_get_rms(dfield);
+        build_values_skew_normal(z, n, avg, rms, args->skew);
+    }
     else {
         g_return_if_reached();
     }
@@ -693,6 +732,12 @@ coerce_do_rows(GwyDataField *dfield, GwyDataField *result,
         rms = gwy_data_field_get_rms(dfield);
         build_values_gaussian(z, xres, avg, rms);
     }
+    else if (args->distribution == COERCE_DISTRIBUTION_SKEW_NORMAL) {
+        gdouble avg, rms;
+        avg = gwy_data_field_get_avg(dfield);
+        rms = gwy_data_field_get_rms(dfield);
+        build_values_skew_normal(z, xres, avg, rms, args->skew);
+    }
     else {
         g_return_if_reached();
     }
@@ -864,10 +909,104 @@ build_values_from_data(gdouble *z, guint n, const gdouble *data, guint ndata)
     g_free(sorted);
 }
 
+static void
+build_values_from_pdf(gdouble *z, guint n,
+                      gdouble a, gdouble b, gdouble *pdf, guint npdf)
+{
+    gdouble s, h, y, xi;
+    guint i, j;
+
+    /* Transform pdf to normalised cumulative distribution function. */
+    pdf[0] = 0.0;  /* We cannot sample to the left anyway. */
+    for (i = 1; i < npdf; i++)
+        pdf[i] += pdf[i-1];
+
+    s = pdf[npdf-1];
+    for (i = 0; i < npdf; i++)
+        pdf[i] /= s;
+
+    /* Create values by linearly sampling the CDF bins. */
+    h = (b - a)/(npdf - 1.0);
+    j = 0;
+    for (i = 0; i < n; i++) {
+        y = (i + 0.5)/n;
+        while (j < npdf && y > pdf[j])
+            j++;
+        if (G_UNLIKELY(j == 0)) {
+            z[i] = a;
+            continue;
+        }
+        if (G_UNLIKELY(j == npdf)) {
+            z[i] = b;
+            continue;
+        }
+        /* Now we have j-1..j interval in which to interpolate. */
+        xi = (y - pdf[j-1])/(pdf[j] - pdf[j-1]);
+        z[i] = a + h*(j + xi - 1.0);
+    }
+}
+
+static gdouble
+solve_alpha_for_skew(gdouble skew)
+{
+    gdouble a = (4.0 - G_PI)/2.0;
+    gdouble a3 = cbrt(a);
+    gdouble s3 = cbrt(skew);
+    gdouble u = (a*a3*s3*s3 - a3*a3*skew*s3 + skew*skew)/(a*a + skew*skew);
+    gdouble alpha = sqrt(0.5*G_PI*u/(2.0 - G_PI*u));
+
+    return alpha;
+}
+
+static void
+build_values_skew_normal(gdouble *z, guint n,
+                         gdouble mean, gdouble rms, gdouble skew)
+{
+    guint i, npdf = 2*n;
+    gdouble *pdf = g_new(gdouble, npdf);
+    gdouble x, s, m, alpha;
+
+    if (fabs(skew) > 0.995)
+        alpha = 100.0;
+    else
+        alpha = solve_alpha_for_skew(skew);
+
+    if (skew < 0.0)
+        alpha = -alpha;
+
+    s = 0.0;
+    for (i = 0; i < npdf; i++) {
+        x = 20.0*i/(npdf - 1.0) - 10.0;
+        pdf[i] = exp(-0.5*x*x) * (erf(alpha*x) + 1.0);
+        s += pdf[i];
+    }
+
+    /* Ensure given mean and rms: This is actually generic and we assume
+     * interval [-1,1]. */
+    m = 0.0;
+    for (i = 0; i < npdf; i++) {
+        x = 2.0*i/(npdf - 1.0) - 1.0;
+        pdf[i] /= s;
+        m += x*pdf[i];
+    }
+    s = 0.0;
+    for (i = 0; i < npdf; i++) {
+        x = 2.0*i/(npdf - 1.0) - 1.0 - m;
+        s += x*x*pdf[i];
+    }
+    s = sqrt(s);
+
+    build_values_from_pdf(z, n,
+                          -(1.0 + m)/s*rms + mean, (1.0 - m)/s*rms + mean,
+                          pdf, npdf);
+    g_free(pdf);
+}
+
 static const gchar distribution_key[] = "/module/coerce/distribution";
 static const gchar level_type_key[]   = "/module/coerce/level_type";
 static const gchar nlevels_key[]      = "/module/coerce/nlevels";
 static const gchar processing_key[]   = "/module/coerce/processing";
+static const gchar skew_key[]         = "/module/coerce/skew";
 
 static void
 sanitize_args(CoerceArgs *args)
@@ -876,6 +1015,7 @@ sanitize_args(CoerceArgs *args)
     args->processing = MIN(args->processing, COERCE_NPROCESSING-1);
     args->level_type = MIN(args->level_type, COERCE_NLEVELTYPES-1);
     args->nlevels = CLAMP(args->nlevels, 2, 16384);
+    args->skew = CLAMP(args->skew, -1.0, 1.0);
     if (args->distribution == COERCE_DISTRIBUTION_DATA
         && !gwy_app_data_id_verify_channel(&args->template))
         args->distribution = coerce_defaults.distribution;
@@ -894,6 +1034,7 @@ load_args(GwyContainer *container,
     gwy_container_gis_int32_by_name(container, nlevels_key, &args->nlevels);
     gwy_container_gis_enum_by_name(container, processing_key,
                                    &args->processing);
+    gwy_container_gis_double_by_name(container, skew_key, &args->skew);
     args->template = template_id;
     sanitize_args(args);
 }
@@ -910,6 +1051,7 @@ save_args(GwyContainer *container,
     gwy_container_set_int32_by_name(container, nlevels_key, args->nlevels);
     gwy_container_set_enum_by_name(container, processing_key,
                                    args->processing);
+    gwy_container_set_double_by_name(container, skew_key, args->skew);
 }
 
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
