@@ -1,6 +1,6 @@
 /*
- *  $Id: col_synth.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2014 David Necas (Yeti).
+ *  $Id: col_synth.c 21410 2018-09-07 14:15:50Z yeti-dn $
+ *  Copyright (C) 2014-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -48,13 +48,13 @@ typedef enum {
 } RelaxationType;
 
 typedef enum {
-    GRAPH_MAX      = 0,
-    GRAPH_RMS      = 1,
-    GRAPH_NMAX     = 2,
-    GRAPH_SKEW     = 3,
-    GRAPH_KURTOSIS = 4,
-    GRAPH_CORRLEN  = 5,
-    GRAPH_NFLAGS,
+    GRAPH_MAX      = (1 << 0),
+    GRAPH_RMS      = (1 << 1),
+    GRAPH_NMAX     = (1 << 2),
+    GRAPH_SKEW     = (1 << 3),
+    GRAPH_KURTOSIS = (1 << 4),
+    GRAPH_CORRLEN  = (1 << 5),
+    GRAPH_ALL      = (1 << 6) - 1,
 } GraphFlags;
 
 typedef struct _ColSynthControls ColSynthControls;
@@ -73,7 +73,7 @@ typedef struct {
     gdouble height_noise;
     RelaxationType relaxation;
     gdouble melting;
-    gboolean graph_flags[GRAPH_NFLAGS];
+    guint graph_flags;
 } ColSynthArgs;
 
 struct _ColSynthControls {
@@ -96,7 +96,7 @@ struct _ColSynthControls {
     GtkObject *height_noise;
     GtkWidget *relaxation;
     GtkObject *melting;
-    GtkWidget *graph_flags[GRAPH_NFLAGS];
+    GSList *graph_flags;
     GwyContainer *mydata;
     GwyDataField *surface;
     gdouble pxsize;
@@ -164,13 +164,17 @@ static void       col_synth_save_args     (GwyContainer *container,
 
 #include "synth.h"
 
-static const gchar* graph_flags[GRAPH_NFLAGS] = {
-    N_("Maximum"),
-    N_("RMS"),
-    N_("Number of maxima"),
-    N_("Skew"),
-    N_("Kurtosis"),
-    N_("Autocorrelation length"),
+static const GwyEnum graph_flags[] = {
+    { N_("Maximum"),                GRAPH_MAX,      },
+    { N_("RMS"),                    GRAPH_RMS,      },
+    { N_("Number of maxima"),       GRAPH_NMAX,     },
+    { N_("Skew"),                   GRAPH_SKEW,     },
+    { N_("Kurtosis"),               GRAPH_KURTOSIS, },
+    { N_("Autocorrelation length"), GRAPH_CORRLEN,  },
+};
+
+enum {
+    GRAPH_NFLAGS = G_N_ELEMENTS(graph_flags)
 };
 
 static const ColSynthArgs col_synth_defaults = {
@@ -181,7 +185,7 @@ static const ColSynthArgs col_synth_defaults = {
     0.0, 1.0,
     1.0, 0.0,
     RELAX_WEAK, 0.0,
-    { FALSE, FALSE, FALSE, },
+    0,
 };
 
 static const GwyDimensionArgs dims_defaults = GWY_DIMENSION_ARGS_INIT;
@@ -191,7 +195,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Generates columnar surfaces by a simple growth algorithm."),
     "Yeti <yeti@gwyddion.net>",
-    "1.5",
+    "1.7",
     "David Nečas (Yeti)",
     "2014",
 };
@@ -322,7 +326,6 @@ run_noninteractive(ColSynthArgs *args,
 
     for (i = 0; i < GRAPH_NFLAGS; i++) {
         GwyGraphModel *gmodel;
-        GwySIUnit *unit;
         gchar *s, *title;
         gboolean is_log = (i != GRAPH_SKEW && i != GRAPH_KURTOSIS);
 
@@ -334,33 +337,29 @@ run_noninteractive(ColSynthArgs *args,
         g_object_unref(gcmodels[i]);
 
         s = gwy_app_get_data_field_title(data, newid);
-        title = g_strdup_printf("%s (%s)", graph_flags[i], s);
+        title = g_strdup_printf("%s (%s)", _(graph_flags[i].name), s);
         g_free(s);
         g_object_set(gmodel,
                      "title", title,
                      "x-logarithmic", TRUE,
                      "y-logarithmic", is_log,
                      "axis-label-bottom", _("Mean deposited thickness"),
-                     "axis-label-left", _(graph_flags[i]),
+                     "axis-label-left", _(graph_flags[i].name),
                      NULL);
         g_free(title);
 
-        unit = gwy_data_field_get_si_unit_z(newfield);
-        unit = gwy_si_unit_duplicate(unit);
-        g_object_set(gmodel, "si-unit-x", unit, NULL);
-        g_object_unref(unit);
-
-        if (i == GRAPH_MAX || i == GRAPH_RMS) {
-            unit = gwy_data_field_get_si_unit_z(newfield);
-            unit = gwy_si_unit_duplicate(unit);
-            g_object_set(gmodel, "si-unit-y", unit, NULL);
-            g_object_unref(unit);
+        if (graph_flags[i].value == GRAPH_MAX
+            || graph_flags[i].value == GRAPH_RMS) {
+            gwy_graph_model_set_units_from_data_field(gmodel, newfield,
+                                                      0, 1, 0, 1);
         }
-        if (i == GRAPH_CORRLEN) {
-            unit = gwy_data_field_get_si_unit_xy(newfield);
-            unit = gwy_si_unit_duplicate(unit);
-            g_object_set(gmodel, "si-unit-y", unit, NULL);
-            g_object_unref(unit);
+        else if (graph_flags[i].value == GRAPH_CORRLEN) {
+            gwy_graph_model_set_units_from_data_field(gmodel, newfield,
+                                                      0, 1, 1, 0);
+        }
+        else {
+            gwy_graph_model_set_units_from_data_field(gmodel, newfield,
+                                                      0, 1, 0, 0);
         }
 
         gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
@@ -378,8 +377,7 @@ col_synth_dialog(ColSynthArgs *args,
     ColSynthControls controls;
     GwyDataField *dfield;
     gboolean finished;
-    gint response;
-    gint row, i;
+    gint response, row;
 
     gwy_clear(&controls, 1);
     controls.in_init = TRUE;
@@ -538,19 +536,12 @@ col_synth_dialog(ColSynthArgs *args,
                      0, 3, row, row+1, GTK_FILL, 0, 0, 0);
     row++;
 
-    for (i = 0; i < GRAPH_NFLAGS; i++) {
-        GtkToggleButton *toggle;
-        controls.graph_flags[i]
-            = gtk_check_button_new_with_label(_(graph_flags[i]));
-        toggle = GTK_TOGGLE_BUTTON(controls.graph_flags[i]);
-        gtk_toggle_button_set_active(toggle, args->graph_flags[i]);
-        gtk_table_attach(GTK_TABLE(table), controls.graph_flags[i],
-                         0, 2, row, row+1, GTK_FILL, 0, 0, 0);
-        g_signal_connect(toggle, "toggled",
-                         G_CALLBACK(gwy_synth_boolean_changed_silent),
-                         &args->graph_flags[i]);
-        row++;
-    }
+    controls.graph_flags
+        = gwy_check_boxes_create(graph_flags, GRAPH_NFLAGS,
+                                 G_CALLBACK(gwy_synth_flag_changed_silent),
+                                 &args->graph_flags, args->graph_flags);
+    row = gwy_check_boxes_attach_to_table(controls.graph_flags,
+                                          GTK_TABLE(table), 3, row);
 
     gtk_widget_show_all(dialog);
     controls.in_init = FALSE;
@@ -616,8 +607,6 @@ static void
 update_controls(ColSynthControls *controls,
                 ColSynthArgs *args)
 {
-    guint i;
-
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->seed), args->seed);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls->randomize),
                                  args->randomize);
@@ -639,10 +628,7 @@ update_controls(ColSynthControls *controls,
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->melting), args->melting);
     gwy_enum_combo_box_set_active(GTK_COMBO_BOX(controls->relaxation),
                                   args->relaxation);
-    for (i = 0; i < GRAPH_NFLAGS; i++) {
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls->graph_flags[i]),
-                                     args->graph_flags[i]);
-    }
+    gwy_check_boxes_set_selected(controls->graph_flags, args->graph_flags);
 }
 
 static void
@@ -687,8 +673,8 @@ scale_to_unit_cubes(GwyDataField *dfield,
 {
     gdouble r;
 
-    *rx = 1.0/gwy_data_field_get_xmeasure(dfield);
-    *ry = 1.0/gwy_data_field_get_ymeasure(dfield);
+    *rx = 1.0/gwy_data_field_get_dx(dfield);
+    *ry = 1.0/gwy_data_field_get_dy(dfield);
     gwy_data_field_set_xreal(dfield, dfield->xres);
     gwy_data_field_set_yreal(dfield, dfield->yres);
     r = sqrt((*rx)*(*ry));
@@ -783,7 +769,6 @@ convolve_periodic_fast3(GwyDataField *data_field,
     gwy_data_field_invalidate(data_field);
 }
 
-
 static gboolean
 col_synth_do(const ColSynthArgs *args,
              GwyDataField *dfield,
@@ -793,29 +778,23 @@ col_synth_do(const ColSynthArgs *args,
 {
     RelaxationType relaxation;
     gint xres, yres;
-    guint i;
     guint64 npart, ip;
     gdouble zmax, zsum, nextgraphx, nextconvolve, melting;
     gdouble lasttime = 0.0, lastpreviewtime = 0.0, currtime;
     GTimer *timer;
-    GwyRandGenSet *rngset;
+    GwyRandGenSet *rngset = NULL;
     GwyDataLine *acf = NULL;
-    GArray **evolution = NULL;
-    gboolean any_graphs = FALSE;
+    GArray **evolution = NULL, *ev;
+    gboolean has_basic_stats = FALSE;
     gboolean finished = FALSE;
 
     timer = g_timer_new();
 
-    evolution = g_new0(GArray*, GRAPH_NFLAGS + 1);
     if (gcmodels) {
-        for (i = 0; i < GRAPH_NFLAGS; i++) {
-            if (args->graph_flags[i]) {
-                evolution[i] = g_array_new(FALSE, FALSE, sizeof(gdouble));
-                any_graphs = TRUE;
-            }
-        }
-        if (any_graphs)
-            evolution[GRAPH_NFLAGS] = g_array_new(FALSE, FALSE, sizeof(gdouble));
+        evolution = gwy_synth_make_evolution_arrays(graph_flags, GRAPH_NFLAGS,
+                                                    args->graph_flags);
+        has_basic_stats = (args->graph_flags
+                           & (GRAPH_RMS | GRAPH_SKEW | GRAPH_KURTOSIS));
     }
 
     xres = gwy_data_field_get_xres(dfield);
@@ -828,8 +807,8 @@ col_synth_do(const ColSynthArgs *args,
     nextconvolve = melting ? 0.0 : G_MAXDOUBLE;
 
     npart = args->coverage * (guint64)(xres*yres);
-    gwy_app_wait_set_message(_("Depositing particles..."));
-    gwy_app_wait_set_fraction(0.0);
+    if (!gwy_app_wait_set_message(_("Depositing particles...")))
+        goto fail;
 
     rngset = gwy_rand_gen_set_new(1);
     gwy_rand_gen_set_init(rngset, args->seed);
@@ -888,68 +867,66 @@ col_synth_do(const ColSynthArgs *args,
             convolve_periodic_fast3(dfield, 0.001*sqrt(melting));
             nextconvolve += 0.0003/sqrt(melting) * args->height;
         }
-        if (any_graphs && ip >= nextgraphx) {
+        if (evolution && ip >= nextgraphx) {
             gwy_data_field_invalidate(dfield);
             height = zsum/(xres*yres) * zscale;
             g_array_append_val(evolution[GRAPH_NFLAGS], height);
-            if (evolution[GRAPH_MAX]) {
+
+            if ((ev = gwy_synth_find_evolution_array(evolution,
+                                                     graph_flags, GRAPH_NFLAGS,
+                                                     GRAPH_MAX))) {
                 height = zmax*zscale;
-                g_array_append_val(evolution[GRAPH_MAX], height);
+                g_array_append_val(ev, height);
             }
-            if (evolution[GRAPH_RMS] || evolution[GRAPH_SKEW]) {
+            if (has_basic_stats) {
                 gdouble rms, skew, kurtosis;
                 gwy_data_field_get_stats(dfield, NULL,
                                          NULL, &rms, &skew, &kurtosis);
                 rms *= zscale;
-                if (evolution[GRAPH_RMS])
-                    g_array_append_val(evolution[GRAPH_RMS], rms);
-                if (evolution[GRAPH_SKEW])
-                    g_array_append_val(evolution[GRAPH_SKEW], skew);
-                if (evolution[GRAPH_KURTOSIS])
-                    g_array_append_val(evolution[GRAPH_KURTOSIS], kurtosis);
+                if ((ev = gwy_synth_find_evolution_array(evolution,
+                                                         graph_flags,
+                                                         GRAPH_NFLAGS,
+                                                         GRAPH_RMS)))
+                    g_array_append_val(ev, rms);
+                if ((ev = gwy_synth_find_evolution_array(evolution,
+                                                         graph_flags,
+                                                         GRAPH_NFLAGS,
+                                                         GRAPH_SKEW)))
+                    g_array_append_val(ev, skew);
+                if ((ev = gwy_synth_find_evolution_array(evolution,
+                                                         graph_flags,
+                                                         GRAPH_NFLAGS,
+                                                         GRAPH_KURTOSIS)))
+                    g_array_append_val(ev, kurtosis);
             }
-            if (evolution[GRAPH_NMAX]) {
+            if ((ev = gwy_synth_find_evolution_array(evolution,
+                                                     graph_flags, GRAPH_NFLAGS,
+                                                     GRAPH_NMAX))) {
                 gdouble nmax;
                 nmax = gwy_data_field_count_maxima(dfield);
-                g_array_append_val(evolution[GRAPH_NMAX], nmax);
+                g_array_append_val(ev, nmax);
             }
-            if (evolution[GRAPH_CORRLEN]) {
+            if ((ev = gwy_synth_find_evolution_array(evolution,
+                                                     graph_flags, GRAPH_NFLAGS,
+                                                     GRAPH_CORRLEN))) {
                 gdouble corrlen = zero_crossing_corrlen(dfield, &acf);
-                g_array_append_val(evolution[GRAPH_CORRLEN], corrlen);
+                g_array_append_val(ev, corrlen);
             }
 
             nextgraphx = 1.2*nextgraphx + 1.0;
         }
     }
 
-    if (gcmodels) {
-        const gdouble *xdata;
-        for (i = 0; i < GRAPH_NFLAGS; i++) {
-            if (!evolution[i]) {
-                gcmodels[i] = NULL;
-                continue;
-            }
-
-            xdata = (const gdouble*)evolution[GRAPH_NFLAGS]->data;
-            gcmodels[i] = gwy_graph_curve_model_new();
-            gwy_graph_curve_model_set_data(gcmodels[i],
-                                           xdata,
-                                           (gdouble*)evolution[i]->data,
-                                           evolution[GRAPH_NFLAGS]->len);
-            g_object_set(gcmodels[i], "description", _(graph_flags[i]), NULL);
-        }
-    }
+    gwy_synth_make_evolution_plots(gcmodels, evolution,
+                                   graph_flags, GRAPH_NFLAGS);
 
     finished = TRUE;
 
 fail:
     g_timer_destroy(timer);
-    gwy_rand_gen_set_free(rngset);
-    for (i = 0; i <= GRAPH_NFLAGS; i++) {
-        if (evolution[i])
-            g_array_free(evolution[i], TRUE);
-    }
-    g_free(evolution);
+    if (rngset)
+        gwy_rand_gen_set_free(rngset);
+    gwy_synth_free_evolution_arrays(evolution, GRAPH_NFLAGS);
     GWY_OBJECT_UNREF(acf);
 
     return finished;
@@ -1097,6 +1074,7 @@ col_synth_sanitize_args(ColSynthArgs *args)
     args->phi_spread = CLAMP(args->phi_spread, 0.0, 1.0);
     args->relaxation = MIN(args->relaxation, RELAX_STRONG);
     args->melting = CLAMP(args->melting, 0.0, 1.0);
+    args->graph_flags &= GRAPH_ALL;
 }
 
 static void
@@ -1104,9 +1082,6 @@ col_synth_load_args(GwyContainer *container,
                     ColSynthArgs *args,
                     GwyDimensionArgs *dimsargs)
 {
-    guint i;
-    gint gflags = 0;
-
     *args = col_synth_defaults;
 
     gwy_container_gis_int32_by_name(container, active_page_key,
@@ -1128,10 +1103,8 @@ col_synth_load_args(GwyContainer *container,
     gwy_container_gis_double_by_name(container, coverage_key, &args->coverage);
     gwy_container_gis_enum_by_name(container, relaxation_key, &args->relaxation);
     gwy_container_gis_double_by_name(container, melting_key, &args->melting);
-    gwy_container_gis_int32_by_name(container, graph_flags_key, &gflags);
-    for (i = 0; i < GRAPH_NFLAGS; i++)
-        args->graph_flags[i] = !!(gflags & (1 << i));
-
+    gwy_container_gis_int32_by_name(container, graph_flags_key,
+                                    &args->graph_flags);
     col_synth_sanitize_args(args);
 
     gwy_clear(dimsargs, 1);
@@ -1144,12 +1117,6 @@ col_synth_save_args(GwyContainer *container,
                     const ColSynthArgs *args,
                     const GwyDimensionArgs *dimsargs)
 {
-    guint i;
-    gint gflags = 0;
-
-    for (i = 0; i < GRAPH_NFLAGS; i++)
-        gflags |= (args->graph_flags[i] ? (1 << i) : 0);
-
     gwy_container_set_int32_by_name(container, active_page_key,
                                     args->active_page);
     gwy_container_set_int32_by_name(container, seed_key, args->seed);
@@ -1169,7 +1136,8 @@ col_synth_save_args(GwyContainer *container,
     gwy_container_set_double_by_name(container, coverage_key, args->coverage);
     gwy_container_set_enum_by_name(container, relaxation_key, args->relaxation);
     gwy_container_set_double_by_name(container, melting_key, args->melting);
-    gwy_container_set_int32_by_name(container, graph_flags_key, gflags);
+    gwy_container_set_int32_by_name(container, graph_flags_key,
+                                    args->graph_flags);
 
     gwy_dimensions_save_args(dimsargs, container, prefix);
 }
