@@ -1,5 +1,5 @@
 /*
- *  $Id: deposit_synth.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: deposit_synth.c 21408 2018-09-07 13:53:53Z yeti-dn $
  *  Copyright (C) 2007,2009,2010,2017 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -160,7 +160,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Generates particles using simple dynamical model"),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "1.5",
+    "1.6",
     "Petr Klapetek",
     "2010",
 };
@@ -869,8 +869,8 @@ deposit_synth_do(const DepositSynthArgs *args,
     gdouble norm;
 
     /* The units must be the same. */
-    gwy_serializable_clone(G_OBJECT(gwy_data_field_get_si_unit_xy(dfield)),
-                           G_OBJECT(gwy_data_field_get_si_unit_z(dfield)));
+    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield),
+                       gwy_data_field_get_si_unit_xy(dfield));
 
     //FIXME renormalize everything for size of field 1x1, including z. Change parameters of potentials.
     norm = 1/gwy_data_field_get_xreal(dfield);
@@ -961,11 +961,12 @@ deposit_synth_do(const DepositSynthArgs *args,
 
     zlfield = gwy_data_field_duplicate(lfield);
     zdfield = gwy_data_field_duplicate(dfield);
-    gwy_app_wait_set_message(_("Initial particle set..."));
+    ndata = steps = 0;
+    if (!gwy_app_wait_set_message(_("Initial particle set...")))
+        goto fail;
 
     /*FIXME determine max number of particles and alloc only necessary field sizes*/
 
-    ndata = steps = 0;
     presetval = args->coverage/100 * xreal*yreal/(G_PI*size*size);
     //printf("%d particles should be deposited\n", presetval);
 
@@ -983,10 +984,9 @@ deposit_synth_do(const DepositSynthArgs *args,
         too_close = 0;
 
         /*sync real to integer positions*/
-        for (k=0; k<ndata; k++)
-        {
-            if (((xpos-xdata[k])*(xpos-xdata[k]) + (ypos-ydata[k])*(ypos-ydata[k]))<(4*disize*disize))
-            {
+        for (k = 0; k < ndata; k++) {
+            if ((xpos-xdata[k])*(xpos-xdata[k])
+                + (ypos-ydata[k])*(ypos-ydata[k]) < 4*disize*disize) {
                 too_close = 1;
                 break;
             }
@@ -1186,6 +1186,7 @@ deposit_synth_do(const DepositSynthArgs *args,
             break;
     }
 
+fail:
     gwy_data_field_copy(zlfield, lfield, 0);
 
     showit(lfield, zdfield,
