@@ -1,6 +1,6 @@
 /*
- *  $Id: diff_synth.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2014 David Necas (Yeti).
+ *  $Id: diff_synth.c 21408 2018-09-07 13:53:53Z yeti-dn $
+ *  Copyright (C) 2014-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -46,9 +46,9 @@ enum {
 };
 
 typedef enum {
-    GRAPH_VAR     = 0,
-    GRAPH_NGRAINS = 1,
-    GRAPH_NFLAGS,
+    GRAPH_VAR     = (1 << 0),
+    GRAPH_NGRAINS = (1 << 1),
+    GRAPH_ALL     = (1 << 2) - 1,
 } GraphFlags;
 
 typedef enum {
@@ -106,7 +106,7 @@ typedef struct {
     gdouble p_stick;
     gdouble p_break;
     gdouble schwoebel; /* Log10 */
-    gboolean graph_flags[GRAPH_NFLAGS];
+    guint graph_flags;
 } DiffSynthArgs;
 
 struct _DiffSynthControls {
@@ -127,7 +127,7 @@ struct _DiffSynthControls {
     GtkObject *height;
     GtkWidget *height_units;
     GtkWidget *height_init;
-    GtkWidget *graph_flags[GRAPH_NFLAGS];
+    GSList *graph_flags;
     GwyContainer *mydata;
     GwyDataField *surface;
     gdouble pxsize;
@@ -191,9 +191,13 @@ static void               diff_synth_save_args     (GwyContainer *container,
 
 #include "synth.h"
 
-static const gchar* graph_flags[GRAPH_NFLAGS + 1/*unused*/] = {
-    N_("Variation"),
-    N_("Number of islands"),
+static const GwyEnum graph_flags[] = {
+    { N_("Variation"),         GRAPH_VAR,    },
+    { N_("Number of islands"), GRAPH_NGRAINS },
+};
+
+enum {
+    GRAPH_NFLAGS = G_N_ELEMENTS(graph_flags)
 };
 
 static const DiffSynthArgs diff_synth_defaults = {
@@ -201,7 +205,7 @@ static const DiffSynthArgs diff_synth_defaults = {
     42, TRUE, TRUE,
     1.0,
     0.25, -10.0, 0.1, 0.01, 0.0,
-    { FALSE, FALSE, },
+    0,
 };
 
 static const GwyDimensionArgs dims_defaults = GWY_DIMENSION_ARGS_INIT;
@@ -211,7 +215,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Generates surfaces by diffusion limited aggregation."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "1.3",
     "David Nečas (Yeti)",
     "2014",
 };
@@ -344,7 +348,6 @@ run_noninteractive(DiffSynthArgs *args,
 
     for (i = 0; i < GRAPH_NFLAGS; i++) {
         GwyGraphModel *gmodel;
-        GwySIUnit *unit;
         gchar *s, *title;
 
         if (!gcmodels[i])
@@ -355,31 +358,24 @@ run_noninteractive(DiffSynthArgs *args,
         g_object_unref(gcmodels[i]);
 
         s = gwy_app_get_data_field_title(data, newid);
-        title = g_strdup_printf("%s (%s)", graph_flags[i], s);
+        title = g_strdup_printf("%s (%s)", _(graph_flags[i].name), s);
         g_free(s);
         g_object_set(gmodel,
                      "title", title,
                      "x-logarithmic", TRUE,
                      "y-logarithmic", i != GRAPH_NGRAINS,
                      "axis-label-bottom", _("Mean deposited thickness"),
-                     "axis-label-left", _(graph_flags[i]),
+                     "axis-label-left", _(graph_flags[i].name),
                      NULL);
         g_free(title);
 
-        unit = gwy_data_field_get_si_unit_z(newfield);
-        unit = gwy_si_unit_duplicate(unit);
-        g_object_set(gmodel, "si-unit-x", unit, NULL);
-        g_object_unref(unit);
-
-        if (i == GRAPH_VAR) {
-            unit = gwy_si_unit_multiply(gwy_data_field_get_si_unit_z(newfield),
-                                        gwy_data_field_get_si_unit_xy(newfield),
-                                        NULL);
-            g_object_set(gmodel, "si-unit-y", unit, NULL);
-            g_object_unref(unit);
+        if (graph_flags[i].value == GRAPH_VAR) {
+            gwy_graph_model_set_units_from_data_field(gmodel, newfield,
+                                                      0, 1, 1, 1);
         }
-        else if (i == GRAPH_NGRAINS) {
-            /* Unitless. */
+        else {
+            gwy_graph_model_set_units_from_data_field(gmodel, newfield,
+                                                      0, 1, 0, 0);
         }
 
         gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
@@ -397,8 +393,7 @@ diff_synth_dialog(DiffSynthArgs *args,
     DiffSynthControls controls;
     GwyDataField *dfield;
     gboolean finished;
-    gint response;
-    gint row, i;
+    gint response, row;
 
     gwy_clear(&controls, 1);
     controls.in_init = TRUE;
@@ -557,19 +552,12 @@ diff_synth_dialog(DiffSynthArgs *args,
                      0, 3, row, row+1, GTK_FILL, 0, 0, 0);
     row++;
 
-    for (i = 0; i < GRAPH_NFLAGS; i++) {
-        GtkToggleButton *toggle;
-        controls.graph_flags[i]
-            = gtk_check_button_new_with_label(_(graph_flags[i]));
-        toggle = GTK_TOGGLE_BUTTON(controls.graph_flags[i]);
-        gtk_toggle_button_set_active(toggle, args->graph_flags[i]);
-        gtk_table_attach(GTK_TABLE(table), controls.graph_flags[i],
-                         0, 2, row, row+1, GTK_FILL, 0, 0, 0);
-        g_signal_connect(toggle, "toggled",
-                         G_CALLBACK(gwy_synth_boolean_changed_silent),
-                         &args->graph_flags[i]);
-        row++;
-    }
+    controls.graph_flags
+        = gwy_check_boxes_create(graph_flags, GRAPH_NFLAGS,
+                                 G_CALLBACK(gwy_synth_flag_changed_silent),
+                                 &args->graph_flags, args->graph_flags);
+    row = gwy_check_boxes_attach_to_table(controls.graph_flags,
+                                          GTK_TABLE(table), 3, row);
 
     gtk_widget_show_all(dialog);
     controls.in_init = FALSE;
@@ -619,8 +607,6 @@ static void
 update_controls(DiffSynthControls *controls,
                 DiffSynthArgs *args)
 {
-    guint i;
-
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->seed), args->seed);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls->randomize),
                                  args->randomize);
@@ -634,10 +620,7 @@ update_controls(DiffSynthControls *controls,
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->p_stick), args->p_stick);
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->schwoebel),
                              args->schwoebel);
-    for (i = 0; i < GRAPH_NFLAGS; i++) {
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls->graph_flags[i]),
-                                     args->graph_flags[i]);
-    }
+    gwy_check_boxes_set_selected(controls->graph_flags, args->graph_flags);
 }
 
 static void
@@ -731,31 +714,22 @@ diff_synth_do(DiffSynthArgs *args,
               gdouble zscale)
 {
     gint xres, yres, k;
-    guint i;
     gdouble nextgraphx;
     gdouble lasttime = 0.0, lastpreviewtime = 0.0, currtime;
     gdouble height, threshold;
     gint *grains = NULL;
     GTimer *timer;
     guint64 workdone, niter, iter;
-    DiffSynthState *dstate;
+    DiffSynthState *dstate = NULL;
     GArray *particles;
-    GArray **evolution = NULL;
-    gboolean any_graphs = FALSE;
+    GArray **evolution = NULL, *ev;
     gboolean finished = FALSE;
 
     timer = g_timer_new();
 
-    evolution = g_new0(GArray*, GRAPH_NFLAGS + 1);
     if (gcmodels) {
-        for (i = 0; i < GRAPH_NFLAGS; i++) {
-            if (args->graph_flags[i]) {
-                evolution[i] = g_array_new(FALSE, FALSE, sizeof(gdouble));
-                any_graphs = TRUE;
-            }
-        }
-        if (any_graphs)
-            evolution[GRAPH_NFLAGS] = g_array_new(FALSE, FALSE, sizeof(gdouble));
+        evolution = gwy_synth_make_evolution_arrays(graph_flags, GRAPH_NFLAGS,
+                                                    args->graph_flags);
     }
 
     xres = gwy_data_field_get_xres(dfield);
@@ -768,8 +742,8 @@ diff_synth_do(DiffSynthArgs *args,
     }
     nextgraphx = 0.0;
 
-    gwy_app_wait_set_message(_("Depositing particles..."));
-    gwy_app_wait_set_fraction(0.0);
+    if (!gwy_app_wait_set_message(_("Depositing particles...")))
+        goto fail;
 
     random_direction(NULL);
 
@@ -815,19 +789,23 @@ diff_synth_do(DiffSynthArgs *args,
             workdone -= WORK_UPDATE_CHECK;
         }
 
-        if (any_graphs && iter >= nextgraphx) {
+        if (evolution && iter >= nextgraphx) {
             copy_hfield_to_data_field(dstate, dfield, zscale);
             height = iter*dstate->fluxperiter * zscale;
             g_array_append_val(evolution[GRAPH_NFLAGS], height);
-            if (evolution[GRAPH_VAR]) {
+            if ((ev = gwy_synth_find_evolution_array(evolution,
+                                                     graph_flags, GRAPH_NFLAGS,
+                                                     GRAPH_VAR))) {
                 gdouble var = gwy_data_field_get_variation(dfield);
-                g_array_append_val(evolution[GRAPH_VAR], var);
+                g_array_append_val(ev, var);
             }
-            if (evolution[GRAPH_NGRAINS]) {
+            if ((ev = gwy_synth_find_evolution_array(evolution,
+                                                     graph_flags, GRAPH_NFLAGS,
+                                                     GRAPH_NGRAINS))) {
                 gdouble ngrains;
                 gwy_clear(grains, xres*yres);
                 ngrains = gwy_data_field_number_grains_periodic(dfield, grains);
-                g_array_append_val(evolution[GRAPH_NGRAINS], ngrains);
+                g_array_append_val(ev, ngrains);
             }
 
             nextgraphx += 0.0001/dstate->flux + MIN(0.2*nextgraphx,
@@ -836,35 +814,17 @@ diff_synth_do(DiffSynthArgs *args,
     }
     finalize_moving_particles(dstate);
 
-    if (gcmodels) {
-        const gdouble *xdata;
-        for (i = 0; i < GRAPH_NFLAGS; i++) {
-            if (!evolution[i]) {
-                gcmodels[i] = NULL;
-                continue;
-            }
-
-            xdata = (const gdouble*)evolution[GRAPH_NFLAGS]->data;
-            gcmodels[i] = gwy_graph_curve_model_new();
-            gwy_graph_curve_model_set_data(gcmodels[i],
-                                           xdata,
-                                           (gdouble*)evolution[i]->data,
-                                           evolution[GRAPH_NFLAGS]->len);
-            g_object_set(gcmodels[i], "description", _(graph_flags[i]), NULL);
-        }
-    }
+    gwy_synth_make_evolution_plots(gcmodels, evolution,
+                                   graph_flags, GRAPH_NFLAGS);
 
     copy_hfield_to_data_field(dstate, dfield, zscale);
     finished = TRUE;
 
 fail:
     g_timer_destroy(timer);
-    diff_synth_state_free(dstate);
-    for (i = 0; i <= GRAPH_NFLAGS; i++) {
-        if (evolution[i])
-            g_array_free(evolution[i], TRUE);
-    }
-    g_free(evolution);
+    if (dstate)
+        diff_synth_state_free(dstate);
+    gwy_synth_free_evolution_arrays(evolution, GRAPH_NFLAGS);
     g_free(grains);
 
     return finished;
@@ -1120,6 +1080,7 @@ diff_synth_sanitize_args(DiffSynthArgs *args)
     args->p_stick = CLAMP(args->p_stick, 0.0, 1.0);
     args->p_break = CLAMP(args->p_break, 0.0, 1.0);
     args->schwoebel = CLAMP(args->schwoebel, -12.0, 0.0);
+    args->graph_flags &= GRAPH_ALL;
 }
 
 static void
@@ -1127,9 +1088,6 @@ diff_synth_load_args(GwyContainer *container,
                      DiffSynthArgs *args,
                      GwyDimensionArgs *dimsargs)
 {
-    guint i;
-    gint gflags = 0;
-
     *args = diff_synth_defaults;
 
     gwy_container_gis_int32_by_name(container, active_page_key,
@@ -1145,9 +1103,8 @@ diff_synth_load_args(GwyContainer *container,
     gwy_container_gis_double_by_name(container, p_stick_key, &args->p_stick);
     gwy_container_gis_double_by_name(container, p_break_key, &args->p_break);
     gwy_container_gis_double_by_name(container, schwoebel_key, &args->schwoebel);
-    gwy_container_gis_int32_by_name(container, graph_flags_key, &gflags);
-    for (i = 0; i < GRAPH_NFLAGS; i++)
-        args->graph_flags[i] = !!(gflags & (1 << i));
+    gwy_container_gis_int32_by_name(container, graph_flags_key,
+                                    &args->graph_flags);
 
     diff_synth_sanitize_args(args);
 
@@ -1161,12 +1118,6 @@ diff_synth_save_args(GwyContainer *container,
                     const DiffSynthArgs *args,
                     const GwyDimensionArgs *dimsargs)
 {
-    guint i;
-    gint gflags = 0;
-
-    for (i = 0; i < GRAPH_NFLAGS; i++)
-        gflags |= (args->graph_flags[i] ? (1 << i) : 0);
-
     gwy_container_set_int32_by_name(container, active_page_key,
                                     args->active_page);
     gwy_container_set_int32_by_name(container, seed_key, args->seed);
@@ -1180,7 +1131,8 @@ diff_synth_save_args(GwyContainer *container,
     gwy_container_set_double_by_name(container, p_stick_key, args->p_stick);
     gwy_container_set_double_by_name(container, p_break_key, args->p_break);
     gwy_container_set_double_by_name(container, schwoebel_key, args->schwoebel);
-    gwy_container_set_int32_by_name(container, graph_flags_key, gflags);
+    gwy_container_set_int32_by_name(container, graph_flags_key,
+                                    args->graph_flags);
 
     gwy_dimensions_save_args(dimsargs, container, prefix);
 }
