@@ -1,5 +1,5 @@
 /*
- *  $Id: domain_synth.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: domain_synth.c 21408 2018-09-07 13:53:53Z yeti-dn $
  *  Copyright (C) 2014 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -162,7 +162,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Generates domain images using a hybrid Ising model."),
     "Yeti <yeti@gwyddion.net>",
-    "1.0",
+    "1.1",
     "David Nečas (Yeti)",
     "2014",
 };
@@ -897,7 +897,7 @@ domain_synth_do(const DomainSynthArgs *args,
     gulong i;
     gdouble lasttime = 0.0, lastpreviewtime = 0.0, currtime;
     GTimer *timer;
-    gint *u, *ubuf;
+    gint *u = NULL, *ubuf = NULL;
     guint k;
     gboolean finished = FALSE;
 
@@ -906,8 +906,8 @@ domain_synth_do(const DomainSynthArgs *args,
     xres = gwy_data_field_get_xres(ufield);
     yres = gwy_data_field_get_yres(ufield);
 
-    gwy_app_wait_set_message(_("Running computation..."));
-    gwy_app_wait_set_fraction(0.0);
+    if (!gwy_app_wait_set_message(_("Running computation...")))
+        goto fail;
 
     gwy_data_field_clear(vfield);
     u = g_new(gint, xres*yres);
