@@ -1,5 +1,5 @@
 /*
- *  $Id: edge.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: edge.c 21410 2018-09-07 14:15:50Z yeti-dn $
  *  Copyright (C) 2003-2007,2012 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -423,8 +423,8 @@ nonlinearity_do(GwyDataField *dfield, GwyDataField *show)
     xres = gwy_data_field_get_xres(dfield);
     yres = gwy_data_field_get_yres(dfield);
     d = gwy_data_field_get_data(show);
-    qx = gwy_data_field_get_xmeasure(dfield);
-    qy = gwy_data_field_get_xmeasure(dfield);
+    qx = gwy_data_field_get_dx(dfield);
+    qy = gwy_data_field_get_dx(dfield);
 
     size = gwy_data_field_get_circular_area_size(r);
     z = g_new(gdouble, size);
@@ -459,8 +459,8 @@ inclination_do(GwyDataField *dfield, GwyDataField *show)
     xres = gwy_data_field_get_xres(dfield);
     yres = gwy_data_field_get_yres(dfield);
     d = gwy_data_field_get_data(show);
-    qx = gwy_data_field_get_xmeasure(dfield);
-    qy = gwy_data_field_get_xmeasure(dfield);
+    qx = gwy_data_field_get_dx(dfield);
+    qy = gwy_data_field_get_dx(dfield);
 
     size = gwy_data_field_get_circular_area_size(r);
     z = g_new(gdouble, size);
@@ -685,7 +685,7 @@ zero_crossing_dialog(ZeroCrossingArgs *args,
     gtk_table_set_row_spacing(GTK_TABLE(table), row, 8);
     row++;
 
-    label = gtk_label_new(gwy_sgettext("verb|Display"));
+    label = gtk_label_new(_("Display:"));
     gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
     gtk_table_attach(GTK_TABLE(table), label,
                      0, 2, row, row+1, GTK_FILL, 0, 0, 0);
