@@ -1,5 +1,5 @@
 /*
- *  $Id: facet_analysis.c 20756 2018-01-24 14:23:24Z yeti-dn $
+ *  $Id: facet_analysis.c 21435 2018-09-11 12:39:53Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -534,8 +534,7 @@ facets_dialog(FacetsArgs *args,
     gwy_selection_set_max_objects(selection, 1024);
     selkey = g_strdup_printf("/%d/select/_facets", id);
     if (gwy_container_gis_object_by_name(data, selkey, &selection)) {
-        gwy_serializable_clone(G_OBJECT(selection),
-                               G_OBJECT(controls.fselection));
+        gwy_selection_assign(controls.fselection, selection);
         selection = controls.fselection;
         restored_selection = TRUE;
     }
@@ -1066,7 +1065,7 @@ refine_facet_selection(FacetsControls *controls)
 
     dist = gwy_container_get_object_by_name(controls->fdata, "/0/data");
     fres = gwy_data_field_get_xres(dist);
-    h = gwy_data_field_get_xmeasure(dist);
+    h = gwy_data_field_get_dx(dist);
     range = GWY_ROUND(fres/controls->q * 0.5/G_SQRT2 * cos(0.5*theta)
                       * controls->args->tolerance);
     x = xy[0]/h;
@@ -1347,8 +1346,7 @@ facet_view_selection_updated(GwySelection *selection,
      * save the current selection as the base for the rotation.  When we are
      * rotating, do not touch fselection0. */
     if (!controls->is_rotating) {
-        gwy_serializable_clone(G_OBJECT(controls->fselection),
-                               G_OBJECT(controls->fselection0));
+        gwy_selection_assign(controls->fselection0, controls->fselection);
         controls->in_update = TRUE;
         gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->rot_theta), 0.0);
         gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->rot_phi), 0.0);
