@@ -1,6 +1,6 @@
 /*
- *  $Id: fit-shape.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2016-2017 David Necas (Yeti).
+ *  $Id: fit-shape.c 21444 2018-09-18 08:09:35Z yeti-dn $
+ *  Copyright (C) 2016-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,10 @@
  */
 
 /* TODO:
+ * - Narrow Gaussians are completely unstable.  All the other bump-like
+ *   functions can be fitted normally, despite having essentially the same
+ *   parameters and similar shape.  Why?  The only difference between Gaussian
+ *   and Lorentzian bump is the expressions exp(-0.5*t) and 1/(1.0 + t).
  * - Gradient adaptation only works for full-range mapping.  Simply enforce
  *   full-range mapping with adapted gradients?
  * - Report style not controllable.
@@ -128,6 +132,7 @@ typedef struct {
     GwyShapeFitPreset *preset;
     gdouble *param;
     gdouble *alt_param;
+    gboolean *param_edited;
     gdouble *param_err;
     gdouble *correl;
     gdouble *secondary;
@@ -210,6 +215,8 @@ static void         fix_changed               (GtkToggleButton *button,
                                                FitShapeControls *controls);
 static void         param_value_activate      (GtkEntry *entry,
                                                FitShapeControls *controls);
+static void         param_value_edited        (GtkEntry *entry,
+                                               FitShapeControls *controls);
 static void         update_all_param_values   (FitShapeControls *controls);
 static void         revert_params             (FitShapeControls *controls);
 static void         calculate_secondary_params(FitShapeControls *controls);
@@ -267,7 +274,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Fits predefined geometrical shapes to data."),
     "Yeti <yeti@gwyddion.net>",
-    "1.4",
+    "1.6",
     "David Nečas (Yeti)",
     "2016",
 };
@@ -561,6 +568,7 @@ finalise:
     GWY_OBJECT_UNREF(controls.results);
     g_free(controls.param);
     g_free(controls.alt_param);
+    g_free(controls.param_edited);
     g_free(controls.param_err);
     g_free(controls.secondary);
     g_free(controls.secondary_err);
@@ -889,6 +897,8 @@ fit_param_table_resize(FitShapeControls *controls)
         g_object_set_data(G_OBJECT(cntrl.value), "id", GUINT_TO_POINTER(i));
         g_signal_connect(cntrl.value, "activate",
                          G_CALLBACK(param_value_activate), controls);
+        g_signal_connect(cntrl.value, "changed",
+                         G_CALLBACK(param_value_edited), controls);
         gwy_widget_set_activate_on_unfocus(cntrl.value, TRUE);
 
         cntrl.value_unit = gtk_label_new(NULL);
@@ -1207,14 +1217,18 @@ function_changed(GtkComboBox *combo, FitShapeControls *controls)
 
     controls->param = g_renew(gdouble, controls->param, nparams);
     controls->alt_param = g_renew(gdouble, controls->alt_param, nparams);
+    controls->param_edited = g_renew(gboolean, controls->param_edited, nparams);
     controls->param_err = g_renew(gdouble, controls->param_err, nparams);
     controls->secondary = g_renew(gdouble, controls->secondary, nsecondary);
     controls->secondary_err = g_renew(gdouble, controls->secondary_err,
                                       nsecondary);
     controls->correl = g_renew(gdouble, controls->correl,
                                (nparams + 1)*nparams/2);
-    for (i = 0; i < nparams; i++)
+    for (i = 0; i < nparams; i++) {
         controls->param_err[i] = -1.0;
+        /* Start from what is shown in the UI. */
+        controls->param_edited[i] = TRUE;
+    }
     fit_param_table_resize(controls);
     fit_correl_table_resize(controls);
     fit_secondary_table_resize(controls);
@@ -1321,6 +1335,26 @@ fix_changed(GtkToggleButton *button, FitShapeControls *controls)
     ctx->param_fixed[i] = fixed;
 }
 
+static gdouble
+transform_value(gdouble v, GwyNLFitParamFlags flags)
+{
+    if (flags & GWY_NLFIT_PARAM_ANGLE)
+        v *= 180.0/G_PI;
+    if (flags & GWY_NLFIT_PARAM_ABSVAL)
+        v = fabs(v);
+    return v;
+}
+
+static gdouble
+transform_value_back(gdouble v, GwyNLFitParamFlags flags)
+{
+    if (flags & GWY_NLFIT_PARAM_ANGLE)
+        v *= G_PI/180.0;
+    if (flags & GWY_NLFIT_PARAM_ABSVAL)
+        v = fabs(v);
+    return v;
+}
+
 static void
 update_param_value(FitShapeControls *controls, guint i)
 {
@@ -1333,10 +1367,7 @@ update_param_value(FitShapeControls *controls, guint i)
     flags = gwy_shape_fit_preset_get_param_flags(controls->preset, i);
     controls->param[i] = g_strtod(gtk_entry_get_text(entry), NULL);
     controls->param[i] *= cntrl->magnitude;
-    if (flags & GWY_NLFIT_PARAM_ANGLE)
-        controls->param[i] *= G_PI/180.0;
-    if (flags & GWY_NLFIT_PARAM_ABSVAL)
-        controls->param[i] = fabs(controls->param[i]);
+    controls->param[i] = transform_value_back(controls->param[i], flags);
 }
 
 static void
@@ -1355,12 +1386,22 @@ param_value_activate(GtkEntry *entry, FitShapeControls *controls)
     update_fit_state(controls);
 }
 
+static void
+param_value_edited(GtkEntry *entry, FitShapeControls *controls)
+{
+    guint i = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(entry), "id"));
+
+    controls->param_edited[i] = TRUE;
+}
+
 static void
 update_all_param_values(FitShapeControls *controls)
 {
     guint i;
-    for (i = 0; i < controls->param_controls->len; i++)
-        update_param_value(controls, i);
+    for (i = 0; i < controls->param_controls->len; i++) {
+        if (controls->param_edited[i])
+            update_param_value(controls, i);
+    }
 }
 
 static void
@@ -1370,8 +1411,11 @@ revert_params(FitShapeControls *controls)
 
     nparams = gwy_shape_fit_preset_get_nparams(controls->preset);
     update_all_param_values(controls);
-    for (i = 0; i < nparams; i++)
+    for (i = 0; i < nparams; i++) {
+        if (controls->param[i] != controls->alt_param[i])
+            controls->param_edited[i] = TRUE;
         GWY_SWAP(gdouble, controls->param[i], controls->alt_param[i]);
+    }
 
     controls->state = FIT_SHAPE_USER;
     calculate_secondary_params(controls);
@@ -1395,6 +1439,7 @@ static void
 update_param_table(FitShapeControls *controls,
                    const gdouble *param, const gdouble *param_err)
 {
+    const GwySIUnitFormatStyle style = GWY_SI_UNIT_FORMAT_VFMARKUP;
     guint i, nparams;
     GwyDataField *dfield;
     GwySIUnit *unit, *xyunit, *zunit;
@@ -1410,15 +1455,13 @@ update_param_table(FitShapeControls *controls,
         FitParamControl *cntrl = &g_array_index(controls->param_controls,
                                                 FitParamControl, i);
         GwyNLFitParamFlags flags;
-        guchar buf[32];
-        gdouble v;
+        guchar buf[64];
+        gdouble v, e;
 
         flags = gwy_shape_fit_preset_get_param_flags(controls->preset, i);
-        v = param[i];
-        if (flags & GWY_NLFIT_PARAM_ANGLE) {
-            v *= 180.0/G_PI;
+        v = transform_value(param[i], flags);
+        if (flags & GWY_NLFIT_PARAM_ANGLE)
             gwy_si_unit_set_from_string(unit, "deg");
-        }
         else {
             g_object_unref(unit);
             unit = gwy_shape_fit_preset_get_param_units(controls->preset, i,
@@ -1428,15 +1471,20 @@ update_param_table(FitShapeControls *controls,
          * that means an unexpected reset to base units. */
         if (G_UNLIKELY(v == 0.0)) {
             gint power10 = GWY_ROUND(log10(cntrl->magnitude));
-            vf = gwy_si_unit_get_format_for_power10(unit,
-                                                    GWY_SI_UNIT_FORMAT_VFMARKUP,
-                                                    power10, vf);
+            vf = gwy_si_unit_get_format_for_power10(unit, style, power10, vf);
+            vf->precision += 3;
+        }
+        else if (param_err && param_err[i]) {
+            e = transform_value(param_err[i], flags);
+            vf = gwy_si_unit_get_format_with_resolution(unit, style,
+                                                        v, fmin(0.1*e, 0.01*v),
+                                                        vf);
         }
         else {
-            vf = gwy_si_unit_get_format(unit, GWY_SI_UNIT_FORMAT_VFMARKUP,
-                                        v, vf);
+            vf = gwy_si_unit_get_format(unit, style, v, vf);
+            vf->precision += 3;
         }
-        g_snprintf(buf, sizeof(buf), "%.*f", vf->precision+3, v/vf->magnitude);
+        g_snprintf(buf, sizeof(buf), "%.*f", vf->precision, v/vf->magnitude);
         gtk_entry_set_text(GTK_ENTRY(cntrl->value), buf);
         gtk_label_set_markup(GTK_LABEL(cntrl->value_unit), vf->units);
         cntrl->magnitude = vf->magnitude;
@@ -1447,10 +1495,8 @@ update_param_table(FitShapeControls *controls,
             continue;
         }
 
-        v = param_err[i];
-        if (flags & GWY_NLFIT_PARAM_ANGLE)
-            v *= 180.0/G_PI;
-        vf = gwy_si_unit_get_format(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, v, vf);
+        v = transform_value(param_err[i], flags);
+        vf = gwy_si_unit_get_format(unit, style, v, vf);
         g_snprintf(buf, sizeof(buf), "%.*f", vf->precision, v/vf->magnitude);
         gtk_label_set_text(GTK_LABEL(cntrl->error), buf);
         gtk_label_set_markup(GTK_LABEL(cntrl->error_unit), vf->units);
@@ -1522,6 +1568,7 @@ update_correl_table(FitShapeControls *controls, GwyNLFitter *fitter)
 static void
 update_secondary_table(FitShapeControls *controls)
 {
+    const GwySIUnitFormatStyle style = GWY_SI_UNIT_FORMAT_VFMARKUP;
     guint i, nsecondary;
     GwyDataField *dfield;
     GwySIUnit *unit, *xyunit, *zunit;
@@ -1539,22 +1586,29 @@ update_secondary_table(FitShapeControls *controls)
         FitParamControl *cntrl = &g_array_index(controls->secondary_controls,
                                                 FitParamControl, i);
         GwyNLFitParamFlags flags;
-        guchar buf[32];
-        gdouble v;
+        guchar buf[64];
+        gdouble v, e;
 
         flags = gwy_shape_fit_preset_get_secondary_flags(controls->preset, i);
-        v = controls->secondary[i];
-        if (flags & GWY_NLFIT_PARAM_ANGLE) {
-            v *= 180.0/G_PI;
+        v = transform_value(controls->secondary[i], flags);
+        if (flags & GWY_NLFIT_PARAM_ANGLE)
             gwy_si_unit_set_from_string(unit, "deg");
-        }
         else {
             g_object_unref(unit);
             unit = gwy_shape_fit_preset_get_secondary_units(controls->preset, i,
                                                             xyunit, zunit);
         }
-        vf = gwy_si_unit_get_format(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, v, vf);
-        g_snprintf(buf, sizeof(buf), "%.*f", vf->precision+3, v/vf->magnitude);
+        if (is_fitted && controls->secondary_err[i]) {
+            e = transform_value(controls->secondary_err[i], flags);
+            vf = gwy_si_unit_get_format_with_resolution(unit, style,
+                                                        v, fmin(0.1*e, 0.01*v),
+                                                        vf);
+        }
+        else {
+            vf = gwy_si_unit_get_format(unit, style, v, vf);
+            vf->precision += 3;
+        }
+        g_snprintf(buf, sizeof(buf), "%.*f", vf->precision, v/vf->magnitude);
         gtk_label_set_text(GTK_LABEL(cntrl->value), buf);
         gtk_label_set_markup(GTK_LABEL(cntrl->value_unit), vf->units);
 
@@ -1564,10 +1618,8 @@ update_secondary_table(FitShapeControls *controls)
             continue;
         }
 
-        v = controls->secondary_err[i];
-        if (flags & GWY_NLFIT_PARAM_ANGLE)
-            v *= 180.0/G_PI;
-        vf = gwy_si_unit_get_format(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, v, vf);
+        v = transform_value(controls->secondary_err[i], flags);
+        vf = gwy_si_unit_get_format(unit, style, v, vf);
         g_snprintf(buf, sizeof(buf), "%.*f", vf->precision, v/vf->magnitude);
         gtk_label_set_text(GTK_LABEL(cntrl->error), buf);
         gtk_label_set_markup(GTK_LABEL(cntrl->error_unit), vf->units);
@@ -1914,6 +1966,7 @@ update_fit_results(FitShapeControls *controls, GwyNLFitter *fitter)
     if (is_fitted) {
         nparams = gwy_shape_fit_preset_get_nparams(controls->preset);
         for (i = 0; i < nparams; i++) {
+            controls->param_edited[i] = FALSE;
             if (ctx->param_fixed[i])
                 controls->param_err[i] = 0.0;
             else
