@@ -1,5 +1,5 @@
 /*
- *  $Id: grain_filter.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: grain_filter.c 20835 2018-03-13 10:54:57Z yeti-dn $
  *  Copyright (C) 2014-2017 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -22,7 +22,6 @@
 #include "config.h"
 #include <string.h>
 #include <gtk/gtk.h>
-#include <glib/gstdio.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
 #include <libgwyddion/gwyexpr.h>
@@ -1078,7 +1077,7 @@ gfilter_load_args(GwyContainer *container,
                   GFilterArgs *args)
 {
     GwyInventory *inventory;
-    gchar *filename, *buffer;
+    gchar *buffer;
     gsize size;
     guint i;
 
@@ -1100,9 +1099,7 @@ gfilter_load_args(GwyContainer *container,
 
     args->ranges_history = g_hash_table_new_full(g_str_hash, g_str_equal,
                                                  NULL, range_record_free);
-    filename = g_build_filename(gwy_get_user_dir(), "grain_filter", "ranges",
-                                NULL);
-    if (g_file_get_contents(filename, &buffer, &size, NULL)) {
+    if (gwy_module_data_load("grain_filter", "ranges", &buffer, &size, NULL)) {
         gchar *p = buffer, *line;
         for (line = gwy_str_next_line(&p); line; line = gwy_str_next_line(&p)) {
             g_strstrip(line);
@@ -1136,7 +1133,6 @@ gfilter_load_args(GwyContainer *container,
         }
         g_free(buffer);
     }
-    g_free(filename);
 
     gfilter_sanitize_args(args);
 }
@@ -1158,7 +1154,6 @@ static void
 gfilter_save_args(GwyContainer *container,
                   GFilterArgs *args)
 {
-    gchar *filename;
     FILE *fh;
     guint i;
 
@@ -1174,18 +1169,10 @@ gfilter_save_args(GwyContainer *container,
                                          g_strdup(args->ranges[i].quantity));
     }
 
-    filename = g_build_filename(gwy_get_user_dir(), "grain_filter", NULL);
-    if (!g_file_test(filename, G_FILE_TEST_IS_DIR))
-        g_mkdir(filename, 0700);
-    g_free(filename);
-
-    filename = g_build_filename(gwy_get_user_dir(), "grain_filter", "ranges",
-                                NULL);
-    if ((fh = gwy_fopen(filename, "w"))) {
+    if ((fh = gwy_module_data_fopen("grain_filter", "ranges", "w", NULL))) {
         g_hash_table_foreach(args->ranges_history, save_range, fh);
         fclose(fh);
     }
-    g_free(filename);
 }
 
 static void
