@@ -1,6 +1,6 @@
 /*
- *  $Id: grain_stat.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2014-2017 David Necas (Yeti), Petr Klapetek, Sven Neumann.
+ *  $Id: grain_stat.c 21368 2018-08-29 12:47:29Z yeti-dn $
+ *  Copyright (C) 2014-2018 David Necas (Yeti), Petr Klapetek, Sven Neumann.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, neumann@jpk.com.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -40,12 +40,13 @@
 #define INSCRIBE_RUN_MODES GWY_RUN_IMMEDIATE
 
 typedef struct {
+    GwyGrainValue *gvalue;
     gdouble mean;
     gdouble median;
     gdouble rms;
     gdouble q25;
     gdouble q75;
-    GwySIValueFormat *vf;
+    GwySIValueFormat *vf;   /* For the treeview */
 } GrainQuantityStats;
 
 typedef struct {
@@ -60,6 +61,9 @@ typedef struct {
 typedef struct {
     GrainStatArgs *args;
     GrainQuantityStats *stats;
+    GwyDataField *dfield;
+    GtkWidget *rexport;
+    GtkWidget *dialog;
     gboolean same_units;
 } GrainStatsControls;
 
@@ -97,6 +101,12 @@ static void                render_grain_stat      (GtkTreeViewColumn *column,
                                                    GtkTreeModel *model,
                                                    GtkTreeIter *iter,
                                                    gpointer user_data);
+static void                rexport_format_changed (GwyResultsExport *rexport,
+                                                   GwyResultsReportType *report_style);
+static void                grain_stats_copy       (GwyResultsExport *rexport,
+                                                   GrainStatsControls *controls);
+static void                grain_stats_save       (GwyResultsExport *rexport,
+                                                   GrainStatsControls *controls);
 static void                summary_load_args      (GwyContainer *container,
                                                    GrainSummaryArgs *args);
 static void                summary_save_args      (GwyContainer *container,
@@ -120,7 +130,7 @@ static GwyModuleInfo module_info = {
     N_("Displays overall grain statistics."),
     "Petr Klapetek <petr@klapetek.cz>, Sven Neumann <neumann@jpk.com>, "
         "Yeti <yeti@gwyddion.net>",
-    "1.3",
+    "1.4",
     "David Nečas (Yeti) & Petr Klapetek & Sven Neumann",
     "2015",
 };
@@ -328,11 +338,11 @@ grain_summary(GwyContainer *data, GwyRunType run)
     gwy_results_export_set_results(GWY_RESULTS_EXPORT(rexport), results);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), rexport,
                        FALSE, FALSE, 0);
+    g_signal_connect(rexport, "format-changed",
+                     G_CALLBACK(rexport_format_changed), &args.report_style);
 
     gtk_widget_show_all(dialog);
     gtk_dialog_run(GTK_DIALOG(dialog));
-    args.report_style
-        = gwy_results_export_get_format(GWY_RESULTS_EXPORT(rexport));
     gtk_widget_destroy(dialog);
     g_object_unref(results);
 
@@ -371,10 +381,12 @@ grain_stat(G_GNUC_UNUSED GwyContainer *data, GwyRunType run)
     controls.args = &args;
     controls.same_units = gwy_si_unit_equal(siunitxy, siunitz);
     controls.stats = calculate_stats(dfield, mfield);
+    controls.dfield = dfield;
 
-    dialog = gtk_dialog_new_with_buttons(_("Grain Summary"), NULL, 0,
+    dialog = gtk_dialog_new_with_buttons(_("Grain Statistics"), NULL, 0,
                                          GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                          NULL);
+    controls.dialog = dialog;
     gtk_window_set_default_size(GTK_WINDOW(dialog), 640, 640);
     gwy_help_add_to_proc_dialog(GTK_DIALOG(dialog), GWY_HELP_DEFAULT);
 
@@ -409,6 +421,17 @@ grain_stat(G_GNUC_UNUSED GwyContainer *data, GwyRunType run)
     gwy_grain_value_tree_view_set_expanded_groups(GTK_TREE_VIEW(treeview),
                                                   args.expanded);
 
+
+    controls.rexport = gwy_results_export_new(args.report_style);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), controls.rexport,
+                       FALSE, FALSE, 0);
+    g_signal_connect(controls.rexport, "format-changed",
+                     G_CALLBACK(rexport_format_changed), &args.report_style);
+    g_signal_connect(controls.rexport, "copy",
+                     G_CALLBACK(grain_stats_copy), &controls);
+    g_signal_connect(controls.rexport, "save",
+                     G_CALLBACK(grain_stats_save), &controls);
+
     gtk_widget_show_all(dialog);
     gtk_dialog_run(GTK_DIALOG(dialog));
     args.expanded = gwy_grain_value_tree_view_get_expanded_groups
@@ -523,7 +546,8 @@ calculate_stats(GwyDataField *dfield, GwyDataField *mfield)
     siunit = NULL;
     for (i = 0; i < n; i++) {
         stat = stats + i;
-        is_angle = (gwy_grain_value_get_flags(gvalues[i])
+        stat->gvalue = gvalues[i];
+        is_angle = (gwy_grain_value_get_flags(stat->gvalue)
                     & GWY_GRAIN_VALUE_IS_ANGLE);
         /* Exclude the zeroth value of the array as no-grain. */
         if (is_angle) {
@@ -543,7 +567,7 @@ calculate_stats(GwyDataField *dfield, GwyDataField *mfield)
             stat->rms = calc_rms(values[i]+1, ngrains, stat->mean);
         }
 
-        quantity = gwy_grain_value_get_quantity(gvalues[i]);
+        quantity = gwy_grain_value_get_quantity(stat->gvalue);
         if (quantity == GWY_GRAIN_VALUE_PIXEL_AREA) {
             stat->vf = gwy_si_unit_value_format_new(1.0, 1,
                                                     _("px<sup>2</sup>"));
@@ -556,8 +580,8 @@ calculate_stats(GwyDataField *dfield, GwyDataField *mfield)
 
         siunitxy = gwy_data_field_get_si_unit_xy(dfield);
         siunitz = gwy_data_field_get_si_unit_z(dfield);
-        powerxy = gwy_grain_value_get_power_xy(gvalues[i]);
-        powerz = gwy_grain_value_get_power_z(gvalues[i]);
+        powerxy = gwy_grain_value_get_power_xy(stat->gvalue);
+        powerz = gwy_grain_value_get_power_z(stat->gvalue);
         siunit = gwy_si_unit_power_multiply(siunitxy, powerxy, siunitz, powerz,
                                             siunit);
         v = fmax(fabs(stat->mean), fabs(stat->median));
@@ -566,7 +590,7 @@ calculate_stats(GwyDataField *dfield, GwyDataField *mfield)
         stat->vf = gwy_si_unit_get_format_with_digits(siunit, style, v, 3,
                                                       NULL);
     }
-    g_object_unref(siunit);
+    gwy_object_unref(siunit);
     g_free(values);
     g_free(gvalues);
 
@@ -844,6 +868,210 @@ grain_exscribe_circles(GwyContainer *data, GwyRunType run)
     g_free(circc);
 }
 
+static void
+rexport_format_changed(GwyResultsExport *rexport,
+                       GwyResultsReportType *report_style)
+{
+    *report_style = gwy_results_export_get_format(rexport);
+}
+
+static void
+append_separator(GString *str, GwyResultsReportType base_type)
+{
+    if (base_type == GWY_RESULTS_REPORT_TABSEP)
+        g_string_append_c(str, '\t');
+    if (base_type == GWY_RESULTS_REPORT_CSV)
+        g_string_append(str, "\",\"");
+}
+
+static void
+format_value(GString *str, gdouble v, GwySIValueFormat *vf)
+{
+    if (vf)
+        g_string_append_printf(str, "%.*f", vf->precision, v/vf->magnitude);
+    else {
+        gchar dbuf[64];
+
+        g_ascii_formatd(dbuf, 64, "%g", v);
+        g_string_append(str, dbuf);
+    }
+}
+
+static gchar*
+format_grain_stat_report(const GrainQuantityStats *stats,
+                         GwyResultsReportType report_style,
+                         GwyDataField *dfield)
+{
+    GwyResultsReportType base_style;
+    const GrainQuantityStats *stat;
+    GwyGrainQuantity quantity;
+    GwyGrainValueGroup idgroup = GWY_GRAIN_VALUE_GROUP_ID;
+    GwySIUnit *siunit, *siunitxy, *siunitz;
+    GwySIUnitFormatStyle style;
+    gboolean is_angle, for_machine;
+    GwySIValueFormat *vf = NULL;
+    GString *str;
+    const gchar *name, *s;
+    gchar *padding = NULL, *unitstr = NULL;
+    gint powerxy, powerz;
+    gdouble v;
+    guint n, i, w, maxwidth;
+
+    for_machine = (report_style & GWY_RESULTS_REPORT_MACHINE);
+    base_style = (report_style & 0xff);
+    n = gwy_inventory_get_n_items(gwy_grain_values());
+    str = g_string_new(NULL);
+    siunit = NULL;
+    style = (for_machine
+             ? GWY_SI_UNIT_FORMAT_PLAIN
+             : GWY_SI_UNIT_FORMAT_VFUNICODE);
+
+    maxwidth = 0;
+    if (base_style == GWY_RESULTS_REPORT_COLON) {
+        for (i = 0; i < n; i++) {
+            stat = stats + i;
+            if (gwy_grain_value_get_group(stat->gvalue) == idgroup)
+                continue;
+            name = gwy_resource_get_name(GWY_RESOURCE(stat->gvalue));
+            if (!for_machine)
+                name = _(name);
+            w = gwy_str_fixed_font_width(name);
+            maxwidth = MAX(w, maxwidth);
+        }
+        padding = g_new(gchar, maxwidth+2);
+        memset(padding, ' ', maxwidth+1);
+        padding[maxwidth+1] = '\0';
+    }
+
+    for (i = 0; i < n; i++) {
+        stat = stats + i;
+        if (gwy_grain_value_get_group(stat->gvalue) == idgroup)
+            continue;
+        is_angle = (gwy_grain_value_get_flags(stat->gvalue)
+                    & GWY_GRAIN_VALUE_IS_ANGLE);
+        quantity = gwy_grain_value_get_quantity(stat->gvalue);
+
+        /* Name */
+        name = gwy_resource_get_name(GWY_RESOURCE(stat->gvalue));
+        if (!for_machine)
+            name = _(name);
+
+        if (base_style == GWY_RESULTS_REPORT_CSV)
+            g_string_append(str, "\"");
+        g_string_append(str, name);
+        if (base_style == GWY_RESULTS_REPORT_COLON) {
+            g_string_append(str, ": ");
+            g_string_append_len(str, padding,
+                                maxwidth - gwy_str_fixed_font_width(name));
+        }
+        append_separator(str, base_style);
+
+        /* Value format */
+        if (!for_machine && is_angle) {
+            if (vf)
+                gwy_si_unit_value_format_free(vf);
+            vf = gwy_si_unit_value_format_new(G_PI/180.0, 2, _("deg"));
+        }
+        else {
+            if (quantity == GWY_GRAIN_VALUE_PIXEL_AREA) {
+                gwy_object_unref(siunit);
+                siunit = gwy_si_unit_new("px^2");
+            }
+            else {
+                siunitxy = gwy_data_field_get_si_unit_xy(dfield);
+                siunitz = gwy_data_field_get_si_unit_z(dfield);
+                powerxy = gwy_grain_value_get_power_xy(stat->gvalue);
+                powerz = gwy_grain_value_get_power_z(stat->gvalue);
+                siunit = gwy_si_unit_power_multiply(siunitxy, powerxy, siunitz,
+                                                    powerz, siunit);
+            }
+            if (for_machine) {
+                g_free(unitstr);
+                unitstr = gwy_si_unit_get_string(siunit, style);
+            }
+            else {
+                v = fmax(fabs(stat->mean), fabs(stat->median));
+                v = fmax(v, stat->rms);
+                v = fmax(v, stat->q75 - stat->q25);
+                vf = gwy_si_unit_get_format_with_digits(siunit,
+                                                        style, v, 3, vf);
+            }
+        }
+
+        format_value(str, stat->mean, vf);
+        if (base_style == GWY_RESULTS_REPORT_COLON)
+            g_string_append(str, " ± ");
+        else
+            append_separator(str, base_style);
+
+        format_value(str, stat->rms, vf);
+        if (base_style == GWY_RESULTS_REPORT_COLON)
+            g_string_append(str, ", ");
+        else
+            append_separator(str, base_style);
+
+        format_value(str, stat->median, vf);
+        if (base_style == GWY_RESULTS_REPORT_COLON)
+            g_string_append(str, " ± ");
+        else
+            append_separator(str, base_style);
+
+        format_value(str, stat->q75 - stat->q25, vf);
+        s = for_machine ? unitstr : vf->units;
+        if (base_style == GWY_RESULTS_REPORT_COLON) {
+            if (*s) {
+                g_string_append(str, " ");
+                g_string_append(str, s);
+            }
+        }
+        else {
+            append_separator(str, base_style);
+            g_string_append(str, s);
+            if (base_style == GWY_RESULTS_REPORT_CSV)
+                g_string_append(str, "\"");
+        }
+        g_string_append(str, "\n");
+    }
+    gwy_object_unref(siunit);
+    g_free(unitstr);
+    g_free(padding);
+    if (vf)
+        gwy_si_unit_value_format_free(vf);
+
+    return g_string_free(str, FALSE);
+}
+
+static void
+grain_stats_copy(G_GNUC_UNUSED GwyResultsExport *rexport,
+                 GrainStatsControls *controls)
+{
+    GtkClipboard *clipboard;
+    GdkDisplay *display;
+    gchar *text;
+
+    text = format_grain_stat_report(controls->stats,
+                                    controls->args->report_style,
+                                    controls->dfield);
+    display = gtk_widget_get_display(controls->dialog);
+    clipboard = gtk_clipboard_get_for_display(display, GDK_SELECTION_CLIPBOARD);
+    gtk_clipboard_set_text(clipboard, text, -1);
+    g_free(text);
+}
+
+static void
+grain_stats_save(G_GNUC_UNUSED GwyResultsExport *rexport,
+                 GrainStatsControls *controls)
+{
+    gchar *text;
+
+    text = format_grain_stat_report(controls->stats,
+                                    controls->args->report_style,
+                                    controls->dfield);
+    gwy_save_auxiliary_data(_("Save Grain Statistics"),
+                            GTK_WINDOW(controls->dialog), -1, text);
+    g_free(text);
+}
+
 static const gchar report_style_key_summary[] = "/module/grain_summary/report_style";
 
 static void
@@ -853,7 +1081,6 @@ summary_load_args(GwyContainer *container, GrainSummaryArgs *args)
 
     gwy_container_gis_enum_by_name(container, report_style_key_summary,
                                    &args->report_style);
-    /* FIXME: sanitize is hairy, make a helper for that. */
 }
 
 static void
@@ -871,7 +1098,6 @@ stat_load_args(GwyContainer *container, GrainStatArgs *args)
 {
     *args = grain_stat_defaults;
 
-    /* FIXME: sanitize is hairy, make a helper for that. */
     gwy_container_gis_enum_by_name(container, report_style_key_stat,
                                    &args->report_style);
     gwy_container_gis_int32_by_name(container, expanded_key, &args->expanded);
