@@ -1,5 +1,5 @@
 /*
- *  $Id: grain_wshed.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: grain_wshed.c 21408 2018-09-07 13:53:53Z yeti-dn $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -101,7 +101,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Marks grains by watershed algorithm."),
     "Petr Klapetek <petr@klapetek.cz>",
-    "1.18",
+    "1.19",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -436,14 +436,16 @@ mask_process(GwyDataField *dfield, GwyDataField *maskfield, WshedArgs *args,
         gwy_data_field_grains_watershed_iteration(state);
         if (oldstate != state->state) {
             if (state->state == GWY_WATERSHED_STATE_MIN)
-                gwy_app_wait_set_message(_("Finding minima..."));
+                ok = gwy_app_wait_set_message(_("Finding minima..."));
             else if (state->state == GWY_WATERSHED_STATE_LOCATE)
-                gwy_app_wait_set_message(_("Locating..."));
+                ok = gwy_app_wait_set_message(_("Locating..."));
             else if (state->state == GWY_WATERSHED_STATE_WATERSHED)
-                gwy_app_wait_set_message(_("Simulating watershed..."));
+                ok = gwy_app_wait_set_message(_("Simulating watershed..."));
             else if (state->state == GWY_WATERSHED_STATE_MARK)
-                gwy_app_wait_set_message(_("Marking boundaries..."));
+                ok = gwy_app_wait_set_message(_("Marking boundaries..."));
             oldstate = state->state;
+            if (!ok)
+                break;
         }
         if (!gwy_app_wait_set_fraction(state->fraction))
             break;
