@@ -87,7 +87,7 @@ module_register(void)
 {
     gwy_process_func_register("latsim",
                               (GwyProcessFunc)&latsim,
-                              N_("/_Tip and Indentation/_Lateral Force..."),
+                              N_("/SPM M_odes/_Force and Indentation/_Lateral Force..."),
                               GWY_STOCK_TIP_LATERAL_FORCE,
                               LATSIM_RUN_MODES,
                               GWY_MENU_FLAG_DATA,
