@@ -1,6 +1,6 @@
 /*
- *  $Id: linematch.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2015 David Necas (Yeti).
+ *  $Id: linematch.c 21590 2018-11-03 21:08:30Z yeti-dn $
+ *  Copyright (C) 2015-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,9 @@
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
 #include <libprocess/gwyprocesstypes.h>
+#include <libprocess/arithmetic.h>
 #include <libprocess/level.h>
+#include <libprocess/correct.h>
 #include <libprocess/stats.h>
 #include <libprocess/linestats.h>
 #include <libprocess/gwyprocesstypes.h>
@@ -98,29 +100,21 @@ static void     linematch_do             (GwyDataField *mask,
 static void     linematch_do_poly        (GwyDataField *dfield,
                                           GwyDataField *mask,
                                           const LineMatchArgs *args);
-static void     linematch_do_median      (GwyDataField *dfield,
+static void     linematch_do_trimmed_mean(GwyDataField *dfield,
                                           GwyDataField *mask,
-                                          const LineMatchArgs *args);
-static void     linematch_do_median_diff (GwyDataField *dfield,
+                                          const LineMatchArgs *args,
+                                          gdouble trim_fraction);
+static void     linematch_do_trimmed_diff(GwyDataField *dfield,
                                           GwyDataField *mask,
-                                          const LineMatchArgs *args);
+                                          const LineMatchArgs *args,
+                                          gdouble trim_fraction);
 static void     linematch_do_modus       (GwyDataField *dfield,
                                           GwyDataField *mask,
                                           const LineMatchArgs *args);
 static void     linematch_do_match       (GwyDataField *dfield,
                                           GwyDataField *mask,
                                           const LineMatchArgs *args);
-static void     linematch_do_trimmed_mean(GwyDataField *dfield,
-                                          GwyDataField *mask,
-                                          const LineMatchArgs *args);
-static void     linematch_do_tmean_diff  (GwyDataField *dfield,
-                                          GwyDataField *mask,
-                                          const LineMatchArgs *args);
 static void     zero_level_row_shifts    (GwyDataLine *shifts);
-static void     slope_level_row_shifts   (GwyDataLine *shifts);
-static void     apply_row_shifts         (GwyDataField *dfield,
-                                          GwyDataField *bg,
-                                          GwyDataLine *shifts);
 static gboolean linematch_dialog         (LineMatchArgs *args,
                                           GwyContainer *data,
                                           GwyDataField *dfield,
@@ -184,7 +178,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Aligns rows by various methods."),
     "Yeti <yeti@gwyddion.net>",
-    "1.6",
+    "1.8",
     "David Nečas (Yeti)",
     "2015",
 };
@@ -303,6 +297,7 @@ linematch_do(GwyDataField *mask,
     GwyMaskingType masking = args->masking;
     GwyDataField *mymask, *myfield;
 
+    gwy_data_field_copy(args->result, args->bg, TRUE);
     if (args->masking == GWY_MASK_IGNORE)
         mask = NULL;
     if (!mask)
@@ -320,27 +315,29 @@ linematch_do(GwyDataField *mask,
         }
     }
 
-    gwy_data_field_resample(args->bg, myfield->xres, myfield->yres,
-                            GWY_INTERPOLATION_NONE);
-    gwy_data_line_resample(args->shifts, args->result->yres,
+    gwy_data_line_resample(args->shifts, gwy_data_field_get_yres(myfield),
                            GWY_INTERPOLATION_NONE);
-    gwy_data_line_set_real(args->shifts, args->result->yreal);
+    gwy_data_line_set_real(args->shifts, gwy_data_field_get_yreal(myfield));
 
     /* Perform the correction. */
-    if (args->method == LINE_LEVEL_POLY)
-        linematch_do_poly(myfield, mymask, args);
+    if (args->method == LINE_LEVEL_POLY) {
+        if (args->max_degree == 0)
+            linematch_do_trimmed_mean(myfield, mymask, args, 0.0);
+        else
+            linematch_do_poly(myfield, mymask, args);
+    }
     else if (args->method == LINE_LEVEL_MEDIAN)
-        linematch_do_median(myfield, mymask, args);
+        linematch_do_trimmed_mean(myfield, mymask, args, 0.5);
     else if (args->method == LINE_LEVEL_MEDIAN_DIFF)
-        linematch_do_median_diff(myfield, mymask, args);
+        linematch_do_trimmed_diff(myfield, mymask, args, 0.5);
     else if (args->method == LINE_LEVEL_MODUS)
         linematch_do_modus(myfield, mymask, args);
     else if (args->method == LINE_LEVEL_MATCH)
         linematch_do_match(myfield, mymask, args);
     else if (args->method == LINE_LEVEL_TRIMMED_MEAN)
-        linematch_do_trimmed_mean(myfield, mymask, args);
+        linematch_do_trimmed_mean(myfield, mymask, args, args->trim_fraction);
     else if (args->method == LINE_LEVEL_TMEAN_DIFF)
-        linematch_do_tmean_diff(myfield, mymask, args);
+        linematch_do_trimmed_diff(myfield, mymask, args, args->trim_fraction);
     else {
         g_assert_not_reached();
     }
@@ -349,32 +346,25 @@ linematch_do(GwyDataField *mask,
     if (args->direction == GWY_ORIENTATION_VERTICAL) {
         GWY_OBJECT_UNREF(mymask);
         gwy_data_field_flip_xy(myfield, args->result, FALSE);
-        gwy_data_field_flip_xy(args->bg, myfield, FALSE);
-        gwy_data_field_resample(args->bg, myfield->xres, myfield->yres,
-                                GWY_INTERPOLATION_NONE);
-        gwy_data_field_copy(myfield, args->bg, FALSE);
         g_object_unref(myfield);
     }
+    gwy_data_field_subtract_fields(args->bg, args->bg, args->result);
 
     args->masking = masking;
-    gwy_data_field_invalidate(args->result);
-    gwy_data_field_invalidate(args->bg);
 }
 
 static void
 linematch_do_poly(GwyDataField *dfield, GwyDataField *mask,
                   const LineMatchArgs *args)
 {
-    GwyDataField *bg;
     GwyDataLine *means;
     GwyMaskingType masking;
     gdouble *xpowers, *zxpowers, *matrix;
     gint xres, yres, degree, i, j, k;
     gdouble xc, avg;
     const gdouble *m;
-    gdouble *d, *b;
+    gdouble *d;
 
-    bg = args->bg;
     means = args->shifts;
     masking = args->masking;
 
@@ -386,7 +376,6 @@ linematch_do_poly(GwyDataField *dfield, GwyDataField *mask,
     d = gwy_data_field_get_data(dfield);
 
     m = mask ? gwy_data_field_get_data_const(mask) : NULL;
-    b = gwy_data_field_get_data(bg);
 
     xpowers = g_new(gdouble, 2*degree+1);
     zxpowers = g_new(gdouble, degree+1);
@@ -437,11 +426,9 @@ linematch_do_poly(GwyDataField *dfield, GwyDataField *mask,
             }
 
             d[j] -= z;
-            b[j] = z;
         }
 
         d += xres;
-        b += xres;
         m = m ? m+xres : NULL;
     }
 
@@ -451,120 +438,39 @@ linematch_do_poly(GwyDataField *dfield, GwyDataField *mask,
 }
 
 static void
-linematch_do_median(GwyDataField *dfield, GwyDataField *mask,
-                    const LineMatchArgs *args)
+linematch_do_trimmed_mean(GwyDataField *dfield, GwyDataField *mask,
+                          const LineMatchArgs *args,
+                          gdouble trimfrac)
 {
-    GwyDataField *bg;
-    GwyDataLine *medians, *line;
-    GwyMaskingType masking;
-    gint xres, yres, i;
-    const gdouble *d, *m;
-    gdouble median, total_median;
-    gdouble *buf;
+    GwyDataLine *shifts;
 
-    bg = args->bg;
-    masking = args->masking;
-    medians = args->shifts;
-
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    total_median = gwy_data_field_area_get_median_mask(dfield, mask, masking,
-                                                       0, 0, xres, yres);
-
-    d = gwy_data_field_get_data(dfield);
-    m = mask ? gwy_data_field_get_data_const(mask) : NULL;
-    line = gwy_data_line_new(xres, 1.0, FALSE);
-    buf = gwy_data_line_get_data(line);
-
-    if (mask) {
-        for (i = 0; i < yres; i++) {
-            const gdouble *row = d + i*xres, *mrow = m + i*xres;
-            gint count = 0, j;
-
-            if (masking == GWY_MASK_INCLUDE) {
-                for (j = 0; j < xres; j++) {
-                    if (mrow[j] > 0.0)
-                        buf[count++] = row[j];
-                }
-            }
-            else {
-                for (j = 0; j < xres; j++) {
-                    if (mrow[j] < 1.0)
-                        buf[count++] = row[j];
-                }
-            }
-
-            median = count ? gwy_math_median(count, buf) : total_median;
-            gwy_data_line_set_val(medians, i, median);
-        }
-    }
-    else {
-        for (i = 0; i < yres; i++) {
-            gwy_data_field_get_row(dfield, line, i);
-            median = gwy_math_median(xres, gwy_data_line_get_data(line));
-            gwy_data_line_set_val(medians, i, median);
-        }
-    }
-
-    apply_row_shifts(dfield, bg, medians);
-
-    g_object_unref(line);
+    shifts = gwy_data_field_find_row_shifts_trimmed_mean(dfield,
+                                                         mask, args->masking,
+                                                         trimfrac, 0);
+    gwy_data_field_subtract_row_shifts(dfield, shifts);
+    gwy_data_line_copy(shifts, args->shifts);
+    g_object_unref(shifts);
 }
 
 static void
-linematch_do_median_diff(GwyDataField *dfield, GwyDataField *mask,
-                         const LineMatchArgs *args)
+linematch_do_trimmed_diff(GwyDataField *dfield, GwyDataField *mask,
+                          const LineMatchArgs *args,
+                          gdouble trimfrac)
 {
-    GwyDataField *bg;
-    GwyDataLine *medians, *line;
-    GwyMaskingType masking;
-    gint xres, yres, i;
-    const gdouble *d, *m;
-    gdouble median;
-    gdouble *buf;
+    GwyDataLine *shifts;
 
-    bg = args->bg;
-    masking = args->masking;
-    medians = args->shifts;
-
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    d = gwy_data_field_get_data(dfield);
-    m = mask ? gwy_data_field_get_data_const(mask) : NULL;
-    line = gwy_data_line_new(xres, 1.0, FALSE);
-    buf = gwy_data_line_get_data(line);
-
-    gwy_data_line_set_val(medians, 0, 0.0);
-    for (i = 0; i+1 < yres; i++) {
-        const gdouble *row = d + i*xres, *mrow = m + i*xres;
-        gint count = 0, j;
-
-        for (j = 0; j < xres; j++) {
-            if ((masking == GWY_MASK_INCLUDE && (mrow[j] <= 0.0
-                                                 || mrow[xres + j] <= 0.0))
-                || (masking == GWY_MASK_EXCLUDE && (mrow[j] >= 1.0
-                                                    || mrow[xres + j] >= 1.0)))
-                continue;
-
-            buf[count++] = row[xres + j] - row[j];
-        }
-        median = gwy_data_line_get_val(medians, i);
-        if (count)
-            median += gwy_math_median(count, buf);
-        gwy_data_line_set_val(medians, i+1, median);
-    }
-
-    slope_level_row_shifts(medians);
-    apply_row_shifts(dfield, bg, medians);
-
-    g_object_unref(line);
+    shifts = gwy_data_field_find_row_shifts_trimmed_diff(dfield,
+                                                         mask, args->masking,
+                                                         trimfrac, 0);
+    gwy_data_field_subtract_row_shifts(dfield, shifts);
+    gwy_data_line_copy(shifts, args->shifts);
+    g_object_unref(shifts);
 }
 
 static void
 linematch_do_modus(GwyDataField *dfield, GwyDataField *mask,
                    const LineMatchArgs *args)
 {
-    GwyDataField *bg;
     GwyDataLine *modi, *line;
     GwyMaskingType masking;
     gint xres, yres, i;
@@ -572,7 +478,6 @@ linematch_do_modus(GwyDataField *dfield, GwyDataField *mask,
     gdouble modus, total_median;
     gdouble *buf;
 
-    bg = args->bg;
     masking = args->masking;
     modi = args->shifts;
 
@@ -624,7 +529,8 @@ linematch_do_modus(GwyDataField *dfield, GwyDataField *mask,
         gwy_data_line_set_val(modi, i, modus);
     }
 
-    apply_row_shifts(dfield, bg, modi);
+    zero_level_row_shifts(modi);
+    gwy_data_field_subtract_row_shifts(dfield, modi);
 
     g_object_unref(line);
 }
@@ -633,7 +539,6 @@ static void
 linematch_do_match(GwyDataField *dfield, GwyDataField *mask,
                    const LineMatchArgs *args)
 {
-    GwyDataField *bg;
     GwyDataLine *shifts;
     GwyMaskingType masking;
     gint xres, yres, i, j;
@@ -642,7 +547,6 @@ linematch_do_match(GwyDataField *dfield, GwyDataField *mask,
     gdouble *s, *w;
 
     shifts = args->shifts;
-    bg = args->bg;
     masking = args->masking;
 
     xres = gwy_data_field_get_xres(dfield);
@@ -708,180 +612,18 @@ linematch_do_match(GwyDataField *dfield, GwyDataField *mask,
 
         s[i] = -lambda + s[i-1];
     }
-    apply_row_shifts(dfield, bg, shifts);
+    zero_level_row_shifts(shifts);
+    gwy_data_field_subtract_row_shifts(dfield, shifts);
 
     g_free(w);
 }
 
-static gdouble
-trimmed_mean_or_median(gdouble *array, guint n, gdouble p)
-{
-    guint nlowest = GWY_ROUND(p*n);
-    guint nhighest = GWY_ROUND(p*n);
-
-    if (nlowest + nhighest >= n)
-        return gwy_math_median(n, array);
-    return gwy_math_trimmed_mean(n, array, nlowest, nhighest);
-}
-
-static void
-linematch_do_trimmed_mean(GwyDataField *dfield, GwyDataField *mask,
-                          const LineMatchArgs *args)
-{
-    GwyDataField *bg;
-    GwyDataLine *means, *line;
-    GwyMaskingType masking;
-    gint xres, yres, i;
-    const gdouble *d, *m;
-    gdouble ptrim, mean, total_median;
-    gdouble *buf;
-
-    bg = args->bg;
-    masking = args->masking;
-    means = args->shifts;
-    ptrim = args->trim_fraction;
-
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    /* FIXME: Use trimmed mean of the entire data here? */
-    total_median = gwy_data_field_area_get_median_mask(dfield, mask, masking,
-                                                       0, 0, xres, yres);
-
-    d = gwy_data_field_get_data(dfield);
-    m = mask ? gwy_data_field_get_data_const(mask) : NULL;
-    line = gwy_data_line_new(xres, 1.0, FALSE);
-    buf = gwy_data_line_get_data(line);
-
-    if (mask) {
-        for (i = 0; i < yres; i++) {
-            const gdouble *row = d + i*xres, *mrow = m + i*xres;
-            gint count = 0, j;
-
-            if (masking == GWY_MASK_INCLUDE) {
-                for (j = 0; j < xres; j++) {
-                    if (mrow[j] > 0.0)
-                        buf[count++] = row[j];
-                }
-            }
-            else {
-                for (j = 0; j < xres; j++) {
-                    if (mrow[j] < 1.0)
-                        buf[count++] = row[j];
-                }
-            }
-
-            if (!count)
-                mean = total_median;
-            else
-                mean = trimmed_mean_or_median(buf, count, ptrim);
-            gwy_data_line_set_val(means, i, mean);
-        }
-    }
-    else {
-        for (i = 0; i < yres; i++) {
-            gwy_data_field_get_row(dfield, line, i);
-            mean = trimmed_mean_or_median(buf, xres, ptrim);
-            gwy_data_line_set_val(means, i, mean);
-        }
-    }
-
-    apply_row_shifts(dfield, bg, means);
-
-    g_object_unref(line);
-}
-
-static void
-linematch_do_tmean_diff(GwyDataField *dfield, GwyDataField *mask,
-                        const LineMatchArgs *args)
-{
-    GwyDataField *bg;
-    GwyDataLine *means, *line;
-    GwyMaskingType masking;
-    gint xres, yres, i;
-    const gdouble *d, *m;
-    gdouble mean, ptrim;
-    gdouble *buf;
-
-    bg = args->bg;
-    masking = args->masking;
-    means = args->shifts;
-    ptrim = args->trim_fraction;
-
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    d = gwy_data_field_get_data(dfield);
-    m = mask ? gwy_data_field_get_data_const(mask) : NULL;
-    line = gwy_data_line_new(xres, 1.0, FALSE);
-    buf = gwy_data_line_get_data(line);
-
-    gwy_data_line_set_val(means, 0, 0.0);
-    for (i = 0; i+1 < yres; i++) {
-        const gdouble *row = d + i*xres, *mrow = m + i*xres;
-        gint count = 0, j;
-
-        for (j = 0; j < xres; j++) {
-            if ((masking == GWY_MASK_INCLUDE && (mrow[j] <= 0.0
-                                                 || mrow[xres + j] <= 0.0))
-                || (masking == GWY_MASK_EXCLUDE && (mrow[j] >= 1.0
-                                                    || mrow[xres + j] >= 1.0)))
-                continue;
-
-            buf[count++] = row[xres + j] - row[j];
-        }
-
-        mean = gwy_data_line_get_val(means, i);
-        if (count)
-            mean += trimmed_mean_or_median(buf, count, ptrim);
-        gwy_data_line_set_val(means, i+1, mean);
-    }
-
-    slope_level_row_shifts(means);
-    apply_row_shifts(dfield, bg, means);
-
-    g_object_unref(line);
-}
-
 static void
 zero_level_row_shifts(GwyDataLine *shifts)
 {
     gwy_data_line_add(shifts, -gwy_data_line_get_avg(shifts));
 }
 
-static void
-slope_level_row_shifts(GwyDataLine *shifts)
-{
-    gdouble a, b;
-
-    gwy_data_line_get_line_coeffs(shifts, &a, &b);
-    gwy_data_line_line_level(shifts, a, b);
-}
-
-static void
-apply_row_shifts(GwyDataField *dfield, GwyDataField *bg,
-                 GwyDataLine *shifts)
-{
-    gint xres, yres, i, j;
-    const gdouble *s;
-    gdouble *d, *b;
-
-    zero_level_row_shifts(shifts);
-
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    d = gwy_data_field_get_data(dfield);
-    b = gwy_data_field_get_data(bg);
-    s = gwy_data_line_get_data(shifts);
-    for (i = 0; i < yres; i++) {
-        gdouble z = s[i];
-        for (j = 0; j < xres; j++) {
-            d[j] -= z;
-            b[j] = z;
-        }
-        d += xres;
-        b += xres;
-    }
-}
-
 static gboolean
 linematch_dialog(LineMatchArgs *args,
                  GwyContainer *data,
