@@ -1,5 +1,5 @@
 /*
- *  $Id: logistic.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: logistic.c 20835 2018-03-13 10:54:57Z yeti-dn $
  *  Copyright (C) 2016-2017 David Necas (Yeti), Petr Klapetek,
  *  Daniil Bratashov.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net,
@@ -22,9 +22,9 @@
  */
 
 #include "config.h"
+#include <string.h>
 #include <gtk/gtk.h>
 #include <glib.h>
-#include <glib/gstdio.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
 #include <libprocess/stats.h>
@@ -34,7 +34,7 @@
 #include <libgwydgets/gwydgetutils.h>
 #include <libgwymodule/gwymodule-process.h>
 #include <app/gwyapp.h>
-#include <string.h>
+#include <app/gwymoduleutils.h>
 
 #define LOGISTIC_RUN_MODES GWY_RUN_INTERACTIVE
 
@@ -985,7 +985,7 @@ static void
 logistic_load_args(GwyContainer *settings,
                    LogisticArgs *args)
 {
-    gchar *filename, *buffer, *line, *p;
+    gchar *buffer, *line, *p;
     gsize size;
     gint i, nfeatures;
     gdouble *thetas;
@@ -1012,10 +1012,7 @@ logistic_load_args(GwyContainer *settings,
         *(thetas + i) = 0.0;
     }
 
-    filename = g_build_filename(gwy_get_user_dir(), "logistic",
-                                "thetas", NULL);
-
-    if (g_file_get_contents(filename, &buffer, &size, NULL)) {
+    if (gwy_module_data_load("logistic", "thetas", &buffer, &size, NULL)) {
         p = buffer;
         i = 0;
         while ((line = gwy_str_next_line(&p)) && (i++ < nfeatures)) {
@@ -1024,7 +1021,6 @@ logistic_load_args(GwyContainer *settings,
         }
         g_free(buffer);
     }
-    g_free(filename);
 
     logistic_sanitize_args(args);
 }
@@ -1033,7 +1029,7 @@ static void
 logistic_save_args(GwyContainer *settings,
                    LogisticArgs *args)
 {
-    gchar *filename, *s;
+    gchar *s;
     gdouble *thetas;
     FILE *fh;
     gint i, nfeatures;
@@ -1051,26 +1047,19 @@ logistic_save_args(GwyContainer *settings,
     gwy_container_set_double_by_name(settings, lambda_key,
                                      args->lambda);
 
-    filename = g_build_filename(gwy_get_user_dir(), "logistic", NULL);
-    if (!g_file_test(filename, G_FILE_TEST_IS_DIR))
-        g_mkdir(filename, 0700);
-    g_free(filename);
-
-    filename = g_build_filename(gwy_get_user_dir(), "logistic",
-                                "thetas", NULL);
     thetas = gwy_data_line_get_data(args->thetas);
-    if ((fh = gwy_fopen(filename, "w"))) {
-        nfeatures = logistic_nfeatures(args);
-        for (i = 0; i < nfeatures; i++) {
-            s = g_malloc(G_ASCII_DTOSTR_BUF_SIZE);
-            g_ascii_dtostr(s, G_ASCII_DTOSTR_BUF_SIZE, *(thetas+i));
-            fputs(s, fh);
-            fputc('\n', fh);
-            g_free(s);
-        }
-        fclose(fh);
+    if (!(fh = gwy_module_data_fopen("logistic", "thetas", "w", NULL)))
+        return;
+
+    nfeatures = logistic_nfeatures(args);
+    for (i = 0; i < nfeatures; i++) {
+        s = g_malloc(G_ASCII_DTOSTR_BUF_SIZE);
+        g_ascii_dtostr(s, G_ASCII_DTOSTR_BUF_SIZE, *(thetas+i));
+        fputs(s, fh);
+        fputc('\n', fh);
+        g_free(s);
     }
-    g_free(filename);
+    fclose(fh);
 }
 
 static void
