@@ -1,6 +1,6 @@
 /*
- *  $Id: median-bg.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
+ *  $Id: median-bg.c 21411 2018-09-07 14:29:17Z yeti-dn $
+ *  Copyright (C) 2004-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,8 @@
 #include <libgwyddion/gwymath.h>
 #include <libprocess/datafield.h>
 #include <libprocess/arithmetic.h>
+#include <libprocess/elliptic.h>
+#include <libprocess/filters.h>
 #include <libgwydgets/gwystock.h>
 #include <libgwydgets/gwydgetutils.h>
 #include <libgwymodule/gwymodule-process.h>
@@ -54,7 +56,6 @@ static void          median                    (GwyContainer *data,
                                                 GwyRunType run);
 static GwyDataField* median_background         (gint size,
                                                 GwyDataField *dfield);
-static gint*         median_make_circle        (gint radius);
 static gboolean      median_dialog             (MedianBgArgs *args);
 static void          radius_changed_cb         (GtkAdjustment *adj,
                                                 MedianBgArgs *args);
@@ -82,7 +83,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Subtracts background using a rank-based algorithm."),
     "Yeti <yeti@gwyddion.net>",
-    "1.6",
+    "1.7",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -123,8 +124,8 @@ median(GwyContainer *data, GwyRunType run)
     median_load_args(gwy_app_settings_get(), &args);
 
     /* FIXME: this is bogus for non-square pixels anyway */
-    xr = gwy_data_field_get_xreal(dfield)/gwy_data_field_get_xres(dfield);
-    yr = gwy_data_field_get_yreal(dfield)/gwy_data_field_get_yres(dfield);
+    xr = gwy_data_field_get_dx(dfield);
+    yr = gwy_data_field_get_dy(dfield);
     args.pixelsize = hypot(xr, yr);
     args.valform
         = gwy_data_field_get_value_format_xy(dfield,
@@ -309,75 +310,27 @@ median_dialog_update(MedianBgControls *controls,
 }
 
 static GwyDataField*
-median_background(gint size,
-                  GwyDataField *dfield)
+median_background(gint size, GwyDataField *dfield)
 {
-    GwyDataField *rfield;
-    gint *circle;
-    gdouble *data, *rdata, *buffer;
-    gint i, j, xres, yres, buflen;
+    GwyDataField *rfield, *kernel;
+    gint kres = 2*size + 1, n;
+    gboolean ok;
 
-    data = gwy_data_field_get_data(dfield);
+    kernel = gwy_data_field_new(kres, kres, 1.0, 1.0, TRUE);
+    n = gwy_data_field_elliptic_area_fill(kernel, 0, 0, kres, kres, 1.0);
     rfield = gwy_data_field_duplicate(dfield);
-    xres = gwy_data_field_get_xres(rfield);
-    yres = gwy_data_field_get_yres(rfield);
-    rdata = gwy_data_field_get_data(rfield);
-
-    buflen = 0;
-    circle = median_make_circle(size);
-    for (i = 0; i < 2*size + 1; i++)
-        buflen += 2*circle[i] + 1;
-    buffer = g_new(gdouble, buflen);
-
-    for (i = 0; i < yres; i++) {
-
-        for (j = 0; j < xres; j++) {
-            gint n, k, from, to;
-
-            n = 0;
-            for (k = MAX(0, i - size); k <= MIN(yres - 1, i + size); k++) {
-                gdouble *row = data + k*xres;
-
-                from = MAX(0, j - circle[k - i + size]);
-                to = MIN(xres - 1, j + circle[k - i + size]);
-                gwy_assign(buffer + n, row + from, to - from + 1);
-                n += to - from + 1;
-            }
-            rdata[i*xres + j] = gwy_math_median(n, buffer);
-        }
-        if (!gwy_app_wait_set_fraction((gdouble)i/yres)) {
-            g_free(circle);
-            g_object_unref(rfield);
-            return NULL;
-        }
-    }
-
-    g_free(circle);
+    ok = gwy_data_field_area_filter_kth_rank(rfield, kernel, 0, 0,
+                                             gwy_data_field_get_xres(rfield),
+                                             gwy_data_field_get_yres(rfield),
+                                             n/2,
+                                             gwy_app_wait_set_fraction);
+    g_object_unref(kernel);
+    if (!ok)
+        GWY_OBJECT_UNREF(rfield);
 
     return rfield;
 }
 
-static gint*
-median_make_circle(gint radius)
-{
-    gint *data;
-    gint i;
-
-    data = g_new(gint, 2*radius + 1);
-
-    for (i = 0; i <= radius; i++) {
-        gdouble u = (gdouble)i/radius;
-
-        if (G_UNLIKELY(u > 1.0))
-            data[radius+i] = data[radius-i] = 0;
-        else
-            data[radius+i] = data[radius-i] = GWY_ROUND(radius*sqrt(1.0 - u*u));
-
-    }
-
-    return data;
-}
-
 static const gchar radius_key[]     = "/module/median-bg/radius";
 static const gchar do_extract_key[] = "/module/median-bg/do_extract";
 
