@@ -1,6 +1,6 @@
 /*
- *  $Id: mfm_field.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2003-2015 David Necas (Yeti), Petr Klapetek.
+ *  $Id: mfm_field.c 21387 2018-09-03 11:24:04Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -27,6 +27,7 @@
 #include <libprocess/arithmetic.h>
 #include <libprocess/inttrans.h>
 #include <libprocess/stats.h>
+#include <libprocess/mfm.h>
 #include <libgwydgets/gwydgetutils.h>
 #include <libgwydgets/gwycombobox.h>
 #include <libgwydgets/gwystock.h>
@@ -48,7 +49,7 @@ typedef enum {
 
 typedef struct {
     GwyMfmFieldOutputType out;
-    GwyMfmProbeType probe;
+    GwyMFMProbeType probe;
     gboolean walls;
     gdouble height;
     gdouble thickness;
@@ -145,7 +146,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Simulation of magnetic field above perpendicular media"),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "1.1",
+    "1.2",
     "David Nečas (Yeti) & Petr Klapetek",
     "2017",
 };
@@ -157,8 +158,8 @@ module_register(void)
 {
     gwy_process_func_register("mfm_field",
                               (GwyProcessFunc)&mfm_field,
-                              N_("/SPM M_odes/_MFM/_Perpendicular Media Field..."),
-                              NULL,
+                              N_("/SPM M_odes/_Magnetic/_Perpendicular Media Field..."),
+                              GWY_STOCK_MFM_PERPENDICULAR,
                               MFM_FIELD_RUN_MODES,
                               GWY_MENU_FLAG_DATA | GWY_MENU_FLAG_DATA_MASK,
                               N_("Compute stray field above perpendicular magnetic medium"));
@@ -266,14 +267,14 @@ mfm_field_dialog(MfmFieldArgs *args, GwyContainer *data)
         { N_("Point charge"), GWY_MFM_PROBE_CHARGE, },
         { N_("Bar"),          GWY_MFM_PROBE_BAR,    },
     };
-    GtkWidget *dialog, *table, *hbox;
+    GtkWidget *dialog, *table, *hbox, *spin;
     MfmFieldControls controls;
     gint response, row, id, datano;
 
     controls.args = args;
 
-
-    dialog = gtk_dialog_new_with_buttons(_("Perpendicular Media Stray Field"), NULL, 0, NULL);
+    dialog = gtk_dialog_new_with_buttons(_("Perpendicular Media Stray Field"),
+                                         NULL, 0, NULL);
     gtk_dialog_add_action_widget(GTK_DIALOG(dialog),
                                  gwy_stock_like_button_new(_("_Update"),
                                                            GTK_STOCK_EXECUTE),
@@ -317,24 +318,27 @@ mfm_field_dialog(MfmFieldArgs *args, GwyContainer *data)
     row = 0;
 
     controls.height = gtk_adjustment_new(args->height, 0, 1000, 1, 10, 0);
-    gwy_table_attach_adjbar(table, row, _("_Output plane height:"), "nm",
-                            controls.height, GWY_HSCALE_SQRT);
+    spin = gwy_table_attach_adjbar(table, row, _("_Output plane height:"), "nm",
+                                   controls.height, GWY_HSCALE_SQRT);
+    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 2);
     g_object_set_data(G_OBJECT(controls.height), "controls", &controls);
     g_signal_connect(controls.height, "value-changed",
                      G_CALLBACK(height_changed), &controls);
     row++;
 
     controls.thickness = gtk_adjustment_new(args->thickness, 0, 1000, 1, 10, 0);
-    gwy_table_attach_adjbar(table, row, _("_Film thickness:"), "nm",
-                            controls.thickness, GWY_HSCALE_SQRT);
+    spin = gwy_table_attach_adjbar(table, row, _("_Film thickness:"), "nm",
+                                   controls.thickness, GWY_HSCALE_SQRT);
+    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 2);
     g_object_set_data(G_OBJECT(controls.thickness), "controls", &controls);
     g_signal_connect(controls.thickness, "value-changed",
                      G_CALLBACK(thickness_changed), &controls);
     row++;
 
     controls.sigma = gtk_adjustment_new(args->sigma, 1, 1000, 1, 10, 0);
-    gwy_table_attach_adjbar(table, row, _("_Magnetic charge:"), "kA/m",
-                            controls.sigma, GWY_HSCALE_SQRT);
+    spin = gwy_table_attach_adjbar(table, row, _("_Magnetic charge:"), "kA/m",
+                                   controls.sigma, GWY_HSCALE_SQRT);
+    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 2);
     g_object_set_data(G_OBJECT(controls.sigma), "controls", &controls);
     g_signal_connect(controls.sigma, "value-changed",
                      G_CALLBACK(sigma_changed), &controls);
@@ -351,16 +355,19 @@ mfm_field_dialog(MfmFieldArgs *args, GwyContainer *data)
     row++;
 
     controls.wall_a = gtk_adjustment_new(args->wall_a, 1, 1000, 1, 10, 0);
-    gwy_table_attach_adjbar(table, row, _("_Exchange constant:"), "pJ/m",
-                            controls.wall_a, GWY_HSCALE_SQRT);
+    spin = gwy_table_attach_adjbar(table, row, _("_Exchange constant:"), "pJ/m",
+                                   controls.wall_a, GWY_HSCALE_SQRT);
+    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 2);
     g_object_set_data(G_OBJECT(controls.wall_a), "controls", &controls);
     g_signal_connect(controls.wall_a, "value-changed",
                      G_CALLBACK(wall_a_changed), &controls);
     row++;
 
     controls.wall_kn = gtk_adjustment_new(args->wall_kn, 1, 1000, 1, 10, 0);
-    gwy_table_attach_adjbar(table, row, _("_Uniaxial anisotropy:"), "kJ/m³",
-                            controls.wall_kn, GWY_HSCALE_SQRT);
+    spin = gwy_table_attach_adjbar(table, row,
+                                   _("_Uniaxial anisotropy:"), "kJ/m³",
+                                   controls.wall_kn, GWY_HSCALE_SQRT);
+    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 2);
     g_object_set_data(G_OBJECT(controls.wall_kn), "controls", &controls);
     g_signal_connect(controls.wall_kn, "value-changed",
                      G_CALLBACK(wall_kn_changed), &controls);
@@ -478,41 +485,63 @@ mfm_field_do(MfmFieldArgs *args, GwyDataField *mfield, GwyDataField *out)
 {
     GwyDataField  *fz=NULL, *fza, *fzb, *fzc;
     gdouble dd = 1.0e-9;
+    gdouble wall_delta = G_PI*sqrt(args->wall_a*1e-12/(args->wall_kn*1e3));
 
-
+    /* FIXME: This could be done directly if we had a function equivalent to
+     * gwy_data_field_mfm_perpendicular_stray_field() which calculated
+     * derivatives by Z. */
     if (args->out == GWY_MFM_FIELD_OUTPUT_FIELD || args->out == GWY_MFM_FIELD_OUTPUT_FORCE) {
-        mfm_perpendicular_stray_field(mfield, out, args->height*1e-9, args->thickness*1e-9, args->sigma,
-                                                      args->walls, args->wall_a*1e-12, args->wall_kn*1e3);
+        gwy_data_field_mfm_perpendicular_stray_field(mfield,
+                                                     out,
+                                                     args->height*1e-9,
+                                                     args->thickness*1e-9,
+                                                     args->sigma*1e3,
+                                                     args->walls,
+                                                     wall_delta);
 
-        if (args->out == GWY_MFM_FIELD_OUTPUT_FORCE)
-        {
+        if (args->out == GWY_MFM_FIELD_OUTPUT_FORCE) {
             fz = gwy_data_field_new_alike(out, TRUE);
-
-            mfm_perpendicular_force_from_field(out, fz,
-                                  args->probe, args->mtip*1e3,
-                                  args->bx*1e-9, args->by*1e-9, args->length*1e-9);
+            gwy_data_field_mfm_perpendicular_medium_force(out, fz,
+                                                          args->probe,
+                                                          args->mtip*1e3,
+                                                          args->bx*1e-9,
+                                                          args->by*1e-9,
+                                                          args->length*1e-9);
             gwy_data_field_copy(fz, out, FALSE);
             gwy_object_unref(fz);
         }
     }
-    else if (args->out == GWY_MFM_FIELD_OUTPUT_FORCE_DX) //this is done numerically now
-    {
+    else if (args->out == GWY_MFM_FIELD_OUTPUT_FORCE_DX) {
+        //this is done numerically now
         fza = gwy_data_field_new_alike(out, TRUE);
         fzb = gwy_data_field_new_alike(out, TRUE);
         fz = gwy_data_field_new_alike(out, TRUE);
 
+        gwy_data_field_mfm_perpendicular_stray_field(mfield, out,
+                                                     args->height*1e-9-dd,
+                                                     args->thickness*1e-9,
+                                                     args->sigma*1e3,
+                                                     args->walls,
+                                                     wall_delta);
+        gwy_data_field_mfm_perpendicular_medium_force(out, fza,
+                                                      args->probe,
+                                                      args->mtip*1e3,
+                                                      args->bx*1e-9,
+                                                      args->by*1e-9,
+                                                      args->length*1e-9);
 
-        mfm_perpendicular_stray_field(mfield, out, args->height*1e-9-dd, args->thickness*1e-9, args->sigma,
-                                                     args->walls, args->wall_a*1e-12, args->wall_kn*1e3);
-        mfm_perpendicular_force_from_field(out, fza,
-                                  args->probe, args->mtip*1e3,
-                                  args->bx*1e-9, args->by*1e-9, args->length*1e-9);
-
-        mfm_perpendicular_stray_field(mfield, out, args->height*1e-9+dd, args->thickness*1e-9, args->sigma,
-                                                     args->walls, args->wall_a*1e-12, args->wall_kn*1e3);
-        mfm_perpendicular_force_from_field(out, fzb,
-                                  args->probe, args->mtip*1e3,
-                                  args->bx*1e-9, args->by*1e-9, args->length*1e-9);
+        gwy_data_field_mfm_perpendicular_stray_field(mfield, out,
+                                                     args->height*1e-9+dd,
+                                                     args->thickness*1e-9,
+                                                     args->sigma*1e3,
+                                                     args->walls,
+                                                     wall_delta);
+        gwy_data_field_mfm_perpendicular_medium_force(out, fzb,
+                                                      args->probe,
+                                                      args->mtip*1e3,
+                                                      args->bx*1e-9,
+                                                      args->by*1e-9,
+                                                      args->length*1e-9);
 
         gwy_data_field_subtract_fields(fz, fza, fzb);
         gwy_data_field_multiply(fz, 0.5/dd);
@@ -532,23 +561,44 @@ mfm_field_do(MfmFieldArgs *args, GwyDataField *mfield, GwyDataField *out)
         fz = gwy_data_field_new_alike(out, TRUE);
 
 
-        mfm_perpendicular_stray_field(mfield, out, args->height*1e-9-dd, args->thickness*1e-9, args->sigma,
-                                                     args->walls, args->wall_a*1e-12, args->wall_kn*1e3);
-        mfm_perpendicular_force_from_field(out, fza,
-                                  args->probe, args->mtip*1e3,
-                                  args->bx*1e-9, args->by*1e-9, args->length*1e-9);
+        gwy_data_field_mfm_perpendicular_stray_field(mfield, out,
+                                                     args->height*1e-9-dd,
+                                                     args->thickness*1e-9,
+                                                     args->sigma*1e3,
+                                                     args->walls,
+                                                     wall_delta);
+        gwy_data_field_mfm_perpendicular_medium_force(out, fza,
+                                                      args->probe,
+                                                      args->mtip*1e3,
+                                                      args->bx*1e-9,
+                                                      args->by*1e-9,
+                                                      args->length*1e-9);
 
-        mfm_perpendicular_stray_field(mfield, out, args->height*1e-9, args->thickness*1e-9, args->sigma,
-                                                     args->walls, args->wall_a*1e-12, args->wall_kn*1e3);
-        mfm_perpendicular_force_from_field(out, fzb,
-                                  args->probe, args->mtip*1e3,
-                                  args->bx*1e-9, args->by*1e-9, args->length*1e-9);
+        gwy_data_field_mfm_perpendicular_stray_field(mfield, out,
+                                                     args->height*1e-9,
+                                                     args->thickness*1e-9,
+                                                     args->sigma*1e3,
+                                                     args->walls,
+                                                     wall_delta);
+        gwy_data_field_mfm_perpendicular_medium_force(out, fzb,
+                                                      args->probe,
+                                                      args->mtip*1e3,
+                                                      args->bx*1e-9,
+                                                      args->by*1e-9,
+                                                      args->length*1e-9);
 
-        mfm_perpendicular_stray_field(mfield, out, args->height*1e-9+dd, args->thickness*1e-9, args->sigma,
-                                                     args->walls, args->wall_a*1e-12, args->wall_kn*1e3);
-        mfm_perpendicular_force_from_field(out, fzc,
-                                  args->probe, args->mtip*1e3,
-                                  args->bx*1e-9, args->by*1e-9, args->length*1e-9);
+        gwy_data_field_mfm_perpendicular_stray_field(mfield, out,
+                                                     args->height*1e-9+dd,
+                                                     args->thickness*1e-9,
+                                                     args->sigma*1e3,
+                                                     args->walls,
+                                                     wall_delta);
+        gwy_data_field_mfm_perpendicular_medium_force(out, fzc,
+                                                      args->probe,
+                                                      args->mtip*1e3,
+                                                      args->bx*1e-9,
+                                                      args->by*1e-9,
+                                                      args->length*1e-9);
 
         gwy_data_field_multiply(fzb, 2.0);
         gwy_data_field_sum_fields(fz, fza, fzc);
