@@ -1,5 +1,5 @@
 /*
- *  $Id: pat_synth.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: pat_synth.c 20983 2018-04-26 15:15:14Z yeti-dn $
  *  Copyright (C) 2010-2017 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -221,7 +221,7 @@ static GwyModuleInfo module_info = {
     N_("Generates surfaces representing simple patterns "
        "(steps, ridges, ...)."),
     "Yeti <yeti@gwyddion.net>",
-    "1.6",
+    "1.7",
     "David Nečas (Yeti)",
     "2010",
 };
@@ -250,6 +250,7 @@ pat_synth(GwyContainer *data, GwyRunType run)
     GwyRandGenSet *rngset;
     GwyDataField *dfield;
     GQuark quark;
+    gboolean ok = TRUE;
     gint id;
 
     g_return_if_fail(run & PAT_SYNTH_RUN_MODES);
@@ -260,11 +261,12 @@ pat_synth(GwyContainer *data, GwyRunType run)
                                      0);
 
     rngset = gwy_rand_gen_set_new(RNG_NRNGS);
-    if (run == GWY_RUN_IMMEDIATE
-        || pat_synth_dialog(&args, &dimsargs, rngset, data, dfield, id)) {
+    if (run == GWY_RUN_INTERACTIVE) {
+        ok = pat_synth_dialog(&args, &dimsargs, rngset, data, dfield, id);
         pat_synth_save_args(gwy_app_settings_get(), &args, &dimsargs);
-        run_noninteractive(&args, &dimsargs, rngset, data, dfield, id, quark);
     }
+    if (ok)
+        run_noninteractive(&args, &dimsargs, rngset, data, dfield, id, quark);
 
     gwy_rand_gen_set_free(rngset);
     gwy_dimensions_free_args(&dimsargs);
@@ -672,44 +674,63 @@ pat_synth_do(const PatSynthArgs *args,
     gwy_data_field_data_changed(dfield);
 }
 
-/* Iterating through rectangles in a growing fashion from the origin to
- * preserve the top left conrer if it's randomly generated.  Field @k holds
- * the current index in the two-dimensional array. */
+/* Iterating through square in a spiral fashion from the origin to preserve the
+ * centre conrer if it's randomly generated.  Field @k holds the current index
+ * in the two-dimensional array.  */
 typedef struct {
-    guint n;
-    guint i, j, k;
-    gboolean horizontal;
+    gint n;
+    gint i, j, k;
+    gint istep, jstep;
+    gint s, segmentend, ntotalstep;
 } GrowingIter;
 
 static inline void
 growing_iter_init(GrowingIter *giter, guint n)
 {
     giter->n = n;
-    giter->k = giter->j = giter->i = 0;
-    giter->horizontal = FALSE;
+    giter->j = giter->i = 0;
+    giter->istep = 0;
+    giter->jstep = -1;
+    giter->ntotalstep = n*n;
+    giter->segmentend = MIN(1, n*n);
+    giter->s = 0;
+    giter->k = (n/2 - giter->i)*n + (giter->j + n/2);
 }
 
 static inline gboolean
 growing_iter_next(GrowingIter *giter)
 {
-    if (giter->horizontal) {
-        if (++giter->j > giter->i) {
-            giter->j = 0;
-            giter->horizontal = FALSE;
-        }
-    }
-    else {
-        if (++giter->j >= giter->i) {
-            if (++giter->i == giter->n)
-                return FALSE;
-            giter->j = 0;
-            giter->horizontal = TRUE;
-        }
-    }
+    giter->i += giter->istep;
+    giter->j += giter->jstep;
+    giter->k = (giter->n/2 - giter->i)*giter->n + (giter->j + giter->n/2);
+    giter->s++;
+    if (giter->s == giter->segmentend) {
+        if (giter->s == giter->ntotalstep)
+            return FALSE;
 
-    giter->k = (giter->horizontal
-                ? giter->i*giter->n + giter->j
-                : giter->j*giter->n + giter->i);
+        if (giter->i == giter->j + 1) {
+            giter->istep = 1;
+            giter->jstep = 0;
+            giter->segmentend = 1 - 2*giter->i;
+        }
+        else if (giter->i == giter->j) {
+            giter->istep = -1;
+            giter->jstep = 0;
+            giter->segmentend = 2*giter->i;
+        }
+        else if (giter->j > 0) {
+            giter->istep = 0;
+            giter->jstep = -1;
+            giter->segmentend = 2*giter->j + 1;
+        }
+        else {
+            giter->istep = 0;
+            giter->jstep = 1;
+            giter->segmentend = 2*giter->i;
+        }
+        giter->segmentend += giter->s;
+        giter->segmentend = MIN(giter->segmentend, giter->ntotalstep);
+    }
     return TRUE;
 }
 
@@ -1612,7 +1633,7 @@ make_pattern_holes(const PatSynthArgs *args,
     GwyDataField *displacement_x, *displacement_y;
     guint n, i, j, xres, yres;
     gdouble *params, *data, *dx_data, *dy_data;
-    gdouble h, c, s, xoff, yoff, pitch, range, margin;
+    gdouble h, c, s, xoff, yoff, pitch, range;
     GrowingIter giter;
 
     h = pargs->height * pow10(dimsargs->zpow10);
@@ -1622,9 +1643,8 @@ make_pattern_holes(const PatSynthArgs *args,
     data = gwy_data_field_get_data(dfield);
 
     pitch = pargs->size + 2*pargs->slope + pargs->distance;
-    margin = (hypot(xres, yres) - MAX(xres, yres) + 4*pargs->sigma + 2);
     range = (xres + yres + 8*pargs->sigma + 4)/pitch;
-    n = GWY_ROUND(range + 1);
+    n = GWY_ROUND(range + 1) | 1;
     params = g_new(gdouble, HOLE_NPARAMS*n*n);
 
     growing_iter_init(&giter, n);
@@ -1657,8 +1677,8 @@ make_pattern_holes(const PatSynthArgs *args,
 
     c = cos(pargs->angle);
     s = sin(pargs->angle);
-    xoff = 0.5*((1.0 - c)*xres + s*yres);
-    yoff = 0.5*(-s*yres + (1.0 - c)*yres);
+    xoff = 0.5*(s*yres - c*xres + pitch*n + pargs->distance);
+    yoff = 0.5*(-s*xres - c*yres + pitch*n + pargs->distance);
     for (i = 0; i < yres; i++) {
         for (j = 0; j < xres; j++) {
             gdouble v[4], x, y, xu, yu, xx, yy;
@@ -1667,8 +1687,8 @@ make_pattern_holes(const PatSynthArgs *args,
 
             xu = xoff + j*c - i*s;
             yu = yoff + j*s + i*c;
-            x = xu + dx_data[i*xres + j] + margin;
-            y = yu + dy_data[i*xres + j] + margin;
+            x = xu + dx_data[i*xres + j];
+            y = yu + dy_data[i*xres + j];
             kx = (gint)floor(x/pitch);
             ky = (gint)floor(y/pitch);
             x -= pitch*(kx + 0.5);
