@@ -1,5 +1,5 @@
 /*
- *  $Id: roddeposit_synth.c 20742 2018-01-20 15:36:54Z yeti-dn $
+ *  $Id: roddeposit_synth.c 21302 2018-08-08 14:33:39Z yeti-dn $
  *  Copyright (C) 2007,2009,2010 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -1268,8 +1268,8 @@ rod_deposit_synth_do(const RodDepositSynthArgs *args,
 //    FILE *fo;
 
     /* The units must be the same. */
-    gwy_serializable_clone(G_OBJECT(gwy_data_field_get_si_unit_xy(dfield)),
-                           G_OBJECT(gwy_data_field_get_si_unit_z(dfield)));
+    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield),
+                       gwy_data_field_get_si_unit_xy(dfield));
 
     //FIXME renormalize everything for size of field 1x1, including z. Change parameters of potentials.
     norm = 1/gwy_data_field_get_xreal(dfield);
