@@ -1,5 +1,5 @@
 /*
- *  $Id: semsim.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: semsim.c 21410 2018-09-07 14:15:50Z yeti-dn $
  *  Copyright (C) 2014 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -117,7 +117,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Simple SEM image simulation from topography."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "1.2",
     "David Nečas (Yeti)",
     "2014",
 };
@@ -184,7 +184,6 @@ semsim(GwyContainer *data, GwyRunType run)
 
     gwy_app_wait_start(gwy_app_find_window_for_channel(data, id),
                        _("SEM image simulation..."));
-    gwy_app_wait_set_fraction(0.0);
 
     if (semsim_do(dfield, showfield, &args)) {
         gwy_app_undo_qcheckpointv(data, 1, &squark);
@@ -238,7 +237,7 @@ semsim_dialog(SEMsimArgs *args, GwyDataField *dfield)
     controls.format_sigma
         = gwy_data_field_get_value_format_xy(dfield,
                                              GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
-    controls.format_sigma->magnitude /= gwy_data_field_get_xmeasure(dfield);
+    controls.format_sigma->magnitude /= gwy_data_field_get_dx(dfield);
     label = gtk_label_new(NULL);
     gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
     gtk_label_set_markup(GTK_LABEL(label), controls.format_sigma->units);
@@ -357,8 +356,8 @@ semsim_do(GwyDataField *dfield,
     gdouble sigma;
     gboolean ok;
 
-    common.dx = gwy_data_field_get_xmeasure(dfield);
-    common.dy = gwy_data_field_get_ymeasure(dfield);
+    common.dx = gwy_data_field_get_dx(dfield);
+    common.dy = gwy_data_field_get_dy(dfield);
     sigma = args->sigma * common.dx;
     common.exth = (gint)ceil(5.5*sigma/common.dx);
     common.extv = (gint)ceil(5.5*sigma/common.dy);
