@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: slope_dist.c 19948 2017-06-22 14:05:41Z yeti-dn $
+ *  $Id: slope_dist.c 20677 2017-12-18 18:22:52Z yeti-dn $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -732,7 +732,8 @@ slope_do_graph_phi(GwyDataField *dfield,
     yd = gwy_data_field_get_data_const(yder);
     for (i = 0; i < n; i++) {
         if (is_counted(mfield, i, args->masking)) {
-            gdouble phi = fmod(atan2(yd[i], -xd[i]) + 2*G_PI, 2*G_PI);
+            gdouble phi = gwy_canonicalize_angle(atan2(yd[i], -xd[i]),
+                                                 TRUE, TRUE);
             gdouble d = (xd[i]*xd[i] + yd[i]*yd[i]);
             gint iphi = floor(args->size*phi/(2.0*G_PI));
 
