@@ -1,5 +1,5 @@
 /*
- *  $Id: sphere-revolve.c 20725 2018-01-16 23:42:48Z yeti-dn $
+ *  $Id: sphere-revolve.c 21411 2018-09-07 14:29:17Z yeti-dn $
  *  Copyright (C) 2004-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -160,8 +160,8 @@ arcrev(GwyContainer *data, GwyRunType run)
     arcrev_load_args(settings, &args);
 
     /* FIXME: this is bogus for non-square pixels anyway */
-    xr = gwy_data_field_get_xreal(dfield)/gwy_data_field_get_xres(dfield);
-    yr = gwy_data_field_get_yreal(dfield)/gwy_data_field_get_yres(dfield);
+    xr = gwy_data_field_get_dx(dfield);
+    yr = gwy_data_field_get_dy(dfield);
     args.pixelsize = hypot(xr, yr);
     args.valform
         = gwy_data_field_get_value_format_xy(dfield,
@@ -245,8 +245,8 @@ sphrev(GwyContainer *data, GwyRunType run)
     sphrev_load_args(settings, &args);
 
     /* FIXME: this is bogus for non-square pixels anyway */
-    xr = gwy_data_field_get_xreal(dfield)/gwy_data_field_get_xres(dfield);
-    yr = gwy_data_field_get_yreal(dfield)/gwy_data_field_get_yres(dfield);
+    xr = gwy_data_field_get_dx(dfield);
+    yr = gwy_data_field_get_dy(dfield);
     args.pixelsize = hypot(xr, yr);
     args.valform
         = gwy_data_field_get_value_format_xy(dfield,
