@@ -1,5 +1,5 @@
 /*
- *  $Id: synth.h 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: synth.h 21102 2018-05-10 15:06:02Z yeti-dn $
  *  Copyright (C) 2009,2010 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -36,6 +36,7 @@
 #endif
 
 #include <libgwydgets/gwydgetutils.h>
+#include <libgwydgets/gwycheckboxes.h>
 
 typedef void (*GwySynthUpdateValueFunc)(GWY_SYNTH_CONTROLS *controls);
 
@@ -197,6 +198,15 @@ gwy_synth_boolean_changed_silent(GtkToggleButton *button,
     *target = gtk_toggle_button_get_active(button);
 }
 
+G_GNUC_UNUSED
+static void
+gwy_synth_flag_changed_silent(GtkWidget *button,
+                              guint *target)
+{
+    GSList *group = gwy_check_box_get_group(GTK_WIDGET(button));
+    *target = gwy_check_boxes_get_selected(group);
+}
+
 G_GNUC_UNUSED
 static void
 gwy_synth_toggle_sensitive(GtkToggleButton *toggle,
@@ -712,6 +722,93 @@ gwy_synth_save_arg_boolean(GwyContainer *settings,
     g_string_truncate(key, len);
 }
 
+G_GNUC_UNUSED
+static GArray**
+gwy_synth_make_evolution_arrays(const GwyEnum *graph_flags_enum,
+                                guint n_graph_flags,
+                                guint flags)
+{
+    GArray **evolution;
+    guint i;
+
+    if (!flags)
+        return NULL;
+
+    evolution = g_new0(GArray*, n_graph_flags+1);
+    for (i = 0; i < n_graph_flags; i++) {
+        if (flags & graph_flags_enum[i].value)
+            evolution[i] = g_array_new(FALSE, FALSE, sizeof(gdouble));
+    }
+    evolution[n_graph_flags] = g_array_new(FALSE, FALSE, sizeof(gdouble));
+
+    return evolution;
+}
+
+G_GNUC_UNUSED
+static GArray*
+gwy_synth_find_evolution_array(GArray **evolution,
+                               const GwyEnum *graph_flags_enum,
+                               guint n_graph_flags,
+                               guint flag)
+{
+    guint i;
+
+    for (i = 0; i < n_graph_flags; i++) {
+        if (graph_flags_enum[i].value == flag)
+            return evolution[i];
+    }
+    g_return_val_if_reached(NULL);
+}
+
+G_GNUC_UNUSED
+static void
+gwy_synth_make_evolution_plots(GwyGraphCurveModel **gcmodels,
+                               GArray **evolution,
+                               const GwyEnum *graph_flags_enum,
+                               guint n_graph_flags)
+{
+    const gdouble *xdata;
+    guint i, n;
+
+    if (!evolution) {
+        if (gcmodels)
+            gwy_clear(gcmodels, n_graph_flags);
+        return;
+    }
+
+    xdata = (const gdouble*)evolution[n_graph_flags]->data;
+    n = evolution[n_graph_flags]->len;
+    for (i = 0; i < n_graph_flags; i++) {
+        if (!evolution[i]) {
+            gcmodels[i] = NULL;
+            continue;
+        }
+
+        gcmodels[i] = gwy_graph_curve_model_new();
+        gwy_graph_curve_model_set_data(gcmodels[i],
+                                       xdata, (gdouble*)evolution[i]->data, n);
+        g_object_set(gcmodels[i],
+                     "description", _(graph_flags_enum[i].name),
+                     NULL);
+    }
+}
+
+G_GNUC_UNUSED
+static void
+gwy_synth_free_evolution_arrays(GArray **evolution, guint n_graph_flags)
+{
+    guint i;
+
+    if (!evolution)
+        return;
+
+    for (i = 0; i <= n_graph_flags; i++) {
+        if (evolution[i])
+            g_array_free(evolution[i], TRUE);
+    }
+    g_free(evolution);
+}
+
 #endif
 
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
