@@ -1,5 +1,5 @@
 /*
- *  $Id: wpour_mark.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: wpour_mark.c 21305 2018-08-09 14:49:32Z yeti-dn $
  *  Copyright (C) 2014 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -610,7 +610,7 @@ set_visible_images(WPourControls *controls)
 
     if (!gwy_container_gis_object_by_name(controls->mydata, "/1/data",
                                           &preproc)) {
-        preproc = gwy_data_field_new_alike(dfield, FALSE);
+        preproc = gwy_data_field_new_alike(dfield, TRUE);
         gwy_container_set_object_by_name(controls->mydata, "/1/data",
                                          preproc);
         g_object_unref(preproc);
