@@ -1,30 +1,16 @@
-#!/usr/bin/python
-# $Id: pygwy-codegen.py 20146 2017-08-02 20:31:56Z yeti-dn $
+#!/usr/bin/python2
+# $Id: pygwy-codegen.py 21284 2018-07-31 19:14:38Z yeti-dn $
 # Runtime for pygtk module codegen.py and overriden functions for Gwyddion data
 # types.
 # Public domain
 
-import sys
-import re
-import string
+import sys, os, re, string
 
-# Add codegen path to our import path
-i = 1
-codegendir = None
-while i < len(sys.argv):
-    arg = sys.argv[i]
-    if arg == '--codegendir':
-        del sys.argv[i]
-        codegendir = sys.argv[i]
-        del sys.argv[i]
-    elif arg.startswith('--codegendir='):
-        codegendir = arg.split('=', 2)[1]
-        del sys.argv[i]
-    else:
-        i += 1
+# Add codegen path to our import path.  NB: codegen does its own argument
+# parsing so we must not an option parser.  Use environment to pass other info.
+codegendir = os.environ['PYGTK_CODEGENDIR'].strip()
 if codegendir:
     sys.path.insert(0, codegendir)
-del codegendir
 
 # Load it
 from codegen import *
