@@ -1,5 +1,5 @@
-#!/usr/bin/python
-# $Id: pygwy-fix-defs-2.py 20605 2017-12-09 11:04:11Z yeti-dn $
+#!/usr/bin/python2
+# $Id: pygwy-fix-defs-2.py 21563 2018-10-30 19:47:45Z yeti-dn $
 # alter defs, fix parameters and return types
 import sys, re
 from optparse import OptionParser
@@ -7,7 +7,7 @@ from optparse import OptionParser
 parser = OptionParser(usage='''\
 pygwy-fix-defs.py --codegendir=DIR
 Reads pygwy.defs.tmp.
-Generated pygwy.defs and boxed.override.''')
+Generates pygwy.defs and boxed.override.''')
 
 parser.add_option('-c', '--codegendir', dest='codegendir', type='string',
                   default='', metavar='DIR',
@@ -34,6 +34,7 @@ gwy_data_field_area_fit_plane(pa, pbx, pby)
 gwy_data_field_area_get_inclination(theta, phi)
 gwy_data_field_area_get_min_max_mask(min, max)
 gwy_data_field_area_get_min_max(min, max)
+gwy_data_field_area_get_dispersion(xcenter, ycenter)
 gwy_data_field_area_get_normal_coeffs(nx, ny, nz)
 gwy_data_field_area_get_stats(avg, ra, rms, skew, kurtosis)
 gwy_data_field_area_get_stats_mask(avg, ra, rms, skew, kurtosis)
@@ -47,6 +48,7 @@ gwy_data_field_fractal_triangulation_dim(a, b)
 gwy_data_field_get_autorange(from_, to)
 gwy_data_field_get_inclination(theta, phi)
 gwy_data_field_get_min_max(min, max)
+gwy_data_field_get_dispersion(xcenter, ycenter)
 gwy_data_field_get_normal_coeffs(nx, ny, nz)
 gwy_data_field_get_stats(avg, ra, rms, skew, kurtosis)
 gwy_data_field_hough_datafield_line_to_polar(rho, theta)
@@ -206,6 +208,7 @@ gwy_data_field_area_row_acf(mask,weights)
 gwy_data_field_area_row_hhcf(mask,weights)
 gwy_data_field_area_row_psdf(mask)
 gwy_data_field_area_row_asg(mask)
+gwy_data_field_get_profile_mask(mask)
 '''
 
 skip_args = '''
@@ -216,6 +219,7 @@ gwy_tip_estimate_full(set_fraction=NULL, set_message=NULL)
 gwy_graph_export_postscript(str=NULL)
 gwy_graph_model_export_ascii(string=NULL)
 gwy_data_field_get_profile(data_line=NULL)
+gwy_data_field_area_filter_kth_rank(set_fraction=NULL)
 gwy_brick_get_value_format_x(format=NULL)
 gwy_brick_get_value_format_y(format=NULL)
 gwy_brick_get_value_format_z(format=NULL)
@@ -253,7 +257,6 @@ gwy_file_save(mode=GWY_RUN_NONINTERACTIVE)
 
 # Lists of functions that should have various attributes
 caller_owns_return_funcs = '''
-gwy_app_file_load
 gwy_brick_duplicate
 gwy_brick_new_alike
 gwy_brick_new_part
@@ -269,6 +272,8 @@ gwy_data_field_area_row_hhcf
 gwy_data_field_area_row_psdf
 gwy_data_field_area_row_asg
 gwy_data_field_duplicate
+gwy_data_field_find_row_shifts_trimmed_diff
+gwy_data_field_find_row_shifts_trimmed_mean
 gwy_data_field_new_alike
 gwy_data_field_new_resampled
 gwy_data_field_new_rotated
