@@ -1,5 +1,5 @@
 /*
- *  $Id: pygwy.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: pygwy.c 21373 2018-08-30 14:27:28Z yeti-dn $
  *  Copyright (C) 2004-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -884,7 +884,7 @@ pygwy_file_load_run(const gchar *filename, GwyRunType mode, GError **error,
     class_name = find_out_class_name(res);
     if (class_name && gwy_strequal(class_name, "Container")) {
         PyGObject *pyg_res = (PyGObject*)res;
-        container = g_object_ref(pyg_res->obj);
+        container = (GwyContainer*)g_object_ref(pyg_res->obj);
     }
     g_free(class_name);
 
