@@ -1,6 +1,6 @@
 /*
- *  $Id: wrap_calls.c 20605 2017-12-09 11:04:11Z yeti-dn $
- *  Copyright (C) 2008 Jan Horak, 2015-2017 David Necas (Yeti)
+ *  $Id: wrap_calls.c 21563 2018-10-30 19:47:45Z yeti-dn $
+ *  Copyright (C) 2008 Jan Horak, 2015-2018 David Necas (Yeti)
  *  E-mail: xhorak@gmail.com, yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -1067,6 +1067,30 @@ gwy_data_field_unrotate_find_corrections_pygwy(GwyDataLine *derdist,
                                                     (gdouble*)correction->data);
 }
 
+GwyDoubleArray*
+gwy_data_field_get_profile_mask_pygwy(GwyDataField *dfield,
+                                      GwyDataField *mask,
+                                      GwyMaskingType masking,
+                                      gdouble xfrom, gdouble yfrom,
+                                      gdouble xto, gdouble yto,
+                                      gint res,
+                                      gint thickness,
+                                      GwyInterpolationType interpolation)
+{
+    GwyDoubleArray *xyarray;
+    gint ndata;
+    GwyXY *xy;
+
+    xy = gwy_data_field_get_profile_mask(dfield, &ndata, mask, masking,
+                                         xfrom, yfrom, xto, yto,
+                                         res, thickness, interpolation);
+    xyarray = g_array_new(FALSE, FALSE, sizeof(gdouble));
+    g_array_set_size(xyarray, 2*ndata);
+    memcpy(xyarray->data, xy, 2*ndata*sizeof(gdouble));
+    g_free(xy);
+    return xyarray;
+}
+
 GwyDoubleArray*
 gwy_data_line_part_fit_polynom_pygwy(GwyDataLine *data_line,
                                      gint n,
@@ -1397,6 +1421,43 @@ gwy_data_field_get_grain_bounding_boxes_pygwy(GwyDataField *data_field,
     return ok;
 }
 
+/**
+ * gwy_data_field_get_grain_bounding_boxes_periodic_pygwy:
+ * @data_field: A data field representing a mask.
+ * @grains: Array of grain numbers.
+ *
+ * Finds bounding boxes of all grains in a mask data field, assuming periodic
+ * boundary condition.
+ *
+ * The array @grains must have the same number of elements as @mask_field.
+ * Normally it is obtained from a function such as
+ * gwy_data_field_number_grains().
+ *
+ * Returns: An array of quadruples of integers, each representing the bounding
+ *          box of the corresponding grain (the zeroth quadrupe does not
+ *          correspond to any as grain numbers start from 1).
+ **/
+GwyArrayFuncStatus
+gwy_data_field_get_grain_bounding_boxes_periodic_pygwy(GwyDataField *data_field,
+                                                       GwyIntArray *grains,
+                                                       GwyIntArrayOutArg bboxes)
+{
+    guint xres = gwy_data_field_get_xres(data_field);
+    guint yres = gwy_data_field_get_yres(data_field);
+    gboolean ok = (grains->len == xres*yres);
+    if (ok) {
+        guint ngrains = find_ngrains(grains);
+        g_array_set_size(bboxes, 4*(ngrains+1));
+        gwy_data_field_get_grain_bounding_boxes_periodic(data_field, ngrains,
+                                                         (gint*)grains->data,
+                                                         (gint*)bboxes->data);
+    }
+    else
+        g_array_free(bboxes, TRUE);
+    g_array_free(grains, TRUE);
+    return ok;
+}
+
 GwyArrayFuncStatus
 gwy_data_field_get_grain_sizes_pygwy(GwyDataField *data_field,
                                      GwyIntArray *grains,
