@@ -1,5 +1,5 @@
 /*
- *  $Id: distance.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: distance.c 21410 2018-09-07 14:15:50Z yeti-dn $
  *  Copyright (C) 2003-2008 Nenad Ocelic, David Necas (Yeti), Petr Klapetek.
  *  E-mail: ocelic@biochem.mpg.de, yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -616,8 +616,8 @@ gwy_tool_distance_create_report(GwyToolDistance *tool)
     xyunit = gwy_data_field_get_si_unit_xy(plain_tool->data_field);
     zunit = gwy_data_field_get_si_unit_z(plain_tool->data_field);
     if (!(report_style & GWY_RESULTS_REPORT_MACHINE)) {
-        dx = gwy_data_field_get_xmeasure(plain_tool->data_field);
-        dy = gwy_data_field_get_ymeasure(plain_tool->data_field);
+        dx = gwy_data_field_get_dx(plain_tool->data_field);
+        dy = gwy_data_field_get_dy(plain_tool->data_field);
         vf_dist = gwy_si_unit_get_format(xyunit, style, fmin(dx, dy), NULL);
         gwy_data_field_get_min_max(plain_tool->data_field, &dx, &dy);
         vf_dz = gwy_si_unit_get_format(zunit, style,
