@@ -1,6 +1,6 @@
 /*
- *  $Id: filter.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
+ *  $Id: filter.c 21344 2018-08-27 15:11:28Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -74,6 +74,7 @@ struct _GwyToolFilter {
     GwyPlainTool parent_instance;
 
     ToolArgs args;
+    gint isel[4];
 
     GwyRectSelectionLabels *rlabels;
     GtkWidget *filter_type;
@@ -109,6 +110,7 @@ static void     gwy_tool_filter_masking_changed  (GtkWidget *button,
                                                   GwyToolFilter *tool);
 static gboolean gwy_tool_filter_is_sized         (FilterType type);
 static gboolean gwy_tool_filter_is_float_sized   (FilterType type);
+static void     update_selected_rectangle        (GwyToolFilter *tool);
 static void     setup_size_adjustment            (GwyToolFilter *tool);
 static void     gwy_tool_filter_apply            (GwyToolFilter *tool);
 static void     filter_area_sharpen              (GwyDataField *dfield,
@@ -117,12 +119,6 @@ static void     filter_area_sharpen              (GwyDataField *dfield,
                                                   gint row,
                                                   gint width,
                                                   gint height);
-static void     filter_area_cicrular_mean        (GwyDataField *dfield,
-                                                  gint size,
-                                                  gint col,
-                                                  gint row,
-                                                  gint width,
-                                                  gint height);
 static void     apply_masking                    (GwyDataField *dfield,
                                                   GwyDataField *orig,
                                                   GwyDataField *mask,
@@ -135,7 +131,7 @@ static GwyModuleInfo module_info = {
     N_("Filter tool, processes selected part of data with a filter "
        "(conservative denoise, mean, median. Kuwahara, minimum, maximum)."),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "3.14",
+    "3.16",
     "David Nečas (Yeti) & Petr Klapetek",
     "2003",
 };
@@ -359,11 +355,7 @@ gwy_tool_filter_data_switched(GwyTool *gwytool,
 static void
 gwy_tool_filter_data_changed(GwyPlainTool *plain_tool)
 {
-    gwy_rect_selection_labels_fill(GWY_TOOL_FILTER(plain_tool)->rlabels,
-                                   plain_tool->selection,
-                                   plain_tool->data_field,
-                                   NULL, NULL);
-    gwy_tool_filter_selection_changed(plain_tool, 0);
+    update_selected_rectangle(GWY_TOOL_FILTER(plain_tool));
 }
 
 static void
@@ -380,22 +372,8 @@ static void
 gwy_tool_filter_selection_changed(GwyPlainTool *plain_tool,
                                   gint hint)
 {
-    GwyToolFilter *tool;
-    gint n = 0;
-
-    tool = GWY_TOOL_FILTER(plain_tool);
     g_return_if_fail(hint <= 0);
-
-    if (plain_tool->selection) {
-        n = gwy_selection_get_data(plain_tool->selection, NULL);
-        g_return_if_fail(n == 0 || n == 1);
-        gwy_rect_selection_labels_fill(tool->rlabels,
-                                       plain_tool->selection,
-                                       plain_tool->data_field,
-                                       NULL, NULL);
-    }
-    else
-        gwy_rect_selection_labels_fill(tool->rlabels, NULL, NULL, NULL, NULL);
+    update_selected_rectangle(GWY_TOOL_FILTER(plain_tool));
 }
 
 static void
@@ -436,6 +414,26 @@ gwy_tool_filter_masking_changed(GtkWidget *button,
     tool->args.masking = gwy_radio_button_get_value(button);
 }
 
+static void
+update_selected_rectangle(GwyToolFilter *tool)
+{
+    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
+    GwySelection *selection = plain_tool->selection;
+    GwyDataField *field = plain_tool->data_field;
+    gint n;
+
+    if (selection) {
+        n = gwy_selection_get_data(selection, NULL);
+        g_return_if_fail(n == 0 || n == 1);
+        gwy_rect_selection_labels_fill(tool->rlabels, selection, field,
+                                       NULL, tool->isel);
+    }
+    else {
+        gwy_rect_selection_labels_fill(tool->rlabels, NULL, NULL,
+                                       NULL, tool->isel);
+    }
+}
+
 static gboolean
 gwy_tool_filter_is_float_sized(FilterType type)
 {
@@ -456,7 +454,9 @@ gwy_tool_filter_needs_kernel(FilterType type)
     return (type == FILTER_MINIMUM
             || type == FILTER_MAXIMUM
             || type == FILTER_OPENING
-            || type == FILTER_CLOSING);
+            || type == FILTER_CLOSING
+            || type == FILTER_MEAN
+            || type == FILTER_MEDIAN);
 }
 
 static void
@@ -494,10 +494,8 @@ gwy_tool_filter_apply(GwyToolFilter *tool)
 {
     GwyPlainTool *plain_tool;
     GwyDataField *dfield, *origfield = NULL, *kernel = NULL;
-    gdouble sel[4];
-    gint isel[4];
     gdouble sigma;
-    gint size;
+    gint size, col, row, w, h, n = 0;
 
     plain_tool = GWY_PLAIN_TOOL(tool);
     size = tool->args.size;
@@ -506,31 +504,19 @@ gwy_tool_filter_apply(GwyToolFilter *tool)
     g_return_if_fail(plain_tool->id >= 0 && dfield != NULL);
     gwy_tool_filter_save_args(tool);
 
-    if (gwy_selection_get_object(plain_tool->selection, 0, sel)) {
-        isel[0] = floor(gwy_data_field_rtoj(dfield, sel[0]));
-        isel[1] = floor(gwy_data_field_rtoi(dfield, sel[1]));
-        isel[2] = floor(gwy_data_field_rtoj(dfield, sel[2]));
-        isel[3] = floor(gwy_data_field_rtoi(dfield, sel[3]));
-
-        if (sel[0] > sel[2])
-            GWY_SWAP(gdouble, sel[0], sel[2]);
-        if (sel[1] > sel[3])
-            GWY_SWAP(gdouble, sel[1], sel[3]);
-        isel[2] -= isel[0] + 1;
-        isel[3] -= isel[1] + 1;
-    }
-    else {
-        isel[0] = isel[1] = 0;
-        isel[2] = gwy_data_field_get_xres(dfield);
-        isel[3] = gwy_data_field_get_yres(dfield);
-    }
+    col = tool->isel[0];
+    row = tool->isel[1];
+    w = tool->isel[2]+1 - tool->isel[0];
+    h = tool->isel[3]+1 - tool->isel[1];
 
     gwy_app_undo_qcheckpoint(plain_tool->container,
                              gwy_app_get_data_key_for_id(plain_tool->id), 0);
 
     if (gwy_tool_filter_needs_kernel(tool->args.filter_type)) {
         kernel = gwy_data_field_new(size, size, size, size, TRUE);
-        gwy_data_field_elliptic_area_fill(kernel, 0, 0, size, size, 1.0);
+        n = gwy_data_field_elliptic_area_fill(kernel, 0, 0, size, size, 1.0);
+        if (tool->args.filter_type == FILTER_MEAN)
+            gwy_data_field_multiply(kernel, 1.0/n);
     }
 
     /*
@@ -547,89 +533,55 @@ gwy_tool_filter_apply(GwyToolFilter *tool)
     if (tool->args.masking != GWY_MASK_IGNORE && plain_tool->mask_field)
         origfield = gwy_data_field_duplicate(dfield);
 
-    switch (tool->args.filter_type) {
-        case FILTER_MEAN:
-        filter_area_cicrular_mean(dfield, size,
-                                  isel[0], isel[1], isel[2], isel[3]);
-        break;
-
-        case FILTER_MEDIAN:
-        gwy_data_field_area_filter_median(dfield, size,
-                                          isel[0], isel[1],
-                                          isel[2], isel[3]);
-        break;
-
-        case FILTER_MINIMUM:
+    if (tool->args.filter_type == FILTER_MEAN) {
+        gwy_data_field_area_ext_convolve(dfield, col, row, w, h, dfield, kernel,
+                                         GWY_EXTERIOR_BORDER_EXTEND, 0.0,
+                                         FALSE);
+    }
+    else if (tool->args.filter_type == FILTER_MEDIAN) {
+        gwy_data_field_area_filter_kth_rank(dfield, kernel, col, row, w, h,
+                                            n/2, NULL);
+    }
+    else if (tool->args.filter_type == FILTER_MINIMUM) {
         gwy_data_field_area_filter_min_max(dfield, kernel,
                                            GWY_MIN_MAX_FILTER_MINIMUM,
-                                           isel[0], isel[1],
-                                           isel[2], isel[3]);
-        break;
-
-        case FILTER_MAXIMUM:
+                                           col, row, w, h);
+    }
+    else if (tool->args.filter_type == FILTER_MAXIMUM) {
         gwy_data_field_area_filter_min_max(dfield, kernel,
                                            GWY_MIN_MAX_FILTER_MAXIMUM,
-                                           isel[0], isel[1],
-                                           isel[2], isel[3]);
-        break;
-
-        case FILTER_CONSERVATIVE:
-        gwy_data_field_area_filter_conservative(dfield, size,
-                                                isel[0], isel[1],
-                                                isel[2], isel[3]);
-        break;
-
-        case FILTER_KUWAHARA:
-        gwy_data_field_area_filter_kuwahara(dfield,
-                                            isel[0], isel[1],
-                                            isel[2], isel[3]);
-        break;
-
-        case FILTER_DECHECKER:
-        gwy_data_field_area_filter_dechecker(dfield,
-                                             isel[0], isel[1],
-                                             isel[2], isel[3]);
-        break;
-
-        case FILTER_GAUSSIAN:
-        gwy_data_field_area_filter_gaussian(dfield, sigma,
-                                            isel[0], isel[1],
-                                            isel[2], isel[3]);
-        break;
-
-        case FILTER_SHARPEN:
-        filter_area_sharpen(dfield, sigma, isel[0], isel[1], isel[2], isel[3]);
-        break;
-
-        case FILTER_OPENING:
+                                           col, row, w, h);
+    }
+    else if (tool->args.filter_type == FILTER_CONSERVATIVE)
+        gwy_data_field_area_filter_conservative(dfield, size, col, row, w, h);
+    else if (tool->args.filter_type == FILTER_KUWAHARA)
+        gwy_data_field_area_filter_kuwahara(dfield, col, row, w, h);
+    else if (tool->args.filter_type == FILTER_DECHECKER)
+        gwy_data_field_area_filter_dechecker(dfield, col, row, w, h);
+    else if (tool->args.filter_type == FILTER_GAUSSIAN)
+        gwy_data_field_area_filter_gaussian(dfield, sigma, col, row, w, h);
+    else if (tool->args.filter_type == FILTER_SHARPEN)
+        filter_area_sharpen(dfield, sigma, col, row, w, h);
+    else if (tool->args.filter_type == FILTER_OPENING) {
         gwy_data_field_area_filter_min_max(dfield, kernel,
                                            GWY_MIN_MAX_FILTER_OPENING,
-                                           isel[0], isel[1],
-                                           isel[2], isel[3]);
-        break;
-
-        case FILTER_CLOSING:
+                                           col, row, w, h);
+    }
+    else if (tool->args.filter_type == FILTER_CLOSING) {
         gwy_data_field_area_filter_min_max(dfield, kernel,
                                            GWY_MIN_MAX_FILTER_CLOSING,
-                                           isel[0], isel[1],
-                                           isel[2], isel[3]);
-        break;
-
-        case FILTER_ASF_OPENING:
+                                           col, row, w, h);
+    }
+    else if (tool->args.filter_type == FILTER_ASF_OPENING) {
         gwy_data_field_area_filter_disc_asf(dfield, size/2, FALSE,
-                                            isel[0], isel[1],
-                                            isel[2], isel[3]);
-        break;
-
-        case FILTER_ASF_CLOSING:
+                                            col, row, w, h);
+    }
+    else if (tool->args.filter_type == FILTER_ASF_CLOSING) {
         gwy_data_field_area_filter_disc_asf(dfield, size/2, TRUE,
-                                            isel[0], isel[1],
-                                            isel[2], isel[3]);
-        break;
-
-        default:
+                                            col, row, w, h);
+    }
+    else {
         g_assert_not_reached();
-        break;
     }
 
     if (origfield) {
@@ -643,20 +595,6 @@ gwy_tool_filter_apply(GwyToolFilter *tool)
     gwy_plain_tool_log_add(plain_tool);
 }
 
-static void
-filter_area_cicrular_mean(GwyDataField *dfield, gint size,
-                          gint col, gint row, gint width, gint height)
-{
-    GwyDataField *kernel = gwy_data_field_new(size, size, size, size, TRUE);
-    gint n = gwy_data_field_get_elliptic_area_size(size, size);
-
-    gwy_data_field_elliptic_area_fill(kernel, 0, 0, size, size, 1.0/n);
-    gwy_data_field_area_ext_convolve(dfield, col, row, width, height,
-                                     dfield, kernel,
-                                     GWY_EXTERIOR_BORDER_EXTEND, 0.0, FALSE);
-    g_object_unref(kernel);
-}
-
 static void
 filter_area_sharpen(GwyDataField *dfield, gdouble sigma,
                     gint col, gint row, gint width, gint height)
