@@ -1,6 +1,6 @@
 /*
- *  $Id: icolorange.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
+ *  $Id: icolorange.c 21344 2018-08-27 15:11:28Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -61,6 +61,8 @@ struct _GwyToolColorRange {
     GwyGraphModel *histogram_model;
     GwyDataLine *heightdist;
     GwySelection *graph_selection;
+    gint isel[4];
+    gdouble rsel[4];
 
     GtkWidget *is_default;
     GtkLabel *min;
@@ -121,6 +123,7 @@ static void   gwy_tool_color_range_spin_changed     (GwyToolColorRange *tool);
 static void   set_range_to_masked                   (GwyToolColorRange *tool);
 static void   set_range_to_unmasked                 (GwyToolColorRange *tool);
 static void   invert_mapping                        (GwyToolColorRange *tool);
+static void   update_selected_rectangle             (GwyToolColorRange *tool);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -129,7 +132,7 @@ static GwyModuleInfo module_info = {
        "color scale should map to, either on data or on height distribution "
        "histogram."),
     "Yeti <yeti@gwyddion.net>",
-    "3.17",
+    "3.18",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -562,11 +565,10 @@ gwy_tool_color_range_make_keys(GwyToolColorRange *tool,
 static void
 gwy_tool_color_range_data_changed(GwyPlainTool *plain_tool)
 {
-    gwy_rect_selection_labels_fill(GWY_TOOL_COLOR_RANGE(plain_tool)->rlabels,
-                                   plain_tool->selection,
-                                   plain_tool->data_field,
-                                   NULL, NULL);
-    gwy_tool_color_range_update_histogram(GWY_TOOL_COLOR_RANGE(plain_tool));
+    GwyToolColorRange *tool = GWY_TOOL_COLOR_RANGE(plain_tool);
+
+    update_selected_rectangle(tool);
+    gwy_tool_color_range_update_histogram(tool);
 }
 
 static void
@@ -584,23 +586,16 @@ static void
 gwy_tool_color_range_selection_changed(GwyPlainTool *plain_tool,
                                        gint hint)
 {
-    GwyToolColorRange *tool;
+    GwyToolColorRange *tool = GWY_TOOL_COLOR_RANGE(plain_tool);
     GwyLayerBasicRangeType range_type;
     gboolean is_selected = FALSE;
     gdouble range[4];
 
-    tool = GWY_TOOL_COLOR_RANGE(plain_tool);
     g_return_if_fail(hint <= 0);
 
-    if (plain_tool->selection) {
-        is_selected = gwy_selection_get_data(plain_tool->selection, range);
-        gwy_rect_selection_labels_fill(tool->rlabels,
-                                       plain_tool->selection,
-                                       plain_tool->data_field,
-                                       NULL, NULL);
-    }
-    else
-        gwy_rect_selection_labels_fill(tool->rlabels, NULL, NULL, NULL, NULL);
+    update_selected_rectangle(tool);
+    if (plain_tool->selection)
+        is_selected = gwy_selection_get_data(plain_tool->selection, NULL);
 
     range_type = gwy_tool_color_range_get_range_type(tool);
     if (range_type != GWY_LAYER_BASIC_RANGE_FIXED)
@@ -780,9 +775,9 @@ gwy_tool_color_range_set_min_max(GwyToolColorRange *tool)
     GwyPlainTool *plain_tool;
     const GwySIValueFormat *vf;
     gboolean clear = FALSE;
+    gdouble sel[2];
+    gint col, row, w, h;
     gchar buf[64];
-    gdouble sel[4];
-    gint isel[4];
 
     if (tool->data_switch)
         return;
@@ -797,23 +792,20 @@ gwy_tool_color_range_set_min_max(GwyToolColorRange *tool)
     switch (tool->range_source) {
         case USE_SELECTION:
         gwy_debug("source: area selection");
+        col = tool->isel[0];
+        row = tool->isel[1];
+        w = tool->isel[2]+1 - tool->isel[0];
+        h = tool->isel[3]+1 - tool->isel[1];
         if (!plain_tool->selection
-            || !gwy_selection_get_object(plain_tool->selection, 0, sel)
-            || sel[0] == sel[2] || sel[1] == sel[3]) {
+            || !gwy_selection_get_data(plain_tool->selection, NULL)
+            || w <= 1 || h <= 1) {
             clear = TRUE;
             break;
         }
-        isel[0] = floor(gwy_data_field_rtoj(plain_tool->data_field, sel[0]));
-        isel[1] = floor(gwy_data_field_rtoi(plain_tool->data_field, sel[1]));
-        isel[2] = floor(gwy_data_field_rtoj(plain_tool->data_field, sel[2]));
-        isel[3] = floor(gwy_data_field_rtoi(plain_tool->data_field, sel[3]));
-
-        gwy_data_field_area_get_min_max(plain_tool->data_field, NULL,
-                                        MIN(isel[0], isel[2]),
-                                        MIN(isel[1], isel[3]),
-                                        ABS(isel[2] - isel[0]) + 1,
-                                        ABS(isel[3] - isel[1]) + 1,
-                                        &sel[0], &sel[1]);
+        gwy_data_field_area_get_min_max_mask(plain_tool->data_field,
+                                             NULL, GWY_MASK_IGNORE,
+                                             col, row, w, h,
+                                             &sel[0], &sel[1]);
         break;
 
         case USE_HISTOGRAM:
@@ -993,5 +985,25 @@ invert_mapping(GwyToolColorRange *tool)
     gwy_tool_color_range_spin_changed(tool);
 }
 
+static void
+update_selected_rectangle(GwyToolColorRange *tool)
+{
+    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
+    GwySelection *selection = plain_tool->selection;
+    GwyDataField *field = plain_tool->data_field;
+    gint n;
+
+    if (selection) {
+        n = gwy_selection_get_data(selection, NULL);
+        g_return_if_fail(n == 0 || n == 1);
+        gwy_rect_selection_labels_fill(tool->rlabels, selection, field,
+                                       NULL, tool->isel);
+    }
+    else {
+        gwy_rect_selection_labels_fill(tool->rlabels, NULL, NULL,
+                                       NULL, tool->isel);
+    }
+}
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
 
