@@ -1,6 +1,6 @@
 /*
- *  $Id: linestats.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2003-2016 David Necas (Yeti), Petr Klapetek.
+ *  $Id: linestats.c 21344 2018-08-27 15:11:28Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -67,6 +67,7 @@ struct _GwyToolLineStats {
 
     GwyDataLine *line;
     GwyDataLine *weights;
+    gint isel[4];
     gint isel_prev[4];
 
     GtkWidget *graph;
@@ -106,14 +107,8 @@ static void     gwy_tool_line_stats_data_changed     (GwyPlainTool *plain_tool);
 static void     gwy_tool_line_stats_mask_changed     (GwyPlainTool *plain_tool);
 static void     gwy_tool_line_stats_selection_changed(GwyPlainTool *plain_tool,
                                                       gint hint);
-static void     fill_selection_labels                (GwyToolLineStats *tool);
+static void     update_selected_rectangle            (GwyToolLineStats *tool);
 static void     update_sensitivity                   (GwyToolLineStats *tool);
-static gboolean obtain_selection                     (GwyPlainTool *plain_tool,
-                                                      gdouble *sel,
-                                                      gint *col,
-                                                      gint *row,
-                                                      gint *w,
-                                                      gint *h);
 static void     update_curve                         (GwyToolLineStats *tool);
 static void     instant_update_changed               (GtkToggleButton *check,
                                                       GwyToolLineStats *tool);
@@ -170,6 +165,8 @@ static const ToolArgs default_args = {
     GWY_APP_DATA_ID_NONE,
 };
 
+/* If you add something here, consider adding it also to
+ * ../volume/volume_linestat.c */
 static const GwyEnum sf_types[] =  {
     { N_("Mean"),               GWY_LINE_STAT_MEAN,      },
     { N_("Median"),             GWY_LINE_STAT_MEDIAN,    },
@@ -520,6 +517,7 @@ static void
 gwy_tool_line_stats_data_changed(GwyPlainTool *plain_tool)
 {
     GwyToolLineStats *tool = GWY_TOOL_LINE_STATS(plain_tool);
+    update_selected_rectangle(tool);
     update_curve(tool);
 }
 
@@ -531,44 +529,35 @@ gwy_tool_line_stats_mask_changed(GwyPlainTool *plain_tool)
 }
 
 static void
-fill_selection_labels(GwyToolLineStats *tool)
+update_selected_rectangle(GwyToolLineStats *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     GwySelection *selection = plain_tool->selection;
     GwyDataField *field = plain_tool->data_field;
     gint n;
 
-    if (!selection) {
-        gwy_rect_selection_labels_fill(tool->rlabels, NULL, NULL, NULL, NULL);
-        return;
+    if (selection) {
+        n = gwy_selection_get_data(selection, NULL);
+        g_return_if_fail(n == 0 || n == 1);
+        gwy_rect_selection_labels_fill(tool->rlabels, selection, field,
+                                       NULL, tool->isel);
+    }
+    else {
+        gwy_rect_selection_labels_fill(tool->rlabels, NULL, NULL,
+                                       NULL, tool->isel);
     }
-
-    n = gwy_selection_get_data(selection, NULL);
-    g_return_if_fail(n == 0 || n == 1);
-    gwy_rect_selection_labels_fill(tool->rlabels, selection, field, NULL, NULL);
 }
 
 static void
 gwy_tool_line_stats_selection_changed(GwyPlainTool *plain_tool,
                                       gint hint)
 {
-    GwySelection *selection = plain_tool->selection;
-    GwyToolLineStats *tool;
-    gdouble sel[4];
-    gint col, row, w, h;
+    GwyToolLineStats *tool = GWY_TOOL_LINE_STATS(plain_tool);
 
-    tool = GWY_TOOL_LINE_STATS(plain_tool);
     g_return_if_fail(hint <= 0);
-
-    fill_selection_labels(tool);
+    update_selected_rectangle(tool);
     if (tool->args.instant_update) {
-        if (selection) {
-            obtain_selection(plain_tool, sel, &col, &row, &w, &h);
-            if (col != tool->isel_prev[0] || row != tool->isel_prev[1]
-                || w != tool->isel_prev[2] || h != tool->isel_prev[3])
-                update_curve(tool);
-        }
-        else
+        if (memcmp(tool->isel, tool->isel_prev, 4*sizeof(gint)) != 0)
             update_curve(tool);
     }
 }
@@ -579,38 +568,6 @@ update_sensitivity(GwyToolLineStats *tool)
     gtk_widget_set_sensitive(tool->update, !tool->args.instant_update);
 }
 
-static gboolean
-obtain_selection(GwyPlainTool *plain_tool, gdouble *sel,
-                 gint *col, gint *row, gint *w, gint *h)
-{
-    GwyDataField *field = plain_tool->data_field;
-    gint xres = gwy_data_field_get_xres(field);
-    gint yres = gwy_data_field_get_yres(field);
-    gint isel[4];
-
-    if (!gwy_selection_get_object(plain_tool->selection, 0, sel)
-        || sel[0] == sel[2] || sel[1] == sel[3]) {
-        *col = *row = 0;
-        *w = xres;
-        *h = yres;
-        sel[0] = sel[1] = 0.0;
-        sel[2] = gwy_data_field_get_xreal(field);
-        sel[3] = gwy_data_field_get_yreal(field);
-        return TRUE;
-    }
-
-    isel[0] = floor(gwy_data_field_rtoj(field, sel[0]));
-    isel[1] = floor(gwy_data_field_rtoi(field, sel[1]));
-    isel[2] = floor(gwy_data_field_rtoj(field, sel[2]));
-    isel[3] = floor(gwy_data_field_rtoi(field, sel[3]));
-    *w = ABS(isel[2] - isel[0]) + 1;
-    *h = ABS(isel[3] - isel[1]) + 1;
-    *col = MIN(isel[0], isel[2]);
-    *row = MIN(isel[1], isel[3]);
-    return (*col == 0 && *row == 0 && *w == xres && *h == yres);
-}
-
-
 static void
 update_curve(GwyToolLineStats *tool)
 {
@@ -618,7 +575,6 @@ update_curve(GwyToolLineStats *tool)
 
     GwyGraphCurveModel *gcmodel;
     GwyDataField *field;
-    gdouble sel[4];
     gchar *result;
     gint n, nsel, col, row, w, h;
     const gchar *title;
@@ -649,12 +605,16 @@ update_curve(GwyToolLineStats *tool)
         }
     }
 
+    if (plain_tool->pending_updates & GWY_PLAIN_TOOL_CHANGED_SELECTION)
+        update_selected_rectangle(tool);
+    plain_tool->pending_updates = 0;
+
+    gwy_assign(tool->isel_prev, tool->isel, 4);
     n = gwy_graph_model_get_n_curves(tool->gmodel);
-    obtain_selection(plain_tool, sel, &col, &row, &w, &h);
-    tool->isel_prev[0] = col;
-    tool->isel_prev[1] = row;
-    tool->isel_prev[2] = w;
-    tool->isel_prev[3] = h;
+    col = tool->isel[0];
+    row = tool->isel[1];
+    w = tool->isel[2]+1 - tool->isel[0];
+    h = tool->isel[3]+1 - tool->isel[1];
     nsel = (w >= 4 && h >= 4) ? 1 : 0;
     gwy_debug("%d x %d at (%d, %d)", w, h, col, row);
 
@@ -670,10 +630,6 @@ update_curve(GwyToolLineStats *tool)
         return;
     }
 
-    if (plain_tool->pending_updates & GWY_PLAIN_TOOL_CHANGED_SELECTION)
-        fill_selection_labels(tool);
-
-    plain_tool->pending_updates = 0;
     gwy_data_field_get_line_stats_mask(field,
                                        plain_tool->mask_field,
                                        tool->args.masking,
