@@ -1,5 +1,5 @@
 /*
- *  $Id: profile.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: profile.c 21410 2018-09-07 14:15:50Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1024,8 +1024,8 @@ gwy_tool_profile_update_curve(GwyToolProfile *tool,
         /* Just create some line when there is none. */
         if (!tool->line)
             tool->line = gwy_data_line_new(1, 1.0, FALSE);
-        r = MAX(r, hypot(gwy_data_field_get_xmeasure(data_field),
-                         gwy_data_field_get_ymeasure(data_field)));
+        r = MAX(r, hypot(gwy_data_field_get_dx(data_field),
+                         gwy_data_field_get_dy(data_field)));
         gwy_data_field_angular_average(data_field, tool->line, mask,
                                        tool->args.masking,
                                        xc, yc, r, lineres);
@@ -1206,8 +1206,8 @@ estimate_angular_variation(GwyDataField *dfield, GwyDataLine *tmp,
     r = MIN(r, MIN(xc, yc));
     r = MIN(r, MIN(dfield->xreal - xc, dfield->yreal - yc));
 
-    dx = gwy_data_field_get_xmeasure(dfield);
-    dy = gwy_data_field_get_ymeasure(dfield);
+    dx = gwy_data_field_get_dx(dfield);
+    dy = gwy_data_field_get_dy(dfield);
     h = 2.0*dx*dy/(dx + dy);
     ir = GWY_ROUND(r/h);
 
@@ -1266,8 +1266,8 @@ symmetrize_at_scale(GwyDataField *dfield,
 {
     gdouble xreal = dfield->xreal, yreal = dfield->yreal;
     gint i, j, besti = 0, bestj = 0;
-    gdouble dx = gwy_data_field_get_xmeasure(dfield);
-    gdouble dy = gwy_data_field_get_ymeasure(dfield);
+    gdouble dx = gwy_data_field_get_dx(dfield);
+    gdouble dy = gwy_data_field_get_dy(dfield);
     gdouble allvar[25], z[9];
     gdouble var, bestvar = G_MAXDOUBLE;
     gboolean ok;
@@ -1353,8 +1353,8 @@ gwy_tool_profile_symmetrize_profile(GwyToolProfile *tool, gint id)
     g_return_if_fail(plain_tool->selection);
     g_return_if_fail(gwy_selection_get_object(plain_tool->selection, id, line));
     dfield = plain_tool->data_field;
-    dx = gwy_data_field_get_xmeasure(dfield);
-    dy = gwy_data_field_get_ymeasure(dfield);
+    dx = gwy_data_field_get_dx(dfield);
+    dy = gwy_data_field_get_dy(dfield);
 
     /* Don't attempt to optimise very short lines. It would end up in tears. */
     if (hypot((line[2] - line[0])/dx, (line[3] - line[1])/dy) < 4.0)
@@ -1384,8 +1384,8 @@ estimate_orthogonal_variation(GwyDataField *dfield,
     ly = line[3] - line[1];
     l = hypot(lx, ly);
 
-    dx = gwy_data_field_get_xmeasure(dfield);
-    dy = gwy_data_field_get_ymeasure(dfield);
+    dx = gwy_data_field_get_dx(dfield);
+    dy = gwy_data_field_get_dy(dfield);
     h = 2.0*dx*dy/(dx + dy);
 
     /* First orthogonal profile is (xfrom0,yfrom0)--(xto0,yto0),
@@ -1505,8 +1505,8 @@ gwy_tool_profile_straighten_profile(GwyToolProfile *tool, gint id)
     g_return_if_fail(plain_tool->selection);
     g_return_if_fail(gwy_selection_get_object(plain_tool->selection, id, line));
     dfield = plain_tool->data_field;
-    dx = gwy_data_field_get_xmeasure(dfield);
-    dy = gwy_data_field_get_ymeasure(dfield);
+    dx = gwy_data_field_get_dx(dfield);
+    dy = gwy_data_field_get_dy(dfield);
     thickness = MAX((thickness + 1)/2, 4);
 
     /* Don't attempt to optimise very short lines. It would end up in tears. */
