@@ -1,5 +1,5 @@
 /*
- *  $Id: readvalue.c 20677 2017-12-18 18:22:52Z yeti-dn $
+ *  $Id: readvalue.c 21410 2018-09-07 14:15:50Z yeti-dn $
  *  Copyright (C) 2003-2007,2014 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -698,12 +698,12 @@ gwy_tool_read_value_calculate(GwyToolReadValue *tool,
     gwy_math_choleski_solve(3, m, z);
     /* The signs may seem odd.  We have to invert y due to coordinate system
      * and then invert both for downward slopes.  As a result x is inverted. */
-    tool->bx = -z[1]/gwy_data_field_get_xmeasure(dfield);
-    tool->by = z[2]/gwy_data_field_get_ymeasure(dfield);
+    tool->bx = -z[1]/gwy_data_field_get_dx(dfield);
+    tool->by = z[2]/gwy_data_field_get_dy(dfield);
 
     calc_curvatures(tool->values, tool->xpos, tool->ypos, n,
-                    gwy_data_field_get_xmeasure(dfield),
-                    gwy_data_field_get_ymeasure(dfield),
+                    gwy_data_field_get_dx(dfield),
+                    gwy_data_field_get_dy(dfield),
                     &tool->k1, &tool->k2);
 }
 
