@@ -1,6 +1,6 @@
 /*
- *  $Id: sfunctions.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
+ *  $Id: sfunctions.c 21410 2018-09-07 14:15:50Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -88,6 +88,7 @@ struct _GwyToolSFunctions {
     GwyRectSelectionLabels *rlabels;
 
     GwyDataLine *line;
+    gint isel[4];
     gint isel_prev[4];
 
     GwyDataField *cached_flipped_field;
@@ -138,14 +139,8 @@ static void     gwy_tool_sfunctions_data_changed     (GwyPlainTool *plain_tool);
 static void     gwy_tool_sfunctions_mask_changed     (GwyPlainTool *plain_tool);
 static void     gwy_tool_sfunctions_selection_changed(GwyPlainTool *plain_tool,
                                                       gint hint);
-static void     fill_selection_labels                (GwyToolSFunctions *tool);
+static void     update_selected_rectangle            (GwyToolSFunctions *tool);
 static void     update_sensitivity                   (GwyToolSFunctions *tool);
-static gboolean obtain_selection                     (GwyPlainTool *plain_tool,
-                                                      gdouble *sel,
-                                                      gint *col,
-                                                      gint *row,
-                                                      gint *w,
-                                                      gint *h);
 static void     update_curve                         (GwyToolSFunctions *tool);
 static void     instant_update_changed               (GtkToggleButton *check,
                                                       GwyToolSFunctions *tool);
@@ -195,7 +190,7 @@ static GwyModuleInfo module_info = {
        "functions (height distribution, correlations, PSDF, Minkowski "
        "functionals) of selected part of data."),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "2.20",
+    "2.21",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -596,21 +591,23 @@ gwy_tool_sfunctions_response(GwyTool *tool,
 }
 
 static void
-fill_selection_labels(GwyToolSFunctions *tool)
+update_selected_rectangle(GwyToolSFunctions *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     GwySelection *selection = plain_tool->selection;
     GwyDataField *field = plain_tool->data_field;
     gint n;
 
-    if (!selection) {
-        gwy_rect_selection_labels_fill(tool->rlabels, NULL, NULL, NULL, NULL);
-        return;
+    if (selection) {
+        n = gwy_selection_get_data(selection, NULL);
+        g_return_if_fail(n == 0 || n == 1);
+        gwy_rect_selection_labels_fill(tool->rlabels, selection, field,
+                                       NULL, tool->isel);
+    }
+    else {
+        gwy_rect_selection_labels_fill(tool->rlabels, NULL, NULL,
+                                       NULL, tool->isel);
     }
-
-    n = gwy_selection_get_data(selection, NULL);
-    g_return_if_fail(n == 0 || n == 1);
-    gwy_rect_selection_labels_fill(tool->rlabels, selection, field, NULL, NULL);
 }
 
 static void
@@ -620,6 +617,7 @@ gwy_tool_sfunctions_data_changed(GwyPlainTool *plain_tool)
 
     GWY_OBJECT_UNREF(tool->cached_flipped_field);
     update_unc_fields(plain_tool);
+    update_selected_rectangle(tool);
     update_curve(tool);
     update_target_graphs(tool);
 }
@@ -638,23 +636,12 @@ static void
 gwy_tool_sfunctions_selection_changed(GwyPlainTool *plain_tool,
                                       gint hint)
 {
-    GwySelection *selection = plain_tool->selection;
-    GwyToolSFunctions *tool;
-    gdouble sel[4];
-    gint col, row, w, h;
+    GwyToolSFunctions *tool = GWY_TOOL_SFUNCTIONS(plain_tool);
 
-    tool = GWY_TOOL_SFUNCTIONS(plain_tool);
     g_return_if_fail(hint <= 0);
-
-    fill_selection_labels(tool);
+    update_selected_rectangle(tool);
     if (tool->args.instant_update) {
-        if (selection) {
-            obtain_selection(plain_tool, sel, &col, &row, &w, &h);
-            if (col != tool->isel_prev[0] || row != tool->isel_prev[1]
-                || w != tool->isel_prev[2] || h != tool->isel_prev[3])
-                update_curve(tool);
-        }
-        else
+        if (memcmp(tool->isel, tool->isel_prev, 4*sizeof(gint)) != 0)
             update_curve(tool);
     }
 }
@@ -684,37 +671,6 @@ update_sensitivity(GwyToolSFunctions *tool)
         gtk_widget_set_sensitive(GTK_WIDGET(l->data), sensitive);
 }
 
-static gboolean
-obtain_selection(GwyPlainTool *plain_tool, gdouble *sel,
-                 gint *col, gint *row, gint *w, gint *h)
-{
-    GwyDataField *field = plain_tool->data_field;
-    gint xres = gwy_data_field_get_xres(field);
-    gint yres = gwy_data_field_get_yres(field);
-    gint isel[4];
-
-    if (!gwy_selection_get_object(plain_tool->selection, 0, sel)
-        || sel[0] == sel[2] || sel[1] == sel[3]) {
-        *col = *row = 0;
-        *w = xres;
-        *h = yres;
-        sel[0] = sel[1] = 0.0;
-        sel[2] = gwy_data_field_get_xreal(field);
-        sel[3] = gwy_data_field_get_yreal(field);
-        return TRUE;
-    }
-
-    isel[0] = floor(gwy_data_field_rtoj(field, sel[0]));
-    isel[1] = floor(gwy_data_field_rtoi(field, sel[1]));
-    isel[2] = floor(gwy_data_field_rtoj(field, sel[2]));
-    isel[3] = floor(gwy_data_field_rtoi(field, sel[3]));
-    *w = ABS(isel[2] - isel[0]) + 1;
-    *h = ABS(isel[3] - isel[1]) + 1;
-    *col = MIN(isel[0], isel[2]);
-    *row = MIN(isel[1], isel[3]);
-    return (*col == 0 && *row == 0 && *w == xres && *h == yres);
-}
-
 static void
 update_curve(GwyToolSFunctions *tool)
 {
@@ -724,7 +680,6 @@ update_curve(GwyToolSFunctions *tool)
     GwyInterpolationType interp = tool->args.interpolation;
     GwyGraphModel *gmodel = tool->gmodel;
     GwyGraphCurveModel *gcmodel, *ugcmodel = NULL;
-    gdouble sel[4];
     gint n, nsel, lineres, col, row, w, h;
     const gchar *title, *xlabel, *ylabel;
     gboolean xy_is_flipped;
@@ -739,12 +694,16 @@ update_curve(GwyToolSFunctions *tool)
         return;
     }
 
+    if (plain_tool->pending_updates & GWY_PLAIN_TOOL_CHANGED_SELECTION)
+        update_selected_rectangle(tool);
+    plain_tool->pending_updates = 0;
+
+    gwy_assign(tool->isel_prev, tool->isel, 4);
     n = gwy_graph_model_get_n_curves(gmodel);
-    obtain_selection(plain_tool, sel, &col, &row, &w, &h);
-    tool->isel_prev[0] = col;
-    tool->isel_prev[1] = row;
-    tool->isel_prev[2] = w;
-    tool->isel_prev[3] = h;
+    col = tool->isel[0];
+    row = tool->isel[1];
+    w = tool->isel[2]+1 - tool->isel[0];
+    h = tool->isel[3]+1 - tool->isel[1];
     nsel = (dfield && w >= 4 && h >= 4) ? 1 : 0;
     gwy_debug("%d x %d at (%d, %d)", w, h, col, row);
 
@@ -758,10 +717,6 @@ update_curve(GwyToolSFunctions *tool)
         return;
     }
 
-    if (plain_tool->pending_updates & GWY_PLAIN_TOOL_CHANGED_SELECTION)
-        fill_selection_labels(tool);
-
-    plain_tool->pending_updates = 0;
     tool->has_uline = FALSE;
     lineres = tool->args.fixres ? tool->args.resolution : -1;
 
@@ -1188,12 +1143,12 @@ gwy_data_field_area_range(GwyDataField *dfield,
     if (direction == GWY_ORIENTATION_HORIZONTAL) {
         res = width-1;
         thickness = height;
-        h = gwy_data_field_get_xmeasure(dfield);
+        h = gwy_data_field_get_dx(dfield);
     }
     else if (direction == GWY_ORIENTATION_VERTICAL) {
         res = height-1;
         thickness = width;
-        h = gwy_data_field_get_ymeasure(dfield);
+        h = gwy_data_field_get_dy(dfield);
     }
     else {
         g_return_if_reached();
