@@ -1,6 +1,6 @@
 /*
- *  $Id: stats.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
+ *  $Id: stats.c 21573 2018-11-01 16:25:32Z yeti-dn $
+ *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -37,8 +37,6 @@
 #include <app/gwyapp.h>
 #include <app/gwymoduleutils.h>
 
-#define ENTROPY_NORMAL 1.41893853320467274178l
-
 #define GWY_TYPE_TOOL_STATS            (gwy_tool_stats_get_type())
 #define GWY_TOOL_STATS(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), GWY_TYPE_TOOL_STATS, GwyToolStats))
 #define GWY_IS_TOOL_STATS(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), GWY_TYPE_TOOL_STATS))
@@ -61,11 +59,11 @@ const gchar *guivalues[] = {
     "min", "max", "median",
     "Sp", "Sv", "Sz",
     /* Hybrid */
-    "projarea", "area",
+    "projarea", "area", "volume",
     "var",
     "theta", "phi",
     /* Other */
-    "entropy", "entropydef",
+    "linedis",
 };
 
 enum { NGUIVALUES = G_N_ELEMENTS(guivalues) };
@@ -92,7 +90,9 @@ struct _GwyToolStats {
     GtkWidget *rexport;
 
     GtkWidget *guivalues[NGUIVALUES];
+    gint isel[4];
     gint isel_prev[4];
+    gdouble rsel[4];
 
     GSList *masking;
     GtkWidget *instant_update;
@@ -127,14 +127,8 @@ static void     gwy_tool_stats_selection_changed(GwyPlainTool *plain_tool,
                                                  gint hint);
 static void     report_style_changed            (GwyToolStats *tool,
                                                  GwyResultsExport *rexport);
-static void     fill_selection_labels           (GwyToolStats *tool);
+static void     update_selected_rectangle       (GwyToolStats *tool);
 static void     update_labels                   (GwyToolStats *tool);
-static gboolean obtain_selection                (GwyPlainTool *plain_tool,
-                                                 gdouble *sel,
-                                                 gint *col,
-                                                 gint *row,
-                                                 gint *w,
-                                                 gint *h);
 static gboolean gwy_tool_stats_calculate        (GwyToolStats *tool);
 static void     calculate_uncertainties         (GwyToolStats *tool,
                                                  StatsUncertanties *unc,
@@ -157,7 +151,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Statistics tool."),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "3.1",
+    "3.2",
     "David Nečas (Yeti) & Petr Klapetek",
     "2003",
 };
@@ -287,7 +281,7 @@ gwy_tool_stats_init(GwyToolStats *tool)
     gwy_results_add_value(results, "Sp", N_("Maximum peak height"),
                           "power-z", 1, "symbol", "Sp", NULL);
     gwy_results_add_value(results, "Sv", N_("Maximum pit depth"),
-                          "power-z", 1, "symbol", "Sp", NULL);
+                          "power-z", 1, "symbol", "Sv", NULL);
     gwy_results_add_value(results, "Sz", N_("Maximum height"),
                           "power-z", 1, "symbol", "Sz", NULL);
     gwy_results_bind_formats(results,
@@ -303,6 +297,10 @@ gwy_tool_stats_init(GwyToolStats *tool)
                           "type", GWY_RESULTS_VALUE_FLOAT,
                           "power-x", 1, "power-y", 1,
                           NULL);
+    gwy_results_add_value(results, "volume", N_("Volume"),
+                          "type", GWY_RESULTS_VALUE_FLOAT,
+                          "power-x", 1, "power-y", 1, "power-z", 1,
+                          NULL);
     gwy_results_add_value(results, "var", N_("Variation"),
                           "type", GWY_RESULTS_VALUE_FLOAT,
                           "power-x", 1, "power-z", 1,
@@ -311,8 +309,7 @@ gwy_tool_stats_init(GwyToolStats *tool)
     gwy_results_add_value_angle(results, "phi", N_("Inclination φ"));
     gwy_results_add_separator(results);
 
-    gwy_results_add_value_plain(results, "entropy", N_("Entropy"));
-    gwy_results_add_value_plain(results, "entropydef", N_("Entropy deficit"));
+    gwy_results_add_value_plain(results, "linedis", N_("Scan line discrepancy"));
 
     gwy_tool_stats_init_dialog(tool);
 }
@@ -404,7 +401,7 @@ gwy_tool_stats_init_dialog(GwyToolStats *tool)
             header = _("Order-Based");
         else if (i == 12)
             header = gwy_sgettext("parameters|Hybrid");
-        else if (i == 17)
+        else if (i == 18)
             header = _("Other");
 
         if (header) {
@@ -522,21 +519,23 @@ update_units(GwyToolStats *tool)
 }
 
 static void
-fill_selection_labels(GwyToolStats *tool)
+update_selected_rectangle(GwyToolStats *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     GwySelection *selection = plain_tool->selection;
     GwyDataField *field = plain_tool->data_field;
     gint n;
 
-    if (!selection) {
-        gwy_rect_selection_labels_fill(tool->rlabels, NULL, NULL, NULL, NULL);
-        return;
+    if (selection) {
+        n = gwy_selection_get_data(selection, NULL);
+        g_return_if_fail(n == 0 || n == 1);
+        gwy_rect_selection_labels_fill(tool->rlabels, selection, field,
+                                       tool->rsel, tool->isel);
+    }
+    else {
+        gwy_rect_selection_labels_fill(tool->rlabels, NULL, NULL,
+                                       tool->rsel, tool->isel);
     }
-
-    n = gwy_selection_get_data(selection, NULL);
-    g_return_if_fail(n == 0 || n == 1);
-    gwy_rect_selection_labels_fill(tool->rlabels, selection, field, NULL, NULL);
 }
 
 static void
@@ -558,7 +557,7 @@ gwy_tool_stats_data_changed(GwyPlainTool *plain_tool)
         && gwy_container_gis_object_by_name(container, zukey, &tool->zunc))
         GWY_TOOL_STATS(plain_tool)->has_calibration = TRUE;
 
-    fill_selection_labels(tool);
+    update_selected_rectangle(tool);
     update_units(tool);
     update_labels(tool);
 }
@@ -590,23 +589,12 @@ static void
 gwy_tool_stats_selection_changed(GwyPlainTool *plain_tool,
                                  gint hint)
 {
-    GwySelection *selection = plain_tool->selection;
-    GwyToolStats *tool;
-    gdouble sel[4];
-    gint col, row, w, h;
+    GwyToolStats *tool = GWY_TOOL_STATS(plain_tool);
 
-    tool = GWY_TOOL_STATS(plain_tool);
     g_return_if_fail(hint <= 0);
-
-    fill_selection_labels(tool);
+    update_selected_rectangle(tool);
     if (tool->args.instant_update) {
-        if (selection) {
-            obtain_selection(plain_tool, sel, &col, &row, &w, &h);
-            if (col != tool->isel_prev[0] || row != tool->isel_prev[1]
-                || w != tool->isel_prev[2] || h != tool->isel_prev[3])
-                update_labels(tool);
-        }
-        else
+        if (memcmp(tool->isel, tool->isel_prev, 4*sizeof(gint)) != 0)
             update_labels(tool);
     }
     else
@@ -629,9 +617,9 @@ update_labels(GwyToolStats *tool)
     }
 
     if (plain_tool->pending_updates & GWY_PLAIN_TOOL_CHANGED_SELECTION)
-        fill_selection_labels(tool);
-
+        update_selected_rectangle(tool);
     plain_tool->pending_updates = 0;
+
     if (!gwy_tool_stats_calculate(tool))
         return;
 
@@ -641,35 +629,63 @@ update_labels(GwyToolStats *tool)
     }
 }
 
-static gboolean
-obtain_selection(GwyPlainTool *plain_tool, gdouble *sel,
-                 gint *col, gint *row, gint *w, gint *h)
+static gdouble
+gwy_data_field_scan_line_discrepancy(GwyDataField *dfield,
+                                     GwyDataField *mask,
+                                     GwyMaskingType masking,
+                                     gint col, gint row,
+                                     gint width, gint height)
 {
-    GwyDataField *field = plain_tool->data_field;
-    gint xres = gwy_data_field_get_xres(field);
-    gint yres = gwy_data_field_get_yres(field);
-    gint isel[4];
+    gint xres = dfield->xres, yres = dfield->yres, i, j, n;
+    const gdouble *drow, *drowp, *drown, *mrow;
+    gdouble v, s2 = 0.0;
 
-    if (!gwy_selection_get_object(plain_tool->selection, 0, sel)
-        || sel[0] == sel[2] || sel[1] == sel[3]) {
-        *col = *row = 0;
-        *w = xres;
-        *h = yres;
-        sel[0] = sel[1] = 0.0;
-        sel[2] = gwy_data_field_get_xreal(field);
-        sel[3] = gwy_data_field_get_yreal(field);
-        return TRUE;
+    if (masking == GWY_MASK_IGNORE)
+        mask = NULL;
+    if (!mask)
+        masking = GWY_MASK_IGNORE;
+
+    /* Cannot calculate discrepancy with only one scan line. */
+    if (yres < 2)
+        return 0.0;
+
+    n = 0;
+    for (i = 0; i < height; i++) {
+        /* Take the two neighbour rows, except for the first and last one,
+         * where we simply calculate the difference from the sole neigbour. */
+        drow = dfield->data + (row + i)*xres + col;
+        drowp = (row + i > 0) ? drow - xres : drow + xres;
+        drown = (row + i < yres-1) ? drow + xres : drow - xres;
+        if (masking == GWY_MASK_INCLUDE) {
+            mrow = mask->data + (row + i)*xres + col;
+            for (j = 0; j < width; j++) {
+                if (mrow[j] > 0.0) {
+                    v = drow[j] - 0.5*(drowp[j] + drown[j]);
+                    s2 += v*v;
+                    n++;
+                }
+            }
+        }
+        else if (masking == GWY_MASK_EXCLUDE) {
+            mrow = mask->data + (row + i)*xres + col;
+            for (j = 0; j < width; j++) {
+                if (mrow[j] <= 0.0) {
+                    v = drow[j] - 0.5*(drowp[j] + drown[j]);
+                    s2 += v*v;
+                    n++;
+                }
+            }
+        }
+        else {
+            for (j = 0; j < width; j++) {
+                v = drow[j] - 0.5*(drowp[j] + drown[j]);
+                s2 += v*v;
+            }
+            n += width;
+        }
     }
 
-    isel[0] = floor(gwy_data_field_rtoj(field, sel[0]));
-    isel[1] = floor(gwy_data_field_rtoi(field, sel[1]));
-    isel[2] = floor(gwy_data_field_rtoj(field, sel[2]));
-    isel[3] = floor(gwy_data_field_rtoi(field, sel[3]));
-    *w = ABS(isel[2] - isel[0]) + 1;
-    *h = ABS(isel[3] - isel[1]) + 1;
-    *col = MIN(isel[0], isel[2]);
-    *row = MIN(isel[1], isel[3]);
-    return (*col == 0 && *row == 0 && *w == xres && *h == yres);
+    return n ? sqrt(s2/n) : 0.0;
 }
 
 static gboolean
@@ -682,20 +698,19 @@ gwy_tool_stats_calculate(GwyToolStats *tool)
     StatsUncertanties unc;
     gdouble xoff, yoff, q;
     gdouble min, max, avg, median, Sa, rms, rms_gw, skew, kurtosis,
-            projarea, area, var, entropy, entropydef, phi, theta;
+            projarea, area, volume, var, phi, theta, linedis;
     gint nn, col, row, w, h;
-    gdouble sel[4];
 
     plain_tool = GWY_PLAIN_TOOL(tool);
     field = plain_tool->data_field;
 
     gwy_results_export_set_actions_sensitive(GWY_RESULTS_EXPORT(tool->rexport),
                                              FALSE);
-    obtain_selection(plain_tool, sel, &col, &row, &w, &h);
-    tool->isel_prev[0] = col;
-    tool->isel_prev[1] = row;
-    tool->isel_prev[2] = w;
-    tool->isel_prev[3] = h;
+    gwy_assign(tool->isel_prev, tool->isel, 4);
+    col = tool->isel[0];
+    row = tool->isel[1];
+    w = tool->isel[2]+1 - tool->isel[0];
+    h = tool->isel[3]+1 - tool->isel[1];
     gwy_debug("%d x %d at (%d, %d)", w, h, col, row);
 
     xoff = gwy_data_field_get_xoffset(field);
@@ -712,7 +727,7 @@ gwy_tool_stats_calculate(GwyToolStats *tool)
         mask = NULL;
     }
 
-    q = gwy_data_field_get_xmeasure(field) * gwy_data_field_get_ymeasure(field);
+    q = gwy_data_field_get_dx(field) * gwy_data_field_get_dy(field);
     if (mask) {
         if (masking == GWY_MASK_INCLUDE)
             gwy_data_field_area_count_in_range(mask, NULL, col, row, w, h,
@@ -737,13 +752,14 @@ gwy_tool_stats_calculate(GwyToolStats *tool)
                                                  col, row, w, h);
     var = gwy_data_field_area_get_variation(field, mask, masking,
                                             col, row, w, h);
-    entropy = gwy_data_field_area_get_entropy(field, mask, masking,
-                                              col, row, w, h);
-    /* Consider δ-function a limit of Gaussian and report deficit of zero. */
-    if (rms > 0.0 && entropy < 0.1*G_MAXDOUBLE)
-        entropydef = (ENTROPY_NORMAL + log(rms) - entropy);
-    else
-        entropydef = 0.0;
+
+    linedis = gwy_data_field_scan_line_discrepancy(field, mask, masking,
+                                                   col, row, w, h);
+    if (linedis > 0.0) {
+        linedis /= sqrt(gwy_data_field_area_get_mean_square(field,
+                                                            mask, masking,
+                                                            col, row, w, h));
+    }
 
     if (tool->same_units) {
         area = gwy_data_field_area_get_surface_area_mask(field, mask, masking,
@@ -752,6 +768,12 @@ gwy_tool_stats_calculate(GwyToolStats *tool)
     else
         area = 0.0;   /* Silly GCC. */
 
+    volume = gwy_data_field_area_get_volume(field, NULL, mask, col, row, w, h);
+    if (masking == GWY_MASK_EXCLUDE) {
+        volume = gwy_data_field_area_get_volume(field, NULL, NULL,
+                                                col, row, w, h) - volume;
+    }
+
     if (tool->same_units && !mask) {
         gwy_data_field_area_get_inclination(field, col, row, w, h,
                                             &theta, &phi);
@@ -761,10 +783,10 @@ gwy_tool_stats_calculate(GwyToolStats *tool)
                             "w", w, "h", h, "x", col, "y", row,
                             NULL);
     gwy_results_fill_format(results, "realsel",
-                            "w", fabs(sel[2] - sel[0]),
-                            "h", fabs(sel[3] - sel[1]),
-                            "x", MIN(sel[0], sel[2]) + xoff,
-                            "y", MIN(sel[1], sel[3]) + yoff,
+                            "w", fabs(tool->rsel[2] - tool->rsel[0]),
+                            "h", fabs(tool->rsel[3] - tool->rsel[1]),
+                            "x", MIN(tool->rsel[0], tool->rsel[2]) + xoff,
+                            "y", MIN(tool->rsel[1], tool->rsel[3]) + yoff,
                             NULL);
     gwy_results_fill_values(results,
                             "masking", !!mask,
@@ -776,9 +798,9 @@ gwy_tool_stats_calculate(GwyToolStats *tool)
                             "Sz", max - min,
                             "rms_gw", rms_gw,
                             "area", area,
+                            "volume", volume,
                             "var", var,
-                            "entropy", entropy,
-                            "entropydef", entropydef,
+                            "linedis", linedis,
                             NULL);
 
     /* Try to have the same format for area and projected area but only if
@@ -841,7 +863,7 @@ calculate_uncertainties(GwyToolStats *tool, StatsUncertanties *unc,
     yres = gwy_data_field_get_yres(field);
     oldx = gwy_data_field_get_xres(tool->xunc);
     oldy = gwy_data_field_get_yres(tool->xunc);
-    //FIXME, functions should work with data of any size
+    // FIXME, functions should work with data of any size
     gwy_data_field_resample(tool->xunc, xres, yres, GWY_INTERPOLATION_BILINEAR);
     gwy_data_field_resample(tool->yunc, xres, yres, GWY_INTERPOLATION_BILINEAR);
     gwy_data_field_resample(tool->zunc, xres, yres, GWY_INTERPOLATION_BILINEAR);
