@@ -1,6 +1,6 @@
 /*
- *  $Id: volume_calibrate.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2013 David Necas (Yeti), Petr Klapetek.
+ *  $Id: volume_calibrate.c 21434 2018-09-11 12:24:22Z yeti-dn $
+ *  Copyright (C) 2013-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -174,7 +174,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Recalibrate volume data dimensions or value range."),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "1.3",
+    "1.4",
     "David Nečas (Yeti) & Petr Klapetek",
     "2013",
 };
@@ -200,12 +200,10 @@ static void
 volcal(GwyContainer *data, GwyRunType run)
 {
     GwyBrick *brick;
-    GwyDataField *dfield;
     gint id, newid;
     VolcalArgs args;
     gboolean ok;
     GwySIUnit *siunitx, *siunity, *siunitz, *siunitw;
-    GQuark quark;
 
     g_return_if_fail(run & VOLCAL_RUN_MODES);
     gwy_app_data_browser_get_current(GWY_APP_BRICK, &brick,
@@ -218,9 +216,7 @@ volcal(GwyContainer *data, GwyRunType run)
     args.xorig = gwy_brick_get_xreal(brick);
     args.yorig = gwy_brick_get_yreal(brick);
     args.zorig = gwy_brick_get_zreal(brick);
-
-    args.worig = gwy_brick_get_max(brick)
-                 - gwy_brick_get_min(brick);
+    args.worig = gwy_brick_get_max(brick) - gwy_brick_get_min(brick);
 
     args.xres = gwy_brick_get_xres(brick);
     args.yres = gwy_brick_get_yres(brick);
@@ -251,24 +247,19 @@ volcal(GwyContainer *data, GwyRunType run)
 
     siunitx = gwy_brick_get_si_unit_x(brick);
     args.xunitorig = gwy_si_unit_get_string(siunitx,
-                                             GWY_SI_UNIT_FORMAT_VFMARKUP);
-    args.xunit = args.xunitorig;
+                                            GWY_SI_UNIT_FORMAT_VFMARKUP);
 
     siunity = gwy_brick_get_si_unit_y(brick);
     args.yunitorig = gwy_si_unit_get_string(siunity,
-                                             GWY_SI_UNIT_FORMAT_VFMARKUP);
-    args.yunit = args.yunitorig;
-
+                                            GWY_SI_UNIT_FORMAT_VFMARKUP);
 
     siunitz = gwy_brick_get_si_unit_z(brick);
     args.zunitorig = gwy_si_unit_get_string(siunitz,
                                             GWY_SI_UNIT_FORMAT_VFMARKUP);
-    args.zunit = args.zunitorig;
 
     siunitw = gwy_brick_get_si_unit_w(brick);
     args.wunitorig = gwy_si_unit_get_string(siunitw,
                                             GWY_SI_UNIT_FORMAT_VFMARKUP);
-    args.wunit = args.wunitorig;
 
     if (run == GWY_RUN_INTERACTIVE) {
         ok = volcal_dialog(&args, brick);
@@ -277,8 +268,6 @@ volcal(GwyContainer *data, GwyRunType run)
             return;
     }
 
-    quark = gwy_app_get_brick_preview_key_for_id(id);
-    dfield = gwy_data_field_duplicate(gwy_container_get_object(data, quark));
     brick = gwy_brick_duplicate(brick);
 
     gwy_brick_set_xreal(brick, args.xreal);
@@ -294,47 +283,25 @@ volcal(GwyContainer *data, GwyRunType run)
     gwy_brick_set_yoffset(brick, args.y0);
     gwy_brick_set_zoffset(brick, args.z0);
 
-    if (args.xunit != args.xunitorig) {
+    if (!gwy_strequal(args.xunit, args.xunitorig)) {
         siunitx = gwy_brick_get_si_unit_x(brick);
         gwy_si_unit_set_from_string(siunitx, args.xunit);
     }
-    if (args.yunit != args.yunitorig) {
+    if (!gwy_strequal(args.yunit, args.yunitorig)) {
         siunity = gwy_brick_get_si_unit_y(brick);
         gwy_si_unit_set_from_string(siunity, args.yunit);
     }
-    if (args.zunit != args.zunitorig) {
+    if (!gwy_strequal(args.zunit, args.zunitorig)) {
         siunitz = gwy_brick_get_si_unit_z(brick);
         gwy_si_unit_set_from_string(siunitz, args.zunit);
     }
-    if (args.wunit != args.wunitorig) {
+    if (!gwy_strequal(args.wunit, args.wunitorig)) {
         siunitw = gwy_brick_get_si_unit_w(brick);
         gwy_si_unit_set_from_string(siunitw, args.wunit);
     }
 
-    /*FIXME: this should probably modify the previous preview, not create new one*/
-    gwy_brick_sum_plane(brick, dfield, 0, 0, 0, gwy_brick_get_xres(brick),
-                                                gwy_brick_get_yres(brick), -1, FALSE);
-
-    gwy_data_field_set_xreal(dfield, args.xreal);
-    gwy_data_field_set_yreal(dfield, args.yreal);
-
-    gwy_data_field_set_xoffset(dfield, args.x0);
-    gwy_data_field_set_yoffset(dfield, args.y0);
-
-
-    /*FIXME: if units are independent for each axis, we are in trouble*/
-    if (args.xunit != args.xunitorig) {
-        siunitx = gwy_data_field_get_si_unit_xy(dfield);
-        gwy_si_unit_set_from_string(siunitx, args.xunit);
-    }
-    if (args.wunit != args.wunitorig) {
-        siunitz = gwy_data_field_get_si_unit_z(dfield);
-        gwy_si_unit_set_from_string(siunitz, args.wunit);
-    }
-
-    newid = gwy_app_data_browser_add_brick(brick, dfield, data, TRUE);
+    newid = gwy_app_data_browser_add_brick(brick, NULL, data, TRUE);
     g_object_unref(brick);
-    g_object_unref(dfield);
 
     gwy_app_set_brick_title(data, newid, _("Recalibrated Data"));
     gwy_app_volume_log_add_volume(data, id, newid);
@@ -344,7 +311,7 @@ static gboolean
 volcal_dialog(VolcalArgs *args,
               GwyBrick *brick)
 {
-    enum { RESPONSE_RESET = 1};
+    enum { RESPONSE_RESET = 1 };
     GtkWidget *dialog, *spin, *table, *label;
     GwySIUnit *unit;
     VolcalControls controls;
@@ -352,7 +319,8 @@ volcal_dialog(VolcalArgs *args,
     GwySIValueFormat *vfx, *vfy, *vfz, *vfw;
     gchar *buf;
 
-    dialog = gtk_dialog_new_with_buttons(_("Volume Dimensions and Units"), NULL, 0,
+    dialog = gtk_dialog_new_with_buttons(_("Volume Dimensions and Units"),
+                                         NULL, 0,
                                          _("_Reset"), RESPONSE_RESET,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          NULL);
@@ -463,11 +431,12 @@ volcal_dialog(VolcalArgs *args,
                                    controls.xreal, GWY_HSCALE_LOG);
     gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 3);
 
-    unit = gwy_brick_get_si_unit_x(brick);
+    unit = gwy_si_unit_new(args->xunit);
     controls.xexponent
         = gwy_combo_box_metric_unit_new(G_CALLBACK(xexponent_changed_cb),
                                         &controls, -15, 6, unit,
                                         args->xexponent);
+    g_object_unref(unit);
     gtk_table_attach(GTK_TABLE(table), controls.xexponent, 2, 3, row, row+1,
                      GTK_FILL | GTK_SHRINK, 0, 0, 0);
 
@@ -483,11 +452,12 @@ volcal_dialog(VolcalArgs *args,
                                    controls.yreal, GWY_HSCALE_LOG);
     gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 3);
 
-    unit = gwy_brick_get_si_unit_y(brick);
+    unit = gwy_si_unit_new(args->yunit);
     controls.yexponent
         = gwy_combo_box_metric_unit_new(G_CALLBACK(yexponent_changed_cb),
                                         &controls, -15, 6, unit,
                                         args->yexponent);
+    g_object_unref(unit);
     gtk_table_attach(GTK_TABLE(table), controls.yexponent,
                      2, 3, row, row+1, GTK_FILL | GTK_SHRINK, 0, 0, 0);
 
@@ -503,11 +473,12 @@ volcal_dialog(VolcalArgs *args,
                                    controls.zreal, GWY_HSCALE_LOG);
     gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 3);
 
-    unit = gwy_brick_get_si_unit_z(brick);
+    unit = gwy_si_unit_new(args->zunit);
     controls.zexponent
         = gwy_combo_box_metric_unit_new(G_CALLBACK(zexponent_changed_cb),
                                         &controls, -15, 6, unit,
                                         args->zexponent);
+    g_object_unref(unit);
     gtk_table_attach(GTK_TABLE(table), controls.zexponent,
                      2, 3, row, row+1, GTK_FILL | GTK_SHRINK, 0, 0, 0);
 
@@ -555,11 +526,12 @@ volcal_dialog(VolcalArgs *args,
                                    controls.wreal, GWY_HSCALE_SQRT);
     gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 2);
 
-    unit = gwy_brick_get_si_unit_w(brick);
+    unit = gwy_si_unit_new(args->wunit);
     controls.wexponent
         = gwy_combo_box_metric_unit_new(G_CALLBACK(wexponent_changed_cb),
                                         &controls, -15, 6, unit,
                                         args->wexponent);
+    g_object_unref(unit);
     gtk_table_attach(GTK_TABLE(table), controls.wexponent,
                      2, 3, row, row+1, GTK_FILL | GTK_SHRINK, 0, 0, 0);
 
@@ -1102,7 +1074,6 @@ volcal_dialog_update(VolcalControls *controls,
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->zreal),
                              args->zreal/pow10(args->zexponent));
 
-
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->x0),
                              args->x0/pow10(args->xexponent));
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->y0),
@@ -1110,7 +1081,6 @@ volcal_dialog_update(VolcalControls *controls,
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->z0),
                              args->z0/pow10(args->zexponent));
 
-
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->wreal),
                              args->wreal/pow10(args->wexponent));
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->wshift),
@@ -1156,32 +1126,47 @@ volcal_dialog_update(VolcalControls *controls,
     else
         g_snprintf(buffer, sizeof(buffer), "× 10<sup>%d</sup>", e);
     gtk_label_set_markup(GTK_LABEL(controls->wpower10), buffer);
-
-    if (args->wratio == 0 || args->wreal == 0)
-        gtk_widget_set_sensitive(controls->ok, FALSE);
-    else
-        gtk_widget_set_sensitive(controls->ok, TRUE);
 }
 
-static const gchar xratio_key[] = "/module/volcal/xratio";
-static const gchar yratio_key[] = "/module/volcal/yratio";
-static const gchar zratio_key[] = "/module/volcal/zratio";
 static const gchar wratio_key[] = "/module/volcal/wratio";
 static const gchar wshift_key[] = "/module/volcal/wshift";
+static const gchar wunit_key[]  = "/module/volcal/wunit";
+static const gchar xratio_key[] = "/module/volcal/xratio";
+static const gchar xunit_key[]  = "/module/volcal/xunit";
+static const gchar yratio_key[] = "/module/volcal/yratio";
+static const gchar yunit_key[]  = "/module/volcal/yunit";
+static const gchar zratio_key[] = "/module/volcal/zratio";
+static const gchar zunit_key[]  = "/module/volcal/zunit";
+
+/* NB: We do not free the unit because it's meant to be taken from settings
+ * or defaults, i.e. const. */
+static void
+sanitize_unit(gchar **unitstr)
+{
+    GwySIUnit *unit;
+
+    unit = gwy_si_unit_new(*unitstr);
+    *unitstr = gwy_si_unit_get_string(unit, GWY_SI_UNIT_FORMAT_PLAIN);
+    g_object_unref(unit);
+}
 
 static void
 volcal_sanitize_args(VolcalArgs *args)
 {
-    args->xratio = CLAMP(args->xratio, 1e-15, 1e15);
-    args->yratio = CLAMP(args->yratio, 1e-15, 1e15);
-    args->zratio = CLAMP(args->zratio, 1e-15, 1e15);
-    args->wratio = CLAMP(args->wratio, -1e15, 1e15);
+    args->xratio = CLAMP(args->xratio, 1e-30, 1e15);
+    args->yratio = CLAMP(args->yratio, 1e-30, 1e15);
+    args->zratio = CLAMP(args->zratio, 1e-30, 1e15);
+    args->wratio = CLAMP(args->wratio, -1e30, 1e15);
     args->wshift = CLAMP(args->wshift, -1e-9, 1e9);
+    sanitize_unit(&args->xunit);
+    sanitize_unit(&args->yunit);
+    sanitize_unit(&args->zunit);
+    sanitize_unit(&args->wunit);
 }
 
 static void
 volcal_load_args(GwyContainer *container,
-                    VolcalArgs *args)
+                 VolcalArgs *args)
 {
     *args = volcal_defaults;
 
@@ -1190,18 +1175,30 @@ volcal_load_args(GwyContainer *container,
     gwy_container_gis_double_by_name(container, zratio_key, &args->zratio);
     gwy_container_gis_double_by_name(container, wratio_key, &args->wratio);
     gwy_container_gis_double_by_name(container, wshift_key, &args->wshift);
+    gwy_container_gis_string_by_name(container, xunit_key,
+                                     (const guchar**)&args->xunit);
+    gwy_container_gis_string_by_name(container, yunit_key,
+                                     (const guchar**)&args->yunit);
+    gwy_container_gis_string_by_name(container, zunit_key,
+                                     (const guchar**)&args->zunit);
+    gwy_container_gis_string_by_name(container, wunit_key,
+                                     (const guchar**)&args->wunit);
     volcal_sanitize_args(args);
 }
 
 static void
 volcal_save_args(GwyContainer *container,
-                    VolcalArgs *args)
+                 VolcalArgs *args)
 {
     gwy_container_set_double_by_name(container, xratio_key, args->xratio);
     gwy_container_set_double_by_name(container, yratio_key, args->yratio);
     gwy_container_set_double_by_name(container, zratio_key, args->zratio);
     gwy_container_set_double_by_name(container, wratio_key, args->wratio);
     gwy_container_set_double_by_name(container, wshift_key, args->wshift);
+    gwy_container_set_const_string_by_name(container, xunit_key, args->xunit);
+    gwy_container_set_const_string_by_name(container, yunit_key, args->yunit);
+    gwy_container_set_const_string_by_name(container, zunit_key, args->zunit);
+    gwy_container_set_const_string_by_name(container, wunit_key, args->zunit);
 }
 
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
