@@ -1,6 +1,6 @@
 /*
- *  $Id: volume_linestat.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2015 David Necas (Yeti).
+ *  $Id: volume_linestat.c 21482 2018-10-12 06:59:29Z yeti-dn $
+ *  Copyright (C) 2015-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -48,12 +48,12 @@ enum {
 };
 
 enum {
-    RESPONSE_RESET = 1,
+    RESPONSE_RESET   = 1,
     RESPONSE_PREVIEW = 2,
 };
 
 typedef enum {
-    OUTPUT_IMAGE = 0,
+    OUTPUT_IMAGE   = 0,
     OUTPUT_PREVIEW = 1,
     NOUTPUTS
 } LineStatOutput;
@@ -88,7 +88,9 @@ typedef struct {
     GSList *output_type;
     GtkWidget *zfrom;
     GtkWidget *zto;
+    GtkWidget *current_value;
     GwySIValueFormat *zvf;
+    GwySIValueFormat *vf;
 } LineStatControls;
 
 typedef struct {
@@ -119,6 +121,7 @@ static void     graph_selection_changed(LineStatControls *controls,
                                         GwySelection *selection);
 static void     quantity_changed       (GtkComboBox *combo,
                                         LineStatControls *controls);
+static void     update_current_value   (LineStatControls *controls);
 static void     range_changed          (GtkWidget *entry,
                                         LineStatControls *controls);
 static void     output_type_changed    (GtkToggleButton *button,
@@ -147,6 +150,7 @@ static const GwyEnum quantities[] =  {
     { N_("Max. position"),      GWY_LINE_STAT_MAXPOS,    },
     { N_("Range"),              GWY_LINE_STAT_RANGE,     },
     { N_("Slope"),              GWY_LINE_STAT_SLOPE,     },
+    /* This is defined, but not unitless without identical units. */
     { N_("tan β<sub>0</sub>"),  GWY_LINE_STAT_TAN_BETA0, },
     { N_("Variation"),          GWY_LINE_STAT_VARIATION, },
     { N_("Ra"),                 GWY_LINE_STAT_RA,        },
@@ -172,7 +176,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Summarizes profiles of volume data to a channel."),
     "Yeti <yeti@gwyddion.net>",
-    "1.2",
+    "1.3",
     "David Nečas (Yeti)",
     "2015",
 };
@@ -264,11 +268,12 @@ line_stat_dialog(LineStatArgs *args, GwyContainer *data, gint id)
     GwySelection *selection;
     GwyBrick *brick;
     const guchar *gradient;
-    gchar key[40];
+    GQuark quark;
     gdouble xy[2];
     gdouble zmax;
     guint nquantities;
 
+    gwy_clear(&controls, 1);
     controls.args = args;
 
     brick = args->brick;
@@ -313,8 +318,8 @@ line_stat_dialog(LineStatArgs *args, GwyContainer *data, gint id)
     gwy_container_set_object_by_name(controls.mydata, "/0/data", dfield);
     g_object_unref(dfield);
 
-    g_snprintf(key, sizeof(key), "/brick/%d/preview/palette", id);
-    if (gwy_container_gis_string_by_name(data, key, &gradient)) {
+    quark = gwy_app_get_brick_palette_key_for_id(id);
+    if (gwy_container_gis_string(data, quark, &gradient)) {
         gwy_container_set_const_string_by_name(controls.mydata,
                                                "/0/base/palette", gradient);
     }
@@ -386,6 +391,13 @@ line_stat_dialog(LineStatArgs *args, GwyContainer *data, gint id)
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), controls.quantity);
     row++;
 
+    controls.current_value = gtk_label_new(NULL);
+    gtk_misc_set_alignment(GTK_MISC(controls.current_value), 1.0, 0.5);
+    gtk_table_attach(GTK_TABLE(table), controls.current_value,
+                     1, 2, row, row+1, GTK_FILL, 0, 0, 0);
+    row++;
+
+    gtk_table_set_row_spacing(GTK_TABLE(table), row-1, 8);
     hbox2 = gtk_hbox_new(FALSE, 6);
     gtk_table_attach(GTK_TABLE(table), hbox2,
                      0, 2, row, row+1, GTK_FILL, 0, 0, 0);
@@ -462,6 +474,8 @@ line_stat_dialog(LineStatArgs *args, GwyContainer *data, gint id)
     else
         gwy_selection_clear(selection);
 
+    update_current_value(&controls);
+
     gtk_widget_show_all(dialog);
     do {
         response = gtk_dialog_run(GTK_DIALOG(dialog));
@@ -484,6 +498,7 @@ line_stat_dialog(LineStatArgs *args, GwyContainer *data, gint id)
 
             case RESPONSE_PREVIEW:
             extract_summary_image(args, controls.image);
+            update_current_value(&controls);
             break;
 
             default:
@@ -495,6 +510,8 @@ line_stat_dialog(LineStatArgs *args, GwyContainer *data, gint id)
     gtk_widget_destroy(dialog);
     g_object_unref(controls.mydata);
     gwy_si_unit_value_format_free(controls.zvf);
+    if (controls.vf)
+        gwy_si_unit_value_format_free(controls.vf);
 
     return TRUE;
 }
@@ -519,6 +536,7 @@ point_selection_changed(LineStatControls *controls,
     gmodel = gwy_graph_get_model(GWY_GRAPH(controls->graph));
     gcmodel = gwy_graph_model_get_curve(gmodel, 0);
     extract_graph_curve(args, gcmodel);
+    update_current_value(controls);
 }
 
 static void
@@ -563,8 +581,10 @@ graph_selection_changed(LineStatControls *controls,
                gwy_brick_ktor_cal(brick, z[1])/zvf->magnitude);
     gtk_entry_set_text(GTK_ENTRY(controls->zto), buf);
 
-    if (args->update)
+    if (args->update) {
         extract_summary_image(controls->args, controls->image);
+        update_current_value(controls);
+    }
 }
 
 static void
@@ -574,6 +594,37 @@ quantity_changed(GtkComboBox *combo, LineStatControls *controls)
 
     args->quantity = gwy_enum_combo_box_get_active(combo);
     extract_summary_image(controls->args, controls->image);
+    update_current_value(controls);
+}
+
+static void
+update_current_value(LineStatControls *controls)
+{
+    LineStatArgs *args = controls->args;
+    GwySIValueFormat *vf;
+    gint xres, yres, col, row;
+    GwySIUnit *unit;
+    gdouble v;
+    gchar *s;
+
+    xres = gwy_brick_get_xres(args->brick);
+    yres = gwy_brick_get_yres(args->brick);
+    col = (args->x >= 0 && args->x < xres) ? args->x : xres/2;
+    row = (args->y >= 0 && args->y < yres) ? args->y : yres/2;
+
+    v = gwy_data_field_get_val(controls->image, col, row);
+    unit = gwy_data_field_get_si_unit_z(controls->image);
+    controls->vf = gwy_si_unit_get_format_with_digits(unit,
+                                                      GWY_SI_UNIT_FORMAT_VFMARKUP,
+                                                      v, 3,
+                                                      controls->vf);
+    vf = controls->vf;
+
+    s = g_strdup_printf("%.*f%s%s",
+                        vf->precision, v/vf->magnitude,
+                        *vf->units ? " " : "", vf->units);
+    gtk_label_set_markup(GTK_LABEL(controls->current_value), s);
+    g_free(s);
 }
 
 static void
@@ -622,8 +673,10 @@ update_changed(LineStatControls *controls, GtkToggleButton *check)
     args->update = gtk_toggle_button_get_active(check);
     gtk_dialog_set_response_sensitive(GTK_DIALOG(controls->dialog),
                                       RESPONSE_PREVIEW, !args->update);
-    if (args->update)
+    if (args->update) {
+        update_current_value(controls);
         extract_summary_image(controls->args, controls->image);
+    }
 }
 
 static void
@@ -755,13 +808,7 @@ extract_summary_image(const LineStatArgs *args, GwyDataField *dfield)
         }
     }
 
-    gwy_brick_extract_plane(brick, dfield, 0, 0, 0, xres, yres, -1, FALSE);
-    if (!lsfunc) {
-        g_warning("Quantity %u is still unimplemented.", quantity);
-        gwy_data_field_clear(dfield);
-        gwy_data_field_data_changed(dfield);
-        return;
-    }
+    gwy_brick_extract_xy_plane(brick, dfield, 0);
 
     /* Use an iterator interface to formally process data profile by profle,
      * but physically extract them from the brick by larger blocks, gaining a
@@ -802,8 +849,7 @@ extract_summary_image(const LineStatArgs *args, GwyDataField *dfield)
 
     if ((quantity == GWY_LINE_STAT_MINPOS)
         || (quantity == GWY_LINE_STAT_MAXPOS)) {
-        gwy_serializable_clone(G_OBJECT(zunit),
-                               G_OBJECT(gwy_data_field_get_si_unit_z(dfield)));
+        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), zunit);
     }
     else if ((quantity == GWY_LINE_STAT_TAN_BETA0)
              || (quantity == GWY_LINE_STAT_SLOPE))
