@@ -1,6 +1,6 @@
 /*
- *  $Id: volume_swaxes.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2017 David Necas (Yeti).
+ *  $Id: volume_swaxes.c 21430 2018-09-11 11:46:00Z yeti-dn $
+ *  Copyright (C) 2017-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -103,7 +103,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Swaps axes of volume data."),
     "Yeti <yeti@gwyddion.net>",
-    "1.0",
+    "1.1",
     "David Nečas (Yeti)",
     "2017",
 };
@@ -370,223 +370,33 @@ static void
 swaxes_do(GwyContainer *data, gint id, GwyBrick *brick, SwapAxesArgs *args)
 {
     AxisType ax = args->x, ay = args->y, az = args->z;
-    AxisType bx = 2*(ax/2), by = 2*(ay/2), bz = 2*(az/2);
+    AxisType bx = 2*(ax/2), by = 2*(ay/2);
+    GwyBrickTransposeType transtype;
     gboolean xinv = ax & 1, yinv = ay & 1, zinv = az & 1;
-    guint oldxres = gwy_brick_get_xres(brick),
-          oldyres = gwy_brick_get_yres(brick),
-          oldzres = gwy_brick_get_zres(brick),
-          xres = oldxres, yres = oldyres, zres = oldzres;
-    gdouble xreal = gwy_brick_get_xreal(brick),
-            yreal = gwy_brick_get_yreal(brick),
-            zreal = gwy_brick_get_zreal(brick),
-            newxreal = xreal, newyreal = yreal, newzreal = zreal;
-    gdouble xoff = gwy_brick_get_xoffset(brick),
-            yoff = gwy_brick_get_yoffset(brick),
-            zoff = gwy_brick_get_zoffset(brick),
-            newxoff = xoff, newyoff = yoff, newzoff = zoff;
     GwyBrick *result;
     GwyDataField *preview = NULL;
-    GwyDataLine *zcal = gwy_brick_get_zcalibration(brick);
-    guint i, j, k, n;
-    const gdouble *bdata = gwy_brick_get_data_const(brick);
-    GwySIUnit *oldunit[4], *unit[4];
-    gdouble *rdata, *r1, *r2;
+    gdouble xoff, yoff;
+    gint xres, yres, newid;
     GQuark quarks[2];
 
-    /* There are 48 different combinations, which is way too much.  Implement
-     * the transformation in two steps:
-     * 1. Reshaping, without regard of axis inversion (6 types, one trivial). */
-    if (bx == AXIS_XPOS && by == AXIS_YPOS && bz == AXIS_ZPOS) {
-        result = gwy_brick_duplicate(brick);
-    }
-    else if (bx == AXIS_XPOS && by == AXIS_ZPOS && bz == AXIS_YPOS) {
-        /* Y <-> Z */
-        GWY_SWAP(guint, yres, zres);
-        GWY_SWAP(gdouble, newyreal, newzreal);
-        GWY_SWAP(gdouble, newyoff, newzoff);
-        result = gwy_brick_new(xres, yres, zres,
-                               newxreal, newyreal, newzreal,
-                               FALSE);
-        rdata = gwy_brick_get_data(result);
-
-        for (i = 0; i < zres; i++) {
-            for (j = 0; j < yres; j++) {
-                for (k = 0; k < xres; k++, rdata++)
-                    *rdata = bdata[(j*oldyres + i)*oldxres + k];
-            }
-        }
-    }
-    else if (bx == AXIS_YPOS && by == AXIS_XPOS && bz == AXIS_ZPOS) {
-        /* X <-> Y */
-        GWY_SWAP(guint, xres, yres);
-        GWY_SWAP(gdouble, newxreal, newyreal);
-        GWY_SWAP(gdouble, newxoff, newyoff);
-        result = gwy_brick_new(xres, yres, zres,
-                               newxreal, newyreal, newzreal,
-                               FALSE);
-        rdata = gwy_brick_get_data(result);
-
-        for (i = 0; i < zres; i++) {
-            for (j = 0; j < yres; j++) {
-                for (k = 0; k < xres; k++, rdata++)
-                    *rdata = bdata[(i*oldyres + k)*oldxres + j];
-            }
-        }
-
-        if (zcal) {
-            zcal = gwy_data_line_duplicate(zcal);
-            gwy_brick_set_zcalibration(result, zcal);
-            g_object_unref(zcal);
-        }
-    }
-    else if (bx == AXIS_ZPOS && by == AXIS_YPOS && bz == AXIS_XPOS) {
-        /* Z <-> X */
-        GWY_SWAP(guint, zres, xres);
-        GWY_SWAP(gdouble, newzreal, newxreal);
-        GWY_SWAP(gdouble, newzoff, newxoff);
-        result = gwy_brick_new(xres, yres, zres,
-                               newxreal, newyreal, newzreal,
-                               FALSE);
-        rdata = gwy_brick_get_data(result);
-
-        for (i = 0; i < zres; i++) {
-            for (j = 0; j < yres; j++) {
-                for (k = 0; k < xres; k++, rdata++)
-                    *rdata = bdata[(k*oldyres + j)*oldxres + i];
-            }
-        }
-    }
-    else if (bx == AXIS_YPOS && by == AXIS_ZPOS && bz == AXIS_XPOS) {
-        /* X -> Y -> Z -> X */
-        xres = oldzres;
-        yres = oldxres;
-        zres = oldyres;
-        newxreal = zreal;
-        newyreal = xreal;
-        newzreal = yreal;
-        newxoff = zoff;
-        newyoff = xoff;
-        newzoff = yoff;
-        result = gwy_brick_new(xres, yres, zres,
-                               newxreal, newyreal, newzreal,
-                               FALSE);
-        rdata = gwy_brick_get_data(result);
-
-        for (i = 0; i < zres; i++) {
-            for (j = 0; j < yres; j++) {
-                for (k = 0; k < xres; k++, rdata++)
-                    *rdata = bdata[(k*oldyres + i)*oldxres + j];
-            }
-        }
-    }
-    else if (bx == AXIS_ZPOS && by == AXIS_XPOS && bz == AXIS_YPOS) {
-        /* X -> Z -> Y -> X */
-        xres = oldyres;
-        yres = oldzres;
-        zres = oldxres;
-        newxreal = yreal;
-        newyreal = zreal;
-        newzreal = xreal;
-        newxoff = yoff;
-        newyoff = zoff;
-        newzoff = xoff;
-        result = gwy_brick_new(xres, yres, zres,
-                               newxreal, newyreal, newzreal,
-                               FALSE);
-        rdata = gwy_brick_get_data(result);
-
-        for (i = 0; i < zres; i++) {
-            for (j = 0; j < yres; j++) {
-                for (k = 0; k < xres; k++, rdata++)
-                    *rdata = bdata[(j*oldyres + k)*oldxres + i];
-            }
-        }
-    }
+    if (ax == AXIS_XPOS && ay == AXIS_YPOS)
+        transtype = GWY_BRICK_TRANSPOSE_XYZ;
+    else if (ax == AXIS_XPOS && ay == AXIS_ZPOS)
+        transtype = GWY_BRICK_TRANSPOSE_XZY;
+    else if (ax == AXIS_YPOS && ay == AXIS_XPOS)
+        transtype = GWY_BRICK_TRANSPOSE_YXZ;
+    else if (ax == AXIS_YPOS && ay == AXIS_ZPOS)
+        transtype = GWY_BRICK_TRANSPOSE_YZX;
+    else if (ax == AXIS_ZPOS && ay == AXIS_XPOS)
+        transtype = GWY_BRICK_TRANSPOSE_ZXY;
+    else if (ax == AXIS_ZPOS && ay == AXIS_YPOS)
+        transtype = GWY_BRICK_TRANSPOSE_ZYX;
     else {
-        g_assert_not_reached();
+        g_return_if_reached();
     }
 
-    /* 2. Reverse along axes (8 types, one trivial). */
-    rdata = gwy_brick_get_data(result);
-    if (!xinv && !yinv && !zinv) {
-        /* Do nothing. */
-    }
-    else if (xinv && yinv && zinv) {
-        n = xres*yres*zres;
-        r1 = rdata;
-        r2 = rdata + n-1;
-        for (i = n/2; i; i--, r1++, r2--)
-            GWY_SWAP(gdouble, *r1, *r2);
-    }
-    else if (!xinv && !yinv && zinv) {
-        n = xres*yres;
-        for (i = 0; i < zres/2; i++) {
-            r1 = rdata + i*n;
-            r2 = rdata + (zres-1 - i)*n;
-            for (j = n; j; j--, r1++, r2++)
-                GWY_SWAP(gdouble, *r1, *r2);
-        }
-    }
-    else if (xinv && !yinv && !zinv) {
-        n = yres*zres;
-        for (i = 0; i < n; i++) {
-            r1 = rdata + i*xres;
-            r2 = r1 + xres-1;
-            for (j = xres/2; j; j--, r1++, r2--)
-                GWY_SWAP(gdouble, *r1, *r2);
-        }
-    }
-    else if (!xinv && yinv && !zinv) {
-        for (i = 0; i < zres; i++) {
-            for (j = 0; j < yres/2; j++) {
-                r1 = rdata + (i*yres + j)*xres;
-                r2 = rdata + (i*yres + yres-1 - j)*xres;
-                for (k = xres; k; k--, r1++, r2++)
-                    GWY_SWAP(gdouble, *r1, *r2);
-            }
-        }
-    }
-    else if (xinv && yinv && !zinv) {
-        n = xres*yres;
-        for (i = 0; i < zres; i++) {
-            r1 = rdata + i*n;
-            r2 = r1 + n-1;
-            for (j = n/2; j; j--, r1++, r2--)
-                GWY_SWAP(gdouble, *r1, *r2);
-        }
-    }
-    else if (xinv && !yinv && zinv) {
-        for (i = 0; i < zres/2; i++) {
-            for (j = 0; j < yres; j++) {
-                r1 = rdata + (i*yres + j)*xres;
-                r2 = rdata + ((zres-1 - i)*yres + j)*xres + xres-1;
-                for (k = xres; k; k--, r1++, r2--)
-                    GWY_SWAP(gdouble, *r1, *r2);
-            }
-        }
-    }
-    else if (!xinv && yinv && zinv) {
-        for (i = 0; i < zres/2; i++) {
-            for (j = 0; j < yres; j++) {
-                r1 = rdata + (i*yres + j)*xres;
-                r2 = rdata + ((zres-1 - i)*yres + (yres-1 - j))*xres;
-                for (k = xres; k; k--, r1++, r2++)
-                    GWY_SWAP(gdouble, *r1, *r2);
-            }
-        }
-    }
-    else {
-        g_assert_not_reached();
-    }
-
-    /* The result can only have a calibration if Z axis was preserved. */
-    if (zinv && (zcal = gwy_brick_get_zcalibration(result))) {
-        gwy_data_line_invert(zcal, TRUE, FALSE);
-    }
-
-    gwy_brick_set_xoffset(result, xoff);
-    gwy_brick_set_yoffset(result, yoff);
-    gwy_brick_set_zoffset(result, zoff);
+    result = gwy_brick_new(1, 1, 1, 1.0, 1.0, 1.0, FALSE);
+    gwy_brick_transpose(brick, result, transtype, xinv, yinv, zinv);
 
     /* Reuse the old preview if the XY plane is preserved. */
     if (gwy_container_gis_object(data, gwy_app_get_brick_preview_key_for_id(id),
@@ -620,41 +430,26 @@ swaxes_do(GwyContainer *data, gint id, GwyBrick *brick, SwapAxesArgs *args)
         }
         GWY_SWAP(GwyDataField*, tmp, preview);
 
-        gwy_data_field_set_xoffset(preview, xoff);
-        gwy_data_field_set_yoffset(preview, yoff);
+        xoff = gwy_data_field_get_xoffset(preview);
+        yoff = gwy_data_field_get_yoffset(preview);
+        gwy_data_field_set_xoffset(preview, yoff);
+        gwy_data_field_set_yoffset(preview, xoff);
     }
     else {
-        preview = gwy_data_field_new(xres, yres, newxreal, yres, FALSE);
-        gwy_brick_mean_plane(result, preview, 0, 0, 0, xres, yres, -1, TRUE);
+        xres = gwy_brick_get_xres(result);
+        yres = gwy_brick_get_yres(result);
+        preview = gwy_data_field_new(xres, yres, xres, yres, FALSE);
+        gwy_brick_mean_xy_plane(result, preview);
     }
 
-    /* Set units. */
-    oldunit[0] = gwy_brick_get_si_unit_x(brick);
-    oldunit[1] = gwy_brick_get_si_unit_y(brick);
-    oldunit[2] = gwy_brick_get_si_unit_z(brick);
-    oldunit[3] = gwy_brick_get_si_unit_w(brick);
-    unit[0] = gwy_brick_get_si_unit_x(result);
-    unit[1] = gwy_brick_get_si_unit_y(result);
-    unit[2] = gwy_brick_get_si_unit_z(result);
-    unit[3] = gwy_brick_get_si_unit_w(result);
-    gwy_serializable_clone(G_OBJECT(oldunit[0]), G_OBJECT(unit[bx/2]));
-    gwy_serializable_clone(G_OBJECT(oldunit[1]), G_OBJECT(unit[by/2]));
-    gwy_serializable_clone(G_OBJECT(oldunit[2]), G_OBJECT(unit[bz/2]));
-    gwy_serializable_clone(G_OBJECT(oldunit[3]), G_OBJECT(unit[3]));
-
     /* Create new channel or modify the current one. */
     if (args->new_channel) {
-        gint newid;
-        const guchar *gradient;
-
         newid = gwy_app_data_browser_add_brick(result, preview, data, TRUE);
         gwy_app_set_brick_title(data, id, _("Rotated Data"));
         gwy_app_volume_log_add_volume(data, id, newid);
-
-        quarks[0] = gwy_app_get_brick_palette_key_for_id(id);
-        quarks[1] = gwy_app_get_brick_palette_key_for_id(newid);
-        if (gwy_container_gis_string(data, quarks[0], &gradient))
-            gwy_container_set_const_string(data, quarks[1], gradient);
+        gwy_app_sync_volume_items(data, data, id, newid,
+                                  GWY_DATA_ITEM_GRADIENT,
+                                  0);
     }
     else {
         quarks[0] = gwy_app_get_brick_key_for_id(id);
