@@ -1,6 +1,6 @@
 /*
- *  $Id: volume_zcal.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2015 David Necas (Yeti).
+ *  $Id: volume_zcal.c 20843 2018-03-13 17:08:48Z yeti-dn $
+ *  Copyright (C) 2015-2018 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -46,12 +46,14 @@ typedef enum {
     ZCAL_EXTRACT = 0,
     ZCAL_REMOVE  = 1,
     ZCAL_ATTACH  = 2,
+    ZCAL_ANOTHER = 3,
     NACTIONS
 } ZCalActionType;
 
 typedef struct {
     ZCalActionType mode;
     gchar *filename;
+    GwyAppDataId source;
     /* Dynamic state. */
     GwyBrick *brick;
     GwyDataLine *calibration;
@@ -63,8 +65,10 @@ typedef struct {
     GtkWidget *dialog;
     GSList *mode;
     GtkWidget *filebutton;
+    GtkWidget *source;
     GtkWidget *errmessage;
     GtkWidget *graph;
+    gboolean has_source;
 } ZCalControls;
 
 static gboolean     module_register             (void);
@@ -73,9 +77,14 @@ static void         zcal                        (GwyContainer *data,
 static gboolean     zcal_dialog                 (ZCalArgs *args);
 static void         mode_changed                (GtkWidget *button,
                                                  ZCalControls *controls);
+static gboolean     filter_source_bricks        (GwyContainer *data,
+                                                 gint id,
+                                                 gpointer user_data);
 static void         file_selected               (GtkFileChooser *chooser,
                                                  ZCalControls *controls);
 static void         update_file_calibration     (ZCalControls *controls);
+static void         source_brick_changed        (GwyDataChooser *chooser,
+                                                 ZCalControls *controls);
 static void         switch_calibration          (ZCalControls *controls);
 static void         setup_graph_from_calibration(ZCalControls *controls);
 static void         setup_graph_do              (GwyGraphModel *gmodel,
@@ -93,17 +102,19 @@ static void         zcal_save_args              (GwyContainer *container,
                                                  ZCalArgs *args);
 
 static const ZCalArgs zcal_defaults = {
-    ZCAL_ATTACH, NULL,
+    ZCAL_ATTACH, NULL, GWY_APP_DATA_ID_NONE,
     /* Dynamic state. */
     NULL, NULL
 };
 
+static GwyAppDataId source_id = GWY_APP_DATA_ID_NONE;
+
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Attaches, extracts and removes volume data z-axis calibration."),
     "Yeti <yeti@gwyddion.net>",
-    "1.0",
+    "1.1",
     "David Nečas (Yeti)",
     "2015",
 };
@@ -171,17 +182,20 @@ zcal_dialog(ZCalArgs *args)
     static const GwyEnum modes[] = {
         { N_("_Extract to a graph"), ZCAL_EXTRACT, },
         { N_("_Remove"),             ZCAL_REMOVE,  },
-        { N_("_Attach from file"),   ZCAL_ATTACH,  },
+        { N_("Attach from _file"),   ZCAL_ATTACH,  },
+        { N_("Copy from _another"),  ZCAL_ANOTHER, },
     };
     GdkColor gdkcolor = { 0, 51118, 0, 0 };
 
-    GtkWidget *dialog, *table, *hbox, *label;
+    GtkWidget *dialog, *table, *hbox, *label, *widget;
+    GwyDataChooser *chooser;
     ZCalControls controls;
     GwyGraphModel *gmodel;
     gint response, row;
 
     gwy_clear(&controls, 1);
     controls.args = args;
+    controls.has_source = TRUE;
 
     dialog = gtk_dialog_new_with_buttons(_("Volume Z Calibration"),
                                          NULL, 0,
@@ -195,7 +209,7 @@ zcal_dialog(ZCalArgs *args)
     hbox = gtk_hbox_new(FALSE, 20);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, TRUE, TRUE, 4);
 
-    table = gtk_table_new(6, 2, FALSE);
+    table = gtk_table_new(8, 2, FALSE);
     gtk_container_set_border_width(GTK_CONTAINER(table), 4);
     gtk_table_set_col_spacings(GTK_TABLE(table), 6);
     gtk_table_set_row_spacings(GTK_TABLE(table), 2);
@@ -212,14 +226,25 @@ zcal_dialog(ZCalArgs *args)
         = gwy_radio_buttons_create(modes, G_N_ELEMENTS(modes),
                                    G_CALLBACK(mode_changed), &controls,
                                    args->mode);
-    row = gwy_radio_buttons_attach_to_table(controls.mode,
-                                            GTK_TABLE(table), 2, row);
-    if (!args->calibration) {
-        gtk_widget_set_sensitive(gwy_radio_buttons_find(controls.mode,
-                                                        ZCAL_EXTRACT), FALSE);
-        gtk_widget_set_sensitive(gwy_radio_buttons_find(controls.mode,
-                                                        ZCAL_REMOVE), FALSE);
-    }
+
+    widget = gwy_radio_buttons_find(controls.mode, ZCAL_EXTRACT);
+    if (!args->calibration)
+        gtk_widget_set_sensitive(widget, FALSE);
+    gtk_table_attach(GTK_TABLE(table), widget,
+                     0, 2, row, row+1, GTK_FILL, 0, 0, 0);
+    row++;
+
+    widget = gwy_radio_buttons_find(controls.mode, ZCAL_REMOVE);
+    if (!args->calibration)
+        gtk_widget_set_sensitive(widget, FALSE);
+    gtk_table_attach(GTK_TABLE(table), widget,
+                     0, 2, row, row+1, GTK_FILL, 0, 0, 0);
+    row++;
+
+    widget = gwy_radio_buttons_find(controls.mode, ZCAL_ATTACH);
+    gtk_table_attach(GTK_TABLE(table), widget,
+                     0, 2, row, row+1, GTK_FILL, 0, 0, 0);
+    row++;
 
     controls.filebutton
         = gtk_file_chooser_button_new(_("Volume Z Calibration"),
@@ -238,6 +263,28 @@ zcal_dialog(ZCalArgs *args)
     gtk_table_attach(GTK_TABLE(table), label,
                      1, 2, row, row+1, GTK_FILL, 0, 0, 0);
 
+    gtk_table_set_row_spacing(GTK_TABLE(table), row-1, 8);
+    widget = gwy_radio_buttons_find(controls.mode, ZCAL_ANOTHER);
+    gtk_table_attach(GTK_TABLE(table), widget,
+                     0, 2, row, row+1, GTK_FILL, 0, 0, 0);
+    row++;
+
+    controls.source = gwy_data_chooser_new_volumes();
+    chooser = GWY_DATA_CHOOSER(controls.source);
+    gwy_data_chooser_set_none(chooser, NULL);
+    gwy_data_chooser_set_filter(chooser, filter_source_bricks, args->brick,
+                                NULL);
+    gwy_data_chooser_set_active_id(chooser, &args->source);
+    if (!gwy_data_chooser_get_active_id(chooser, &args->source)) {
+        gtk_widget_set_sensitive(widget, FALSE);
+        controls.has_source = FALSE;
+    }
+    gtk_table_attach(GTK_TABLE(table), controls.source,
+                     1, 2, row, row+1, GTK_FILL, 0, 0, 0);
+    g_signal_connect(chooser, "changed",
+                     G_CALLBACK(source_brick_changed), &controls);
+    row++;
+
     gmodel = gwy_graph_model_new();
     controls.graph = gwy_graph_new(gmodel);
     g_object_unref(gmodel);
@@ -285,15 +332,36 @@ static void
 mode_changed(GtkWidget *button, ZCalControls *controls)
 {
     ZCalArgs *args = controls->args;
-    gboolean is_attach;
+    gboolean is_attach, is_source;
 
     args->mode = gwy_radio_button_get_value(button);
     is_attach = (args->mode == ZCAL_ATTACH);
     gtk_widget_set_sensitive(controls->filebutton, is_attach);
     gtk_widget_set_sensitive(controls->errmessage, is_attach);
+    is_source = (controls->has_source && args->mode == ZCAL_ANOTHER);
+    gtk_widget_set_sensitive(controls->source, is_source);
     switch_calibration(controls);
 }
 
+static gboolean
+filter_source_bricks(GwyContainer *data,
+                     gint id,
+                     gpointer user_data)
+{
+    GwyBrick *brick, *source;
+
+    brick = GWY_BRICK(user_data);
+    source = gwy_container_get_object(data, gwy_app_get_brick_key_for_id(id));
+    if (source == brick)
+        return FALSE;
+    if (!gwy_brick_get_zcalibration(source))
+        return FALSE;
+    if (gwy_brick_get_zres(source) != gwy_brick_get_zres(brick))
+        return FALSE;
+
+    return TRUE;
+}
+
 static void
 file_selected(GtkFileChooser *chooser, ZCalControls *controls)
 {
@@ -324,15 +392,33 @@ update_file_calibration(ZCalControls *controls)
     switch_calibration(controls);
 }
 
+static void
+source_brick_changed(GwyDataChooser *chooser, ZCalControls *controls)
+{
+    gwy_data_chooser_get_active_id(chooser, &controls->args->source);
+    switch_calibration(controls);
+}
+
 static void
 switch_calibration(ZCalControls *controls)
 {
     ZCalArgs *args = controls->args;
+    GwyBrick *source;
+    GwyContainer *data;
+    GQuark quark;
 
     gwy_object_unref(args->calibration);
 
     if (args->mode == ZCAL_ATTACH)
         args->calibration = controls->filecalibration;
+    else if (args->mode == ZCAL_ANOTHER) {
+        if (args->source.datano >= 0 && args->source.id >= 0) {
+            data = gwy_app_data_browser_get(args->source.datano);
+            quark = gwy_app_get_brick_key_for_id(args->source.id);
+            source = gwy_container_get_object(data, quark);
+            args->calibration = gwy_brick_get_zcalibration(source);
+        }
+    }
     else
         args->calibration = gwy_brick_get_zcalibration(args->brick);
 
@@ -442,6 +528,19 @@ zcal_do(GwyContainer *data, ZCalArgs *args)
             g_object_unref(gmodel);
         }
     }
+    else if (args->mode == ZCAL_ANOTHER) {
+        if (args->source.datano >= 0 && args->source.id >= 0) {
+            GwyContainer *sdata = gwy_app_data_browser_get(args->source.datano);
+            GQuark quark = gwy_app_get_brick_key_for_id(args->source.id);
+            GwyBrick *source = gwy_container_get_object(sdata, quark);
+            GwyDataLine *calibration = gwy_brick_get_zcalibration(source);
+
+            calibration = gwy_data_line_duplicate(calibration);
+            gwy_brick_set_zcalibration(args->brick, calibration);
+            gwy_brick_data_changed(args->brick);
+            g_object_unref(calibration);
+        }
+    }
     else {
         g_assert_not_reached();
     }
@@ -536,6 +635,8 @@ load_calibration_from_file(const gchar *filename, GwyBrick *brick,
     calibration = gwy_brick_get_zcalibration(brick);
     if (calibration)
         unit = gwy_data_line_get_si_unit_y(calibration);
+    else
+        unit = gwy_brick_get_si_unit_z(brick);
     unit = gwy_si_unit_duplicate(unit);
     gwy_data_line_set_si_unit_y(dline, unit);
     g_object_unref(unit);
@@ -551,6 +652,7 @@ zcal_sanitize_args(ZCalArgs *args)
 {
     args->mode = MIN(args->mode, NACTIONS-1);
     args->filename = g_strdup(args->filename);
+    gwy_app_data_id_verify_volume(&args->source);
 }
 
 static void
@@ -562,6 +664,7 @@ zcal_load_args(GwyContainer *container,
     gwy_container_gis_enum_by_name(container, mode_key, &args->mode);
     gwy_container_gis_string_by_name(container, filename_key,
                                      (const guchar**)&args->filename);
+    args->source = source_id;
     zcal_sanitize_args(args);
 }
 
@@ -569,6 +672,7 @@ static void
 zcal_save_args(GwyContainer *container,
                 ZCalArgs *args)
 {
+    source_id = args->source;
     gwy_container_set_enum_by_name(container, mode_key, args->mode);
     if (args->filename)
         gwy_container_set_const_string_by_name(container, filename_key,
