@@ -1,5 +1,5 @@
 /*
- *  $Id: xyz_drift.c 20676 2017-12-18 18:19:09Z yeti-dn $
+ *  $Id: xyz_drift.c 21408 2018-09-07 13:53:53Z yeti-dn $
  *  Copyright (C) 2016 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -1916,10 +1916,13 @@ estimate_drift(XYZDriftControls *controls, GwyXYZ *points, GwyXYZ *corpoints, gi
 //    bdiff = 1e-5; //gen, exp
 //    cdiff = 100;
 
-    gwy_app_wait_start(GTK_WINDOW(controls->dialog), _("Fitting in progress..."));
+    gwy_app_wait_start(GTK_WINDOW(controls->dialog),
+                       _("Fitting in progress..."));
+
     total = (gdouble)(2*(gint)controls->args->fit_xdrift + 2*(gint)controls->args->fit_ydrift + (gint)controls->args->fit_zdrift)*controls->args->iterations;
     controls->fraction = sofar = 0;
-    gwy_app_wait_set_fraction(sofar/total);
+    if (!gwy_app_wait_set_fraction(sofar/total))
+        goto fail;
 
     //successively minimize all the variables
     iteration = 0;
@@ -2126,7 +2129,9 @@ estimate_drift(XYZDriftControls *controls, GwyXYZ *points, GwyXYZ *corpoints, gi
        iteration++;
     } while (iteration<controls->args->iterations);
 
+fail:
     gwy_app_wait_finish();
+    /* XXX: And if user cancelled the operation, what happens now? */
 
     rdata->xdrift_b_result = bx;
     rdata->xdrift_c_result = cx;
@@ -2270,14 +2275,8 @@ xyzdrift_do(XYZDriftData *rdata,
                                 FALSE);
     gwy_data_field_set_xoffset(dfield, args->xmin);
     gwy_data_field_set_yoffset(dfield, args->ymin);
-    gwy_serializable_clone(G_OBJECT(gwy_surface_get_si_unit_xy(surface)),
-                           G_OBJECT(gwy_data_field_get_si_unit_xy(dfield)));
-    gwy_serializable_clone(G_OBJECT(gwy_surface_get_si_unit_z(surface)),
-                           G_OBJECT(gwy_data_field_get_si_unit_z(dfield)));
-
-
-    gwy_data_field_average_xyz(dfield, NULL,
-                          rdata->corpoints, rdata->npoints);
+    gwy_surface_copy_units_to_data_field(surface, dfield);
+    gwy_data_field_average_xyz(dfield, NULL, rdata->corpoints, rdata->npoints);
 
     //printf("interpolated through %d points to %d x %d\n", rdata->npoints, args->xres, args->yres);
 
