@@ -1,5 +1,5 @@
 /*
- *  $Id: xyz_raster.c 20724 2018-01-16 14:03:47Z yeti-dn $
+ *  $Id: xyz_raster.c 21301 2018-08-08 14:30:28Z yeti-dn $
  *  Copyright (C) 2016-2018 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -445,7 +445,7 @@ xyzras_dialog(XYZRasArgs *args,
     gwy_data_view_set_base_layer(GWY_DATA_VIEW(controls.view), player);
 
     gtype = g_type_from_name("GwyLayerRectangle");
-    vlayer = GWY_VECTOR_LAYER(g_object_newv(gtype, 0, NULL));
+    vlayer = GWY_VECTOR_LAYER(g_object_new(gtype, NULL));
     gwy_vector_layer_set_selection_key(vlayer, "/0/select/rectangle");
     gwy_data_view_set_top_layer(GWY_DATA_VIEW(controls.view), vlayer);
     selection = gwy_vector_layer_ensure_selection(vlayer);
@@ -965,10 +965,7 @@ xyzras_do(XYZRasData *rdata,
                                 FALSE);
     gwy_data_field_set_xoffset(dfield, args->xmin);
     gwy_data_field_set_yoffset(dfield, args->ymin);
-    gwy_serializable_clone(G_OBJECT(gwy_surface_get_si_unit_xy(surface)),
-                           G_OBJECT(gwy_data_field_get_si_unit_xy(dfield)));
-    gwy_serializable_clone(G_OBJECT(gwy_surface_get_si_unit_z(surface)),
-                           G_OBJECT(gwy_data_field_get_si_unit_z(dfield)));
+    gwy_surface_copy_units_to_data_field(surface, dfield);
 
     if ((gint)args->interpolation == GWY_INTERPOLATION_FIELD) {
         if (window)
